// $Id: DJAgentImpl.java,v 1.2 2000/05/19 19:55:25 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.
package net.hivecell.hive.agent;

import org.w3c.dom.*;
import net.hivecell.hive.description.HiveXML;

import net.hivecell.hive.*;
import net.hivecell.hive.agent.*;
import net.hivecell.hive.agent.EventTranslatingAgentImpl;
import net.hivecell.hive.agent.cell.CellControlAgent;
import net.hivecell.hive.agent.desktop.MusicPlayingAgent;
import net.hivecell.hive.agent.desktop.MusicUIAgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.cell.*;
import net.hivecell.hive.event.*;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.shadow.*;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.MalformedHiveURLException;

import java.lang.*;
import java.util.*;
import java.io.*;
import java.rmi.*;

public class DJAgentImpl
extends EventReceivingAgentImpl 
implements Signalable, DJAgent {

    /**
     * the handle to the dj shadow
     */
    private transient DJShadow shadow;

    /**
     * the song map we are going to use
     */
    protected Hashtable agentSongMap = new Hashtable();

    /**
     * the next song we are going to play
     */
    protected String nextSong = null;
    
    /**
     * whether we are in continuous mode or not
     */
    protected boolean continuous = false;

    /**
     * whether the shadow is playing a song now
     */
    protected boolean playingSongNow = false;

    /**
     * i have no clue what this variable is -- raffi 
     */
    protected Enumeration agentRequestKeys = null;

    /** 
     * What songs can this agent play? 
     */
    protected Vector playableSongList = null;

    /**
     * some hack from nelson
     */
    private final boolean useDescGunk = false;    // nelson hack

    
    /**
     * the constructor 
     *
     * @exception RemoteException if something goes wrong
     */
    public DJAgentImpl() 
    throws RemoteException { 
	super(); 
    
    }

    /**
     * get the handle back to our cell
     *
     * @return the cell that we are on
     */
    public RemoteCell getCell() {
	return myCell;

    }

    /**
     * we use this to return a RemoteDJAgentImpl which is configured
     * to connect right back to us for control over us
     *
     * @param requester the agent that is asking for the remote control
     * @param destination the cell that the remote control should be stuck on
     * @return the remote control agent that we can get the handle to
     */
    public Agent sendUIAgent( Agent requester, CellAddress destination ) {
	try {
	    MusicUIAgentImpl muai = new MusicUIAgentImpl();
	    muai.setMusicPlayingAgent( (MusicPlayingAgent)myCell.getRemoteObject( this ) );
	    
	    Debug.notice( "shipping the music ui agent to " + destination );
	    return myCell.moveAgent( muai, destination );

	} catch( Exception error ) {
	    Debug.error( "problem instantiating and serializing a remote interface to send over", error );

	}
	return null;

    }

    /**
     * set up our data structures
     *
     * @exception AgentInitializationException will never be thrown
     */
    public void doLocalSetup() 
    throws AgentInitializationException  {
        super.doLocalSetup();
	agentRequestKeys = agentSongMap.keys();

    }
    
    /**
     * actually do stuff -- like get our handle to the shadow
     */
    public void doBehavior() {
	// Don't let it run without a description
        if( useDescGunk ) {
            if( this.getDescription() == null ) {
                Debug.error( "The DJ Agent needs a description to run." );
                myCell.killAgent( (AgentImpl)this );

            }

        }

	try {
	    shadow = (DJShadow)myCell.getShadow( (DJShadowImpl.class).getName() );
	    shadow.addListener( this );

	} catch( ShadowNotFoundException error ) {
	    Debug.error( "could not bind ourselves to a dj shadow" );
	    myCell.killAgent( (AgentImpl)this );
	    return;

	}

    }

    /** 
     * Load a song list from the file. Current format is
     * <?xml version="1.0"?>
     * <config>
     *   <songList>
     *     <song>~nelson/hive/mp1/1.mp3</song>
     *     <song>~nelson/hive/mp1/2.mp3</song>
     *     <song>~nelson/hive/mp1/3.mp3</song>
     *   </songList>
     * </config>
     * Would be nice to structure this more, put in title and stuff.
     *
     * @param doc the document that we are loading from
     */
    public void configure( Document doc ) {
        if( playableSongList != null ) {
            Debug.warning( "Someone configured DJ agent twice!" );
            return;

        }

	Node base = HiveXML.selectFirst( doc, "config" );
        Node list = HiveXML.selectFirst( base, "songList" );
	NodeList songs = HiveXML.select( list, "song" );
        playableSongList = new Vector( songs.getLength() + 1 );
	for( int i=0;i<songs.getLength();i++ ) {
	    Node current = songs.item(i);
            String file = HiveXML.getFirstNonWhitespaceChild( current ).getNodeValue();
            playableSongList.addElement( file );

        }

    }

    /**
     * tell us whether or not the shadow is playing a song
     *
     * @return true if the shadow is playing a song
     */
    public boolean getPlayingSong() {
        return shadow.getPlayingSong();

    }

    /**
     * the signal pass through from the shadow
     *
     * @param source the source of our signal -- its coming from the dj shadow
     */
    public void signal( EventPublisherShadow source ) {
	signal( source, null );

    }

    /**
     * if we get a signal, then we are finished playing, so we should
     * send the next song out
     *
     * @param source the source of the signal
     * @param info the information that was sent along with the signal 
     */
    public void signal( EventPublisherShadow source, Object info ) {
        if( ( info != null ) && ( info instanceof String ) ) {
            String report = (String)info;
            if( !shadow.getPlayingSong() ) {
                Debug.notice( "Calling getNextSong" );
                nextSong = getNextSong();
                if( nextSong != null )
                    shadow.playSong( nextSong );

            }    

        }

    }

    /**
     * return what the next song is we are going to play
     *
     * @return the name of the next song
     */
    private String getNextSong() {
        String nextsong = null;
        if( !agentRequestKeys.hasMoreElements() ) 
            agentRequestKeys = agentSongMap.keys();  // Will this reset the enumerator?

        if( agentRequestKeys.hasMoreElements() ) {
            Object requestKey = agentRequestKeys.nextElement();
            Vector songList = (Vector)agentSongMap.get( requestKey );
            if( songList.isEmpty() ) {
                Debug.warning( "Song list for agent is empty, but not removed from the agentSongMap!" );
                return null ;

            }

	    Debug.notice( "songList = " + songList );
	    nextsong = (String)songList.firstElement();
	    Debug.notice( "nextsong = " + nextsong );
                
	    songList.removeElementAt( 0 );
	    if( songList.isEmpty() ) {
		agentSongMap.remove( requestKey );
		Debug.notice( "Removed agent request list 'cause we played all their songs" );

	    }

        }

        return nextsong;

    }

    /**
     * add a song to be played
     *
     * @param requestor the agent that wants the song added
     * @param add the song that we want added
     */
    private void addNextSong( Agent requestor, String song ) {
        Vector songList = null;
        songList = (Vector)agentSongMap.get( requestor );
        if( songList == null ) {
            songList = new Vector();
            songList.addElement( song );
            agentSongMap.put( requestor, songList );
            Debug.notice( "adding new agent " + requestor + ", with song " + song );

        } else {
            songList.addElement( song );
            Debug.notice( "appending song " + song + " for agent " + requestor );

        }

    }

    /**
     * request a song to be played
     *
     * @param requestingAgent the agent erquesting teh song
     * @param song the song that we are adding
     */
    public void requestSong( Agent requestingAgent, String song ) {
        if( useDescGunk ) {
            Description DJDesc = this.getDescription();
	    if( DJDesc.selectFirst( "semantic" ) ) {  // If lazy and don't care about accuracy, could just use a findFirst("room"), but then you'd be evil.
		DJDesc = DJDesc.getMatch();
		if( DJDesc.selectFirst( "location" ) ) {
		    DJDesc = DJDesc.getMatch();
		    if( DJDesc.selectFirst( "room" ) ) {
			DJDesc = DJDesc.getMatch();
			Debug.notice( "[REQUESTING <" + song + "> SONG IN ROOM <" + DJDesc.getText() + ">]" );

		    }

		}

	    }

        } else 
            Debug.notice( "[REQUESTING <" + song + "> SONG" );

        
        if( song != null ) {
            if( !shadow.getPlayingSong() )    // If no one's waiting, just play it
                shadow.playSong( song );

            else
                addNextSong( requestingAgent, song );

        }

    }

    /**
     * remvoe all the requests froma  certain agent
     *
     * @param requestingAgent the agent that is requesting its stuff be removed
     */
    public void removeRequests( Agent requestingAgent ) {
        if( agentSongMap.containsKey( requestingAgent ) ) {
            agentSongMap.remove( requestingAgent );
            Debug.notice( "[Removing remaining song requests from <" + requestingAgent + ">]" );

        }

    }        

    /**
     * stop the dj shadow from doing anything
     *
     * @param requestingAgent the agent that is requesting the stop
     */
    public void stopDJ( Agent requestingAgent ) {
        shadow.stop();

    }
    
    /**
     * we got an event from another agent
     *
     * @param event the event that we received
     */
    public void notify( HiveEvent event ) {
        super.notify( event );
        String song = (String)event.getEventData();
        Agent requestingAgent = (Agent)event.getSource();
        requestSong( requestingAgent, song );

    }

    /**
     * play this song
     *
     * @param song the song that we want played
     */
    public void play( Object song ) {
	if( song == null ) return;
	if( song instanceof String ) {
	    myCell.report( this, "Now Playing: " + song );
	    requestSong( this, (String)song );

	} else 
	    Debug.error( "Error: not a string!" );

    }

    /**
     * stop playing
     */
    public void stop() {
	myCell.report( this, "Now stopping" );
	stopDJ( this );
    }

    /**
     * pause the shadow
     */
    public void pause() {
	myCell.report( this, "PAUSING NOT YET IMPLEMENTED" );

    }

    /**
     * get the song list of thigns we can play
     *
     * @return a vector of the songs
     */
    public Vector getSongList() {
	if( playableSongList == null ) 
	    return new Vector();
        return playableSongList;

    }

    /**
     * get the song title as keyed by the object of the song
     *
     * @param song the song that we want the title of
     */
    public String getSongTitle( Object song ) {
	String ret = (String)song;
	int slashIndex = ret.lastIndexOf( '/' );
	int dotIndex = ret.lastIndexOf( '.' );
	ret = ret.substring( slashIndex + 1, dotIndex );
	return ret;

    }

}

