// $Id: ChatAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/**
 ** $Id: ChatAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
 ** Moderately more advanced chat system, with chat groups managed
 ** by servers.  Allows agents to send private messages to subsets 
 ** of the groups.
 **/
package net.hivecell.hive.agent.desktop;

import java.util.*;
import java.awt.*;
import java.rmi.RemoteException;
import java.io.*;
import java.awt.event.ActionEvent;

import org.w3c.dom.*;
import com.docuverse.dom.*;

import net.hivecell.hive.*;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.ActionCommand;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.TwoWayConnectionAgentImpl;
import net.hivecell.hive.agent.desktop.ChatAgent;
import net.hivecell.hive.agent.desktop.ChatMessage;
import net.hivecell.hive.agent.desktop.ChatDisplayContainerNoSwing;
import net.hivecell.hive.agent.desktop.ChatDisplayContainerSwing;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.event.HiveEvent;


/*
  TODO:
  (later)
  VH1 get rid of slowdown in html display as messages get longer
  H2 handle servers suspending (somehow manually do time outs? or ask for notification of received?)

  Bugs to Fix:
  H1 re-fix the scrollbar bug (comes back after the SB is scrolled up sometimes)  ****SEE BELOW****
  RELATED(?) Screen freezes when selecting <public> after being on private.  This causes the Swing one's scrollbar act up too (see H3)
    b Actually occurs second time you use select, even if you don't change it from public either time.
    z Think this might be an AWT bug.  Remove listener, still has problem.  
  MH3 Odd JButton->Hashtable null pointer in Swing, (Last one is always ok, it is just the first 2 that aren't.  Thread unsafety again?)
      b actually occurs in AgentThreadGroup.java:18, might require a Thread.sleep()
  E4 Are the UID's correct in the warnings given to the CA's when another CA cuts out?
  M5 CGSAI *still* doesn't format non-chat data correctly
  E6 when try to connect 2 CA's that are already connected, throws null pointer.  Shouldn't let it happen, but fix exc.

  E7 AUGHHHHH! It frikkin February.  Anyway, the swing mode doesn't render colors correctly.  After it was switched to sending private,
     It rendered everything as if it were receiving private messages.  Bad. Probably simple, but damned annoying.

  NOTE: Description for an agent is never null anymore.  How set it in constructor?

  Features to Add:
  1) begin group chat security: CGSAIs only allow certain users from certain hosts, or any user from a certain host to connect.
  1a) Implement by having ChatAgent pass a ChatSecurity Object (for now just containing user name) to CGSAI.  Later replace with Crypto.
  2) For script: have make newAgent generate RDF files, don't do it automatically.  (if user wants to change someone else's default, don't
      overwrite that every time you compile.
  NNN 3) DON'T WORRY ABOUT FOR NOW:  Add cut and paste capabilities to Swing window.


  BUGS NOT IN CA:
  ?0 Labyrinth agent: on startup, PropertySheet.java cannot find property editor for 'description'.  Looks similar to 'chatname, chatName'
     problem.  If so, it might be another bug in Description I need to fix.
  E1 hiveui.AgentGraphCanvas.mouseDragged(), throws a bunch of null's (if mouse dragged over area with no icons?)
     Happens when holding click mouse2 on an agentIcon, then click mouse1 and move the mouse.
  H1 In Description: Constructor is called twice: when a Description is created from a file, and when the Agent's description is set to 
     be that description.  Not fatal, but slow and redundant.  Change setDescription() in AgentImpl to use clone() if possible?
*/

/*RDF*
<?xml version="1.0"?>
<rdf:RDF xmlns="http://www.media.mit.edu/hive-agent-syntax#"
         xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	 xmlns:menu='http://www.media.mit.edu/hive-syntax#'>
<rdf:Description about="net.hivecell.hive.agent.desktop.ChatAgentImpl">

	<name>null</name>
	<iconFile>null</iconFile>
	<mode>noswing</mode>
	<menuString>Chat Agent</menuString>    <!-- not used yet -->

	<sendPriv>GRAY:I</sendPriv>            <!-- Color:Style -->
	<sendPub>BLACK:PLAIN</sendPub>
	<recPriv>GREEN:I</recPriv>
	<recPub>BLUE:B</recPub>

</rdf:Description>
</rdf:RDF>
*RDF*/


public class ChatAgentImpl extends TwoWayConnectionAgentImpl implements ChatAgent {
    protected transient ChatDisplayContainer cdc;
    protected transient ComponentManagerShadow acms;
    protected boolean useSwing = true;

    protected String chatUserName = null;
    protected static int numChatAgents = 0;
    protected int UID = -1;
    protected Frame chatFrame = null;

    protected ChatGroupServerAgent myChatServer = null;
    protected Vector privateList = null;
    protected Vector connectedList = null;  // The other ChatAgents this is connected to, either directly or thru a chatServer.

    protected Hashtable textColor;
    protected Hashtable textStyle;


    public ChatAgentImpl() throws RemoteException {

	super();

	textColor = new Hashtable();
        textStyle = new Hashtable();

	UID = numChatAgents++;

	privateList = new Vector();          // Will all connections break? if yes, might want to put these in doLocalSetup().
	connectedList = new Vector();
	chatUserName = System.getProperty("user.name");  // Default name

	textColor.put("sPrivate", "GRAY");
	textColor.put("sPublic", "BLACK");
	textColor.put("rPrivate", "GREEN");
	textColor.put("rPublic", "BLUE");

	textStyle.put("sPrivate", "I");
	textStyle.put("sPublic", "PLAIN");
	textStyle.put("rPrivate", "I");
	textStyle.put("rPublic", "B");

	// NOTE: DO THIS AFTER CONVERTED AGENT!
	// NOTE: This is bad.  Description for an agent is never null anymore.  What to do?
	Description desc = this.getDescription();  // Has the description already been set on startup?  If yes (desc != null) use that to config.
	if (true) { //desc == null) {
	    // Try and open user's config file.
	    String fs = System.getProperty("file.separator");
	    String fileName = System.getProperty("user.home") + fs + ".hive" + fs + "agent" + fs + "ChatAgentImpl.xml";
	    
	    
	    InputStream is = null; // Try to open the user's config file.
	    try {
		is = new FileInputStream(fileName);
		
	    } catch (IOException e) {
		Debug.notice("Personalized ChatAgent configuration file '" + fileName + "' could not be opened.  Trying default.");
		is = getClass().getResourceAsStream("xml" + fs + "ChatAgentImpl.xml"); // Try to open the base config file.
		if (is == null)
		    Debug.notice("Configuration files " + fileName + 
				 " and ~/.hive/ChatAgentImpl.xml not found.  Using system defaults.");
		else fileName = "xml" + fs + "ChatAgentImpl.xml";
	    }
	    
	    // Read the config file if you got it.
	    if (is != null)
		{
		    DOM dom = new DOM();
		    Document doc = dom.openDocument(is);
		    desc = new Description(this, doc);

		    setDescription(desc);

		    desc.addSemantic("role", "HTML enhanced text chat, with private and group messaging", null, null);
		}
	}

	Description config = desc.configContext();
	if (config != null) {
	    configureConfig(desc);
	    configureStyle(desc);
	}



	// Allow the agent to create a new CGSA from its popup menu and connect to it.
	addActionCommand(new ActionCommand() {
	    public String getName() { return ("Create a CGSA..."); }
	    
	    public void invoke() {
		Debug.notice("Creating a Chat Group Server Agent...");
		AgentImpl cgsa  = null;
		try {
		    cgsa = myCell.createNewAgent("net.hivecell.hive.agent.desktop.ChatGroupServerAgentImpl");
		    cgsa.blockUntilReady();
		    connectTo(cgsa);
		} catch (Exception ex) { 
		    Debug.notice("Creation of CGSA failed.", ex);
		}
	    }
	});
    }
    
    
    public void doLocalSetup() throws AgentInitializationException 
    {
	super.doLocalSetup();
	
	//   MOVED CONFIG FROM RDF TO CONSTRUCTOR TO MAKE LOADING OF ICON THRU XML FILE POSSIBLE.

	try {
	    acms = getComponentManagerShadow();
	} catch (Exception e) {
	    throw new AgentInitializationException("Can't Create a ChatAgent without a GUI!");
	};

	if (useSwing)  // True == SwingMode
	    cdc = new ChatDisplayContainerSwing(this);
	else
	    cdc = new ChatDisplayContainerNoSwing(this);

	chatFrame = acms.acceptComponentFrom(this, (Component)cdc, "HIVEChat: " + chatUserName);
	chatFrame.setLocation(500, 0);
    }


    public void doBehavior()
    {
	/* This tests the querySet stuff.
	AgentSet set = null;
	try {
	    set = myCell.queryAgents(this, null, null);
	} catch (ClassNotFoundException e) {
	    Debug.failure("Didn't work.");
	    myCell.killAgent( (Agent)this );
	}

	String[] synt = { "net.hivecell.hive.agent.desktop.ChatAgentImpl" };
	String[] sems = { "myName", "shoNuff" };
	String[] semVals = { "Silent Like Thunder", "This is the answer!" };
	
	set = set.queryValues(synt, sems, semVals);

	Vector names = set.getNames();
	Enumeration ags = names.elements();
	while (ags.hasMoreElements()) {
	    String ag = (String)ags.nextElement();
	    Debug.failure("Agent name: " + ag + " has tag <myName> with value 'Silent Like Thunder'." + 
			  " and tag named <shoNuff> with value 'This is the answer!'");
	}
	*/

	waitUntilDeath();
    }
    
    
    public void configure(Description desc) {
	if (desc == null) return;  // This should never happen, but be careful.
	desc = desc.configContext();

	if (desc != null) {
	    //	    Description base = desc.getMatch();

	    //	    configureConfig(base);
	    //	    configureStyle(base);
	    configureConfig(desc);
	    configureStyle(desc);
	}	
    }
    

    public boolean connectTo(Agent otherAgent)
    {
	boolean connected = false;
	try {
	    
	    // Currently can only subscribe to one chat Server at a time
	    if (otherAgent instanceof ChatGroupServerAgent)
		{
		    // First, check to make sure this is not connected to any agent which connected connected to this CGSA:
		    Vector serverChatAgents = ((ChatGroupServerAgent)otherAgent).getChatAgents();
		    for (int i = 0; i < connectedList.size(); i++)
			{
			    for (int j = 0; j < serverChatAgents.size(); j++)
				{
				    if (serverChatAgents.elementAt(j).equals(connectedList.elementAt(i)))
					{
					    Debug.warning("Cannot connect to server, " + 
							  ((ChatAgent)connectedList.elementAt(i)).getChatName() + 
							  " is connected to " + chatUserName + 
							  " (this) and the server.  Disconnect from it first.");
					    return false;
					}
				}
			}

		    if (myChatServer == null)
			connected = super.connectTo(otherAgent);
		    else 
			{
			    // Allow previous chatServer to update this and everyone else subscribed to it:
			    disconnectFrom(myChatServer);
			    connected = super.connectTo(otherAgent);		
			}
		    if (connected)
			myChatServer = (ChatGroupServerAgent)otherAgent;
		}

	    else
		{
		    if (!connectedList.contains(otherAgent))
			{
			    connected = super.connectTo(otherAgent);
			    if (connected && (otherAgent instanceof ChatAgent))
				{
				    addToConnectedList((ChatAgent)otherAgent);
				    ((ChatAgent)otherAgent).addToConnectedList(this);
				}
			}
		    else Debug.warning("Already connected to " + ((ChatAgent)otherAgent).getChatName());
		}
	} catch (RemoteException e) { Debug.error("Remote Exception in connectTo(): ", e); }

	return (connected);
    }


    public void disconnectFrom(Agent otherAgent)
    {
	super.disconnectFrom(otherAgent);

	if (otherAgent instanceof ChatAgent)
	    {
		try {
		    if (myChatServer != null && myChatServer.isAgentConnected(otherAgent) == false)
			{  // Don't remove it from your private list if you are connected to it on a ChatGroupServer.
			    //  !!!!!!!!!!!!!!!!!!!!!!! THIS (/|\)SHOULD NEVER OCCUR !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			    removeFromConnectedList((ChatAgent)otherAgent);
			    ((ChatAgent)otherAgent).removeFromConnectedList(this);
			}
		    else if (connectedList.contains(otherAgent)) // !!!!!!!!!! THIS SHOULD ALWAYS BE TRUE !!!!!!!!!!!!!!!
			{
			    removeFromConnectedList((ChatAgent)otherAgent);
			}
		} catch (RemoteException e) { Debug.error("Remote Exception in disconnectFrom()", e); }
	    }

	else if (otherAgent.equals(myChatServer))
	    myChatServer = null;
    }
        


    public void eventSendingError( Object o, Exception error ) {
	super.eventSendingError(o, error);
	String who = "ChatGroupServerAgent";
	if (o instanceof ChatAgent)
	    {
		try {
		    who = removeFromConnectedList((ChatAgent)o);
		} catch (RemoteException e) { Debug.error("Caught a remote exception in eventSendingError.  Surprising.", e); }
	    }
	
	else if (o.equals(myChatServer))
	    myChatServer = null;  // It will find out that it can't contace the other agents publically when it tries to message them.
	
	cdc.displayOutgoingMessage("<FONT COLOR=RED><B>WARNING: MESSAGE NOT RECEIVED BY<BR>" + who + " </B></FONT>");
    }

    

    // Dumb functions:
    public String getChatName() throws RemoteException { return chatUserName; }
    public int getUID() throws RemoteException { return (UID); }


    
    /** Adds otherAgent to the list of ChatAgents it chats with privately.
     **/  
    public void addToPrivateList(ChatAgent otherAgent) throws RemoteException
    {
	if (connectedList.contains(otherAgent))
	    {
		if (privateList.contains(otherAgent))
		    Debug.warning("Attempt to re-add current member, " + otherAgent + " to " + chatUserName + "'s privateList.");
		else privateList.addElement(otherAgent);
	    }
	else Debug.warning("Agent " + otherAgent + " not connected, cannot be private.");
    }
    
    
    
    /** Removes otherAgent from the list of ChatAgents it chats with privately.
     **/    
    public void removeFromPrivateList(ChatAgent otherAgent) throws RemoteException
    {
	if (connectedList.contains(otherAgent))
	    {
		if (privateList.contains(otherAgent))
		    privateList.removeElement(otherAgent);
		else 
		    Debug.warning("Attempting to remove non-private member " + otherAgent + " from private list.");
	    }
	else 
	    Debug.warning("Attempting to remove agent from private list that " + chatUserName + " isn't connected to.");
    }

    public void clearPrivateList() throws RemoteException { privateList.removeAllElements(); }


    /**
     ** Updates the list of agents this is connected to thru a ChatGroupServerAgent or by a 
     ** manual connection to another ChatAgent. 
     ** Currently only ChatAgent's and descendent types can connect. This may or may not
     ** need to be changed.
     **/
    public void addToConnectedList(ChatAgent newAgent) throws RemoteException
    {      
	if (!connectedList.contains(newAgent))
	    {
		connectedList.addElement(newAgent);
		cdc.addChatAgent(newAgent);
		chatFrame.pack();
	    }
    }
    

    public String removeFromConnectedList(ChatAgent oldAgent) throws RemoteException
    {
	String who = "unknown";
	
	if (connectedList.contains(oldAgent))
	    {
		connectedList.removeElement(oldAgent);
		who = cdc.removeChatAgent(oldAgent);
		chatFrame.pack();
	    }
	
	// Also remove the now-absent agent from your private list:
	if (privateList.contains(oldAgent))
	    privateList.removeElement(oldAgent);

	return (who);
    }

    
    
    
    public static boolean isChatAgent(Object CAWannabe) throws RemoteException 
    {
	try { // Attempt a cast
	    if (((ChatAgent)CAWannabe) == null);
	} catch (ClassCastException e) { return (false); }
	return true;
    }
    
    
    
    public void send(String s)
    {
	if (!privateList.isEmpty())
	    {
		for (int i = 0; i < privateList.size(); i++)
		    { 
			try { 
			    if (isChatAgent(privateList.elementAt(i)))
				{ 
				    if (privateList.elementAt(i) != null)
					{
					    try {
						EventReceivingAgent[] destinationAgent = { (ChatAgent)privateList.elementAt(i) };
						eventSendingHelper.broadcastEventTo( new ChatMessage( this, s, true ), destinationAgent, null );
										    
					    } catch (RemoteException e) { Debug.warning("Message not successfully created: ", e); }
					}
				}
			} catch (RemoteException e) { Debug.warning("Remote Exception thrown here in send: ", e); }
		    }
		cdc.displayOutgoingMessage(formatOutgoingChatString(chatUserName, s, true));
	    }	
	else 
	    {
		try {
		    eventSendingHelper.broadcastEvent(new ChatMessage(this, s, false));
		    cdc.displayOutgoingMessage(formatOutgoingChatString(chatUserName, s, false));
		} catch (RemoteException e) { Debug.warning("Message not successfully created: ", e); }
	    }
    }
  
  

  public synchronized void notify(HiveEvent event)
    {
      super.notify(event);
      // Make sure not to display a message that's coming from itself (it is echoed by default):

      if (event.getEventData() instanceof ChatMessage)
	  {  
	      ChatMessage msg = (ChatMessage)event.getEventData();
	      try {
		  // ie, it WAS a ChatAgent and it WASN'T this one, then read the message
		  if ( !(( isChatAgent(msg.getSender()) )  &&  ( ((ChatAgent)msg.getSender()).equals(this) )) )
		      cdc.displayIncomingMessage( formatIncomingChatString(msg.getSName(), msg.read(), msg.isPrivate()) );
		  
	      } catch (RemoteException e) { Debug.error("Remote Exception while trying to notify:", e); }
	  }
      else cdc.displayIncomingMessage("(Non-ChatMessage data): " + (event.getEventData()).toString());
    }





    /** 
     ** Convenience function to format a string for display on the screen.
     **/
    private String formatOutgoingChatString(String sender, String message, boolean isPrivate)
    {
	if (isPrivate)
	    return ("<FONT COLOR=\"" + textColor.get("sPrivate") + "\"><" + textStyle.get("sPrivate") + ">" +
		    sender + ": " + message + "</" + textStyle.get("sPrivate") + "></FONT><BR>");
	
	else
	    return ("<FONT COLOR=\"" + textColor.get("sPublic") + "\"><" + textStyle.get("sPublic") + ">" +
		    sender + ": " + message + "</" + textStyle.get("sPublic") + "></FONT><BR>");
    }

    private String formatIncomingChatString(String sender, String message, boolean isPrivate)
    {
	if (isPrivate)
	    return ("<FONT COLOR=\"" + textColor.get("rPrivate") + "\"><" + textStyle.get("rPrivate") + ">" +
		    sender + ": " + message + "</" + textStyle.get("rPrivate") + "></FONT><BR>");
	
	else
	    return ("<FONT COLOR=\"" + textColor.get("rPublic") + "\"><" + textStyle.get("rPublic") + ">" +
		    sender + ": " + message + "</" + textStyle.get("rPublic") + "></FONT><BR>");
    }



    /**
     ** Convenience function to initialize this ChatAgent from the config file.
     **/
    private void configureConfig(Description desc) {
	
	if (desc.findFirst("name", Description.ANYWHERE)) { // Get the chatName
		Description match = desc.getMatch();
		String name = match.getText();
		if (name != null && !name.equals("null"))
		    chatUserName = name;
	}
	
	if (desc.findFirst("iconFile", Description.ANYWHERE)) { // Get the icon
		Description match = desc.getMatch();
		String name = match.getText();
		if (name != null && !name.equals("null"))
		    setIconName("icons/" + name);
	}

	if (desc.findFirst("mode", Description.ANYWHERE)) {  // Swing mode or AWT mode?
	    Description match = desc.getMatch();
	    String name = match.getText();

	    if (name != null && !name.equals("null")) {
		if (name.equals("noswing"))
		    useSwing = false;
	    }
	}
	
	/*  Old way:
	set.upContext();
	set = set.select(Description.HIVE_AGENT + "mode"); // Get the icon
	if (set.matches() > 0)
	    {
		if (!set.getValue(0).equals("null"))
		    {
			if (set.getValue(0).equals("noswing"))
			    useSwing = false;
		    }
	    }
	*/
    }


    private void configureStyle(Description desc) {

	if (desc.findFirst("sendPriv", Description.ANYWHERE)) {  // Send Private style
	    Description match = desc.getMatch();
	    String val = match.getText();
	    
	    if (val != null && !val.equals("")) {
		int col = val.indexOf(':');
		if (col != -1) {
		    textColor.put("sPrivate", val.substring(0, col));
		    textStyle.put("sPrivate", val.substring(col + 1));
		}
		else {
		    textColor.put("sPrivate", val);
		    textStyle.put("sPrivate", "INVALIDTAG");  // Won't be rendered.
		}
	    }
	}

	if (desc.findFirst("sendPub", Description.ANYWHERE)) {  // Send Public style
	    Description match = desc.getMatch();
	    String val = match.getText();

	    if (val != null && !val.equals("")) {
		int col = val.indexOf(':');
		if (col != -1) {
		    textColor.put("sPublic", val.substring(0, col));
		    textStyle.put("sPublic", val.substring(col + 1));
		}
		else {
		    textColor.put("sPublic", val);
		    textStyle.put("sPublic", "INVALIDTAG");
		}
	    }
	}

	if (desc.findFirst("recPriv", Description.ANYWHERE)) {  // Receive Private style
	    Description match = desc.getMatch();
	    String val = match.getText();

	    if (val != null && !val.equals("")) {
		int col = val.indexOf(':');
		if (col != -1) {
		    textColor.put("rPrivate", val.substring(0, col));
		    textStyle.put("rPrivate", val.substring(col + 1));
		}
		else {
		    textColor.put("rPrivate", val);
		    textStyle.put("rPrivate", "INVALIDTAG");
		}
	    }
	}

	
	if (desc.findFirst("recPriv", Description.ANYWHERE)) {  // Receive Public style
	    Description match = desc.getMatch();
	    String val = match.getText();

	    if (val != null && !val.equals("")) {
		int col = val.indexOf(':');
		if (col != -1) {
		    textColor.put("rPublic", val.substring(0, col));
		    textStyle.put("rPublic", val.substring(col + 1));
		}
		else {
		    textColor.put("rPublic", val);
		    textStyle.put("rPublic", "INVALIDTAG");
		}
	    }
	}
	
	/*  Old way:
	set.upContext();
	set = set.select(Description.HIVE_AGENT + "recPub"); // Receive Public style
	if (set.matches() > 0)
	    {
		String val = set.getValue(0);
		if (!val.equals(""))
		    {
			int col = val.indexOf(':');
			if (col != -1)
			    {
				textColor.put("rPublic", val.substring(0, col));
				textStyle.put("rPublic", val.substring(col + 1));
			    }
			else 
			    {
				textColor.put("rPublic", val);
				textStyle.put("rPublic", "INVALIDTAG");
			    }
		    }
	    }
	*/
    }
};

// TIMING BUG WHEN STARTING A CELL:
// Pauses after <NOTICE HTTPCellShadow.java:93> Blocking for HTTP connection (5 lines before lockup) when NOT working.
// Maybe, I;m not sure. I think so (6/4 - locked up after waiting for a long time after the above notice.)
