// $Id: ChatDisplayContainerNoSwing.java,v 1.1 2000/05/18 22:04:48 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/** Helper class: ChatAgent UI
 **
 ** This should go in the ChatAgent class file, but for now it is being used
 ** in slightly different configurations by the ChatAgentImpl and the
 ** PrivateChatAgentImpl due the the current limitation on agents only being
 ** able to use one frame.  Later, the PrivateChatAgentImpl will
 ** open up a window (A la AOL chat), and this class can be re-streamlined and
 ** put back into ChatAgentImpl.
 **/

package net.hivecell.hive.agent.desktop;

import java.awt.*;
import java.awt.event.*;
import java.awt.Toolkit;

import java.util.*;
import java.rmi.RemoteException;

import net.hivecell.hive.*;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.agent.desktop.ChatAgentImpl;
import net.hivecell.hive.agent.desktop.ChatDisplayContainer;


class ChatDisplayContainerNoSwing extends Container implements ChatDisplayContainer {

    protected ChatAgentImpl chatAgent;
    
    protected TextArea sendText;
    protected TextArea receiveText;
    protected Button Sbutton = new Button("Send Message");
    protected Button Cbutton = new Button("Clear");
    
    protected Choice pChoice = new Choice();
    protected Hashtable agentList = null;  // Hash of the ChatAgents, CGSAI, and ChatGroupLists (not yet implemented) connected to this's chatAgent with key = unique name.
    protected Hashtable agentListStrings = null;  // Hash of the ChatAgents unique id's (agent.toString) and human readable names, in case a server goes down.
    protected GridBagConstraints gbcs = new GridBagConstraints();
    
    
    /** Set up the two text areas: one are to type in
     ** and one to receive messages in.
     ** Also set up default canvas properties.
     **/
    public ChatDisplayContainerNoSwing(ChatAgentImpl c) {  // PASS CONSTRUCTOR CHATGROUPLISTS FROM CAI's CONFIG FILE LATER, MAKE VECTOR OF CHAT FRIENDS, AND TREAT THAT VECTOR THE SAME AS MULTI-PERSON PRIVATE CHATS WERE TREATED BEFORE.
	super();
	
	agentList = new Hashtable();
	agentListStrings = new Hashtable();
        
	chatAgent = c;
	
	setBackground(Color.white);          // Set the defaults
	setFont(Global.defaultFont);
	setSize(100, 100);
	setVisible(true);
	
	// Set up the layout, using the convenience functions below.
	// NOTE: THE ORDER OF THE FUNCTIONS BELOW MATTERS!

	// Panels:  The layout manager containing the text areas, private Choice, and buttons
	initPanels();
        	
	// Incoming and outgoing text areas:
	initTextBoxes();
	
	// Clear and Send button:
	initButtons();
    }
    
    
    private String formatRawText(String s, boolean inc) {
	String textOut = "";
	boolean skipping = false;

	for (int i = 0; i < s.length(); i++)
	    {
		if (s.charAt(i) == '<')
		    {
			if ( (s.charAt(i + 1) == 'B' && s.charAt(i + 2) == 'R') || (s.charAt(i + 1) == 'P' && s.charAt(i + 2) == '>') )
			    {
				i += 3;
				textOut += "\n";
				continue;
			    }
			skipping = true;
			if (inc)                     // Is it an incoming message?
			    {
				if (s.charAt(i + 1) == 'B')
				    textOut += '*';                 // receivePub (never <BR>, because continue'ing if find <BR> above)
				else if (s.charAt(i + 1) == 'I')    // receivePriv
				    textOut += '#';
			    }
			else
			    {
				if (s.charAt(i + 1) == 'P')         // sendPub  (1st letter of 'PLAIN')
				    textOut += '=';
				else if (s.charAt(i + 1) == 'I')    // sendPriv
				    textOut += '$';
			    }
		    }

		else if (skipping == false)
		    textOut += s.charAt(i);

		else if (s.charAt(i) == '>')
		    skipping = false;
	    }
	return (textOut + "\n");
    }

    /** Display a message sent to this chat agent.
     **/
    public void displayIncomingMessage(String s) { 

	receiveText.appendText(formatRawText(s, true)); 
	Toolkit.getDefaultToolkit().beep();
    }
    
    public void displayOutgoingMessage(String s)
    {
	receiveText.appendText(formatRawText(s, false));
	sendText.setText("");
    }
    
    
    /** 
     ** Adds a chat agent to the agentList(Strings) and to pChoice, so that chatAgent can select who it wants
     ** to have a private chat with.
     **/
    public void addChatAgent(ChatAgent agent)
    {
        String cbName = null;
        int rport = -1;
	    
        try {
            rport = agent.getCell().getAddress().getPort();
            cbName = agent.getChatName() + " (" + agent.getUID() + "@" + agent.getCell().getAddress().getHost() +
                (rport == 23235 ? "" : ":" + rport) + ")";
        } catch (RemoteException e) { Debug.error("Unable to get information from server: ", e); }

        agentList.put(cbName, agent);
        agentListStrings.put(agent.toString(), cbName);
        pChoice.addItem(cbName);

    }
    
    

    /** Removes an agent from the connected list (even if the agent died impolitely), and returns its name. */
    public String removeChatAgent(ChatAgent agent)
    {    	
	// Remove it from the hashtables and connected list.
	String ret = (String)agentListStrings.get(agent.toString());
	pChoice.remove(ret);
	agentList.remove(agentListStrings.get(agent.toString()));
	agentListStrings.remove(agent.toString());

	return (ret);
    }




    /** Private functions to initialize UI, and make initialization easier to read. */
    private void initPanels()
    {
	setLayout(new GridBagLayout());
	
	gbcs.gridx = gbcs.gridy = 0;             // Text panel is on left half of frame,
	gbcs.gridheight = gbcs.gridwidth = 5;    // is a 5x5 rectangular grid elements
	gbcs.weightx = gbcs.weighty = 1.0;       // and should expand to fill free space.
	
	pChoice.addItem("<Public: Sending to all Listeners>");
	
	pChoice.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		try {
		    chatAgent.clearPrivateList();
		    if (!((String)e.getItem()).equals("<Public: Sending to all Listeners>"))
			chatAgent.addToPrivateList((ChatAgent)agentList.get((String)e.getItem()));
		} catch (RemoteException f) { 
		    Debug.error("RemoteException updating agent's private list.  Somehow the agent died, but the ui didn't...", f); 
		}
	    }
	});

	
	gbcs.gridx = 2; gbcs.gridy = 6;  // put pChoice right below send button
	gbcs.weightx = gbcs.weighty = 0; // Cannot change size when more space is available
	add(pChoice, gbcs);
    }


    private void initTextBoxes()
    {
	gbcs.fill = GridBagConstraints.BOTH;   // Components with non-zero gbcs.weight(x/y) can expand in both directions
	gbcs.insets = new Insets(5, 5, 5, 5);  // 5 pixel space between components
	
	// Incoming text area:
	receiveText = new TextArea("", 10, 40, TextArea.SCROLLBARS_VERTICAL_ONLY);
	receiveText.setFont(new Font("Times New Roman", Font.BOLD, Global.defaultFont.getSize() + 4));
	receiveText.setEditable(false);
	gbcs.gridx = gbcs.gridy = 0;          // Incoming text displayed at the top
	gbcs.gridwidth = gbcs.gridheight = 4; // specifies default size, but won't matter because
	gbcs.weightx = gbcs.weighty = 1.0;    // will expand in both directions to fill the space
	add(receiveText, gbcs);
	
	
	// Outgoing text area:
	sendText = new TextArea("", 4, 40, TextArea.SCROLLBARS_VERTICAL_ONLY);

	// Hot key support:
	sendText.addKeyListener(new KeyListener() {
	    public void keyPressed(KeyEvent e) {
		
		if (e.getKeyChar() == KeyEvent.VK_ENTER && (e.getModifiers() == KeyEvent.ALT_MASK || e.getModifiers() == KeyEvent.CTRL_MASK))
		    Sbutton.dispatchEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "click, you bitch"));
	    }
	    
	    public void keyReleased(KeyEvent e) {;}
	    public void keyTyped(KeyEvent e) {;}
	}); 


	sendText.setFont(new Font(Global.defaultFont.getName(), Font.PLAIN, Global.defaultFont.getSize() + 2));
	sendText.setEditable(true);
	gbcs.gridx = 0; gbcs.gridy = 4;  // Outgoing text displayed beneath incoming.
	gbcs.gridheight = 2;             // Same width as incoming, not as tall, and don't change weights.
	gbcs.weighty = 0;                // Cannot grow taller, only longer.
	add(sendText, gbcs);
    }


    private void initButtons()
    {
	// "Clear" button
	Cbutton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		sendText.setText("");          // Clear the text from the sendText area.
		sendText.requestFocus();
	    }
	});
	gbcs.gridx = 0; gbcs.gridy = 6;  // put buttons right below outgoing text area.
	gbcs.gridwidth = gbcs.gridheight = 1; // Make buttons default to 1x1 grid element
	gbcs.weightx = gbcs.weighty = 0; // buttons cannot change size when more space is available
	add(Cbutton, gbcs);
	
	// "Send Message" button:
	Sbutton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		chatAgent.send(sendText.getText());
		sendText.requestFocus();
	    }
	});
	gbcs.gridx = 1; gbcs.gridy = 6;  // put buttons right below outgoing text area.
	gbcs.gridwidth = gbcs.gridheight = 1; // Make buttons default to 1x1 grid element
	gbcs.weightx = gbcs.weighty = 0; // buttons cannot change size when more space is available
	add(Sbutton, gbcs);
    }
};









