// $Id: ChatDisplayContainerSwing.java,v 1.1 2000/05/18 22:04:48 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/** Helper class: ChatAgent UI
 **
 ** This should go in the ChatAgent class file, but for now it is being used
 ** in slightly different configurations by the ChatAgentImpl and the
 ** PrivateChatAgentImpl due the the current limitation on agents only being
 ** able to use one frame.  Later, the PrivateChatAgentImpl will
 ** open up a window (A la AOL chat), and this class can be re-streamlined and
 ** put back into ChatAgentImpl.
 **/

package net.hivecell.hive.agent.desktop;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;

import java.util.*;
import java.io.IOException;
import java.rmi.RemoteException;

import net.hivecell.hive.*;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.agent.desktop.ChatAgentImpl;
import net.hivecell.hive.agent.desktop.ChatDisplayContainer;


class ChatDisplayContainerSwing extends Container implements ChatDisplayContainer {
    
    protected ChatAgentImpl chatAgent;

    protected TextArea sendText;  // Use awt here.  Don't need to have it rendered, and get cut and paste automatically.
    protected JTextPane receiveText;
    protected JScrollPane receiveScroll;

    protected JButton Sbutton = null;
    protected JButton Cbutton = null;

    protected Choice pChoice = null;   // Not thread safe as JComboBox.  Do that Later.
    protected Hashtable agentList = null; // Hash of the ChatAgents connected to this's chatAgent with key = unique name.
    protected Hashtable agentListStrings = null;  // Hash of the ChatAgents unique id's (agent.toString) and human readable names, in case a server goes down.
        
    protected GridBagConstraints gbcs = null;
    protected int agentLoc = 1;  // y-component for laying out where the agent checkboxes go as agents join the group
    
    
    // Stuff for the textPane and ScrollPane
    protected HTMLDocument doc = null;
    protected HTMLEditorKit ek = null;
    protected RepaintManager rpm = null; 
    

    // For the swing TextPane to copy stuff to.
    //    protected Clipboard clipboard = null;
    
    // Stuff for the keymaps: the actions a text component can support (Redo this later)
    //    protected Hashtable actions;
    
    /** Set up the two text areas: one are for you to type in
     ** and one for you to receive messages to.
     ** Also set up default canvas properties.
     **/
    public ChatDisplayContainerSwing(ChatAgentImpl c) {
	super();

	Sbutton = new JButton("Send Message");
	Cbutton = new JButton("Clear");
	
	pChoice = new Choice();   // Not thread safe as JComboBox.  Do that Later.
	
	gbcs = new GridBagConstraints();


	//	clipboard = getToolkit().getSystemClipboard();
	
	// Double buffering off:
	//(rpm = RepaintManager.currentManager(this)).setDoubleBufferingEnabled(false);
	// Double buffering on:
	rpm = RepaintManager.currentManager(this);
	
	agentList = new Hashtable();
	agentListStrings = new Hashtable();
	
	chatAgent = c;
	
	setBackground(Color.white);          // Set the defaults
	setFont(Global.defaultFont);
	setSize(100, 100);
	setVisible(true);
	
	// Set up the layout, using the convenience functions below.
	// NOTE: THE ORDER OF THE FUNCTIONS BELOW MATTERS!
	
	// Panels:  The box containing the text areas and buttons, and the box containing the agent checkbox list.
	initPanels();
	
	// Incoming and outgoing text areas:
	initTextBoxes();
	
	// Clear and Send button:
	initButtons();

	// Set up some keyboard bindings to cut & paste, as well as open menu's, etc.
	//	initKeyboardBindings();
    }
    
    
    /**
     ** Utility class to get around Swing's non-thread safety.  This is called to display a string in the
     ** receiveText pane, because when it was called from notify asynchonously, Swing didn't update the Scrollbars correctly.
     ** Currently used in displayIncomingMessage (and thus disp.Outg.Mess. too)
     **/
    private class sendString implements Runnable {
	
	protected String s;
	
	public sendString(String str) { s = str; }
	
	public void run() {
	    doc = (HTMLDocument)receiveText.getDocument();
	    
	    try {
		// Insert HTML rendered text with the HTMLEditorKit:
		ek.insertHTML(doc, doc.getLength(), s, 0, 0, HTML.Tag.HTML);

	    } catch (IOException e) { Debug.error("IOException inserting HTML text.", e);
	    } catch (BadLocationException b) { Debug.error("BadLocationException inserting HTML text.", b); }
	    
	    
	    // YES!!! NO TEXT LAG! AND FLICKERS TO LET YOU KNOW A MESSAGE HAS COME IN! (IF DOUBLE BUFFERING IS OFF. FEATURE, NOT BUG)
	    // Now, try to speed this up somehow (only repaint the visible area, if possible)
	    // Too non-trivial.  Only deal with it if lag time is a big problem.
	    rpm.markCompletelyDirty(receiveScroll.getViewport());
	    
	    JScrollBar vsb = receiveScroll.getVerticalScrollBar();
	    vsb.setValue(vsb.getMaximum());
	}
    }
    
    
    /** Display a message sent to this chat agent.
     ** Just writes the string in the Incoming message portion of the 
     ** chat area.
     **/
    public void displayIncomingMessage(String s)
    {
	sendString srr = new sendString(s);
	SwingUtilities.invokeLater(srr);
	Toolkit.getDefaultToolkit().beep();
    }
    
    
    public void displayOutgoingMessage(String s)
    {	
	sendString srr = new sendString(s);
	SwingUtilities.invokeLater(srr);
	sendText.setText("");
    }
    

    /**
     ** Adds a chat agent to the agentList(Strings) and to pChoice, so that chatAgent can select who it wants
     ** to have a private chat with.
     **/
    public void addChatAgent(ChatAgent agent)
    {
        String cbName = null;
        int rport = -1;
	    
        try {
            rport = agent.getCell().getAddress().getPort();
            cbName = agent.getChatName() + " (" + agent.getUID() + "@" + agent.getCell().getAddress().getHost() +
                (rport == 23235 ? "" : ":" + rport) + ")";
        } catch (RemoteException e) { Debug.error("Unable to get information from server: ", e); }
	    
        agentList.put(cbName, agent);
        agentListStrings.put(agent.toString(), cbName);
        pChoice.addItem(cbName);
    }
    
    
    
    public String removeChatAgent(ChatAgent agent)
    {      
	// Remove it from the hashtables and connected list.
	String ret = (String)agentListStrings.get(agent.toString());
	pChoice.remove(ret);
	agentList.remove(agentListStrings.get(agent.toString()));
	agentListStrings.remove(agent.toString());

	return (ret);
	/*
	// Remove it from the hashtables and connected list.
	pChoice.remove((String)agentListStrings.get(agent.toString()));
	agentList.remove(agentListStrings.get(agent.toString()));
	agentListStrings.remove(agent.toString());
	*/
    }    
    
        
    
    
    
    /**
     ** Convenience functions to initialize UI
     **/
    
    private void initPanels()
    {
	setLayout(new GridBagLayout());
	
	gbcs.gridx = gbcs.gridy = 0;             // Text panel is on left half of frame,
	gbcs.gridheight = gbcs.gridwidth = 5;    // is a 4x4 rectangular grid elements
	gbcs.weightx = gbcs.weighty = 1.0;       // and should expand to fill free space, but doesn't seem to at the moment.

	pChoice.addItem("<Public: Sending to all Listeners>");
	
	pChoice.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		try {
		    chatAgent.clearPrivateList();
		    if (!((String)e.getItem()).equals("<Public: Sending to all Listeners>"))
			chatAgent.addToPrivateList((ChatAgent)agentList.get((String)e.getItem()));
		} catch (RemoteException f) { 
		    Debug.error("RemoteException updating agent's private list.  Somehow the agent died, but the ui didn't...", f);
		}
		sendText.requestFocus();
	    }
	});

	
	gbcs.gridx = 2; gbcs.gridy = 6;  // put pChoice right next to send button
	gbcs.weightx = gbcs.weighty = 0; // Cannot change size when more space is available
	add(pChoice, gbcs);
    }

    /*
    private class SwingChatKeyListener implements KeyListener, ClipboardOwner {
	public void keyPressed(KeyEvent e) {
	    Debug.failure("A key was pressed: " + e.getKeyChar());
	    
	    if (e.getKeyChar() == 'k' || e.getKeyChar() == 'K')
		{
		    String srcData = receiveText.getSelectedText();
		    Debug.failure("src data = " + srcData);
		    if (srcData != null) {
			StringSelection contents = new StringSelection(srcData);
			clipboard.setContents(contents, this);
			// Currently doesn't work, needs a ClipboardOwner in setContents!
		    }
		    
		}
	    else if (e.getKeyChar() == 'p' || e.getKeyChar() == 'P')
		{
		    Transferable content = clipboard.getContents(this);
		    if (content != null)
			{
			    try {
				String dstData = (String)content.getTransferData(DataFlavor.stringFlavor);
				dstText.append(dstData);
			    } catch (Exception r) {
				System.out.println("Couldn't get contents in format: "+
						   DataFlavor.stringFlavor.getHumanPresentableName()); 
			    }
			}		    
		}
	}
	
	public void keyReleased(KeyEvent e) {;}
	public void keyTyped(KeyEvent e) {;}

	/**
	 ** For copy and paste from clipboard.
	 **
	
	public void lostOwnership(Clipboard clipboard, Transferable contents) 
	{
	System.out.println("Clipboard contents replaced");
	}	
	}		
    */

    private void initTextBoxes()
    {
	gbcs.fill = GridBagConstraints.BOTH;   // Components with non-zero gbcs.weight(x/y) can expand in both directions
	gbcs.insets = new Insets(5, 5, 5, 5);  // 5 pixel space between components
	
	doc = new HTMLDocument();
	ek = new HTMLEditorKit();
	
	// Incoming Text area:
	receiveText = new JTextPane();
	receiveText.setFont(new Font(Global.defaultFont.getName(), Font.BOLD, Global.defaultFont.getSize() + 4));
	receiveText.setEditable(false);

	//	receiveText.addKeyListener(new SwingChatKeyListener());
	
	/*
	receiveText.addKeyListener(new KeyListener() {
	    public void keyPressed(KeyEvent e) {
		
		if (e.getKeyChar() == KeyEvent.VK_K && (e.getModifiers() == KeyEvent.CTRL_MASK))
		    {
			String srcData = receiveText.getSelectedText();
			if (srcData != null) {
			    StringSelection contents = new StringSelection(srcData);
			    ChatDisplayContainerSwing.setClipboardContents(contents);
			    // clipboard.setContents(contents, this);
			    // Currently doesn't work, needs a ClipboardOwner in setContents!
			}
			
		    }
	    }
	    
	    public void keyReleased(KeyEvent e) {;}
	    public void keyTyped(KeyEvent e) {;}
	});
	*/
	receiveText.setDocument(doc);
	receiveText.setEditorKit(ek);
	ek.install(receiveText);
	
	receiveText.setContentType("text/html");	
	receiveScroll = new JScrollPane(receiveText);
	receiveScroll.setVerticalScrollBarPolicy(
						 JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	receiveScroll.setPreferredSize(new Dimension(250, 200));
	
	
	gbcs.gridx = gbcs.gridy = 0;          // Incoming text displayed at the top
	gbcs.gridwidth = gbcs.gridheight = 3; // specifies default size, but won't matter because
	gbcs.weightx = gbcs.weighty = 1.0;    // will expand in both directions to fill the space  (should, doesn't work right now...)
	add(receiveScroll, gbcs);
	
	
	// Outgoing text area:
	sendText = new TextArea("", 4, 40, TextArea.SCROLLBARS_VERTICAL_ONLY);

	// Allows for hot key access of Send button, since Sbutton's 'setMnemonic()' doesn't work from an AWT TextArea.
	sendText.addKeyListener(new KeyListener() {
	    public void keyPressed(KeyEvent e) {
		
		if (e.getKeyChar() == KeyEvent.VK_ENTER && (e.getModifiers() == KeyEvent.ALT_MASK || 
							    e.getModifiers() == KeyEvent.CTRL_MASK))
		    Sbutton.doClick(100);
	    }
	    
	    public void keyReleased(KeyEvent e) {;}
	    public void keyTyped(KeyEvent e) {;}
	});
	sendText.setFont(new Font(Global.defaultFont.getName(), Font.PLAIN, Global.defaultFont.getSize() + 2));
	sendText.setEditable(true);
	
	gbcs.gridx = 0; gbcs.gridy = 4;  // Outgoing text displayed beneath incoming.
	gbcs.gridheight = 2;             // Same width as incoming, not as tall.
	gbcs.weighty = 0;                // Cannot grow taller, only longer.
	add(sendText, gbcs);
    }
    
    
    private void initButtons()
    {
	// "Send Message" button:
	Sbutton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		chatAgent.send(sendText.getText());
		sendText.requestFocus();
	    }
	});
	gbcs.gridx = 1; gbcs.gridy = 6;       // put buttons right below outgoing text area.
	gbcs.gridwidth = gbcs.gridheight = 1; // Make buttons default to 1x1 grid element
	gbcs.weightx = gbcs.weighty = 0;      // buttons cannot change size when more space is available
	add(Sbutton, gbcs);
	Sbutton.setMnemonic(KeyEvent.VK_ENTER);
	
	// "Clear" button
	Cbutton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		sendText.setText("");          // Clear the text from the sendText area.
		sendText.requestFocus();
	    }
	});
	gbcs.gridx = 0; gbcs.gridy = 6;       // put buttons right below outgoing text area.
	gbcs.gridwidth = gbcs.gridheight = 1; // Make buttons default to 1x1 grid element
	gbcs.weightx = gbcs.weighty = 0;      // buttons cannot change size when more space is available
	add(Cbutton, gbcs);
    }



    // These are the failed attempts to allow cut and paste in the Swing windows. May they Rest In Peace...
    

    
    /*
    private void initKeyboardBindings()
    {
	createActionTable(receiveText);
	Keymap keymap = receiveText.addKeymap("ChatAgentKeyMap", receiveText.getKeymap());
	
	Action action = getActionByName(DefaultEditorKit.copyAction);
	KeyStroke key = KeyStroke.getKeyStroke(KeyEvent.VK_K, Event.CTRL_MASK);
	keymap.addActionForKeyStroke(key, action);
	
	receiveText.setKeymap(keymap);
    }


    // The following two methods allow us to find an
    // action provided by the editor kit by its name.
    // Used in initKeyboardBindings().
    // Taken straight from the online java tutorial at
    // http://java.sun.com/docs/books/tutorial/uiswing/components/example-swing/TextComponentDemo.java
    private void createActionTable(JTextComponent textComponent) {
        actions = new Hashtable();
        Action[] actionsArray = textComponent.getActions();
        for (int i = 0; i < actionsArray.length; i++) {
            Action a = actionsArray[i];
            actions.put(a.getValue(Action.NAME), a);
        }
    }

    private Action getActionByName(String name) {
        return (Action)(actions.get(name));
    }
    */
};






/* Data transfer stuff: 


import java.awt.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;

public class ClipboardTest extends Frame 
    implements ClipboardOwner, ActionListener {
    
    TextArea srcText, dstText;
    Button copyButton, pasteButton;
    
    Clipboard clipboard = getToolkit().getSystemClipboard();
    
    public ClipboardTest() {
	super("Clipboard Test");
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridbag);
	
	srcText = new TextArea(8, 32);
	c.gridwidth = 2;
	c.anchor = GridBagConstraints.CENTER;
	gridbag.setConstraints(srcText, c);
	add(srcText);
	
	copyButton = new Button("Copy Above");
	copyButton.setActionCommand("copy");
	copyButton.addActionListener(this);
	c.gridy = 1;
	c.gridwidth = 1;
	gridbag.setConstraints(copyButton, c);
	add(copyButton);
	
	pasteButton = new Button("Paste Below");
	pasteButton.setActionCommand("paste");
	pasteButton.addActionListener(this);
	pasteButton.setEnabled(false);
	c.gridx = 1;
	gridbag.setConstraints(pasteButton, c);
        add(pasteButton);
	
        dstText = new TextArea(8, 32);
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 2;
        gridbag.setConstraints(dstText, c);
        add(dstText); 
	
        pack();
    }
    
    public void actionPerformed(ActionEvent evt) {
        String cmd = evt.getActionCommand();
	
        if (cmd.equals("copy")) { 
	    // Implement Copy operation
	    String srcData = srcText.getText();
	    if (srcData != null) {
                StringSelection contents = new StringSelection(srcData);
                clipboard.setContents(contents, this);
                pasteButton.setEnabled(true);
            }
        } else if (cmd.equals("paste")) {
            // Implement Paste operation
            Transferable content = clipboard.getContents(this);
            if (content != null) {
                try {
                    String dstData = (String)content.getTransferData(DataFlavor.stringFlavor);
                    dstText.append(dstData);
                } catch (Exception e) {
                    System.out.println("Couldn't get contents in format: "+
				       DataFlavor.stringFlavor.getHumanPresentableName()); 
                }
	    }
        }
    }
    public void lostOwnership(Clipboard clipboard, Transferable contents) {
	System.out.println("Clipboard contents replaced");
    }
    public static void main(String[] args) {
        ClipboardTest test = new ClipboardTest();
        test.show();
    }
}
*/
