// $Id: ChatGroupServerAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/**
 ** $Id: ChatGroupServerAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
 ** This agent acts as a server for a group of ChatAgents to allow them to talk to each other
 ** without directly connecting to each of them.
 **/

package net.hivecell.hive.agent.desktop;

import java.rmi.RemoteException;
import java.util.*;

//TESTING
import java.io.*;

import net.hivecell.hive.*;
import net.hivecell.hive.Global;
import net.hivecell.hive.support.Debug;

import net.hivecell.hive.*;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.agent.TwoWayConnectionAgentImpl;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.desktop.ChatGroupServerAgent;
import net.hivecell.hive.agent.desktop.ChatMessage;


public class ChatGroupServerAgentImpl extends TwoWayConnectionAgentImpl implements ChatGroupServerAgent {
    
    protected Vector chatAgents; // ChatAgents connected to this server
    protected String groupName;  // Name of this CGSAI
    protected static int numGroups = 0;
    
    public ChatGroupServerAgentImpl() throws RemoteException {
	super();
	groupName = "ChatGroup" + ++numGroups;
    }
    
    public void doLocalSetup() throws AgentInitializationException 
    {
	super.doLocalSetup();
	chatAgents = new Vector();
    }
    
    
    public void doLocalCleanup() { super.doLocalCleanup(); }
    
    public void doBehavior() { waitUntilDeath(); }
    
    public synchronized void notify(HiveEvent event)
    {
	super.notify(event);

	if ( (event.getEventData() instanceof ChatMessage) )
	    {  // if !(the sender of the message was a ChatCellAgent && that sender was this), then forward the message
		ChatMessage msg = (ChatMessage)event.getEventData();
		try {	      
		    if ( !(( isChatGroupServerAgent(msg.getSender()) )  &&  ( ((ChatGroupServerAgent)msg.getSender()).equals(this) )) )
			eventSendingHelper.broadcastEvent(event.getEventData());
		} catch (RemoteException e) { Debug.error("Remote exception while broadcasting event from CGSAI: ", e); }
	    }
	else // It isn't a chat message? Well, make it one.
	    {
		try {
		    eventSendingHelper.broadcastEvent(new ChatMessage(this, "(Non-ChatMessage data): " + event.getEventData(), false));
		} catch(RemoteException e) { Debug.error("Remote exception while broadcasting event from CGSAI: ", e); }
	    }
    }
    
    /*   Doesn't work with non-chatMessage data, could handle being connected to multiple CGSAI's
    public synchronized void notify(HiveEvent event)
    {
	super.notify(event);

	if ( (event.getEventData() instanceof ChatMessage) )
	    {  // if !(the sender of the message was a ChatCellAgent && that sender was this), then forward the message
		ChatMessage msg = (ChatMessage)event.getEventData();
		try {	      
		    if ( !(( isChatGroupServerAgent(msg.getSender()) ) &&
			   ( ((ChatGroupServerAgent)msg.getSender()).equals(this) )))
			eventSendingHelper.broadcastEvent(event.getEventData());	  	   
		} catch (RemoteException e) { Debug.error("Remote exception while broadcasting event from CGSAI: ", e); }
	    }
	else // It isn't a chat message? Well, make it one.
	    {
		try {
		    eventSendingHelper.broadcastEvent(new ChatMessage(this, "(Non-ChatMessage data): " + 
								      event.getEventData(), false));
		} catch(RemoteException e) { Debug.error("Remote exception while broadcasting event from CGSAI: ", e); }
	    }
    }
    */
    
    public void setConnecting() { super.setConnecting(); }
    public void setDisconnecting() { super.setDisconnecting(); }


    public boolean connectTo(Agent otherAgent)
    {
	if (!(otherAgent instanceof ChatGroupServerAgent))  // For now, ignore the interesting routing problem.
	    {
		boolean connected = super.connectTo(otherAgent);
		if (connected && (otherAgent instanceof ChatAgent))
		    {
			// Let everyone know that a new ChatAgent has joined the group
			try {
			    addChatAgentNotify((ChatAgent)otherAgent);
			} catch (RemoteException e) { Debug.error(e.getMessage()); }
			
			// Let the newcomer know who else is on this server
			Enumeration e = chatAgents.elements();
			while (e.hasMoreElements())
			    {	    
				try {
				    ((ChatAgent)otherAgent).addToConnectedList( (ChatAgent)e.nextElement() );
				} catch (RemoteException re) { Debug.error(re.getMessage()); }
			    }
			
			// Put the newcomer in the list of all the ChatAgents on this server
			chatAgents.addElement(otherAgent);
		    }
		else if (!connected)
		    super.disconnectFrom(otherAgent);
		
		return (connected);
	    }
	return (false);
    }
    
    
    /** Remove the list of connections from the sender, and let everyone else know that the sender is leaving.
     ** Currently going to have a problem if connected to both the server and an agent on the server.
     ** Will currently lose both connections.
     **/  
    public void disconnectFrom(Agent otherAgent)
    {
	if (otherAgent instanceof ChatAgent)
	    {
		// Remove the agents from this group from the otherAgent:
		Enumeration e = chatAgents.elements();
		while (e.hasMoreElements())
		    {
			// This is where the problem is: will remove the connection from the server and to the agent.
			// NO LONGER A PROBLEM.  CANNOT BE CONNECTED TO BOTH THE SERVER AND ANOTHER AGENT.
			try {
			    ((ChatAgent)otherAgent).removeFromConnectedList((ChatAgent)e.nextElement());
			} catch (RemoteException re) { Debug.error(re.getMessage()); }
		    }
	    }
	
	// Remove otherAgent from the list of ChatAgents on this server
	chatAgents.removeElement(otherAgent);
	// Let everyone know that otherAgent is gone.
	try {
	    removeChatAgentNotify((ChatAgent)otherAgent);
	} catch (RemoteException e) { Debug.error(e.getMessage()); }
	super.disconnectFrom(otherAgent);
    }
    

    public void eventSendingError( Object o, Exception error ) {
	super.eventSendingError(o, error);
	
	if (o instanceof ChatAgent)
	    {
		// Remove otherAgent from the list of ChatAgents on this server
		chatAgents.removeElement(o);

		// Let everyone know that otherAgent is gone.
		try {
		    removeChatAgentNotify((ChatAgent)o);
		} catch (RemoteException e) { Debug.error(e.getMessage()); }
	    }
    }

    

    public String getGroupName() throws RemoteException { return (groupName); }
    
        
    /**
     ** Returns all connected listeners, even those that aren't ChatAgents.
     **/
    public Vector getConnected() { return (eventSendingHelper.getListeners()); }
    
    /**
     ** Returns only the ChatAgents that are connected.
     **/
    public Vector getChatAgents() { return (chatAgents); }

    
    /**
     ** Test to see if CGSAWannabe can be cast to a CGSA 
     **/
    public static boolean isChatGroupServerAgent(Object CGSAWannabe) throws RemoteException
    {
	try { // Attempt a cast
	    if (((ChatGroupServerAgent)CGSAWannabe) == null);
	} catch (ClassCastException e) { return (false); }
	return true;
    }  

    
    /**
     ** Tell all of the Chat Agents connected to the server that a new Chat Agent has arrived.
     **/
    public void addChatAgentNotify(ChatAgent newAgent) throws RemoteException
    {
	Enumeration e = chatAgents.elements();
	while (e.hasMoreElements())
	    {
		((ChatAgent)e.nextElement()).addToConnectedList(newAgent);
	    }
    }
    
    /**
     ** Tell all of the Chat Agents connected to the server that a Chat agent left.
     **/
    public void removeChatAgentNotify(ChatAgent oldAgent) throws RemoteException
    {
	//      Vector listeners = eventSendingHelper.getListeners();
	Enumeration e = chatAgents.elements();      
	while (e.hasMoreElements())
	    {
		((ChatAgent)e.nextElement()).removeFromConnectedList(oldAgent);
	    }
    }
    
    /**
     ** Determine whether or not a ChatAgent by the name of agentName is connected to this server,
     ** and get a reference to it if it is.
     **/
    public ChatAgent getAgent(String agentName) throws RemoteException
    {
	Enumeration e = chatAgents.elements();
	ChatAgent agent;
	
	while (e.hasMoreElements())
	    {
		if ((agent = (ChatAgent)e.nextElement()).getChatName().equals(agentName))
		    return(agent);
	    }
	
	return null;
    }
    
    
    /**
     ** Determine whether or not an agent is connected to this server,
     ** and get a reference to it if it is.
     **/
    public boolean isAgentConnected(Agent agentName) throws RemoteException
    {      
	Enumeration e = chatAgents.elements();
	while (e.hasMoreElements())
	    {
		if (((ChatAgent)e.nextElement()).equals(agentName))
		    return(true);
	    }
	
	return false;
    }
};
