// $Id: ClickAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.EventSendingAgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.SerializableImage;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.rmi.RemoteException;

/** 
 * Simple agent that makes a canvas and listens for mouse clicks there.
 * When it receives a mouse click, it passes on an event. Useful
 * tool for generating events.
 *
 * @author Nelson Minar <nelson@media.mit.edu>
 * @version $Revision: 1.2 $
 */
public class ClickAgentImpl 
extends EventSendingAgentImpl {
 
    /**
     * the color that we are using for the button 
     */
    protected Color buttonColor;

    /**
     * the hook to our agent component manager shadow
     */
    protected transient ComponentManagerShadow acms = null;

    /**
     * the canvas that we are using for the display of our clicking surface
     */
    protected transient Canvas c = null;
    
    public ClickAgentImpl() throws RemoteException { super(); }

    /**
     * do our local setup
     */
    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();

	// test to make sure we can pop up agent components
	try {
	    acms = getComponentManagerShadow();
	} catch( ShadowNotFoundException e ) {
	    throw new AgentInitializationException("this agent cannot run on a cell without a UI." );
	}

	// set up our description
	Description desc = this.getDescription();
	desc.addSemantic("role", "screen-widget", null, null);
	desc.addSemantic("role",  "button", null, null);
	setDescription( desc );
	
	//this.getDescription().printTree();

	// Our canvas is an anonymous class with paint overridden
	c = new Canvas() {
	    public void paint( Graphics g ) {
		drawCenterText( g, "Click me!", this.getSize().width, this.getSize().height - 10 );
	    }
	    private void drawCenterText( Graphics g, String s, int width, int y ) {
		FontMetrics fm = g.getFontMetrics();
		g.drawString( s, width / 2 - g.getFontMetrics().stringWidth(s) / 2, y );
	    }
	};
	c.setSize( 50, 50 );
	c.setBackground( buttonColor );
	c.setForeground( Color.white );
	c.addMouseListener( new MouseAdapter() {
	    public void mouseClicked( MouseEvent evt ) {
		mouseClick( evt );
	    }
	});

	// display us
	acms.acceptComponentFrom( this, c, "Click me!" );

    }

    public void doBehavior() {
	//	Debug.failure(getDescription().toString());

	waitUntilDeath(); }

    /**
     * on receiving a mouse click -- broadcast that out to the agents
     *
     * @param evt the mouse event we receive
     */
    public void mouseClick( MouseEvent evt ) { eventSendingHelper.broadcastEvent( evt ); }

    /**
     * override the load icon for this class so we can load the icon
     * and then morph the colors so the icon color is closer to the
     * button color we use -- the only problem with this algorithm is
     * that it ends up scaling the white area to something incorrect 
     */
    protected void loadIcon() {

	super.loadIcon();
	
	// choose a color for the icon
	if( buttonColor == null ) buttonColor = Color.getHSBColor( Global.random.nextFloat(), 0.8f, 0.5f );

	// recalculate the icon's color to be more like the button color
	Color baseLine = new Color( -8423911 );
	double maxValue = 0;
	int[] data = icon.getRep();
	double[] newRed = new double[data.length];
	double[] newGreen = new double[data.length];
	double[] newBlue = new double[data.length];
	for( int count=0;count < data.length;count++ ) {
	    Color currentColor = new Color( data[count] );
	    newRed[count] = (double)currentColor.getRed() / (double)baseLine.getRed() * (double)buttonColor.getRed();
	    if( newRed[count] > 255 ) newRed[count] = 255;

	    newGreen[count] = (double)currentColor.getGreen() / (double)baseLine.getGreen() * (double)buttonColor.getGreen();
	    if( newGreen[count] > 255 ) newGreen[count] = 255;

	    newBlue[count] = (double)currentColor.getBlue() / (double)baseLine.getBlue() * (double)buttonColor.getBlue();
	    if( newBlue[count] > 255 ) newBlue[count] = 255;
	}
	for( int count=0;count < data.length;count++ )
	    data[count] = ( new Color( (int)newRed[count], (int)newGreen[count], (int)newBlue[count] ) ).getRGB();
	    
	// reconstruct the icon
	icon = new SerializableImage( data, icon.getSize() );

    }

}






