// $Id: CreditsAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.cell.CellListAgent;
import net.hivecell.hive.agent.cell.CellListInfo;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.support.BaseCanvas;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.PPM;
import net.hivecell.hive.support.SerializableImage;

import java.awt.*;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.*;


public class CreditsAgentImpl
extends AgentImpl {

    private transient ComponentManagerShadow acms;

    private transient JPanel creditsPanel;  // the actual panel we pop up on the screen
    private SerializableImage teamImage;
    
    private transient JTextField textField; // used to display the credits

    private static creditText[] credits = {
	new creditText( "Hive", creditText.UP, 5000 ),
	new creditText( "brought to you by", creditText.UP, 2000 ),
	new creditText( "Nelson Minar", creditText.RIGHT, 3500 ),
	new creditText( "Raffi Krikorian", creditText.LEFT, 3500),
	new creditText( "Tucker Sylvestro", creditText.LEFT, 3500 ),
	new creditText( "Orton Huang", creditText.LEFT, 3500 ),
	new creditText( "Matthew Gray", creditText.LEFT, 3500 ),
	new creditText( "and", creditText.UP, 1500 ),
	new creditText( "Oliver Roup", creditText.RIGHT, 3500 ),
	new creditText( "", creditText.UP, 4000 )
    };

    private transient int numberLoopsUntilJump;
    private transient int countForJump;


    public CreditsAgentImpl() 
    throws RemoteException { 
	super(); 
    }

    public void doLocalSetup() throws AgentInitializationException  {
	
	this.acms = null;
	try {
	    this.acms = getComponentManagerShadow();
	} catch( ShadowNotFoundException error ) {
	    Debug.error( "This agent needs a graphical context to run" );
	    runAway();
	    return;
	}

	// set up the actual panel we want to display
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	creditsPanel = new JPanel( gbl );
	creditsPanel.setBackground( Color.black );
	creditsPanel.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );

	// get the team's pic
	
	if( teamImage == null ) {
	    try {
		teamImage = new PPM( getClass().getResourceAsStream( "/net/hivecell/hive/support/hivegroup1.ppm" ) );
	    } catch( UnsupportedEncodingException error ) {
		throw new AgentInitializationException( "PPM seems to be encoded in an unsupported way" );
		
	    } catch( IOException error ) {
		throw new AgentInitializationException( "IOException while trying to read the image for the canvas" );

	    }
	}

	drawImageCanvas dic = new drawImageCanvas( teamImage.getImage(), teamImage.getSize() );
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.fill = GridBagConstraints.NONE;
	gbc.anchor = GridBagConstraints.CENTER;
	gbl.setConstraints( dic, gbc );
	creditsPanel.add( dic );
	
	// set up the scrolling credits bar
	this.textField = new JTextField(); 
	this.textField.setFont( new Font( "Helvetica", Font.PLAIN, 12 ) );
	this.textField.setColumns( 25 );
	this.textField.setEditable( false );
	this.textField.setHorizontalAlignment( JTextField.CENTER );
	this.textField.setBackground( Color.black );
	this.textField.setForeground( Color.white );
	
	gbc.gridx = 0;
	gbc.gridy = 3;
	gbc.insets = new Insets( 5, 0, 0, 0 );
	gbc.fill = GridBagConstraints.NONE;
	gbl.setConstraints( this.textField, gbc );
	creditsPanel.add( this.textField );
	
	// pop the frame up
	Frame f = this.acms.acceptComponentFrom( this, creditsPanel, "Hive Credits" );
	f.show();
	
	// how many credit loops should we go thorugh until we jump?  anywhere from 1 to 3...
	numberLoopsUntilJump = Math.abs( Global.random.nextInt() % 3 + 1 );
	countForJump = 0;

    }

    private int creditsPosition = 0, intraCreditsPosition = 0;
    public void doBehavior() { 
	while( !timeToStop ) {

	    long sleepTime;

	    if( intraCreditsPosition == 0 && credits[creditsPosition].getDirection() == creditText.UP ) {
		this.textField.setText( credits[creditsPosition].getText() );
		sleepTime = credits[creditsPosition].getLength();
		creditsPosition = ( creditsPosition + 1 ) % credits.length;
		
	    } else if( credits[creditsPosition].getDirection() == creditText.LEFT && intraCreditsPosition != credits[creditsPosition].getText().length() ) {
		this.textField.setText( credits[creditsPosition].getText().substring( 0, ++intraCreditsPosition) );
		sleepTime = 150;

	    } else if( credits[creditsPosition].getDirection() == creditText.RIGHT && intraCreditsPosition != credits[creditsPosition].getText().length() ) {
		this.textField.setText( credits[creditsPosition].getText().substring( credits[creditsPosition].getText().length() - ++intraCreditsPosition, credits[creditsPosition].getText().length() ) );
		sleepTime = 150;

	    } else {
		sleepTime = credits[creditsPosition].getLength();
		intraCreditsPosition = 0;
		creditsPosition = ( creditsPosition + 1 ) % credits.length;
 
	    }

	    testForMovement();

	    try {
		Thread.sleep( sleepTime );
	    } catch( InterruptedException error ) { }
	    
	}
    }

    public void onDying() {
       
	Debug.notice( "We've been given the notification to die -- attempting to run away" );
	runAway();

    }

    private boolean testForMovement() {

	if( ( creditsPosition == 0 ) && ( ++countForJump >= numberLoopsUntilJump ) )
	    return runAway();

	return false;

    }
    
    private boolean runAway() {

	try {
	    String[] cellListAgentType = { "net.hivecell.hive.agent.cell.CellListAgent" };
	    AgentSet ds = myCell.queryAgents( this, cellListAgentType, null );
	    CellListAgent cla = (CellListAgent)ds.pickOne();
	    Vector cellListInfo = cla.listCells( this );
	    Vector upCells = new Vector();
	    Enumeration e = cellListInfo.elements();
	    while( e.hasMoreElements() ) {
		CellListInfo cli = (CellListInfo)e.nextElement();
		if( cli.isUp() && !cli.getCell().equals( myCell ) )
		    upCells.addElement( cli.getAddress() );
	    }
	    if( upCells.size() == 0 ) {
		// Debug.notice( "No other servers to move to" );
	    } else {
		CellAddress toGoTo = (CellAddress)upCells.elementAt( Math.abs( Global.random.nextInt() % upCells.size() ) );
		// Debug.notice( "Moving to " + toGoTo );
		myCell.moveAgent( this, toGoTo );
	    }
	} catch( Exception error ) {
	    Debug.critical( "Could not move to new server!", error );
            return false;
	}
	
	numberLoopsUntilJump = Math.abs( Global.random.nextInt() % 3 + 1 );
	countForJump = 0;
	
	return true;
	
    }
		
}

class drawImageCanvas 
extends BaseCanvas {

    private Image image;
    private Dimension imageSize;

    public drawImageCanvas( Image toDraw, Dimension imageSize ) {
	super();
	setVisible( true );
	setBackground( Color.black );
	setSize( imageSize );
	this.image = toDraw;
	this.imageSize = imageSize;
    }

    public void render( Graphics g ) {

	int thisWidth = getSize().width;
	int thisHeight = getSize().height;
       
	int drawHeight = thisHeight;
	int drawWidth = (int)( (double)thisHeight / (double)imageSize.height * (double)imageSize.width );
	if( drawWidth > thisWidth ) {
	    drawHeight = (int)( (double)thisWidth / (double)imageSize.width * (double)imageSize.height );
	    drawWidth = thisWidth;
	}
	g.drawImage( image, ( thisWidth - drawWidth ) / 2, ( thisHeight - drawHeight ) / 2, drawWidth, drawHeight, this );

    }
    
}

class creditText {

    public static int LEFT = 1;
    public static int RIGHT = 2;
    public static int UP = 3;

    private String text;
    private int direction;
    private long length;

    public creditText( String text, int direction, long length ) {
	this.text = text;
	this.direction = direction;
	this.length = length;
    }

    public String getText() { return text; }
    public int getDirection() { return direction; }
    public long getLength() { return length; }

}
