package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent;
import net.hivecell.hive.agent.image.ImageDisplayCanvas;
import net.hivecell.hive.agent.image.ImageSourceAgent;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.SerializableImage;
 
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;


public class ImageTakingDisplayingAgentImpl 
extends AgentImpl {

    protected transient ComponentManagerShadow cms = null;
    protected transient ImageDisplayCanvas idc = null;
    protected transient JFrame frame = null;

    protected CellAddress isaCellAddress = null;
    protected transient ImageSourceAgent isa = null;

    
    public ImageTakingDisplayingAgentImpl() throws RemoteException { super(); }

    public void setImageSourceAgent( ImageSourceAgent isa ) {
	try {
	    this.isaCellAddress = isa.getCell().getAddress();
	} catch( RemoteException error ) {
	    Debug.error( "something went wrong when trying to assign a image agent for us to connect back to", error );

	}
    }

    public void doLocalSetup() throws AgentInitializationException  {
	try {
	    cms = getComponentManagerShadow();
	    
	    JPanel panel = new JPanel( new BorderLayout() );
	    panel.setBackground( Color.black );
	    panel.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );
	    panel.add( idc = new ImageDisplayCanvas() );
	    JPanel buttonPanel = new JPanel( new FlowLayout() );
	    buttonPanel.setBackground( Color.black );
	    JButton takePicture = new JButton( "take picture" );
	    buttonPanel.add( takePicture );
	    panel.add( buttonPanel, "South" );
	    takePicture.addActionListener( new ActionListener() {
		    public void actionPerformed( ActionEvent ae ) {
			if( isa != null ) {
                            idc.imagePending();
			    try {
				SerializableImage si = isa.getImage( ImageTakingDisplayingAgentImpl.this );
                                if (si == null)
                                    Debug.warning("No image!");
                                else
                                    idc.showImage( si.getImage(), si.getSize() );
			    } catch( RemoteException error ) {
				Debug.error( "problem talking to the image source agent on " + isaCellAddress + " to get a picture", error );

			    }
			}
		    }
		});
	    frame = (JFrame)cms.acceptComponentFrom( this, panel, "Camera display" );
	    frame.setSize( frame.getPreferredSize() );
	    frame.setVisible( true );

	} catch( ShadowNotFoundException error ) {
	    throw new AgentInitializationException("This agent cannot run on a cell without a UI.");
	}

    }

    public void doBehavior() {
	if( isaCellAddress != null ) {
	    Debug.notice( "connecting to image source agent" );
	    try {
		String[] amaSyntatic = { (AgentMonitoringAgent.class).getName() };
		AgentSet ds = myCell.queryAgents( this, amaSyntatic, null );
		AgentMonitoringAgent ama = (AgentMonitoringAgent)ds.pickOne();
		if( ama != null ) {
		    CellAddress[] address = { isaCellAddress };
		    String[] className = { (ImageSourceAgent.class).getName() };
		    ds = ama.queryAgents( this, address, className, null );
		    this.isa = (ImageSourceAgent)ds.pickOne();
		} else
		    Debug.error( "there is no image source agent at " + isaCellAddress );
	    } catch( Exception error ) {
		Debug.error( "problem connecting to the image source agent at the specified " + isaCellAddress, error );
	    }

	} 

	waitUntilDeath();

    }

    public void doLocalCleanup() { 
	super.doLocalCleanup();
	frame.dispose();

    }


}
