package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.ActionCommand;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.desktop.EchoAgent;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.MalformedHiveURLException;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;


public class LatencyTestingAgentImpl
extends EventTransceivingAgentImpl {

    private ComponentManagerShadow cms;
    
    private JTextField hostToTest;
    private CellAddress testingAddress;
    private EchoAgent echoAgent;
    
    private boolean runningTest = false;

    private HiveEvent eventsIn[];
    private Long eventsInTime[];
    private int currentEventIn = 0;
    

    public LatencyTestingAgentImpl() throws RemoteException { super(); }

    public void doLocalSetup() throws AgentInitializationException {
	super.doLocalSetup();

	try {
	    cms = getComponentManagerShadow();
	} catch( ShadowNotFoundException error ) {
	    throw new AgentInitializationException( "no component manager shadow found on cell -- this agent needs one to run" );
	}

	startTest();

    }
    
    public void doBehavior() { waitUntilDeath(); }

    public void notify( HiveEvent he ) {
	if( runningTest && ( currentEventIn < 100 ) && he.getSource().equals( echoAgent ) ) {
	    synchronized( eventsIn ) {
		eventsIn[currentEventIn] = he;
		eventsInTime[currentEventIn++] = new Long( System.currentTimeMillis() );
	    }
	    if( currentEventIn < 100 ) {
		EventReceivingAgent[] destinationAgent = { echoAgent };
		eventSendingHelper.broadcastEventTo( new Long( System.currentTimeMillis() ), destinationAgent, null );

	    }

	    else {
		Debug.error( "100 packet test over" );
		runningTest = false;
       		try {
  		    echoAgent.diePlease();
  		} catch( RemoteException error ) {
  		    Debug.error( "remote exception while trying to kill the echo agent", error );

  		}

		HiveEvent eEvent = eventsIn[0];
  		double minimum = (eventsInTime[0].longValue() - ((Long)((HiveEvent)eventsIn[0].getEventData()).getEventData()).longValue());
  		double maximum = minimum;
  		double average = minimum / 100;

		for( int count=1;count < 100;count++ ) {
    		    eEvent = eventsIn[count];
    		    double proposed = (eventsInTime[count].longValue() - ((Long)((HiveEvent)eventsIn[count].getEventData()).getEventData()).longValue());
    		    average += proposed / 100.0;
    		    if( minimum > proposed )
    			minimum = proposed;
    		    if( maximum < proposed )
    			maximum = proposed;

    		}

		GridLayout gl = new GridLayout( 0, 1 );
		gl.setVgap( 2 );
		final JPanel resultsPanel = new JPanel( gl );
		resultsPanel.setBorder( new EmptyBorder( 7, 7, 7, 7 ) );
		resultsPanel.add( new JLabel( "100 packet latency test to " + testingAddress, JLabel.CENTER ) );
		String averageString = average + "";
		averageString = averageString.substring( 0, averageString.indexOf( "." ) + 2 );
		resultsPanel.add( new JLabel( "average time : " + averageString + " milliseconds", JLabel.CENTER ) );
		String maximumString = maximum + "";
		maximumString = maximumString.substring( 0, maximumString.indexOf( "." ) + 2 );
		resultsPanel.add( new JLabel( "maximum time : " + maximumString + " milliseconds", JLabel.CENTER ) );
		String minimumString = minimum + "";
		minimumString = minimumString.substring( 0, minimumString.indexOf( "." ) + 2 );
		resultsPanel.add( new JLabel( "minimum time : " + minimumString + " milliseconds", JLabel.CENTER ) );
		
		JButton anotherTestButton = new JButton( "run another test" );
		anotherTestButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) {
			    cms.removeComponentFrom( LatencyTestingAgentImpl.this, resultsPanel );
			    startTest();

			}
		    });
		resultsPanel.add( anotherTestButton );

		JButton quitButton = new JButton( "quit" );
		quitButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) {
			    cms.removeComponentFrom( LatencyTestingAgentImpl.this, resultsPanel );
			    diePlease();
			    return;
			}
		    });
		resultsPanel.add( quitButton );
		
		cms.acceptComponentFrom( LatencyTestingAgentImpl.this, resultsPanel, "latency test results" );

	    }
	}

    }

    private void startTest() {

	final JPanel echoAgentSetupPanel = new JPanel( new FlowLayout() );
	echoAgentSetupPanel.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );
	echoAgentSetupPanel.add( new JLabel( "host to test:" ) );
	
	hostToTest = new JTextField( myCell.getAddress().toString(), 30 );
	echoAgentSetupPanel.add( hostToTest );
	
	JButton startButton = new JButton( "start test" );
	startButton.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent ae ) {

		    cms.removeComponentFrom( LatencyTestingAgentImpl.this, echoAgentSetupPanel );

		    testingAddress = null;
		    try {
			testingAddress = new CellAddress( hostToTest.getText().trim() );
			RemoteCell rc = testingAddress.getRemoteCell();
			    
			String[] echoAgentSyntatic = { (EchoAgentImpl.class).getName() };
			AgentSet ds = null;
			try { 
			    ds = rc.queryAgents( LatencyTestingAgentImpl.this, echoAgentSyntatic, null );
			} catch( ClassNotFoundException internalError ) {
			    Debug.error( "cell at \"" + testingAddress + "\" doesn't seem to know about EchoAgents" );
			    ds = null;

			}
			if( ( ds == null ) || ( ds.size() == 0 ) ) {
			    try {
				Debug.notice( "shipping the echo agent to " + testingAddress );
				echoAgent = (EchoAgent)myCell.moveAgent( new EchoAgentImpl(), testingAddress );
				echoAgent.blockUntilReady();

			    } catch( Exception error ) {
				Debug.error( "problem serializing an echo agent to send to " + testingAddress, error );
				return;
				
			    }
    
			} else
			    echoAgent = (EchoAgent)ds.pickOne();

			Debug.warning( "starting test -- sending 100 events to EchoAgent at \"" + testingAddress + "\"" );
			eventsIn = new HiveEvent[100];
			eventsInTime = new Long[100];
			currentEventIn = 0;
			runningTest = true;
			EventReceivingAgent[] destinationAgent = { echoAgent };
			eventSendingHelper.broadcastEventTo( new Long( System.currentTimeMillis() ), destinationAgent, null );
		    } catch( MalformedHiveURLException error ) {
														       
			Debug.error( "\"" + hostToTest.getText().trim() + "\" is not a valid cell address to test" );
			return;

		    } catch( CellConnectException error ) {
			Debug.error( "having problems connecting to the cell at \"" + testingAddress + "\"", error );
			return;

		    } catch( RemoteException error ) {
			Debug.error( "remote exception while trying to connect to cell at \"" + testingAddress + "\"", error );
			return;

		    }

		}
	    });
	echoAgentSetupPanel.add( startButton );

	cms.acceptComponentFrom( this, echoAgentSetupPanel, "start latency test" );

    }

}











