// $Id: PocketServerAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

//stock grabber agent written by Blair Dunn 3/14/99
//Based on Will Logan's PocketServer code.

package net.hivecell.hive.agent.desktop;


import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.*;
import net.hivecell.hive.agent.EventSendingAgentImpl;
import net.hivecell.hive.support.pocketserver.NASDAQGrabber;

import java.rmi.RemoteException;


/* Agent that queries a particular stock price every few minutes
 * and compares the results to a high/low/delta.  default values
 * are requested at creation.
 */

public class PocketServerAgentImpl 
extends EventSendingAgentImpl 
implements PocketServerAgent {

    //these are the constants
    final static int        DEFAULTTIMERDELAY    = 10;

    //these are the variables
    String symbol;
    int delay;
    double highthreshold;
    double lowthreshold;
    double netchangelowthreshold;
    double netchangehighthreshold;
    boolean loadDefaultValues;
    static NASDAQGrabber nasdaq;
    
    
    public PocketServerAgentImpl() throws RemoteException {
        super();

        symbol = new String("MSFT");        
        nasdaq = new NASDAQGrabber();

        setLoadDefaultValues(false);
        setDelay(DEFAULTTIMERDELAY);
   }

    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();
        setLoadDefaultValues(true);
    }

       
    public void doBehavior() {
	
	while (!timeToStop) {
	    try {
		Thread.sleep(delay*1000);
		checkValues();
	    } catch (Exception e) { System.out.println("Exception: "+e); }
	}
    }


    public String getSymbol() { return symbol.toString(); }
    public void setSymbol(String s) { symbol = s; }

    public int getDelay() { return delay; }
    public void setDelay(int d) {
        if (d > 0) {
            delay = d;
        }
    }


    public double getHighthreshold() { return highthreshold; }
    public void setHighthreshold(double v) {
        if (!loadDefaultValues)
            highthreshold = v;
        else highthreshold = nasdaq.getHigh(symbol);

    }

    public double getLowthreshold() { return lowthreshold; }
    public void setLowthreshold(double v) {
        if (!loadDefaultValues)
            lowthreshold = v;
        else lowthreshold = nasdaq.getLow(symbol);

    }

    public double getNetchangehighthreshold() { return netchangehighthreshold; }
    public void setNetchangehighthreshold(double v) {
        if (!loadDefaultValues)
            netchangehighthreshold = v;
        else netchangehighthreshold = nasdaq.getNetChange(symbol);

    }

    public double getNetchangelowthreshold() { return netchangelowthreshold; }
    public void setNetchangelowthreshold(double v) {
        if (!loadDefaultValues)
            netchangelowthreshold = v;
        else netchangelowthreshold = nasdaq.getNetChange(symbol);

    }

    public boolean getLoadDefaultValues() { return loadDefaultValues; }
    public void setLoadDefaultValues(boolean b) {
        loadDefaultValues = b;
        if (loadDefaultValues) {
            setHighthreshold(1.0);
            setLowthreshold(1.0);
            setNetchangelowthreshold(1.0);
            setNetchangehighthreshold(1.0);
            loadDefaultValues = false;
        }        
    }
    
    
    public void checkValues() {
	/*	System.out.print("High is " + nasdaq.getHigh(symbol) + "\n");
	System.out.print("Low is " + nasdaq.getLow(symbol) + "\n");
	System.out.print("Today's Close was " + nasdaq.getClose(symbol) + "\n");
	System.out.print("Previous Close was " + nasdaq.getPrevClose(symbol) + "\n");
	System.out.print("Net Change is " + nasdaq.getNetChange(symbol) + "\n");  
	System.out.print("Today's Volume was " + nasdaq.getVolume(symbol) + "\n\n"); */
	
	double num = -1;
        if ((num = nasdaq.getHigh(symbol)) > highthreshold) {
            eventSendingHelper.broadcastEvent("high: " + num);
            System.out.println("Stock Grabber Sent HIGH");
        }

        if ((num = nasdaq.getLow(symbol)) < lowthreshold) {
            eventSendingHelper.broadcastEvent("low: " + num);
            System.out.println("Stock Grabber Sent LOW");
        }

        if ((num = nasdaq.getNetChange(symbol)) < netchangelowthreshold) {
            eventSendingHelper.broadcastEvent("high& net change: " + num);
            System.out.println("Stock Grabber Sent NETCHNGLOW");
        }

        if ((num = nasdaq.getNetChange(symbol)) > netchangehighthreshold) {
            eventSendingHelper.broadcastEvent("low& net change: " + num);
            System.out.println("Stock Grabber Sent NETCHNGHIGH");
        }
    }
}
