// $Id: RandomDotAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.desktop;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.Global;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import java.awt.*;
import java.util.*;
import net.hivecell.hive.description.Description;

/** Agent that listens for events, draws a dot for each event received.
 ** Dots shrink over time. If the event object is of type PointColor,
 ** then we use that data to builld the dot; otherwise the dot is random.
 **
 ** @author Nelson Minar <nelson@media.mit.edu>
 ** @version $Revision: 1.2 $
 **/
public class RandomDotAgentImpl 
extends EventReceivingAgentImpl {

    protected transient RandomDotCanvas displayCanvas;

    // Agent parameters
    protected int updateInterval = 2000;
    protected transient ComponentManagerShadow acms = null;

    public void setUpdateInterval(int i) { updateInterval = i; }
    public int getUpdateInterval() { return updateInterval; }

    public RandomDotAgentImpl() throws java.rmi.RemoteException { super(); }
    
    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();
	try {
	    acms = getComponentManagerShadow();
	}
	catch (ShadowNotFoundException e) {
	    throw new AgentInitializationException("This agent cannot run on a cell without a UI.");
	}      
	displayCanvas = new RandomDotCanvas();
	acms.acceptComponentFrom(this, displayCanvas, "Random Dot Display");

	Description desc = new Description(this, "RandomDotAgent");
	desc.addSemantic("role", "screen-widget", null, null);
	desc.addSemantic("role", "event-display", null, null);
	setDescription(desc);
    }

    /** Behavior for random dot agent; create a canvas, periodically update
     ** the display.
     **/
    public void doBehavior() {
	while (!timeToStop) {
	    displayCanvas.updateDisplay();
	    try {
		Thread.sleep(updateInterval);
	    } catch (InterruptedException e) {}
	}
	doLocalCleanup();
    }
  
    /** When an event is received, create a dot. If the eventdata is of
     ** type PointColor, then we use that data to build the dot.
     **/
    public synchronized void notify(HiveEvent event) {
	super.notify(event);
   
	Object data = event.getEventData();
	if (data != null && data instanceof PointColor) {
	    PointColor info = (PointColor)data;
	    displayCanvas.addDot( info.x, info.y, info.color );
	} else
	    displayCanvas.addDot();
    }
}

