// $Id: RandomDotCanvas.java,v 1.1 2000/05/18 22:04:48 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

// Canvas for random dot agent, evolved from FlashingDotCanvas

package net.hivecell.hive.agent.desktop;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import net.hivecell.hive.support.BaseCanvas;
import net.hivecell.hive.support.Debug;

public class RandomDotCanvas 
extends BaseCanvas {

  protected RandomDotAgentImpl agent;
  protected Vector dots;
  protected Color colors[];
  protected static final int dotScale = 5;
  protected static final int maxDots = 200;
  
  public RandomDotCanvas() {
    dots = new Vector(maxDots + 2);

    // Create the colour palette - a bunch of different hues at same S, B
    int i, maxColors = 20;
    float hueMin = 0.33f, hueMax = 0.66f;
    colors = new Color[maxColors];
    for (i = 0; i < maxColors; i++) {
      float hue = hueMin + i * (hueMax - hueMin) / maxColors;
      colors[i] = new Color(Color.HSBtoRGB(hue, 0.5f, 1.0f));
    }

    setBackground(Color.black);
    setForeground(Color.green);
    setSize(150, 150);
    setVisible(true);
    setDoubleBuffering(true);
  }
  
  // Add a dot to the graph at a particular spot. x, y, and colour
  // should be in the range [0, 1). The colour stuff is only 1 dimensional,
  // my apologies.
  public void addDot(float x, float y, float colour) {
    Debug.assert(x >= 0.0 && x < 1.0 &&
                 y >= 0.0 && y < 1.0 &&
                 colour >= 0.0 && colour < 1.0);

    // too many dots already? Remove the last dot.
    if (dots.size() >= maxDots)
      dots.removeElementAt(dots.size()-1);

    // create the dot and put it at the beginning of the list
    Dot d = new Dot(x, y, 1.0f, (int)(colour*colors.length));
    dots.insertElementAt(d, 0);
    render();
  }

  // Add a dot with random position and colour
  public void addDot() {
    this.addDot((float)Math.random(), (float)Math.random(),
                (float)Math.random());
  }

  // Display updates make the displayed dots (exponential) smaller
  public void updateDisplay() {
    Dimension s = getSize();
    float minUsefulSize = 1.0f * dotScale / Math.min(s.width, s.height);
    
    Enumeration e = dots.elements();
    while(e.hasMoreElements()) {
      Dot d = (Dot)e.nextElement();
      d.size *= 0.9f;                             // make it smaller
      if (d.size < minUsefulSize)
        dots.removeElement(d);
    }
    
    render();
  }
  
  public void render(Graphics g) {
    Dimension s = getSize();
    
    // clear the window
    g.setColor(getBackground());                  
    g.fillRect(0, 0, s.width, s.height);

    // calculate scaling parameters
    int dotSizeScale = Math.min(s.width, s.height) / dotScale;

    // Draw all the dots
    Enumeration e = dots.elements();
    while(e.hasMoreElements()) {
      Dot d = (Dot)e.nextElement();
      int dotSize = (int)(d.size * dotSizeScale);
      if (dotSize % 2 == 0)
        dotSize++;                                // force odd size
      int x = (int)(d.x * s.width);
      int y = (int)(d.y * s.height);
      g.setColor(colors[d.color]);
      g.fillOval(x - dotSize / 2, y - dotSize / 2, dotSize, dotSize);
      g.setColor(Color.black);
      g.drawOval(x - dotSize / 2, y - dotSize / 2, dotSize, dotSize);
    }
  }
}

class Dot extends Object {
  float x, y, size;
  int color;
  Dot (float ix, float iy, float is, int ic) {
    x = ix; y = iy; size = is; color = ic;
  }
}
