// $Id: RunningGraphAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.desktop;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.*;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.FloatSourceAgent;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import java.awt.*;
import java.util.*;
import java.rmi.RemoteException;

public class RunningGraphAgentImpl 
extends AgentImpl {

    protected transient FloatSourceAgent infoAgent = null;
    protected transient RunningGraphCanvas displayCanvas;

    // Agent parameters
    protected int updateInterval = 5000;
    float scaleFactor = 1;

    protected transient ComponentManagerShadow acms;

    public RunningGraphAgentImpl() throws RemoteException { super(); }

    public int getUpdateInterval() { return updateInterval; }
    public void setUpdateInterval(int ui) { updateInterval = ui; }
    public float getScaleFactor() { return scaleFactor; }
    public void setScaleFactor(float s) {
	scaleFactor = s;
	if (displayCanvas != null)
	    displayCanvas.setBarInterval(scaleFactor);
    }
  
    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();
	try {
	    acms = getComponentManagerShadow();
	}
	catch (ShadowNotFoundException e) {
	    throw new AgentInitializationException("This agent cannot run on a cell without a UI.");
	}      

	displayCanvas = new RunningGraphCanvas();
	displayCanvas.setBarInterval(scaleFactor);
	acms.acceptComponentFrom(this, displayCanvas, "Running Graph");
    }

    public void doBehavior() {
	// main loop - every N seconds query the info agent, draw the number
	while (!timeToStop) {
	    float f = 0.0f;
	    if (infoAgent != null) {
		try {
		    f = infoAgent.getFloat(this);
		} catch (RemoteException re) {
		    Debug.warning("Error contacting our float source ", re);
		}
		myCell.noteAgentMessage( this, "getFloat", this, infoAgent );
		myCell.noteAgentMessage( this,"getFloat reply", infoAgent, this );
	    }

	    displayCanvas.updateDisplay(f);
	    try {
		Thread.sleep(updateInterval);
	    } catch (InterruptedException e) {}
	}
	doLocalCleanup();
    }

    /** Connects agent to FloatSourceAgent.
     * This method should probably be synchronized with the main loop.
     **/
    public boolean connectTo(Agent agent) {
	FloatSourceAgent floatSource = null;
	try {
	    floatSource = (FloatSourceAgent)agent;
	    myCell.noteAgentConnect( this, this, agent );
	    // The actual moment of swap, should be synchronized?
	    infoAgent = floatSource;
	    return true;
	} catch (ClassCastException ex) {
	    Debug.error("RunningGraphAgent was told to connect to a non-FloatSource.");
	    return false;
	}
    }

    /** Disconnects this agent from its sources. Doesn't send any messages,
     * because the source doesn't really know about this agent.
     **/
    public void disconnectFrom(Agent agent) {
	if (agent != infoAgent) {
	    Debug.warning("We were told to disconnect from someone we weren't connected to.");
	}
	infoAgent = null;
	myCell.noteAgentDisconnect( this, this, agent );
    }

    /** Disconnects this agent from its sources. **/
    public void disconnectFromAll() {
	disconnectFrom(infoAgent);
    }
}




