// $Id: RunningGraphCanvas.java,v 1.1 2000/05/18 22:04:48 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.desktop;

import java.awt.*;
import java.awt.event.*;

import net.hivecell.hive.support.BaseCanvas;
import net.hivecell.hive.support.Debug;

public class RunningGraphCanvas 
extends BaseCanvas {

  protected int scan = 0;
  protected float[] values;
  protected float barInterval = 1.0f;
  protected Font legendFont;
  protected String legend;
  protected Dimension legendSize;
  protected Point legendPos;
  protected Rectangle legendBox;

  public RunningGraphCanvas() {
    final int width = 120;
    
    values = new float[width];
    for (int i = 0; i < width; i++)
      values[i] = 0;
    
    setBackground(Color.black);
    setForeground(Color.orange);
    setSize(width, 60);
    setVisible(true);

    legendFont = new Font("Helvetica", Font.PLAIN, 8);

    setDoubleBuffering( true );

  }

  // handle a resize - copy the data structure over and all
  public synchronized void adjustDisplayToNewSize() {
    Dimension newSize = getSize();
    float newValues[] = new float[newSize.width];
    int i;

    // Move the stored data into a new array
    if (values.length < newSize.width) {          // new window bigger?
      for (i = 0; i < values.length; i++)         // copy all old data
        newValues[i] = values[(i+scan) % values.length];
      scan = i;
      for (; i < newSize.width; i++)              // fill in the rest with 0
        newValues[i] = 0;
      values = newValues;
    } else if (values.length == newSize.width) {  // new window same size?
      // do nothing - the data is the same size already
    } else {                                      // new window smaller
      for (i = 0; i < newSize.width; i++)         // copy all that will fit
        newValues[i] = values[(i+scan+(values.length-newSize.width)) % values.length];
      scan=0;
      values=newValues;
    }

    render();

  }
  
  public synchronized void updateDisplay(float lav) {
    values[scan] = lav;
    scan = (scan + 1) % values.length;            // increment scan
    render();
  }

  // redraw the entire window
  public synchronized void render( Graphics g ) {
      
      Dimension s = getSize();
      // clear the window
      g.setColor( getBackground() );                  
      g.fillRect( 0, 0, s.width, s.height );

      // figure out the max value
      int i;
      float maxValue = 0;
      for( i=0;i<values.length;i++ )
	  if( maxValue < values[i] )
	      maxValue = values[i];

      // now make the top of the graph be an even multiple of barInterval
      float maxDataDisplay = (float)( barInterval * Math.ceil( maxValue / barInterval ) );

      // Draw all data points
      g.setColor( getForeground() );
      for( i = 0;i < s.width;i++ ) {
	  float thisVal = values[ ( i + scan ) % values.length ];
	  int barHeight = (int)( ( thisVal / maxDataDisplay ) * s.height );
	  g.drawLine( i, s.height, i, s.height - barHeight );
      }

      Debug.assert( barInterval > 0 );
      // Now draw markers - horizontal lines at every requested interval
      g.setColor( Color.darkGray );
      float marker;
      for( marker = barInterval; marker < maxDataDisplay;marker += barInterval ) {
	  int markerHeight = (int)( ( marker / maxDataDisplay ) * s.height );
	  g.drawLine( 0, s.height - markerHeight, s.width, s.height - markerHeight );
      }

      // And draw the scaling
      if( getSize().width > legendSize.width * 2 ) {
	  g.setColor( Color.black );
	  g.fillRect( legendBox.x, legendBox.y, legendBox.width, legendBox.height );
      
	  g.setColor( Color.lightGray );
	  g.setFont( legendFont );
	  g.drawString( "scale=" + barInterval, legendPos.x, legendPos.y );
      }
  }

  public void setBarInterval(float i) {
    barInterval = i;
    legend = "scale" + barInterval;
    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(legendFont);
    legendSize = new Dimension(fm.stringWidth(legend+"0"), fm.getAscent());
    legendPos = new Point(5, 5 + fm.getAscent());
    legendBox = new Rectangle(3, 3,
                              legendSize.width + 5, legendSize.height + 5);
  }
}
