// $Id: SpokeAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.desktop;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.Global;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.cell.Cell;
import java.util.Hashtable;
import java.util.Enumeration;

/** Agent that listens for events, draws a spoke for each event source.
 ** 
 ** @author Nelson Minar <nelson@media.mit.edu>
 ** @version $Revision: 1.2 $
 **/
public class SpokeAgentImpl 
extends EventReceivingAgentImpl {

    /** The canvas the agent displays on. **/
    protected transient SpokeCanvas displayCanvas;
    /** A hashtable associating Agents with Spokes. **/
    protected transient Hashtable spokes = new Hashtable();

    protected static final float defaultAccel = 1.1f;
    protected static final float defaultDecel = 0.99f;
    
    protected int updateInterval = 100;

    protected transient ComponentManagerShadow acms = null;

    public void setUpdateInterval(int i) { updateInterval = i; }
    public int getUpdateInterval() { return updateInterval; }

    public SpokeAgentImpl() 
    throws java.rmi.RemoteException {
        super();
        Description desc = new Description(this, "SpokeAgent");
        desc.addSemantic("role", "screen-widget", null, null);
        desc.addSemantic("role", "event-display", null, null);
        setDescription(desc);
    }

    /** When the agent arrives at its destination, it creates a canvas
     ** for itself.
     **/
    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();
        displayCanvas = new SpokeCanvas();
	try {
	    acms = getComponentManagerShadow();
	}
	catch (ShadowNotFoundException e) {
	    throw new AgentInitializationException("This agent cannot run on a cell without a UI.");
	}      
        acms.acceptComponentFrom(this, displayCanvas, "Spokes");
    }

    /** The agent's behavior is largely an animation loop. **/
    public void doBehavior() {
	
        // Now main loop
        while (!timeToStop) {
            displayCanvas.updateDisplay();
            try {
                Thread.sleep(updateInterval);
            } catch (InterruptedException e) {}
            Enumeration e = spokes.elements();
            while (e.hasMoreElements())
                ((Spoke)e.nextElement()).accelerateTheta(defaultDecel);
        }
	doLocalCleanup();
    }

    public void notify(HiveEvent event) {
        super.notify(event);
        updateSpokeForEvent(event);
    }

    /** When an event is received, manage the spoke associated with that
     ** event source's agent.
     **/
    protected void updateSpokeForEvent(HiveEvent event) {
        Object source = (Object)event.getSource();
        Spoke s = ensureSpoke(source);
        s.accelerateTheta(defaultAccel);
    }

    public boolean connectTo(Agent otherAgent) {
        ensureSpoke(otherAgent);
        return super.connectTo(otherAgent);
    }

    /** Make sure we have a spoke for the given object. **/
    protected synchronized Spoke ensureSpoke(Object obj) {
        Spoke s = (Spoke)spokes.get(obj);
        if (s == null) {
            s = displayCanvas.createStoppedSpoke();
            spokes.put(obj, s);
        }
        return s;
    }
}
