// $Id: SpokeCellEventAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.event.cell.*;
import net.hivecell.hive.Global;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.cell.Cell;

import java.util.Hashtable;
import java.util.Enumeration;

/** Agent that conveys a general idea of the activity at a server.
 ** It listens for events from a AgentMonitoringAgent, draws a
 ** spoke for each agent heard about.
 ** 
 ** @author Nelson Minar <nelson@media.mit.edu>
 ** @version $Revision: 1.2 $
 **/
public class SpokeCellEventAgentImpl 
extends SpokeAgentImpl {

    public SpokeCellEventAgentImpl() 
    throws java.rmi.RemoteException {
        super();
    }

    /** When this agent arrives at a server, it looks for the
     ** AgentMonitoringAgent there and connects to it. It also asks the
     ** server for a list of all agents. The agent then draws spokes,
     ** one per agent on the server. The rate of the spoke's rotation conveys
     ** how active that agent has been in messaging.
     **/
    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();

        // Try to find the server event info agent and connect to it.
        String[] amaName = {"net.hivecell.hive.agent.cell.AgentMonitoringAgent"};
        AgentMonitoringAgent ama;
        try {
            ama = (AgentMonitoringAgent)myCell.queryAgents(this, amaName, null).pickOne();
        } catch (Exception ex) {
            ama = null;
        }
        if (ama == null) {
	    throw new AgentInitializationException("Couldn't find a AgentMonitoringAgent, oh well.");
        }

        this.connectTo((Agent)ama);
    }
    
    /** When an event is received, if it's from a AgentMonitoringAgent then
     ** do something special with it. Otherwise, pass it up to superclass.
     **/
    public void updateSpokeForEvent(HiveEvent event) {
        Object source = (Object)event.getSource();

        // First, check - is the source a server event info agent?
        if (source instanceof AgentMonitoringAgent) {
            // Yep! First, accelerate our spoke for the info agent..
            ensureSpoke(source).accelerateTheta(defaultAccel);
            // And for ourselves.
            ensureSpoke(this).accelerateTheta(defaultAccel);

            // Now let's figure out what agents were involved in that event,
            // and create/accelerate their spokes.
            CellEvent se = (CellEvent)event;
            long code = se.eventCode();
            if (code == CellEvent.AGENTBORN)
                ensureSpoke(((AgentBornCellEvent)se).getAgent());
            else if (code == CellEvent.AGENTDYING)
                displayCanvas.removeSpoke((Spoke)spokes.get(((AgentDeathCellEvent)se).getAgent()));
            else if (code == CellEvent.AGENTMESSAGE) {
                ensureSpoke(((AgentMessageCellEvent)se).getSourceAgent()).accelerateTheta(defaultAccel);
                ensureSpoke(((AgentMessageCellEvent)se).getDestinationAgent()).accelerateTheta(defaultAccel);
            }
        } else {
            super.updateSpokeForEvent(event);
        }
    }

    /** When connected to a AgentMonitoringAgent, query its server for
     ** all other agents that are there.
     **/
    public boolean connectTo(Agent a) {
        if (a instanceof AgentMonitoringAgent) {
            try {
                RemoteCell rs = a.getCell();
                Enumeration e = rs.queryAgents(this, null, null).elements();
                while (e.hasMoreElements()) {
		    Object o = e.nextElement();
                    ensureSpoke( o );
		}
            } catch (Exception error) {
                Debug.warning("Spoke agent couldn't get a list of remote agents, oh well.", error);
            }
        }
        return super.connectTo(a);
    }
}



