// $Id: StringDisplayAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.
package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.EventReceivingDescription;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.BaseCanvas;
import net.hivecell.hive.support.Debug;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

/** An agent that receives events and displays the event data as a string. **/
public class StringDisplayAgentImpl
extends EventReceivingAgentImpl 
implements EventReceivingDescription {

    protected transient StringDisplayCanvas sdc;
    protected transient ComponentManagerShadow acms;

    public StringDisplayAgentImpl() throws java.rmi.RemoteException { super(); }
    
    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();
	try {
	    acms = getComponentManagerShadow();
	} catch( ShadowNotFoundException e ) {
	    throw new AgentInitializationException("This agent cannot run on a cell without a UI.");
	}      

	Description desc = new Description( this, "StringDisplayAgent" );
	desc.addSemantic("role", "string-display", null, null);
	setDescription(desc);
				    
	sdc = new StringDisplayCanvas();
	acms.acceptComponentFrom( this, sdc, "String Display" );

    }
    
    public void doBehavior() { waitUntilDeath(); }

    public Class[] getAgentSpecificEventClasses() { 
	Class[] classes = { (String.class) };
	return classes;
    }

    public synchronized void notify( HiveEvent event ) {
	super.notify( event );
        Object data = event.getEventData();
        if( data != null )
            sdc.display( data.toString() );
        else
            sdc.display( "null " + event.getClass().getName() );
    }
}

/** A canvas that displays strings for the string display agent. **/
class StringDisplayCanvas 
extends BaseCanvas {
    String mystring = new String( "" );

    public StringDisplayCanvas() {
	super();
	setBackground( Color.black );
        setFont( Global.defaultFont );
	setSize( 200, 120 );
	setVisible( true );
    }

    public void display( String s ) {
	mystring = s;
	render();
    }

    public void render( Graphics g ) {
	Dimension s = getSize();
	g.setColor( getBackground() );
	g.fillRect( 0, 0, s.width, s.height );
	g.setColor( Color.white );

        // draw the string centered
        int width = g.getFontMetrics().stringWidth( mystring );
	g.drawString( mystring, s.width / 2 - width / 2, s.height / 2 );
    }
}
