package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.agent.EventTransceivingAgent;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;

import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.support.Debug;

import java.rmi.RemoteException;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;


public class ThroughputCalculatingAgentImpl
extends EventTransceivingAgentImpl {

    private EventTransceivingAgent tta = null;
    private int eventCounter = 0;

    private HiveEvent[] eventsIn = new HiveEvent[100];
    private Long[] eventsInTime = new Long[100];


    public ThroughputCalculatingAgentImpl() throws RemoteException { super(); }

    public void setThroughputTestingAgent( EventTransceivingAgent tta ) {
	if( this.tta == null ) this.tta = tta;

    }

    public void doLocalSetup() throws AgentInitializationException  { 
	super.doLocalSetup(); 
	setIsReady( true ); 

    }

    public void doBehavior() { waitUntilDeath(); }

    public void notify( HiveEvent event ) {
	super.notify( event );
	
	if( ( eventCounter < 100 ) && event.getSource().equals( tta ) ) {
	    synchronized( eventsIn ) {
		eventsIn[eventCounter] = event;
		eventsInTime[eventCounter++] = new Long( System.currentTimeMillis() );
	    }
	    
	    if( eventCounter == 100 ) {
		double throughput = 0;
		EventReceivingAgent[] destinationAgent = { tta };
		try {
		    for( int count=0;count < 100;count++ ) {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream( baos );
			oos.writeObject( eventsIn[count].getEventData() );
			throughput += (double)baos.size();
		    }
		} catch( IOException error ) {
		    Debug.error( "io exception while trying to calculate average event data size", error );
		    eventSendingHelper.broadcastEventTo( new Double( -1.0 ), destinationAgent, null );
		    return;
		    
		}

		long maxTime = eventsInTime[0].longValue(), minTime = eventsInTime[0].longValue();
		for( int count=0;count < 100;count++ ) {
		    long newTime = eventsInTime[count].longValue();
		    if( maxTime < newTime ) maxTime = newTime;
		    if( minTime > newTime ) minTime = newTime;
		}

		throughput /= ( maxTime - minTime ) / 1000;
		eventSendingHelper.broadcastEventTo( new Double( throughput ), destinationAgent, null );
		
	    }

	} 

    }
    
}



