package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.agent.EventTransceivingAgent;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.MalformedHiveURLException;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.description.AgentSet;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.rmi.RemoteException;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;


public class ThroughputTestingAgentImpl 
extends EventTransceivingAgentImpl {

    private ComponentManagerShadow cms;
    private JTextField hostToTest;

    private EventTransceivingAgent tca;

    private CellAddress testingAddress;


    public ThroughputTestingAgentImpl() throws RemoteException { super(); }
    
    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();

	try {
	    cms = getComponentManagerShadow();
	} catch( ShadowNotFoundException error ) {
	    throw new AgentInitializationException( "no component manager shadow found on cell -- this agent needs one to run" );
	}

	startTest();

    }
    
    public void doBehavior() { waitUntilDeath(); }

    public void notify( HiveEvent he ) {
	if( !he.getSource().equals( tca ) || !( he.getEventData() instanceof Double ) )
	    return;

	Debug.error( "100 packet test over" );
	try {
	    tca.diePlease();
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to kill the throughput testing agent", error );
	    
	}
	
	double throughput = ((Double)he.getEventData()).doubleValue();
	GridLayout gl = new GridLayout( 0, 1 );
	gl.setVgap( 2 );
	final JPanel resultsPanel = new JPanel( gl );
	resultsPanel.setBorder( new EmptyBorder( 7, 7, 7, 7 ) );
	if( throughput == -1 )
	    resultsPanel.add( new JLabel( "100 packet throughput test to " + testingAddress + " failed", JLabel.CENTER ) );
	
	else {
	    resultsPanel.add( new JLabel( "100 packet throughput test to " + testingAddress, JLabel.CENTER ) );
	    String throughputString = throughput + "";
	    throughputString = throughputString.substring( 0, throughputString.indexOf( "." ) + 2 );
	    resultsPanel.add( new JLabel( "throughput of " + throughputString + " bytes/sec (calculated of event data)", JLabel.CENTER ) );

	}
	    
	JButton anotherTestButton = new JButton( "run another test" );
	anotherTestButton.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent ae ) {
		    cms.removeComponentFrom( ThroughputTestingAgentImpl.this, resultsPanel );
		    startTest();
		    
		}
	    });
	resultsPanel.add( anotherTestButton );
	
	JButton quitButton = new JButton( "quit" );
	quitButton.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent ae ) {
		    cms.removeComponentFrom( ThroughputTestingAgentImpl.this, resultsPanel );
		    diePlease();
		    return;
		}
	    });
	resultsPanel.add( quitButton );
	
	cms.acceptComponentFrom( ThroughputTestingAgentImpl.this, resultsPanel, "throughput test results" );
	
    }

    private void startTest() {
	
	final JPanel throughputAgentSetupPanel = new JPanel( new FlowLayout() );
	throughputAgentSetupPanel.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );
	throughputAgentSetupPanel.add( new JLabel( "host to test:" ) );
	
	hostToTest = new JTextField( myCell.getAddress().toString(), 30 );
	throughputAgentSetupPanel.add( hostToTest );
	
	JButton startButton = new JButton( "start test" );
	startButton.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent ae ) {

		    cms.removeComponentFrom( ThroughputTestingAgentImpl.this, throughputAgentSetupPanel );

		    testingAddress = null;
		    try {
			testingAddress = new CellAddress( hostToTest.getText().trim() );
			RemoteCell rc = testingAddress.getRemoteCell();
			    
			String[] throughputAgentSyntatic = { (ThroughputCalculatingAgentImpl.class).getName() };
			AgentSet ds = null;
			try { 
			    ds = rc.queryAgents( ThroughputTestingAgentImpl.this, throughputAgentSyntatic, null );
			} catch( ClassNotFoundException internalError ) {
			    Debug.error( "cell at \"" + testingAddress + "\" doesn't seem to know about ThroughputCalculatingAgents" );
			    ds = null;

			}
			if( ( ds == null ) || ( ds.size() == 0 ) ) {
			    try {
				Debug.notice( "shipping the throughput calculating agent to " + testingAddress );
				ThroughputCalculatingAgentImpl tcai = new ThroughputCalculatingAgentImpl();
				tcai.setThroughputTestingAgent( (EventTransceivingAgent)myCell.getRemoteObject( ThroughputTestingAgentImpl.this ) );
				tca = (EventTransceivingAgent)myCell.moveAgent( tcai, testingAddress );
				tca.blockUntilReady();

			    } catch( Exception error ) {
				Debug.error( "problem serializing a throughput calculating agent to send to " + testingAddress, error );
				return;
				
			    }
    
			} else
			    tca = (EventTransceivingAgent)ds.pickOne();

			Debug.warning( "starting test -- sending 100 events to ThroughputCalculatingAgent at \"" + testingAddress + "\"" );
			EventReceivingAgent[] destinationAgent = { tca };
			for( int count=0;count < 100;count++ )
			    eventSendingHelper.broadcastEventTo( new Long( System.currentTimeMillis() ), destinationAgent, null );
														       
		    } catch( MalformedHiveURLException error ) {
			Debug.error( "\"" + hostToTest.getText().trim() + "\" is not a valid cell address to test" );
			return;

		    } catch( CellConnectException error ) {
			Debug.error( "having problems connecting to the cell at \"" + testingAddress + "\"", error );
			return;

		    } catch( RemoteException error ) {
			Debug.error( "remote exception while trying to connect to cell at \"" + testingAddress + "\"", error );
			return;

		    }

		}
	    });
	throughputAgentSetupPanel.add( startButton );

	cms.acceptComponentFrom( this, throughputAgentSetupPanel, "start throughput test" );

    }

}
