// $Id: WheresBradAgentImpl.java,v 1.2 2000/05/19 19:55:27 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.desktop;

import net.hivecell.hive.Global;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.BaseCanvas;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.PPM;
import net.hivecell.hive.support.SerializableImage;

import java.util.Hashtable;
import java.awt.*;

/** An agent that draws a map of where Brad might be. Takes
 ** strings that are locust IDs, maps them onto a display and
 ** draws them.
 **/
public class WheresBradAgentImpl 
extends EventReceivingAgentImpl {

    protected transient MapCanvas c;
    protected transient ComponentManagerShadow acms = null;
    protected Hashtable idToPoint;                // mapping
    protected static String nowhere;
    private static String mapName = "jungleMap.ppm";
    
    public WheresBradAgentImpl() 
    throws java.rmi.RemoteException {
        super();

        // initialize the mapping
        idToPoint = new Hashtable();
        nowhere = "NOLOC";
        idToPoint.put("Loc  18. 85. 0. 1", new Point(175, 138));
        idToPoint.put("Loc  18. 85. 0. 2", new Point(399, 138));
        idToPoint.put("Loc  18. 85. 0. 3", new Point(294, 138));
    }

    public void doLocalSetup() throws AgentInitializationException  {

	super.doLocalSetup();
        try {
            acms = getComponentManagerShadow();
        }
        catch (ShadowNotFoundException e) {
	    throw new AgentInitializationException("This agent cannot run on a cell without a UI.");
        }      
        c = new MapCanvas(mapName);
        acms.acceptComponentFrom(this, c, "Where is Brad?");
    }

    public void doBehavior() {
	waitUntilDeath();
    }

    public void notify(HiveEvent event) {
        super.notify(event);

        Object data = event.getEventData();
        if (!(data instanceof String)) {
            Debug.notice("Ignoring unknown event " + event);
            return;
        }

        if (nowhere.equals(data))
            c.updateSpot(null);
        else {
            Point p = (Point)idToPoint.get(data);
            if (p == null)
                Debug.notice("Ignoring unknown location " + data);
            else {
                c.updateSpot(p);
            }
        }
    }

    public static void main(String args[]) {
        MapCanvas c = new MapCanvas("jungleMap.ppm");
        Frame f = new Frame();
        f.add(c);
        f.pack();
        f.show();
        c.updateSpot(new Point (131, 120));
        try { Thread.sleep(5000); }
        catch(Exception ex) { }
    }
}
    
class MapCanvas extends BaseCanvas {
    protected Image mapImage;
    Point spot = null;
    
    public MapCanvas(String mapName) {
        SerializableImage si;
        try {
            si = new PPM(getClass().getResourceAsStream(mapName));
        } catch (Exception ex) {
            Debug.error("Couldn't load map! ", ex);
            return;
        }
        
        mapImage = si.getImage();
        setSize(si.getSize());
        setDoubleBuffering(true);
        setVisible(true);
        setForeground(Color.red);
        setBackground(Color.white);
    }

    public void updateSpot(Point p) {
        spot = p;
        render();
    }

    public void render(Graphics g) {
        g.drawImage(mapImage, 0, 0, this);
        if (spot != null) {
            g.setColor(getForeground());
            g.fillOval(spot.x - 6, spot.y - 6, 12, 12);
            g.setColor(getBackground());
            g.fillOval(spot.x - 4, spot.y - 4, 8, 8);
            g.setColor(getForeground());
            g.drawLine(spot.x - 10, spot.y, spot.x + 10, spot.y);
            g.drawLine(spot.x, spot.y - 10, spot.x, spot. y + 10);
        }
    }
}
