// $Id: HoneyIShrunkTheCDs.java,v 1.2 2000/05/19 19:55:29 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.filter;


import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.MalformedHiveURLException;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.*;
import net.hivecell.hive.event.*;
import net.hivecell.hive.cell.*;
import net.hivecell.hive.agent.EventTranslatingAgentImpl;
import net.hivecell.hive.agent.*;
import net.hivecell.hive.description.AgentSet;
import java.lang.*;
import java.util.*;
import java.io.*;

public class HoneyIShrunkTheCDs 
extends EventTranslatingAgentImpl {

    transient Hashtable tagSongMap = new Hashtable();
    String lastsong = null;
    boolean continuous = false;

    public HoneyIShrunkTheCDs() 
    throws java.rmi.RemoteException { 
	super(); 
    }

    public void doLocalSetup() throws AgentInitializationException  {
	super.doLocalSetup();
	loadMap();
    }

    private void loadMap() {
	try {
	    File map = Global.getConfigAsFile("songMap");
	    BufferedReader br = new BufferedReader(new FileReader(map));
	    String line;
	    while((line = br.readLine()) != null){
		line.trim();
		if((line.length() == 0) || (line.charAt(0) == '#'))
		    continue;
		int space = line.indexOf(" ");
		String tag = line.substring(0,space);
		String song = line.substring(space+1);
		tagSongMap.put(tag, song);
	    }
	}
	catch (Exception e) {
	    Debug.warning("Failed to load song map, using defaults", e);
	    tagSongMap.put("00071FFC2", "Yoda");
	    tagSongMap.put("003E6F541", "CMD:pause");
	    tagSongMap.put("003E72277", "CMD:loud");
	    tagSongMap.put("003E708EC", "CMD:midvolume");
	    tagSongMap.put("003E6F985", "Beatles");
	    tagSongMap.put("003E71476", "Love");
	    tagSongMap.put("003E5E82E", "CMD:quiet");
	    tagSongMap.put("003E70497", "mp3");
	}
    }

    public void doBehavior() {
	// Here's where it looks for the
	// Pia Demo Tagreader and
	// Pia Jukebox
	// and does it's wiring
	EventSendingAgent tagreader=null;
	EventReceivingAgent jukebox=null;

	while(true){
	    String[] cName = {"net.hivecell.hive.agent.EventSendingAgent"};
	    Debug.notice("Looking for event sender");
	    AgentSet tagset = null;
	    try {
		tagset = myCell.queryAgents(this, cName, null);
	    } catch (ClassNotFoundException e) {
		Debug.warning("exception:", e);
	    }

	    Debug.notice("Eventsendingagents: "+tagset.size());
	    AgentSet taggedtags = tagset.querySet(null, "nickname");
	    //	    for(int i = 0; i < 5; i++){
	    //		Debug.notice("ESA: "+tagset.getValue(i));
	    //	    }
	    //	    Debug.notice("Tagged: "+taggedtags.matches());
	    //	    Debug.notice("Tagged value: "+taggedtags.getValue(0));

	    if(tagset !=null)
		tagset = tagset.queryValues(null, "nickname", "Pia Demo Tagreader");
	    Debug.notice("looking for jukebox");
	    CellAddress ca = null;
	    try {
		ca = new CellAddress("hive://jukebox.media.mit.edu:23235/");
	    }
	    catch(MalformedHiveURLException e) {
		Debug.critical("A known HiveAddress is now badly formed.");
	    }
	    AgentSet jbset = null;
	    try {
		RemoteCell jbCell = ca.getRemoteCell();
		String[] st = {"net.hivecell.hive.agent.EventReceivingAgent"};
		try {
		    jbset = jbCell.queryAgents(this, st, null);
		} catch(Exception err) {
		    Debug.warning("Unable to queryAgents at: " + ca.toString());
		}
	    }
	    catch(CellConnectException err) {
		Debug.warning("Unable to connect to Cell at: " + ca.toString());
	    }
	    if(jbset != null)
		jbset = jbset.queryValues(null, "nickname", "Pia Jukebox");
	    else
		Debug.notice("Jbset is null");

	    if((tagset == null) || (jbset == null)){
		Debug.notice("Nothing to hookup, yet");
	    }
	    else if((tagset.size() > 0) && (jbset.size() > 0)){
		Debug.notice("I've got one each!");
		EventSendingAgent esa = (EventSendingAgent) tagset.pickOne();
		EventReceivingAgent era = (EventReceivingAgent) jbset.pickOne();
		EventSendingAgent esa2 = (EventSendingAgent) era;
		if((!esa.equals(tagreader)) || (!era.equals(jukebox))){
		    try {
			Debug.notice(esa + " is not "+tagreader+", or "+era+" is not "+jukebox+", so I'm wiring");
			connectTo(esa);
			era.connectTo(this);
			connectTo(esa2);
			tagreader = esa;
			jukebox = era;
		    } catch (java.rmi.RemoteException re){
			// Do nothing and try again later
		    }
		}
	    }
	    else{
		Debug.notice("Tag reader matches: "+tagset.size());
		Debug.notice("Jukebox matches: "+jbset.size());
		//		Debug.notice(jbset.getValue(0));
		//		Debug.notice(jbset.getValue(1));
	    }
	    try { Thread.sleep(10000); } catch (Exception e) { }
	}
    }


    public void translate(HiveEvent event){
	if(event.getEventData() instanceof Boolean){
	    if(((Boolean) event.getEventData()).booleanValue())
		eventSendingHelper.broadcastEvent("mp3");
	    return;
	}
	String tag = (String) event.getEventData();
	if(tag.equals("003E6F285")){
	    loadMap();
	}
	else if(tag.equals("REPORT:song end")){
	    if(continuous)
		eventSendingHelper.broadcastEvent(lastsong);
	    return;
	}
	String song = (String) tagSongMap.get(tag);
	Debug.notice("Saw tag "+tag+" which maps to song "+song);
	if(song != null){
	    if(song.equals("CMD:continuous")){
		continuous = true;
	    }
	    else {
		if(song.equals("pause"))
		    continuous = false;
		Debug.notice("Supposedly sending the event "+song);
		if(song.length() >= 4)
		    if(!song.substring(0, 4).equals("CMD:"))
			lastsong = song;
		eventSendingHelper.broadcastEvent(song);
		Debug.notice("Supposedly sent the event "+song);
	    }
	}
    }
}
