// $Id: CreditsControllingHuntingAgentImpl.java,v 1.2 2000/05/19 19:55:29 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.games.hunter;

import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.desktop.CreditsAgentImpl;
import net.hivecell.hive.agent.cell.CellControlAgent;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;

import java.awt.*;
import java.rmi.RemoteException;
import javax.swing.*;
import javax.swing.border.*;


public class CreditsControllingHuntingAgentImpl
extends EventReceivingAgentImpl 
implements CreditsControllingHuntingAgent {

    private ComponentManagerShadow acms;

    private transient JLabel currentLocation;
    
    private transient JTextField hunterMessageArea;
    

    public CreditsControllingHuntingAgentImpl()
    throws RemoteException {  
	super();
    }

    public void doLocalSetup() throws AgentInitializationException  {

	super.doLocalSetup();

	// try to get the GUI
	acms = null;
	try {
	    acms = getComponentManagerShadow();
	} catch( ShadowNotFoundException error ) {
	    throw new AgentInitializationException("This agent cannot run on a cell without a UI.");
	}

	// set up the ui
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	JPanel displayPanel = new JPanel( gbl );
	displayPanel.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );
    
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.gridwidth = 5;
	gbc.gridheight = 1;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.insets = new Insets( 3, 3, 3, 3 );
	currentLocation = new JLabel( "CreditsAgent in unknown location" );
	gbl.setConstraints( currentLocation, gbc );
	displayPanel.add( currentLocation );

	gbc.gridx = 0;
	gbc.gridy = 1;
	gbc.gridwidth = 5;
	gbc.fill = GridBagConstraints.NONE;
	hunterMessageArea = new JTextField( 35 );
	hunterMessageArea.setEditable( false );
	hunterMessageArea.setAutoscrolls( true );
	gbl.setConstraints( hunterMessageArea, gbc );
	displayPanel.add( hunterMessageArea );

	Frame f = acms.acceptComponentFrom( this, displayPanel, "Credits Agent Hunter" );
	f.setResizable( false );

    }

    public void doBehavior() { 
	// need to start up the CreditsControllingAgentImpl
	CellControlAgent sca = null;
	try {
	    String[] cellControlAgentName = { "net.hivecell.hive.agent.cell.CellControlAgent" };
	    AgentSet ds = myCell.queryAgents( this, cellControlAgentName, null );
	    sca = (CellControlAgent)ds.pickOne();
	    sca.createAgent( "net.hivecell.hive.agent.games.hunter.CreditsHuntingAgentImpl" );
	} catch( ClassNotFoundException error ) {
	    Debug.error( "Huh?  This should never happen!", error );

	} catch( InstantiationException error ) {
	    Debug.error( "Huh?  This should never happen!", error );

	} catch( RemoteException error ) {
	    Debug.error( "Remote exception while trying to talk to our local server?", error );
	    diePlease();
	    return;

	}

	waitUntilDeath(); 
    }

    public void notify( HiveEvent he ) {

	if( !(he.getSource() instanceof CreditsHuntingAgent) ) {
	    Debug.notice( "Tossing away an event from a non credits hunting agent" );
	    return;
	}

	CellAddress location = ((CreditsLocationInformation)he.getEventData()).getAddress();
	if( location != null )
	    currentLocation.setText( "CreditAgent location : " + ((CreditsLocationInformation)he.getEventData()).getAddress().toString() );
	else
	    currentLocation.setText( "CreditAgent location : unknown" );
	hunterMessageArea.setText( ((CreditsLocationInformation)he.getEventData()).getMessage() );

    }

}













