// $Id: CreditsHuntingAgentImpl.java,v 1.2 2000/05/19 19:55:29 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.games.hunter;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Vector;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.agent.EventSendingAgentImpl;
import net.hivecell.hive.agent.desktop.CreditsAgentImpl;
import net.hivecell.hive.agent.cell.CellListAgent;
import net.hivecell.hive.agent.cell.CellListInfo;
import net.hivecell.hive.cell.AgentDeliveryException;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.Debug;

public class CreditsHuntingAgentImpl
extends EventSendingAgentImpl 
implements CreditsHuntingAgent {

    private CellAddress homeAddress = null;
    private transient CreditsControllingHuntingAgent ccha; // our hook back to our controller


    public CreditsHuntingAgentImpl() 
    throws RemoteException {
	super();
    }

    public void doLocalSetup() throws AgentInitializationException  {

	super.doLocalSetup();

	if( homeAddress == null )
	    homeAddress = myCell.getAddress();

	// get in contact with our controlling agent
	ccha = null;
	try {
	    if( ( ccha = getControllingAgent() ) == null ) {
		throw new AgentInitializationException( "Couldn't find the controlling agent on start up!  Have to die..." );
	    }
	} catch( Exception error ) {
	    throw new AgentInitializationException( "Exception while trying to get to our controlling agent -- committing suicide" );
	}
	
	// tell it that we are initializing
	messageController( "Initializing..." );

    }

    public void doBehavior() {
	
	try {
	    if( ( ccha == null ) && ( ( ccha = getControllingAgent() ) == null ) ) {
		Debug.error( "Cannot acquire our controlling agent -- committing suicide!" );
		diePlease();
		return;
	    }
	} catch( Exception error ) {
	    Debug.error( "Exception while trying to get to our controlling agent -- committing suicide", error );
	    diePlease();
	    return;
	}

	while( !timeToStop ) {

	    // attempt to acquire the credits agent on the local server
	    Agent ca = null;
	    try {
		ca = getCreditsAgentImpl( myCell.getAddress() );
	    } catch( RemoteException error ) {
		Debug.error( "Remote exception while trying to locate the credits agent locally -- coping", error );
	    } catch( CellConnectException error ) {
		Debug.error( "This shouldn't ever happen as we are talking to the local server", error );
	    }

	    // if the credits agent is no longer on the same server we
	    // aree, so begin the search!
	    if( ca == null ) {
		messageController( "Lost acquisition of credits agent -- beginning search", null );

		Vector possibleAddresses = null;
		try {
		    possibleAddresses = getUpCells();
		} catch( RemoteException error ) {
		    Debug.warning( "Cannot get a list of up servers from the local server list agent due to a remote exception", error );
		}
		if( possibleAddresses != null ) {

		    Enumeration e = possibleAddresses.elements();
		    while( e.hasMoreElements() ) {
			CellAddress possibleAddress = (CellAddress)e.nextElement();
			messageController( "Searching for credits agent on " + possibleAddress, null );

			// look for the prey
			Agent thePrey = null;
			try {
			    thePrey = getCreditsAgentImpl( possibleAddress );
			} catch( Exception error ) {
			    Debug.error( "Problem searching on " + possibleAddress + ".  Ignoring.", error );
			} 

			
			// have we found the prey?  if so -- move towards it!!!
			if( thePrey != null ) {
			    messageController( "Found the credits agent on " + possibleAddress + ".  Moving there!", possibleAddress );
			    try {
				moveTo( possibleAddress );
			    } catch( AgentDeliveryException error ) {
				Debug.error( "Delivery exception -- ignoring", error );
				messageController( "Problem moving -- will attempt to reacquire and try again!", null );
				break;
			    }
			}
		    }
		}

		messageController( "Did not move to the credits agent -- sleeping a while and then attempt to find and move again", null );
	    } else

		messageController( "Credits agent still on our cell -- " + myCell.getAddress() );

	    
	    try {
		messageController( "Halting for 15 seconds" );
		Thread.sleep( 5000 );
	    } catch( InterruptedException error ) {
	    }

	}

    }

    public void onDying() {

	messageController( "On cell " + myCell.getAddress() + " and being killed" );

    }


    // helper methods

    private void messageController( String message ) {
	EventReceivingAgent[] destinationAgent = { ccha };
	eventSendingHelper.broadcastEventTo( new CreditsLocationInformation( message, myCell.getAddress() ), destinationAgent, null );
    }
    
    private void messageController( String message, CellAddress address ) { 
	EventReceivingAgent[] destinationAgent = { ccha };
	eventSendingHelper.broadcastEventTo( new CreditsLocationInformation( message, address ), destinationAgent, null );
    }

    private CreditsControllingHuntingAgent getControllingAgent() 
    throws CellConnectException, RemoteException {

	Debug.notice( "Looking for the controlling agent at " + homeAddress );
	CreditsControllingHuntingAgent ccha = null;
	try {
	    String[] creditsControllingHuntingAgentImplName = { "net.hivecell.hive.agent.games.hunter.CreditsControllingHuntingAgentImpl" };
	    AgentSet ds = homeAddress.getRemoteCell().queryAgents( this, creditsControllingHuntingAgentImplName, null );
	    ccha = (CreditsControllingHuntingAgent)ds.pickOne();
	} catch( ClassNotFoundException error ) {
	    Debug.error( "Huh?  This should never be thrown!!", error );
	}
	return ccha; 
    }

    private Agent getCreditsAgentImpl( CellAddress address ) 
    throws CellConnectException, RemoteException {
	Agent ca = null;
	try {
	    String[] CreditsAgentImplName = { "net.hivecell.hive.agent.desktop.CreditsAgentImpl" };
	    AgentSet ds = address.getRemoteCell().queryAgents( this, CreditsAgentImplName, null ); 
	    ca = (Agent)ds.pickOne();
	} catch( ClassNotFoundException error ) {
	    Debug.error( "Huh?  This should never be thrown!!", error );
	} 
	return ca;
    }

    private Vector getUpCells()  
    throws RemoteException {
	try {
	    String[] serverListAgentType = { "net.hivecell.hive.agent.cell.CellListAgentImpl" };
	    AgentSet ds = myCell.queryAgents( this, serverListAgentType, null );
	    CellListAgent cla = (CellListAgent)ds.pickOne();
	    Vector serverListInfo = cla.listCells( this );
	    Vector upCells = new Vector();
	    Enumeration e = serverListInfo.elements();
	    while( e.hasMoreElements() ) {
		CellListInfo cli = (CellListInfo)e.nextElement();
		if( cli.isUp() && !cli.getCell().equals( myCell ) )
		    upCells.addElement( cli.getAddress() );
	    }
	    return upCells;
	} catch( ClassNotFoundException error ) {
	    Debug.notice( "Huh?  This should never happen!!", error );
	    return null;
	}
    }

}






