package net.hivecell.hive.agent.hiveui.remoteui;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.DisplayExportingAgent;
import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.agent.EventSendingHelper;
import net.hivecell.hive.agent.cell.CellAgent;
import net.hivecell.hive.agent.cell.CellControlAgent;
import net.hivecell.hive.agent.cell.UnableToMonitorCellException;
import net.hivecell.hive.agent.hiveui.BaseUIAgentImpl;
import net.hivecell.hive.agent.service.AutoMonitoringAgentImpl;
import net.hivecell.hive.cell.MobileAgentInputStream;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.event.cell.CellReportCellEvent;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.MalformedHiveURLException;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.Global;

import java.awt.CardLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayInputStream;
import java.net.URL;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;


public class AutoUIAgentImpl 
extends BaseUIAgentImpl 
implements EventReceivingAgent {
    
    /**
     * the frame
     */
    protected JFrame frame;

    /** 
     * the top most panel -- this contains the card layout which
     * contains the other cells 
     */
    protected JPanel panel;
    protected CardLayout cl;

    /**
     * the component manager shadow
     */
    protected ComponentManagerShadow cms;

    /**
     * the cell address of the cell we are currently monitoring
     */
    transient protected CellAddress currentCell;

    /**
     * this hashtable maps cell addresses to panels which will have
     * the buttons for each agent 
     */
    protected Hashtable cards = new Hashtable();

    /**
     * this hashtable maps the agent to its button
     */
    protected Hashtable buttons = new Hashtable();
    
    /**
     * the event sending helper we will use to send events to the
     * proper agents 
     */
    protected EventSendingHelper esh;

    /**
     * the reference to our cell control agent.  this is not always
     * needed, but if we have it -- then we will keep the reference.
     * we might want to directly birth an AgentImpl, which is not
     * something the BaseUIAgentImpl will need to do 
     */
    private transient CellControlAgent cca;

    /**
     * this is just a vector of the agents we have created as remote
     * control agents.  we need to kill these off when we switch off
     * to another cell 
     */
    private transient Vector remoteControls;

    /** 
     * the constant we will use to put find our splash screen inthe
     * cards and hte layout manager 
     */
    private final static String SPLASH_SCREEN = "final screen";


    public AutoUIAgentImpl() throws RemoteException { super(); }

    /**
     * setup this agent -- it will instantiate an
     * AutoMonitoringAgentImpl so we will continusouly be getting all
     * the cells in the system.  
     */
    public void doLocalSetup() throws AgentInitializationException  {
	
	super.doLocalSetup();
 
	try {
	    cms = getComponentManagerShadow();
	} catch( ShadowNotFoundException error ) {
	    throw new AgentInitializationException( "this agent needs a component manager shadow to function" );
	}

	esh = new EventSendingHelper( this, myCell );

	try {
	    String[] syntatic = { (AutoMonitoringAgentImpl.class).getName() };
	    // not using the ama here, because it is local, this should be faster
	    AgentSet ds = myCell.queryAgents( this, syntatic, null );
	    if( ds.size() == 0 ) {
		Debug.notice( "no automonitoring agent found on local cell -- instantiating one" );
		createNewAgentCommand( (AutoMonitoringAgentImpl.class).getName(), myCell.getAddress() );
	    }

	} catch( ClassNotFoundException error ) {
	    // this means that we do not have an auto monitroing agent
	    // class file on our system.  fine then, we won't
	    // instantiate it

	} catch( InstantiationException error ) {
	    // this means that we couldn't create the auto monitoring
	    // agent.  something is really messed up if this is teh
	    // case.

	}

	cl = new CardLayout();
	panel = new JPanel( cl );
	JPanel autoUIAgentFrame = new JPanel( new GridLayout( 0, 1 ) );
	autoUIAgentFrame.add( new JLabel( "AutoUIAgent", new ImageIcon( getAgentIcon( this ).getImage() ), JLabel.CENTER ) );
	autoUIAgentFrame.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );
	autoUIAgentFrame.doLayout();
	cards.put( SPLASH_SCREEN, autoUIAgentFrame );
	panel.add( autoUIAgentFrame, SPLASH_SCREEN );
	panel.setBorder( new EmptyBorder( 3, 3, 3, 3 ) );
	cl.show( panel, SPLASH_SCREEN );
	this.currentCell = null;

	frame = (JFrame)cms.acceptComponentFrom( this, panel, "AutoUIAgent" );
	frame.setSize( frame.getPreferredSize() );

	readyFlag = true;
	synchronized( this ) { notifyAll(); }

	// make sure we have all our agents
	Enumeration agents = ag.listAllAgents();
	while( agents.hasMoreElements() ) renderNewAgent( (Agent)agents.nextElement() );
	
	remoteControls = new Vector();

    }

    /**
     * override the method to create the menus so we don't bother
     * creating them -- we don't need them in our implementation 
     */
    public void constructMenus() { }

    /**
     * this is how we can react to stuff.  we can take an event -- and
     * if it is a string, and the sting is a cell address, then we
     * should start monitoring the cell.  this provides an easy
     * interface to the cricket location stuff.  works better for our
     * "ad hoc" computing.  but first the UI portion of this is given
     * the chance to parse this.  if you do not send it a hive URL,
     * then it just switches to the splash screen
     *
     * @param event the event that is coming into this agent 
     */
    public synchronized void notify( HiveEvent event ) {
	super.notify( event );
	
	if( ( event.getEventData() != null ) && ( event.getEventData() instanceof String ) ) {
	    String eventData = (String)event.getEventData();
	    CellAddress ca = null;
	    try {
//  		if (eventData.indexOf("://") <= 0) {
//  		    throw new MalformedHiveURLException();
//  		}

		ca = new CellAddress( eventData );
		if( ( this.currentCell == null ) || !ca.equals( currentCell ) ) {
		    if( !cards.containsKey( ca ) ) {
			Debug.notice( "starting to monitor " + ca );
			monitorCell( ca );
			cl.show( panel, ca.toString() );
			
		    } else {
			Debug.notice( "displaying " + ca );
			cl.show( panel, ca.toString() );
			
		    }
		    this.currentCell = ca;
		    frame.setSize( frame.getPreferredSize() );

		    // kill the remote controls that we have created
		    Enumeration killAgents = remoteControls.elements();
		    while( killAgents.hasMoreElements() )
			killAgentCommand( (Agent)killAgents.nextElement() );

		}

	    } catch( MalformedHiveURLException error ) {
		cl.show( panel, SPLASH_SCREEN );
		this.currentCell = null;
		frame.setSize( frame.getPreferredSize() );
		Enumeration killAgents = remoteControls.elements();
		while( killAgents.hasMoreElements() )
		    killAgentCommand( (Agent)killAgents.nextElement() );

		return;

	    } catch( UnableToMonitorCellException error ) {
		Debug.error( "problem beginning monitoring " + ca, error );
		return;

	    }
	    
	}
	
    }

    private void pauseForReadyFlag() {
	while( !readyFlag ) 
		try { 
		    synchronized( this ) {
			wait();
		    }
		} catch( InterruptedException error ) { }
    }

    /**
     * render a new agent to the display.  we first need to figure out
     * which cell the agnet is on, then we need to pull the panel that
     * the cell address refers to, and then we add to that one.
     *
     * @param a the agent tha we are adding 
     */
    public synchronized void renderNewAgent( final Agent a ) { 

	pauseForReadyFlag();

	if( buttons.containsKey( a ) ) return;
	
	JPanel cellPanel = null;
	synchronized( cards ) {
	    cellPanel = (JPanel)cards.get( getAgentAddress( a ).toString() );
	    if( cellPanel == null ) {
		cellPanel = new JPanel( new GridLayout( 0, 1 ) );
		cards.put( getAgentAddress( a ).toString(), cellPanel );
		panel.add( cellPanel, getAgentAddress( a ).toString() );
	    }
	}

        // (nelson) added code here to parse out the image from the
        // description object. Raffi, use this to do something useful,
        // probably replace the if test right below. If an agent has
        // this property, we should display it, otherwise don't.
        String autoUIImageURL = getAutoUIImage( a );
        if( autoUIImageURL == null ) {
            Debug.notice( "no Auto UI Image for " + Global.shortString( a ) );
	    return;
	}

//  	if( !( ( a instanceof EventReceivingAgent ) ||
//  	       ( a instanceof DisplayExportingAgent ) ) ||
//  	    ( a instanceof CellAgent ) ||
//  	    ( a instanceof BaseUIAgentImpl ) )
//  	    return;
	
	Image image = null;
	try {
	    URL imageURL = new URL( autoUIImageURL );
	    image = Toolkit.getDefaultToolkit().getImage( imageURL );
	    if( image == null ) {
		Debug.error( imageURL + " did not return a valid image for " + Global.shortString( a ) );
		return;
	    }
	} catch( MalformedURLException error ) {
	    Debug.error( autoUIImageURL + " is not a valid URL to get the image for " + Global.shortString( a ) );
	    return;
	}

	JButton agentButton = new JButton( new ImageIcon( image ) );
	if( a instanceof DisplayExportingAgent )
	    agentButton.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent ae ) {

		    try {
			Agent createdAgent = ((DisplayExportingAgent)a).sendUIAgent( (Agent)AutoUIAgentImpl.this, myCell.getAddress() );
			if( createdAgent != null ) 
			    remoteControls.addElement( createdAgent );
			else
			    Debug.error( "problem getting the exporter display agent" );

		    } catch( Exception error ) {
			Debug.error( "problem getting the exporter display agent", error );

		    }
		}
	    });
	else
	    agentButton.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent ae ) {
		    EventReceivingAgent[] destinationAgent = { (EventReceivingAgent)a };
		    esh.broadcastEventTo( null, destinationAgent, null );
		}
	    });
	agentButton.show();
	buttons.put( a, agentButton );
	cellPanel.add( agentButton );
	cellPanel.doLayout();

    }
    
    /** 
     * remove a button from a certain card.  the card is not removed, however
     *
     * @param a the agent that we are removing from the card
     * @param ca the cell address that we are removing from
     * @param incoming the array of agents that we have incoming edges from
     * @param outgoing the array of agents that we have outgoing edges to
     */
    public void renderRemoveAgent( Agent a, CellAddress ca, Agent[] incoming, Agent[] outgoing ) { 

	if( !( ( a instanceof EventReceivingAgent ) &&
	       ( a instanceof DisplayExportingAgent ) ) ||
	    ( a instanceof CellAgent ) ||
	    ( a instanceof BaseUIAgentImpl ) )
	    return;

	JPanel cellPanel = (JPanel)cards.get( ca.toString() );
	if( cellPanel == null ) {
	    Debug.error( "can't find the card panel for " + ca );
	    return;
	}

	JButton button = (JButton)buttons.get( a );
	if( button == null ) {
	    Debug.error( "can't find the button for " + a + " to remove from the card panel" );
	    return;
	}
	cellPanel.remove( button );
	buttons.remove( a );
	cellPanel.doLayout();

    }

    public void renderConnectAgents( Agent s, Agent d ) { }
    public void renderDisconnectAgents( Agent s, Agent d ) { }
    public void renderDisconnectFromAll( Agent a, Agent[] incoming, Agent[] outgoing ) { }
    public void renderAgentMessage( Agent s, Agent d ) { }
    public void renderCellReport( CellReportCellEvent report ) { }

    /** Given an agent, check its RDF description for an autoUIAgent property.
     ** Sample RDF description input:
     ** <?xml version="1.0"?>
     ** <rdf:RDF xmlns="http://www.media.mit.edu/hive-agent-syntax#"
     **          xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
     ** <rdf:Description about="net.hivecell.hive.agent.thing.CricketAgentImpl">
     ** <autoUIImage>file://localhost/mas/u/nelson/hive/autoui-pics/FlagCricket.gif</autoUIImage>
     ** </rdf:Description>
     ** </rdf:RDF>
     **/
    protected String getAutoUIImage( Agent a ) {
        // Get the description from the agent
        Description description = null;
        try {
            description = a.getDescription();
        } catch( RemoteException re ) {
            Debug.warning( "Couldn't get description for agent " + Global.shortString(a));
        }
        if( description == null ) return null;

        // Now parse out the autoUIImage property from the RDF
	if (description.selectFirst("config")) {
	    description = description.getMatch();

	    if (description.selectFirst("autoUIImage")) {
		description = description.getMatch();
		return description.getText();
	    }
        }

	return (null);
    }
}
