// $Id: ContrastFilterAgentImpl.java,v 1.2 2000/05/19 19:55:31 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.image;

import net.hivecell.hive.support.*;
import net.hivecell.hive.agent.AgentInitializationException;
import java.awt.*;

public class ContrastFilterAgentImpl 
extends GenericFilterAgentImpl 
implements ContrastFilterAgent {

    int nPixels=0;
    int redMean, blueMean, greenMean;
    float redTot, blueTot, greenTot;
    int redMin=255, blueMin=255, greenMin=255;
    int redMax=0, blueMax=0, greenMax=0;

    float contrast = (float) 0.0;
    
    /**
       * Get the value of contrast.
       * @return Value of contrast.
       */
    public float getContrast() { return contrast; }
    
    /**
       * Set the value of contrast.
       * @param v  Value to assign to contrast.
       */
    public void setContrast(float v) { this.contrast = v; }
    
    public ContrastFilterAgentImpl() 
    throws java.rmi.RemoteException {
	super();
    }

    public void doLocalSetup() throws AgentInitializationException  {
        super.doLocalSetup();
    }

    public SerializableImage filter( SerializableImage si ) {
	SerializableImage newSI = (SerializableImage) si.clone();
	int[] rep = newSI.getRep();
	for (int i = 0; i < rep.length; i++) {
	    distribution(rep[i]);
	}
	finishDistribution();
	System.out.println("Red: "+redMin+", "+redMean+", "+redMax);
	System.out.println("Green: "+greenMin+", "+greenMean+", "+greenMax);
	System.out.println("Blue: "+blueMin+", "+blueMean+", "+blueMax);
	for (int i = 0; i < rep.length; i++) {
	    rep[i] = contrastify(rep[i]);
	}

	return newSI;
    }

    private void finishDistribution(){
	redMean = Math.round(redTot/nPixels);
	greenMean = Math.round(greenTot/nPixels);
	blueMean = Math.round(blueTot/nPixels);
    }

    private void distribution(int pixel){
	float r = (float) ((pixel >> 16) & 255);
	float g = (float) ((pixel >>  8) & 255);
	float b = (float) ((pixel      ) & 255);

	nPixels++;
	redTot+=r;
	greenTot+=g;
	blueTot+=b;
	if(r > redMax)
	    redMax = (int) r;
	if(g > greenMax)
	    greenMax = (int) g;
	if(b > blueMax)
	    blueMax = (int) b;

	if(r < redMin)
	    redMin = (int) r;
	if(g < greenMin)
	    greenMin = (int) g;
	if(b < blueMin)
	    blueMin = (int) b;
    }

    private int contrastify(int pixel) {
	float r = (float) ((pixel >> 16) & 255);
	float g = (float) ((pixel >>  8) & 255);
	float b = (float) ((pixel      ) & 255);

	int red=128, green=128, blue=128;

	if(r >= redMean)
	    red = 128 + Math.round((r-redMean)+(((contrast*(redMax-redMean))*(127/(redMax-redMean)))/(1+contrast)));
	if(g >= greenMean)
	    green = 128 + Math.round((g-greenMean)+(((contrast*(greenMax-greenMean))*(127/(greenMax-greenMean)))/(1+contrast)));
	if(b >= blueMean)
	    blue = 128 + Math.round((b-blueMean)+(((contrast*(blueMax-blueMean))*(127/(blueMax-blueMean)))/(1+contrast)));

	if(r < redMean)
	    red = 128 + Math.round((r-redMean)-(((contrast*(redMin-redMean))*(127/(redMin-redMean)))/(1+contrast)));
	if(g < greenMean)
	    green = 128 + Math.round((g-greenMean)-(((contrast*(greenMin-greenMean))*(127/(greenMin-greenMean)))/(1+contrast)));
	if(b < blueMean)
	    blue = 128 + Math.round((b-blueMean)-(((contrast*(blueMin-blueMean))*(127/(blueMin-blueMean)))/(1+contrast)));

	if(red > 255)
	    red = 255;
	if(blue > 255)
	    blue = 255;
	if(green > 255)
	    green = 255;
	if(red < 0)
	    red = 0;
	if(blue < 0)
	    blue = 0;
	if(green < 0)
	    green = 0;
	int p = (255 << 24) + (red << 16) + (green << 8) + blue;
	return p;
    }

}
