// $Id: FakeCameraAgentImpl.java,v 1.1 2000/05/18 22:04:58 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.image;
import java.awt.*;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.*;
import net.hivecell.hive.event.*;
import net.hivecell.hive.support.*;
import net.hivecell.hive.shadow.*;

/** 
 * This is a FakeCamera which pretends to be a camera, but provides
 * images from an internal source. It is useful for testing when you
 * don't really have a QuickCam hooked up to the Hive Cell.
 */
public class FakeCameraAgentImpl 
extends ImageCreationAgentImpl {
    /** 
     * The shadow fake camera source
     */
    FakeCameraSource fcs;

    /** 
     * This constructor is just here to throw RemoteException
     * 
     * @exception java.rmi.RemoteException 
     */
    public FakeCameraAgentImpl() 
    throws java.rmi.RemoteException {
	super();
    }

    /** Set up the fake camera source.
     * 
     */
    public void init() {
	fcs = new FakeCameraSource();
    }
    
    /** Return a SerializableImage from the fake camera source. This is a blocking
     *  call that takes several seconds to return.
     * 
     * @return the new image.
     */
    public SerializableImage fetchImage() { 
	return fcs.getImg();
    }
}

/** This is a convenience class that loads several test PPMs and makes them available as a fake camera.
 * 
 */
class FakeCameraSource {
    // This class is a fake camera to test the CameraAgent because I don't
    // really have a quickcam on my system. 

    /** The index of the most recent image returned.
     * 
     */
    int count;
    /** The image store.
     * 
     */
    SerializableImage[] si = null;
    /** The places to get the test images.
     * 
     */
    public static final String[] testPPM = {
      
	"/net/hivecell/hive/support/hivegroup1.ppm",
	"/net/hivecell/hive/support/hivegroup2.ppm"
    };

    /** Construct the fake camera source and load the test images.
     * 
     */
    public FakeCameraSource() {
	try {
	    si = new SerializableImage[testPPM.length];
	    for (int i = 0; i < si.length; i++) {
		si[i] = (SerializableImage)(new PPM(getClass().getResourceAsStream(testPPM[i])));
	    }
	    count = 0;
	}
	catch (Exception e) {
	    Debug.println(Debug.WARNING, "Unable to load test ppm.", e);
	}
    }
    /** Wait 5 seconds to simulate a camera, and then return the PPM.
     * 
     */
    public SerializableImage getImg() {
	// To simulate the idea that it takes a while to get an image,
	// This method takes a bit of time to return.
	try {
	    Thread.sleep(1000);
	}
	catch(InterruptedException e) {
	}
	return si[count++ % testPPM.length];
    }   
}





