// $Id: GrayscaleFilterAgentImpl.java,v 1.1 2000/05/18 22:04:58 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.image;
import net.hivecell.hive.support.*;
import net.hivecell.hive.support.Debug;
import java.awt.*;

/** 
 * This filter implements a grayscale filter for images. The "magic"
 * formula is that for every pixel, gray = 0.299 * red + 0.587 * green
 * + 0.114 * blue.
 */
public class GrayscaleFilterAgentImpl 
extends GenericFilterAgentImpl {

    /** Construct a new filter.
     * 
     * @exception java.rmi.RemoteException 
     */
    public GrayscaleFilterAgentImpl() 
    throws java.rmi.RemoteException {
	super();
    }
    
    /** Filter the image.
     * 
     * @param si The source image
     * @return The filtered image
     */
    public SerializableImage filter(SerializableImage si) {
	SerializableImage newSI = (SerializableImage) si.clone();
	int[] rep = newSI.getRep();
	for (int i = 0; i < rep.length; i++) {
	    rep[i] = grayPixel(rep[i]);
	}
	return newSI;
    }

    /** gray a single pixel
     * 
     * @param pixel the color pixel
     * @return the gray pixel
     */
    private static int grayPixel(int pixel) {
	float r = (float) getRed(pixel);
	float g = (float) getGreen(pixel);
	float b = (float) getBlue(pixel);
	
	int gray = Math.round((0.299f * r) + (0.587f * g) + (0.114f * b));
	return setAlpha(255) + setRed(gray) + setGreen(gray) + setBlue(gray);
    }
    


}



