// $Id: ImageCreationAgentImpl.java,v 1.2 2000/05/19 19:55:31 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.image;
import java.awt.*;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.*;
import net.hivecell.hive.event.*;
import net.hivecell.hive.support.*;
import net.hivecell.hive.shadow.*;

/** This is an abstract class that represents an image CREATION agent.
 * Any Agent that creates images either through a camera or web retrieval
 * or algorithmic generation should subclass this class, implementing the
 * init() and fetchImage() methods.
 * 
 */
public abstract class ImageCreationAgentImpl 
extends ImageSourceAgentImpl {

    /** The index of the most recently requested image. Used to determine whether more images need to be generated.
     * 
     */
    protected int requested;

    /** Set up the ImageCreationAgent.
     * 
     * @exception java.rmi.RemoteException 
     */
    public ImageCreationAgentImpl() 
    throws java.rmi.RemoteException {
	super();
	requested = -1;
    }

    /** When an event of any kind comes in, if the Agent is busy, broadcast
     * an ignored message. If the agent is not busy, then request a new image
     * and send out a pending notification.
     * 
     * @param event The incoming event.
     */
    public void translate(HiveEvent event) {
	if (isBusy()) {
	    eventSendingHelper.broadcastEvent
		(new ImageEvent(ImageEvent.IGNORED, rb.getLatestIndex()));
	}
	else {
	    requested++;
	    eventSendingHelper.broadcastEvent(new ImageEvent(ImageEvent.PENDING, 
							   rb.getLatestIndex() + 1));
	}			      
    }


    /** This method should handle any work that needs to be done before the first image
     * can be retrieved.
     * 
     */
    public abstract void init();

    /** This method should do the work of retrieving or constructing the new image. It
     * is expected to block.
     * 
     */
    public abstract SerializableImage fetchImage();

    public SerializableImage getImage(Agent sender, int i) {
	myCell.noteAgentMessage( this, "getImage", sender, this ); // the message
	myCell.noteAgentMessage( this, "getImage reply", this, sender ); // the reply
	return fetchImage();
    }

    public SerializableImage getImage(Agent sender) {
	myCell.noteAgentMessage( this, "getImage", sender, this ); // the message
	myCell.noteAgentMessage( this, "getImage reply", this, sender ); // the reply
	return getImage( this, 0 );
    }

    public void doLocalSetup() throws AgentInitializationException  {
        super.doLocalSetup();
        init();
    }

    /** If an image has been requested, set the busy flag, get the
     * image, notify listeners of the new image, clear the busy flag
     * and loop. If no image has been requested, sleep for a bit and
     * check again.
     * 
     */
    public void doBehavior() {
	while(!timeToStop) {
	    if(requested > rb.getLatestIndex()) {
		setBusy(true);
		rb.addElement(fetchImage());
		eventSendingHelper.broadcastEvent(new ImageEvent
						  (ImageEvent.READY,
						   rb.getLatestIndex()));
		setBusy(false);
	    }
	    else {
		try {
		    Thread.sleep(200);
		}
		catch(InterruptedException e) {
		    ;;
		}
	    }
	}
    }
}







