// $Id: ImageDisplayAgentImpl.java,v 1.2 2000/05/19 19:55:31 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

// Simple agent to display iamges.
package net.hivecell.hive.agent.image;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.EventReceivingDescription;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.*;
import net.hivecell.hive.support.Debug;

import java.rmi.RemoteException;

/** 
 * This agent provides a panel that displays any images it
 * receives. When it gets a pending notification, it grays out the
 * current image, and then when the image arrives it displays it. It
 * centers the image and also doubles the size if it can do so without
 * cutting off the image.
 */
public class ImageDisplayAgentImpl 
extends EventReceivingAgentImpl {

    /** 
     * The actual canvas on which to display the image.
     */
    ImageDisplayCanvas displayCanvas;

    protected transient ComponentManagerShadow cms = null;

    /** 
     * Create a new ImageDisplayAgent
     *
     * @exception RemoteException 
     */
    public ImageDisplayAgentImpl() throws RemoteException { super(); }
    
    public void doLocalSetup() throws AgentInitializationException  {
        super.doLocalSetup();
        try {
	    cms = getComponentManagerShadow();
	} catch( ShadowNotFoundException e ) {
	    throw new AgentInitializationException( "This agent cannot run on a cell without a UI." );
	}
	displayCanvas = new ImageDisplayCanvas();
	cms.acceptComponentFrom( this, displayCanvas, "Image Display" );
    }

    /** 
     * Simply create the canvas and hand it to the cell.
     */
    public void doBehavior() { waitUntilDeath(); }

    /** 
     * Only allow connections to ImageSourceAgents
     * 
     * @param a the agent that is attempting to connect
     */
    public boolean connectTo( Agent a ) {
	if( !( a instanceof ImageSourceAgent ) ) {
	    Debug.warning( "ImageDisplayAgent told to connect to a non-ImageSource." );
	    return false;
	} else
	    return super.connectTo( a );
    }

    public Class[] getAgentSpecificEventClasses() {
	Class[] expectedClasses = { SerializableImage.class };
	return expectedClasses;
    }

    // callback method. Manage display;
    /** 
     * This is where the work is done. If the source is a
     * ImageSourceAgent and the event is a ImageEvent, then the
     * ImageEvent is checked for it's code.  If it is a PENDING
     * notification, then the agent puts up a pending notice. If it is
     * a READY notification, then the display retrieves the image and
     * displayes it. If it is an IGNORED message, then it is ignored.
     * 
     * @param event The incoming event.  
     */
    public void notify( HiveEvent event ) {
        super.notify( event );

	ImageSourceAgent imageSource = null;
	ImageEvent ie = null;
	try {
	    imageSource = (ImageSourceAgent)event.getSource();
	    ie = (ImageEvent)event.getEventData();
	} catch( ClassCastException ex ) {
	    Debug.notice( "Display Agent got a non ImageEvent" );
	    return;
	}

	SerializableImage si = null;
	if( ie.getStatus() == ie.PENDING )
	    displayCanvas.imagePending();

	else if( ie.getStatus() == ie.READY ) {
	    try {
		si = imageSource.getImage( this, ie.getFrameNum() );
	    } catch( RemoteException re ) {
		Debug.error( re );
	    }
	    myCell.noteAgentMessage( this, "getImage", this, imageSource );
	    myCell.noteAgentMessage( this, "getImage reply", imageSource, this );
	    if( si != null )
		displayCanvas.showImage( si.getImage(), si.getSize() );
	    else
		Debug.warning( "Image to display is null" );

	} else if( ie.getStatus() == ie.IGNORED ) {
	    // Safely ignoring.

	} else 
	    Debug.warning( "ImageDisplayAgent received unrecognized ImageEvent" );

    }
}
