// $Id: ImageSourceAgent.java,v 1.1 2000/05/18 22:04:58 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.image;
import net.hivecell.hive.support.SerializableImage;
import net.hivecell.hive.agent.DisplayExportingAgent;
import java.rmi.RemoteException;
import net.hivecell.hive.agent.Agent;


/** ImageSourceAgent is an interface which describes agents that can provide
 * images.  Generally such an agent holds a buffer of such images, but
 * there is no reason that buffer cannot be of size 1.
 * 
 */
public interface ImageSourceAgent 
extends Agent, DisplayExportingAgent {
    /** Get the image of index i;
     * 
     * @exception RemoteException 
     * @param i The index of the image to retrieve.
     * @param sender The sender of the request.
     * @return The retrieved image
     */
    public SerializableImage getImage(Agent sender, int i) throws RemoteException;

    /** Get the most recent image
     * 
     * @exception RemoteException 
     * @param sender The sender of the request.
     * @return The retrieved image
     */
    public SerializableImage getImage(Agent sender) throws RemoteException;

    /** Get the index of the most recent image.
     * 
     * @exception RemoteException 
     * @param sender 
     */
    public int getLatestIndex(Agent sender) throws RemoteException;

    /** Get the index of the earliest image still in the buffer.
     * 
     * @exception RemoteException 
     * @param sender 
     */
    public int getEarliestIndex(Agent sender) throws RemoteException;

    /** Get the size of the buffer that the Agent holds.
     * 
     * @exception RemoteException 
     * @param sender 
     */
    public int getBufSize(Agent sender) throws RemoteException;

    /** Returns true if the source is busy.
     * 
     * @exception RemoteException 
     * @param sender 
     */
    public boolean isBusy(Agent sender) throws RemoteException;
}
