// $Id: MotionColorFilterAgentImpl.java,v 1.2 2000/05/19 19:55:31 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.image;
import net.hivecell.hive.support.*;
import net.hivecell.hive.agent.AgentInitializationException;
import java.awt.*;

public class MotionColorFilterAgentImpl 
extends GenericFilterAgentImpl {

    protected SerializableImage last = null;
    protected double threshold = 0.15;

    public MotionColorFilterAgentImpl() 
    throws java.rmi.RemoteException {
	super();
    }

    public void doLocalSetup() throws AgentInitializationException  {
        super.doLocalSetup();
    }
    
    public SerializableImage filter(SerializableImage si) {
	SerializableImage newSI = (SerializableImage) si.clone();
	if ((last == null) ||
	    (last.getRep().length != newSI.getRep().length)) {
	    last = newSI;
	}
	else {          
	    int[] newRep = newSI.getRep();
	    int[] lastRep = last.getRep();
	    last = (SerializableImage) si.clone();
	    for (int i = 0; i < newRep.length; i++) {
		newRep[i] = maskPixel(newRep[i],
				      grayPixel(absDiffPixel(newRep[i],
							     lastRep[i])));
	    }
	}
	
	return newSI;
    }

    protected double getThreshold() {
	return threshold;
    }

    protected void setThreshold(double threshold) {
	this.threshold = threshold;
    }
    
    protected static int grayPixel(int pixel) {
	float r = (float) getRed(pixel);
	float g = (float) getGreen(pixel);
	float b = (float) getBlue(pixel);

	int gray = Math.round((0.299f * r) + (0.587f * g) + (0.114f * b));
	return setAlpha(255) + setRed(gray) + setGreen(gray) + setBlue(gray);
    }

    protected static int absDiffPixel(int pixel1, int pixel2) {
	int a = Math.abs(getAlpha(pixel1) - getAlpha(pixel2));
	int r = Math.abs(getRed(pixel1) - getRed(pixel2));
	int g = Math.abs(getGreen(pixel1) - getGreen(pixel2));
	int b = Math.abs(getBlue(pixel1) - getBlue(pixel2));
	return setAlpha(a) + setRed(r) + setGreen(g) + setBlue(b);
    }
    
    protected int maskPixel(int pixel1, int pixel2) {
	float sa = ((float) getAlpha(pixel2)) / 255.0f; 
	float sr = ((float) getRed(pixel2)) / 255.0f; 
	float sg = ((float) getGreen(pixel2)) / 255.0f; 
	float sb = ((float) getBlue(pixel2)) / 255.0f; 
	int gray = (int) Math.round(grayPixel(pixel1) & 255);
	
	int a = (sa > threshold) ? getAlpha(pixel1) : 255;
	int r = (sr > threshold) ? getRed(pixel1) : gray;
	int g = (sg > threshold) ? getGreen(pixel1) : gray;
	int b = (sb > threshold) ? getBlue(pixel1) : gray;
	return setAlpha(a) + setRed(r) + setGreen(g) + setBlue(b);
    }

}
