// $Id: MotionDetectorAgentImpl.java,v 1.1 2000/05/18 22:04:58 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/** This agent should be connected to an ImageCreationAgent. It will
 * periodically poll for a new image, and when the image differs
 * Significantly from the previous one, it will announce the availability
 * of a new and different image.
 */

package net.hivecell.hive.agent.image;
import java.rmi.RemoteException;
import net.hivecell.hive.support.*;
import net.hivecell.hive.agent.*;
import net.hivecell.hive.event.*;

public class MotionDetectorAgentImpl 
extends MotionColorFilterAgentImpl {

    protected SerializableImage last = null;

    public MotionDetectorAgentImpl() 
    throws java.rmi.RemoteException {
	super();
    }

    /** We override this method so that the agent only doesn't rebroadcast
     * the pending and ignored messages it will get. Also, it needs to request
     * another image when the previous image is available.
     */

    public void translate(HiveEvent event) {
	ImageSourceAgent source = null;
	ImageEvent ce = null;
	try {
	    source = (ImageSourceAgent) event.getSource();
	    ce = (ImageEvent) event.getEventData();
	}
	catch (ClassCastException e) {
	    Debug.println(Debug.WARNING,
			  "Recieved an Event from a non ImageCreationAgent.", e);
	    return;
	}
	if (ce.getStatus() == ce.READY) {
	    if (!isBusy()) {
		EventReceivingAgent[] destinationAgent = { (EventReceivingAgent)source };
		eventSendingHelper.broadcastEventTo("Get new image", destinationAgent, null );
		fetchQ.enQueue(new QueueEntry(source, ce));
		if (fetchQ.getSize() >= BUFSIZE) {
		    setBusy(true);
		}
	    }
	}
    }	

    public SerializableImage filter(SerializableImage si) {
	SerializableImage newSI = (SerializableImage) si.clone();
	// The first image or the first of this size..
	if ((last == null) ||
	    (last.getRep().length != newSI.getRep().length)) {
	    last = newSI;
	    return null;
	}
	else {          
	    int[] newRep = newSI.getRep();
	    int[] lastRep = last.getRep();
	    last = (SerializableImage) si.clone();
	    double lastAvg = 0.0;
	    double curAvg = 0.0;
	    double repLen = (double) newRep.length;
	    for (int i = 0; i < newRep.length; i++) {
		lastAvg += (double) getBlue(grayPixel(lastRep[i])) / 
		    (double) repLen;
		curAvg += (double) getBlue(grayPixel(newRep[i])) /
		    (double) repLen;
	    }
	    double dif = Math.abs(curAvg - lastAvg) / lastAvg;
	    Debug.notice("MotionDetector Threshold: " + dif);
	    if (dif >= threshold) {
		return newSI;
	    }
	    else {
		return null;
	    }
	}
    }
}

