// $Id: CricketAgentImpl.java,v 1.2 2000/05/19 19:55:33 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.agent.thing;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.desktop.ClickAgentImpl;
import net.hivecell.hive.agent.GenericSignalPassthroughAgentImpl;

import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.thing.CricketShadow;
import net.hivecell.hive.shadow.thing.CricketShadowImpl;
import net.hivecell.hive.shadow.EventPublisherShadow;

import net.hivecell.hive.shadow.Signalable;

import net.hivecell.hive.support.Debug;
import net.hivecell.hive.description.HiveXML;
import org.w3c.dom.Document;

import java.util.*;
import java.rmi.RemoteException;

public class CricketAgentImpl 
extends EventTransceivingAgentImpl 
implements Signalable, CricketAgent {

    /** 
     * A handle to our shadow. 
     */
    protected transient CricketShadow cricketShadow;

    /** 
     * The name of the shadow for crickets. 
     */
    private final String shadowName = (CricketShadowImpl.class).getName();
    protected byte defaultByte = 0x00;
        
    /** 
     * When a signal is received, broadcast the event.
     *
     * @param source the source of this signal
     * @param information what was sent along with this signal
     */
    public void signal( EventPublisherShadow source, Object information ) {
        eventSendingHelper.broadcastEvent( information );

    }
    
    /** 
     * Convenience method, if the signal source doesn't send an object
     * then just send null along.
     *
     * @param source the event pubisher that sent us the information
     */
    public void signal( EventPublisherShadow source ) {
        signal( source, null );

    }
    

    /**
     * construct a new cricket agent
     *
     * @exception RemoteException if something went wrong
     */
    public CricketAgentImpl() 
    throws RemoteException { 
	super(); 

    }

    /**
     * configure this agent
     *
     * @param doc the document that we are configuring by
     */
    public void configure( Document doc ) {
        String v = HiveXML.getConfigValue( doc, "defaultByte" );
        try {
            defaultByte = Byte.parseByte( v );

        } catch( Exception ex ) {
            Debug.warning( "defaultByte value " + v + " not valid." );

        }

    }
    
        
    /** 
     * When the agent starts, it just finds the cricket shadow from the
     * database.
     *
     * @exception AgentInitializationException if something goes wrong getting the shadow
     */
    public void doLocalSetup() 
    throws AgentInitializationException {
        super.doLocalSetup();
        
	try {
            cricketShadow = (CricketShadow)myCell.getShadow( shadowName );

        } catch( ShadowNotFoundException error ) {
            Debug.failure( "Couldn't find the CricketShadow, cricket agent will not work." );
	    throw new AgentInitializationException();

        }
        
        cricketShadow.addListener( this );

    }
    
    /**
     * we don't really do anything except sit here
     */
    public void doBehavior() {
	waitUntilDeath();

    }

    /** 
     * When an event is received, send data out the IR. If there's a
     * number in the event data, send it. Otherwise, send default.
     *
     * @param event the event that somebody else sent us
     */
    public synchronized void notify( HiveEvent event ) {
        super.notify( event );
        
        byte data = defaultByte;                 // default, send a 0.

        if( event.getEventData() instanceof Number )
            data = ((Number)event.getEventData()).byteValue();
        cricketShadow.sendByteOnIR( data );

    }

    /** 
     * Set the default value that we use to send to the cricket. 
     *
     * @param b the byte that is our default
     */
    public void setDefaultByte( byte b ) {
        defaultByte = b;

    }

    /**
     * Get the default value sent to the cricket on event. 
     *
     * @return the byte
     */
    public byte getDefaultByte() {
        return defaultByte;

    }

}
