// $Id: GenericResourceFinderAgentImpl.java,v 1.2 2000/05/19 19:55:33 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.wearable;

import net.hivecell.hive.*;
import net.hivecell.hive.shadow.*;
import net.hivecell.hive.agent.*;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.agent.EventTranslatingAgentImpl;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import java.rmi.RemoteException;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.agent.ConnectAgentHelper;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.cell.*;
import java.util.Vector;

public class GenericResourceFinderAgentImpl extends EventReceivingAgentImpl {
    String resourceType = null;
    ConnectAgentHelper connectAgentHelper = null;
    HiveEvent he;
    
    public GenericResourceFinderAgentImpl() throws RemoteException { super(); }
  
    /**
       * Get the value of resourceType
       * @return Value of resourceType.
       */
    public String getResourceType() {return resourceType;}

    /**
       * Set the value of resourceType
       * @param r  Value to assign to resourceType.
       */
    public void setResourceType(String r) {
        this.resourceType = r;}

    public void doLocalSetup() throws AgentInitializationException {
	super.doLocalSetup();
        connectAgentHelper = new ConnectAgentHelper(this, myCell);
    }

    /**
      * Main thread of execution for the agent
      */
    public void doBehavior() {
        // Wire it up to someone who will give us services in this area
        // This is for startup -- Give up after a while
        for (int i=1; 
             (i < 5) && (!connectAgentHelper.connectMeTo("net.hivecell.hive.agent.wearable.LocationServiceRequestAgent",null));
             i++) {
            Debug.notice("Still waiting for a LocationServiceRequestAgent to hook up to...");
            try { Thread.sleep(1000 * i); } catch (InterruptedException e) {}
        }
    }

    /**
       * Action to take on resources in this location that match the kind requested.
       * This is to be overridden by subclasses.
       */
    public void resourceAction(AgentSet resources) {
    }

    // We should be receiving a DescSet of services for this room.
    public void notify(HiveEvent event){
        super.notify(event);
        he = event;
	Object ed = event.getEventData();
	Debug.notice("Notified of "+ed);
	if (ed instanceof AgentSet) {
            AgentSet resources = (AgentSet) ed;
            //resources.printTree();
            //this looks through the rdf files for the 'nickname' tag and then selects for the resourceType
	    //            resources = Lookup.require(resources, Description.HIVE+"nickname", resourceType);

	    // NOTE: 2/27, tucker: This does the same thing as the comment specifies, but I don't know if it will work the 
	    // same as the RDF does, although it should (requires only those AgentSets which have nickname AND resourceType)
	    // That is what the RDF specifies in the description, but it MAY actually require that nickname == resourceType, 
	    // which this way doesn't.  Testing will show which it is.
	    //
	    // UPDATE: queryValues should do this right.
	    resources = resources.queryValues(null, "nickname", resourceType);
            
            Debug.notice("[RESOURCES DESCSET (USING NICKNAME) OF RESOURCETYPE <" + resourceType + ">]:         " + resources.getNames());
            //resources = Lookup.requireRole(resources, Description.HIVE+resourceType);
            if (resources.size() > 0) Debug.notice("Agents of type "+resourceType+" at this location: "+resources.size());
            else Debug.notice("No agents of type "+resourceType+" found at location.");
            resourceAction(resources);
	}
    }
}

