// $Id: LocationServiceRequestAgent.java,v 1.2 2000/05/19 19:55:34 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.wearable;

import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.event.cell.*;
import java.rmi.RemoteException;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.cell.*;
import net.hivecell.hive.agent.cell.*;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent;
import net.hivecell.hive.agent.cell.CellListInfo;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.cell.CellListAgentImpl;
import net.hivecell.hive.agent.ConnectAgentHelper;
import net.hivecell.hive.agent.wearable.Location;
import java.util.Vector;

/** This Agent takes in locations, represented as strings containing the MIT room number
   (e.g. "e15-305").  It then produces an AgentSet of services associated with that location.

   The right thing to do is allow a "Location" class to be sent as an event, which handles
   translation to and from bldg-room, GPS, address, zip-code, etc.  In the meantime, we'll
   just use room numbers and strings (this should always be a fall-back anyway).
**/

public class LocationServiceRequestAgent extends EventTransceivingAgentImpl {
    Vector allCells = null;      // Vector of type CellListInfo
    CellAddress monitoredCells = null;
    Vector alreadyConnected = new Vector();
    ConnectAgentHelper connectAgentHelper = null;
    AgentMonitoringAgent ama;
    CellListAgent sla;
    
    public LocationServiceRequestAgent() throws java.rmi.RemoteException { super(); }

               
    private void updateAllCells() {
        // First, find the CellListAgent for this cell
	AgentSet slaSet = new AgentSet();
	String[] className = { "net.hivecell.hive.agent.cell.CellListAgentImpl" };
	try {
	    slaSet = myCell.queryAgents(this, className, null);
	} catch (ClassNotFoundException e) {
	    // NOTE: Queryagents shouldn't throw this.  Maybe a RemoteExeption, but not this.
	    Debug.warning("ClassNotFoundException thrown:", e);
	}
	//        DescSet slaSet = Lookup.getAgentSet(myCell, this,
	//                                            "net.hivecell.hive.agent.cell.CellListAgent", null);
        // put all the servers online into allCells        
        if (slaSet.size() > 0) {
            try {
                sla = (CellListAgent)slaSet.pickOne();
                sla.addListener(this);
                allCells = sla.listCells(this);
                Debug.notice("Got "+allCells.size()+" other servers: "+allCells);
            } catch (Exception e) {
                Debug.warning("Failed to get list of other servers", e);
            }
        }
        else {
            Debug.notice("Couldn't find the CellListAgent");
        }        
        //        allCells.addElement(mySLI);  // My own server counts too...
        Debug.notice("Found a total of "+allCells.size()+" servers");
    }

    
    private AgentSet localizedAgents() {        
        AgentSet agentsAtCell = null;
        CellAddress[] cas;
        
        try {
            cas = ama.getMonitoredCells(this);
            agentsAtCell = ama.queryAgents( this, cas, null, null );
        } catch (RemoteException e) {
            Debug.error("cannot monitor other cells! doh!");
        } catch( ClassNotFoundException error ) {

	}

        return agentsAtCell;
    }

    
    public void doLocalSetup() throws AgentInitializationException {
        super.doLocalSetup();
        connectAgentHelper = new ConnectAgentHelper(this, myCell);
        
        String[] agentMonitoringAgentString = {"net.hivecell.hive.agent.cell.AgentMonitoringAgent"};

        updateAllCells();
        
        try {
            AgentSet ds = myCell.queryAgents( this, agentMonitoringAgentString, null);
            ama = (AgentMonitoringAgent)ds.pickOne();
        } catch (ClassNotFoundException cnfe) {
            Debug.error("doh! no AgentMonitoringAgent on this server!");
        }
        
        //monitor all cells listed on the CellListInfo Agent
        
        try {
            for (int i = 0; i < allCells.size(); i++) {
                CellListInfo cli = (CellListInfo)(allCells.elementAt(i));
                if ((cli != null) && (cli.getCell() != null)) {
                    try {
                        ama.monitorCell(this, cli.getCell().getAddress());
                        Debug.notice("----------->[MONITORING CELL]:        " + cli.getCell().getAddress().toString());
                    } catch (UnableToMonitorCellException utmce) {
                        Debug.error("cannot monitor Hive cell!");
                    }   
                }
            }
        } catch (RemoteException error) {
            Debug.error("cannot get address of a server!");
        } 
    }
    

   /**
     * Main thread of execution for the agent
     */
    public void doBehavior() {
        
        updateAllCells();  // First, update what servers are around right now
        
    }
    
    /** When an event is received, find the agents associated with that 
        location and broadcast them. 
    **/
    public void notify(HiveEvent event) {
        super.notify(event);
        
        Object ed = event.getEventData();
        Debug.notice("[Notified of]:......... "+ed);

        Object source = (Object)event.getSource();

        if (event instanceof CellEvent) {
            CellEvent se = (CellEvent)event;
            switch((int)se.eventCode()) {
            case (int)CellEvent.SERVERBORN:
                {
                    String serverBornAddress = ((CellBornCellEvent)se).getAddress().toString();
                    Debug.notice("---------->[FROM <" + source + "> GOT A SERVER BORN!] @ " + serverBornAddress);
                    try {
                        ama.monitorCell(this, ((CellBornCellEvent)se).getAddress());
                        Debug.notice("---------->[ALL OF THE COMBINED AGENTS ARE NOW]:    " + (localizedAgents()).getNames() );
                    } catch (RemoteException error) {
                        Debug.error("Remote Exception while getting address of new server!");
                    } catch (UnableToMonitorCellException error) {
                        Debug.error("Unable to Monitor new cell!");
                    }
                }
            }
        }            
        
        if(ed instanceof Location){
            Location location = (Location) ed;
            AgentSet sources = localizedAgents();
            String bldg = null;
            String room = null;
            
            //sources.printTree();
            Debug.notice("--------------->[BUILDING]:        " + location.getBldg());
            Debug.notice("--------------->[ROOM]:            " + location.getRoom());

            if (location.getBldg() != null) {
		sources = sources.querySet(null, "location");
		sources = sources.queryValues(null, "building", location.getBldg());
	    }

	    //                sources = Lookup.require (sources, Description.HIVE+"location", 
	    //                                          Description.HIVE+"building", location.getBldg());    
            if (location.getRoom() != null) {
		sources = sources.querySet(null, "location");
		sources = sources.queryValues(null, "room", location.getRoom());
		//                sources = Lookup.require (sources, Description.HIVE+"location", 
		//                                          Description.HIVE+"room", location.getRoom());
	    }
            
            //Debug.notice("------------------->[NUMBER OF AGENTS FOUND IN BUILDING/ROOM]: "+location.getBldg()+", room "+location.getRoom()+": "+sources.matches());

            Debug.notice("------------------->[AGENTS FOUND IN ROOM]: "+sources.getNames());
            eventSendingHelper.broadcastEvent(sources);
        }   
    }
}


