package net.hivecell.hive.event;

import org.w3c.dom.*;

/**
 * this is asubclass of the hive event which is specially made just to
 * transmit around the XML DOM stuff so we can do the external events
 * from outside sources.  
 */
public abstract class ExternalEvent
extends HiveEvent {
    
    /**
     * the value of the data tag for the XML passed to us
     */
    protected String dataValue;

    /**
     * create a new ExternalEvent that takes the source of the event,
     * the event number, and as the event data is the Document that is
     * the top level of the event data
     *
     * @param UIDOrigin the UID which is the origin of the 
     * @param source the source of the event
     * @param eventNumber the evnet number
     * @param eventData the document node that we want to stuff
     */
    public ExternalEvent( Object source, long eventNumber, Document eventData ) {
	super( source, eventNumber, (Object)eventData );
	
	// search and extract the data tag from the XML document
	Node outgoingEventNode = eventData.getFirstChild();
	if( outgoingEventNode != null ) {
	    NodeList nodeList = outgoingEventNode.getChildNodes();
	    for( int count=0;count<nodeList.getLength();count++ ) {
		Node searchingNode = nodeList.item( count );
		if( searchingNode.getNodeName().equals( "data" ) ) {
		    if( searchingNode.getFirstChild() != null )
			dataValue = searchingNode.getFirstChild().getNodeValue();
		    break;
		    
		}
		
	    }

	}

    }

    /** 
     * Returns any event-specific data.  In this caes, it returns the
     * info tag of the eventData
     *
     * @return the info tag (which will be a string) 
     */
    public Object getEventData() {
	return dataValue;

    }

    /**
     * Get the who XML document that we are holding
     *
     * @return the XML document
     */
    public Document getXMLDocument() {
	return (Document)super.getEventData();

    }

}
