package net.hivecell.hive.event;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.support.CellAddress;

import com.docuverse.dom.BasicDocument;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * this is asubclass of the hive event which is specially made just to
 * transmit around the XML DOM stuff so we can do the external events
 * from outside sources.  
 */
public class IncomingExternalEvent
extends ExternalEvent {
    
    /**
     * the value of the data tag for the XML passed to us
     */
    protected String dataValue;

    /**
     * the UID which originated this event
     */
    protected String UID;


    /**
     * create a new ExternalEvent that takes the source of the event,
     * the event number, and as the event data is the Document that is
     * the top level of the event data
     *
     * @param source the origin of the event -- only agents can send events right now (not sure if this is the right solution)
     * @param address teh address of teh agent
     * @param UIDDestination the UID which is the origin of the 
     * @param count the evnet number
     * @param eventData the document node that we want to stuff
     */
    public IncomingExternalEvent( Agent source, CellAddress address, String UIDDestination, long count, Object eventData ) {
	super( source, count, new BasicDocument() );

	this.dataValue = eventData.toString();
	this.UID = UIDDestination;

	Document top = (Document)getXMLDocument();
	Element enclosingNode = top.createElement( "incomingevent" );
	enclosingNode.setAttribute( "syntactic", source.getClass().getName() );
	enclosingNode.setAttribute( "hostname", address.toString() );
	Element dataNode = top.createElement( "data" );
	Node dataValueNode = top.createTextNode( eventData.toString() );
	dataNode.appendChild( dataValueNode );
	enclosingNode.appendChild( dataNode );
	top.appendChild( enclosingNode );

    }

    /**
     * return the UID that this event is meant for
     *
     * @return the UID for the sender 
     */
    public String getDestinationUID() {
	return this.UID;

    }


}
