// $Id: DJShadowImpl.java,v 1.1 2000/05/18 22:05:03 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow;

import net.hivecell.hive.Global;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.support.Debug;

import java.io.*;
import java.util.Vector;

// Most of this is based on the JukeBoxShadow, but with some enhancements
public class DJShadowImpl
extends EventPublisherShadowImpl
implements DJShadow {

    /**
     * the process which runs the player
     */
    private Process player = null;

    /**
     * a flag which will notify us to tell us whether the player has
     * been killed 
     */
    private boolean killed = false;

    /**
     * a flag stating whether we are playing a song
     */
    private boolean playingSong = false;


    /**
     * tell us whether or not we are playing a song
     *
     * @return true if we are playing a song, false otherwise
     */
    public synchronized boolean getPlayingSong() {
        return playingSong ;

    }

    /**
     * set whether we are playing a song or not
     * 
     * @param newval true if we are supposidely playing a song
     */
    private synchronized void setPlayingSong( boolean newval ) {
        Debug.notice( "Setting playingSong to " + newval );
        playingSong = newval;

    }

    /**
     * the "is device there" method -- since we are using a software
     * player, it is always "there".  we shoudl modify this to check
     * to make sure we have the binary though
     *
     * @return true always 
     */
    public boolean isDeviceThere(){
	return true;

    }

    /**
     * initialize the shadow.  this doesn't really have to do
     * anything.  perhaps this should check to see if the binary we
     * need is available
     *
     * @exception ShadowInitializationException but it doesn't really 
     */
    public void init()
    throws ShadowInitializationException {
	setReady( true );
	
    }

    /**
     * pause the player
     */
    public void pause() {
	stop();

    }

    /**
     * ask the player to start playing again -- currently unsupported 
     */
    public void play() {
	Debug.notice( "Play unsupported, currently" );

    }

    /**
     * stop playing the mp3 -- this will also notify all our
     * subscribers that we have stopped playing 
     */
    public void stop() {
	Debug.notice( "asked to stop playing the mp3" );
	notifySubscribers( "REPORT:stop" );
        setPlayingSong( false );
	if( player != null ) {
	    killed = true;
	    player.destroy();
	    try {
		player.waitFor();

	    } catch( Exception e ) {
                Debug.warning( e );

            }

	    player = null;

	}

    }

    /**
     * set the volume on the mp3 -- we make a call out to the oss
     * mixer with the volume number
     *
     * @param v the volume we want to set it to 
     */
    public void volume( int v ) {
	if( ( v >= 0 ) && ( v <= 100 ) ) {
	    String[] vexec = { "/usr/lib/oss/mixer", "pcm", v + ":" + v };
	    try {
		Runtime.getRuntime().exec( vexec );

	    } catch( Exception e ) { 
	    
	    }

	}

    }

    /**
     * play a particular song.  if the song is not one of our choices,
     * then the mp3 player will just exit and nothing will happen --
     * not necessarilyt the best thing.  something real should happen
     * -- like an exception or something
     *
     * @param pick the song that we want to play 
     */
    public void playSong( String pick ) {
        if( getPlayingSong() )
            stop();
        setPlayingSong( true );
        Debug.notice( "playing " + pick );
        try {
            String[] playsong = { "mpg123",  pick };
            notifySubscribers( "REPORT:" + pick );
            player = Runtime.getRuntime().exec( playsong );

            (new Thread( new Runnable() {
                public void run() {
                    int me = Global.random.nextInt();
                    try {
                        Debug.notice( "waiting for player to exit " + me );
                        player.waitFor();

                    } catch( Exception e ) {
			// something else should happen here!  this
			// exception doesn't seem like its in the
			// right place!
                        Debug.notice( "hmmmmmm... something isn't good here." );

                    }

                    // Spit out an end of song event, maybe
                    setPlayingSong( false );
                    player = null;
                    Debug.notice( "The player exited (killed? " + killed + ") id: " + me );
                    if( killed ) {
                        killed = false;
                        Debug.notice( "someone killed it (now " + killed + ")" );
                        notifySubscribers( "REPORT:song end" );

                    } else {
                        notifySubscribers( "REPORT:song end" );
                        Debug.notice( "Song ended normally (id " + me + ")" );

                    }

                }

            })).start();
            
        } catch( Exception e ) {
            Debug.notice( "Error playing song", e );
            notifySubscribers( "REPORT:error playing song" );
            stop();

        }

    }

}
