// $Id: ExternalInterfaceShadow.java,v 1.1 2000/05/18 22:05:03 tucker Exp $
// Hive. Copyright (c) 1998-2000, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow;

import net.hivecell.hive.event.IncomingExternalEvent;

/**
 * the external interface shadow listens for connections coming from
 * an external source.  the other source can connect to us and send
 * events to us, and we can spit events out through the socket.
 * because event sending is unreliable, these events may or may not
 * get to the remote agent.  eventually we will need to actually send
 * an exception down the stream, but right now i don't want to deal
 * with that
 *
 * to deal with events the device want to send out, we need XML which
 * looks like this:
 * <outgoingevent>
 *  <destination>
 *   <hostname>...</hostname>
 *   <syntatic>...</syntatic>
 *   <semantic>
 *     .
 *     tags
 *     .
 *   </semantic>
 *  </destinaton>
 *  <data>
 *   .
 *   .
 *  </data>
 * </outgoingevent>
 *
 * the destination tag needs to be there, but only one of the syntatic
 * or the semantic is necessary.  the hostname is optional, if left
 * out we will query the local cell for the appropriate agent.
 *
 * now the return, events can be sent to these devices and the stream
 * coming off this shadow will look like:
 * <incomingevent syntatic="the class coming from" hostname="the host coming from">
 *  <data>
 *   .
 *   .
 *  </data>
 * </incomingevent>
 *
 * the device has to keep the stream open if it ever wants the ability
 * to receive events.  if a device connects, disconnects, and
 * reconnects then it will not be "teh same" device here, and it will
 * not receive events meant for the previous connection of it (or
 * probably not) 
 */
public interface ExternalInterfaceShadow
extends EventPublisherShadow {

    /**
     * spit an external event off to the device specified by the UID
     *
     * @param UID the idnetiifer that we want to spit the event to
     * @param event the ExternalEvent that we want to spit down
     */
    public void sendEvent( IncomingExternalEvent event );

}
