// $Id: LoadAverageShadowImpl.java,v 1.1 2000/05/18 22:05:03 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

// Get the load average and report it.

package net.hivecell.hive.shadow;

import net.hivecell.hive.Global;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.support.*;

import java.io.*;

public class LoadAverageShadowImpl
extends ShadowImpl 
implements LoadAverageShadow {

    /**
     * just make sure we are not attempting to do this in windows --
     * if we are then we just have to just throw an exception and punt
     *
     * @exception ShadowInitializationException if this is a ewindows machien 
     */
    public void init() 
    throws ShadowInitializationException {
	if( Global.windowsOS ) {
	    Debug.warning( "No LoadAverageShadow for windows currently." );
	    throw new ShadowInitializationException( "no load average shadow for windows currently" );
	}

	setReady( true );

    }


    /**
     * this actually calcualtes the load by parsing teh proc system
     * 
     * @return the load of the machien as a float if we are a non windows machine
     */
    public float getLoad() {
	if( Global.windowsOS )
	    return 1.0f;
	
	// Simpler code to do /proc/loadavg on Linux boxes
	FileReader data = null;
	try {
	    data = new FileReader( "/proc/loadavg" );
	    StreamTokenizer st = new StreamTokenizer( data );
	    st.nextToken();
	    return (float)st.nval;

	} catch( FileNotFoundException ex ) { 
	    Debug.critical( ex ); 

	} catch( IOException ex ) { 
	    Debug.critical( ex ); 
	
	} finally {
	    return 0;

	}
	
    }
    
}
