// $Id: LocustShadowImpl.java,v 1.1 2000/05/18 22:05:03 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow;

import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.Signalable;
import net.hivecell.hive.support.Debug;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

public class LocustShadowImpl
extends SerialStreamShadowImpl
implements EventPublisherShadow, LocustShadow {

    /**
     * the event publisher that we are helping
     */
    protected EventPublisherHelper eph = new EventPublisherHelper( this );

    /**
     * create a new locust shadow -- this will set the serial port and
     * the baud rate -- perhaps this should be done in the init, and
     * perhaps this should be read from our description
     */
    public LocustShadowImpl() {
	super();
	setSerialPort( 1 );
	setBaudRate( 9600 );

    }
  
    /**
     * Clients can call this to add themselves as a listener to this
     * type of event
     *
     * @param o the client 
     */
    public void addListener( Signalable o ) { 
	eph.addListener( o ); 

    }

    /**
     * Clients can call this to remove themselves as listeners of this
     * event
     *
     * @param o the client
     * @return whether or not the object was actually removed 
     */
    public boolean removeListener( Signalable o ) { 
	return eph.removeListener( o ); 

    }

    /**
     * Get a count of all the events sent out
     *
     * @return the count
     */
    public int getCount() { 
	return eph.getCount(); 

    }

    /**
     * Send a null event out to all subscribers
     */
    public void notifySubscribers() { 
	eph.notifySubscribers(); 

    }

    /**
     * Send a specific object out to the subscribers
     *
     * @param information the object to send
     */
    public void notifySubscribers( Object information ) { 
	eph.notifySubscribers( information ); 

    }

    /**
     * test to see if the device is there
     *
     * @return this jsut returns true to fake it out
     */
    public boolean isDeviceThere() { 
	return true; 

    }

    /**
     * do the initialization for the shadow
     *
     * @exception ShadowInitializationException if somethign goes wrong in the start up
     */
    public void init() 
    throws ShadowInitializationException {
	super.init(); // get all the serial stuff coming in

	(new Thread( new Runnable() {
	    public void run() {
		LocustListener ll;
		Thread llThread = new Thread( ( ll = new LocustListener( is ) ) );
		llThread.start();
		String lastLocSent = null;
		while( true ) {
		    try { 
			Thread.sleep( 100 ); 
		    
		    } catch( Exception e ) { 

		    }
		    String n = ll.bestGuessLoc;
            
		    if( !llThread.isAlive() ) {
			Debug.failure( "LocustListener died." );
			return;

		    } else if( ( n != null ) && !n.equals( lastLocSent ) ) {
			Debug.notice( "BestGuessLoc: " + n );
			notifySubscribers( n );
			lastLocSent = n;

		    } else if( ( ll.lastSeen < ( System.currentTimeMillis() - 120000 ) ) && !lastLocSent.equals( "NOLOC" ) ) {
			// It's been ten seconds since a reading, let's assume 
			// we're out of range of a locust
			Debug.notice( "120-second timeout" );
			ll.bestGuessLoc = "NOLOC";
			lastLocSent = "NOLOC";
			notifySubscribers( "NOLOC" );

		    }

		}

	    }
	}, "LocustShadow" )).start();

	setReady( true );

    }

}

/**
 * the listener for the locust
 */
class LocustListener 
implements Runnable {

    /**
     * the reader that we are going to listen over
     */
    BufferedReader listen;

    /**
     * the last location beacon we have seen
     */
    public String lastLoc = "NOLOC";

    /**
     * where are we, pappa smurf?
     */
    public String bestGuessLoc = "NOLOC";

    /**
     * when did we last see a beacon?
     */
    public long lastSeen = 0;
    
    /**
     * stop if we start reading nulls, it means our reader is screwed
     */
    private boolean threadShouldStop = false;

    /**
     * how many of lastloc have we seen in a row?
     */
    int numberLocsInRowSeen = 0;

    
    /**
     * set this locust listener to listen to this input stream
     *
     * @param s the input stream that we weant to listen to
     */
    public LocustListener( InputStream s ) { 
	listen = new BufferedReader( new InputStreamReader( s ) ); 

    }

    // Our best guess is the last beacon we've seen at least 3 times in a row
    // Yes, this can be wrong if we go from one beacon to two right next to 
    // eachother, but it's not a bad first approximation.
    public void run() {
	while( !threadShouldStop ) {
	    try {
		String locid = listen.readLine();

                if( locid != null ) {
                    lastSeen = System.currentTimeMillis();
                    if( !locid.equals( lastLoc ) ) {
			Debug.notice( "Newloc: " + locid );
                        lastLoc = locid;
                        numberLocsInRowSeen = 1;

                    } else {
                        Debug.notice( "reploc: " + locid );
                        numberLocsInRowSeen++;
                        if( numberLocsInRowSeen > 3 ) 
                            bestGuessLoc = lastLoc;

                    }

                } else {
		    Debug.notice( "Doh! locid = null!" );
                    threadShouldStop = true;
                    // Thread.sleep(1000); // Doh!, start throttling

		}

	    } catch( Exception e ) { 
		System.out.println( "Io error: " + e ); 

	    }

	}

    }

}



