// $Id: FileGrowthShadowImpl.java,v 1.1 2000/05/18 22:05:04 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive.shadow.sysinfo;

import net.hivecell.hive.agent.*;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.EventPublisherShadowImpl;
import net.hivecell.hive.shadow.ShadowInitializationException;
import net.hivecell.hive.support.Debug;

import java.io.*;

/** 
 * Monitor that fires events when a file gets longer.  This is like a
 * really dumb "tail -f" 
 */
public class FileGrowthShadowImpl
extends EventPublisherShadowImpl
implements Runnable, FileGrowthShadow {

    /**
     * the file that we want to monitor
     */
    protected File file;

    /**
     * the thread that we are monitoring the file with
     */
    protected Thread testThread;

    /**
     * the interval that we are sleepgina dn teting at in milliseconds
     */
    protected final int sleepInterval = 1000;       // milliseconds

    /**
     * the last length of the file when we checked it
     */
    private long lastLength;

    /**
     * a flag stating that the thread should reinitialize
     */
    private boolean threadShouldReinit = false;

    /**
     * a flag stating that the thread shouls stop
     */
    private boolean threadShouldStop = false;
    
    /**
     * by default, the file that we should be monitoring
     */
    protected String fileToMonitor = "/mnt/web-logs/httpd/logs/latest/access_log";
    

    /**
     * Get the value of fileToMonitor.
     *
     * @return Value of fileToMonitor.
     */
    public String getFileToMonitor() { 
	return fileToMonitor; 

    }
    
    /**
     * Set the value of fileToMonitor.
     *
     * @param v Value to assign to fileToMonitor.
     */
    public void setFileToMonitor( String v ) {
	if( isReady() ) 
	    return;
	this.fileToMonitor = v;
	if( testThread == null ) {
	    testThread = new Thread( this, "File Growth Shadow" );
	    testThread.start();

	} else
	    threadShouldReinit = true;

    }
    
    public void init() 
    throws ShadowInitializationException {
	if( fileToMonitor != null )
	    file = new File( fileToMonitor );
	else
	    return;

	Debug.assert( testThread == null );
	if( file.exists() == false)
	    Debug.error( "File " + file.getName() + " does not exist." );

	else {
	    testThread = new Thread( this, "File Growth Shadow" );
	    testThread.start();

	}

	setReady( true );

    }

    // main loop - test the file's length to see if it's changed, then sleep.
    // This seems awfully ugly and expensive, but it's the way GNU tail -f works.
    public void run() {
	lastLength = file.length();

	while( !threadShouldStop ) {
	    while( !threadShouldReinit ) {
		try {
		    Thread.sleep( sleepInterval );

		} catch( InterruptedException e ) { 
		    Debug.critical( e ); }
	    
		long thisLength = file.length();
		if( thisLength != lastLength ) {             // aha, a change!
		    if( thisLength == 0 ) {                    // 0 can mean file is gone
			if( !file.exists() ) {
			    Debug.error( "File " + file.getName() + " disappeared!" );
			    threadShouldStop = true;

			}
		    
		    } else if( thisLength < lastLength ) {
			lastLength = thisLength;                // file got smaller -- not 
			super.notifySubscribers();              // exactly sure what to do
		    
		    } else { // this is now the normal case with the file growing
			String read = ""; // the string we will store it in
			try {
			
			    FileInputStream fis = new FileInputStream( file ); // open it to read
			    fis.skip( lastLength ); // skip ahead to the final position of the last time around
			
			    byte[] reading = new byte[1024]; // seems like a good length for reading
			    while( fis.available() > 0 ) { // read until done
				if( fis.available() > reading.length )
				    fis.read( reading, 0, reading.length );

				else
				    fis.read( reading, 0, fis.available() ); // this will prevent it from blocking
			    
				read += reading; // create the string

			    }
			    fis.close();
			
			} catch( IOException error ) {
			    // i guess if we hit here, we should just die
			    Debug.error( "IOException in reading! -- " + error.getMessage() );
			    threadShouldStop = true;

			}
			lastLength = thisLength;
		    
			// hmm - nelson mentioned something about emitting a whole
			// bunch of events right here if we get multiple line
			// additions.  i'm not sure if that is the right thing, so
			// i'll just emit one event that will be caught by the
			// translator that will split the lines and emit multiple
			// events
			super.notifySubscribers( (Object) read );
		    
		    }

		}

	    }

	    if( fileToMonitor != null ) {
		file = new File( fileToMonitor );
		lastLength = file.length();
		threadShouldReinit = false;

	    }

	}

    }

}
