// $Id: CricketShadowImpl.java,v 1.1 2000/05/18 22:05:04 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.



package net.hivecell.hive.shadow.thing;

import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.SerialStreamEventPublishingShadowImpl;
import net.hivecell.hive.shadow.Shadow;
import net.hivecell.hive.shadow.ShadowInitializationException;
import net.hivecell.hive.support.Debug;

import java.io.*;

/** 
 * A simple shadow to talk to a cricket over the serial port.
 * Currently, this shadow only *sends* bytes to the cricket, it can't
 * receive from the cricket.
 */
public class CricketShadowImpl
extends SerialStreamEventPublishingShadowImpl
implements CricketShadow {
    /**
     * create a new cricket shadow and just set teh serial port and
     * the baud rate.  perhaps this stuff shoudl be moved into the
     * init?  
     */
    public CricketShadowImpl() {
	super();
	setSerialPort( 1 );
	setBaudRate( 9600 );

    }

    /**
     * initailise this shadow -- we just launch off a thread to listen
     * to the serial port
     *
     * @exception ShadowInitializationException if something happens -- but it won't 
     */
    public void init() 
    throws ShadowInitializationException {
        super.init();
        CricketListener cl;
        Thread clThread = new Thread( ( cl = new CricketListener( this, is ) ) );
        clThread.start();
        setReady( true );

    }

    /** 
     * Fake out isDeviceThere - ideally, we should handshake. 
     *
     * @return always returns true
     */
    public boolean isDeviceThere() {
        return true;

    }

    /** 
     * The main method that this shadow exports; sending a byte out
     * on the infrared of the serial port cricket.
     *
     * @param b the byte that we want to send out
     */
    public void sendByteOnIR( byte b ) {
        try {
            os.write( (int)b );

        } catch( IOException ex ) { 
	    Debug.error( ex ); 

	}

    }
    
    /**
     * read a byte from the ir
     *
     * @return the byte that we read
     */
    public int readByteOnIR() {
        int data = 0;
        
        try {
            data = is.read();

        } catch( IOException ex ) {
            Debug.error( ex );

        }

        return data;

    }

}

/** 
 * This Class helps the CricketShadow listen to the input stream
 * coming from the cricket. It takes all the data on the input stream
 * and publishes it to the subscribers.  
 */
class CricketListener
implements Runnable {
    /**
     * the shadow that is listening to the cricket
     */
    CricketShadow cshadow;

    /**
     * the reader that we are going to use to listen to the cricket
     */
    BufferedReader listen;

    /**
     * create a new listener
     *
     * @param t the shadow that is listening to the cricket
     * @param s the input stream that we have
     */
    public CricketListener( CricketShadow t, InputStream s ) {
        cshadow = t;
        listen = new BufferedReader( new InputStreamReader( s ) );

    }

    /**
     * start out listener off
     */
    public void run() {
        while( true ) {
            try {
                int dat = listen.read();
                cshadow.notifySubscribers( new Integer( dat ) );

            } catch( Exception e ) {
                Debug.notice( "Caught Exception while trying to listen to input stream", e );

            }

        }

    }

}
