// $Id: JukeBoxShadowImpl.java,v 1.1 2000/05/18 22:05:04 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow.thing;

import net.hivecell.hive.Global;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.EventPublisherShadowImpl;
import net.hivecell.hive.shadow.ShadowInitializationException;
import net.hivecell.hive.support.Debug;

import java.io.*;
import java.util.Vector;

public class JukeBoxShadowImpl
extends EventPublisherShadowImpl
implements JukeBoxShadow {
    
    /**
     * the song list
     */
    private String[] songlist;

    /**
     * the process that is running the mp3 player
     */
    private Process player = null;

    /**
     * signal whether the player has been killed
     */
    private boolean killed = false;

    
    /**
     * this is supposed to tell us that the device is there -- but
     * since it is software, it is always there
     *
     * @return true at all tiems 
     */
    public boolean isDeviceThere() {
	return true;

    }

    /**
     * initialize the shadow
     *
     * @exception ShadowInitializationException is thrown if something is wrong, but there own't be
     */
    public void init()
    throws ShadowInitializationException {
	songlist = ( new File( "/MP3" ) ).list();
	setReady( true );

    }

    /**
     * pause the player
     */
    public void pause() {
	stop();

    }

    /**
     * play a song
     */
    public void play() {
	Debug.notice( "Play unsupported, currently" );

    }

    /**
     * stop playing on the mp3
     */
    public void stop() {
	Debug.notice( "Stop was called!!!!" );
	notifySubscribers( "REPORT:stop" );
	if( player != null ) {
	    killed = true;
	    player.destroy();
	    try {
		player.waitFor();

	    } catch( Exception e ) { 

	    }
	    player = null;

	}

    }

    /**
     * set the volume of the player
     *
     * @param v the volume
     */
    public void volume( int v ) {
	if( ( v >= 0 ) && ( v <= 100 ) ) {
	    String[] vexec = { "/usr/lib/oss/mixer", "pcm", v + ":" + v };
	    try {
		Runtime.getRuntime().exec( vexec );

	    } catch( Exception e ) { 

	    }

	}

    }

    /**
     * play a song which matches this string
     *
     * @param s the string for the song
     */
    public void playMatch( String s ) {
	Vector matches = new Vector();
	String pick = null;
	for( int i=0;i<songlist.length;i++ ) {
	    if( songlist[i].indexOf( s, 0 ) >= 0 )
		matches.addElement( songlist[i] );

	}
	Debug.notice( "Matches: " + matches.size() );
	if( matches.size() > 0 ) {
	    int pickN = (int)( matches.size() * Global.random.nextDouble() );
	    pick = (String)matches.elementAt(pickN);
	    stop();
	    Debug.notice( "Playing song " + pick );
	    try {
		String[] playsong = { "mpg123", "/MP3/" + pick };
		notifySubscribers( "REPORT:" + pick );
		player = Runtime.getRuntime().exec( playsong );
		(new Thread( new Runnable() {
		    public void run() {
			int me = Global.random.nextInt();
			try {
			    Debug.notice( "Waiting for player to exit " + me );
			    player.waitFor();

			} catch( Exception e ) { 

			}
			// Spit out an end of song event, maybe
			player = null;
			Debug.notice( "The player exited (killed? " + killed + ") id: " + me );
			if( killed ) {
			    killed = false;
			    Debug.notice( "someone killed it (now " + killed + ")" );

			} else {
			    notifySubscribers( "REPORT:song end" );
			    Debug.notice( "Song ended normally (id " + me + ")" );

			}

		    }
		})).start();
	    } catch( Exception e ) {
		Debug.notice( "Error playing song", e );

	    }

	}

    }

}
