// $Id: QuickCamImpl.java,v 1.1 2000/05/18 22:05:04 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow.thing;

import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.Shadow;
import net.hivecell.hive.shadow.ShadowImpl;
import net.hivecell.hive.shadow.ShadowInitializationException;
import net.hivecell.hive.support.*;

import java.io.*;
import java.util.*;

/** 
 * A shadow for a quickcam, works by invoking a program (cqcam by default)
 * that returns a PPM image on stdout.
 */
public class QuickCamImpl
extends ShadowImpl
implements QuickCam {

    /**
     * the program we are going to exec in order to run the quickcam
     * -- perhaps this should be renamed to cqcam shadow 
     */
    private String quickCamCommand = "cqcam";

    /**
     * the array that we are going to use to exec the cqcam
     */
    private String[] cmd = { quickCamCommand };
    
    /**
     * Get the value of quickCamCommand.
     *
     * @return Value of quickCamCommand.
     */
    public String getQuickCamCommand() {
	return quickCamCommand;

    }
    
    /**
     * Set the value of quickCamCommand.
     *
     * @param v Value to assign to quickCamCommand.
     */
    public void setQuickCamCommand( String v ) {
	if( isReady() ) return;
	this.quickCamCommand = v;
	StringTokenizer st = new StringTokenizer(quickCamCommand);
	int ntok;
	cmd = new String[ntok = st.countTokens()];
	for(int i=0;i < ntok;i++)
	    cmd[i] = st.nextToken();

    }

    /**
     * initialize the shadow -- we should at least check to make sure
     * that cqcam is a valid command in the system -- but that can be
     * added in later
     *
     * @exception ShadowInitializationException this should be eventually thrown if we know that cqcam is not on this cell 
     */
    public void init()
    throws ShadowInitializationException {
	setReady( true );

    }

    /**
     * get a ppm from the cqcam -- we run the program, get the iamge
     * and return it out
     *
     * @return the PPM from the quickcam 
     */
    public PPM getPPM() {
	Debug.notice( "Quickcam taking picture with command " + cmd );
	if( cmd == null )
	    return null;

	else {
	    try {
		Process process = Runtime.getRuntime().exec( cmd );
		InputStream ppmin = process.getInputStream();
		return new PPM( ppmin );
 
	    } catch( UnsupportedEncodingException e ) {
                Debug.error( "Taking picture failed, UnsupportedEncodingException: " + e );
                return null;

            } catch( IOException e ) { 
		Debug.error( "Taking picture failed, IOException: " + e );
		return null; 

	    }

	}

    }

}






