// $Id: ImageRingBuffer.java,v 1.1 2000/05/18 22:05:05 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/** This class implements a non resizable ringbuffer of objects.
 * Adding elements and fetching elements is very efficient.
 */

package net.hivecell.hive.support;

/** This class is a fixed size data structure that contains serializable images.
 *  If the ringbuffer is full and a new image is added, the least recently added
 *  image is removed.
 * 
 */
public class ImageRingBuffer implements java.io.Serializable {

    /** The index of the most recent image in the buffer.
     * 
     */
    protected int highval = 0;

    /** The array where the images are stored.
     */
    protected SerializableImage[] store;

    /** Constructs a ImageRingBuffer of size;
     */
    public ImageRingBuffer(int size) {
	store = new SerializableImage[size];
	for (int i = 0; i < size; i++) {
	    store[i] = null;
	}
    }

    /** Get the size of the ringbuffer.
     */
    public int getSize() {
	return store.length;
    }

    /** Get the index of the most recent image.
     */
    public int getLatestIndex() {
	return highval - 1;
    }

    /** Get the index of the earliest image still in the buffer
     */
    public int getEarliestIndex() {
	return highval - store.length;
    }

    /** return whether the RingBuffer contains this index
     */
    public boolean containsIndex(int index) {
	return ((index < highval) && (index >= (highval - store.length)));
    }

    /** Return the element with index i.
     * If it is not in the buffer return null.
     */
    public SerializableImage getElement(int i) {
	if ((i < highval) &&
	    (i >= (highval - store.length))) {
	    return store[i % store.length];
	}
	else {
	    return null;
	}
    }

    /** Add an image to the buffer and return the index it was assigned.
     */
    public int addElement(SerializableImage si) {
	store[highval++ % store.length] = si;
	return highval;
    }
}

