// $Id: NASDAQGrabber.java,v 1.1 2000/05/18 22:05:06 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/*
http://hive.www.media.mit.edu/projects/hive/
/mas/u/nelson/hive/release/
*/
package net.hivecell.hive.support.pocketserver;
import java.net.*;
import java.io.*;
import java.lang.*;
import net.hivecell.hive.support.pocketserver.StockGrabber;

public class NASDAQGrabber implements StockGrabber {
    
    final static String URLSTRING       = "http://www.nasdaq.com/asp/quotes_multi.asp?mode=Stock&symbol=";
    /*    
    final static String HIGHSTRING      = "Today's High:</font></td><td align=right><font size=2><nobr><b>$ ";
    final static String LOWSTRING       = "Today's Low:</font></td><td align=right><font size=2><nobr><b>$ ";
    final static String LASTSTRING      = "Last Sale:</font></td><td align=right><font size=2><nobr><b>$ ";
    final static String NETCHANGESTRING = "Net Change:</font></td><td align=right><font size=2><nobr><b>&nbsp;<font color=\"";
    final static String VOLUMESTRING    = "Share Volume:</font></td><td align=right><font size=2><b>";
    */

    final static String HIGHSTRING      = "Today's High:";
    final static String LOWSTRING       = "Today's Low:";
    final static String TODAYSCLOSE     = "Last Sale:";
    final static String LASTSTRING      = "Previous Close:";
    final static String NETCHANGESTRING = "Net Change:";
    final static String VOLUMESTRING    = "Share Volume:";

    //********************************************************
    // Constructors
    //********************************************************
    /** Public: Takes in nothing.
	Returns a new NASDAQGrabber Object.
	NOTE: Used for future expansion */
    public NASDAQGrabber () {
    }
    
    //********************************************************
    // Public Interface Methods
    //********************************************************
    /** Public: Takes in a stock symbol as a String.
	Returns the double value of today's highest sale value. */
    public double getHigh(String symbolstring) {
	return getValue(symbolstring, HIGHSTRING, "$");
    }
    
    
    /** Public: Takes in a stock symbol as a String.
	Returns the double value of today's lowest sale value. */
    public double getLow(String symbolstring) {
	return getValue(symbolstring, LOWSTRING, "$");
    }
    
    
    /** Public: Takes in a stock symbol as a String.
	Returns the double value of yesterday's closing value. */
    public double getPrevClose(String symbolstring) {
	return getValue(symbolstring, LASTSTRING, "$");
    }


    /** Public: Takes in a stock symbol as a String.
	Returns the double value of today's closing value. */
    public double getClose(String symbolstring) {
	return getValue(symbolstring, TODAYSCLOSE, "$");
    }
    
    
    /** Public: Takes in a stock symbol as a String.
	Returns the double value of today's net change in stock value. */
    public double getNetChange(String symbolstring) {
	return (getValue(symbolstring, TODAYSCLOSE, "$") - getValue(symbolstring, LASTSTRING, "$"));
	//return getNetChangeValue(symbolstring);
    }
    
    
    /** Public: Takes in a stock symbol as a String.
	Returns the double value of today's volume of stocks traded. */
    public double getVolume(String symbolstring) {
	return getValue(symbolstring, VOLUMESTRING, "<b>");
    }
    
    
    /** Public: Takes in nothing.
	Returns the root website of the stock search "www.nasdaq.com" */
    public String toString() {
	return (URLSTRING);
    }
    
    //********************************************************
    // Private methods
    //********************************************************
    /** Private:  Takes in a stock symbolstring and a searchstring for parsing the HTML.
	Returns the double value the the search string directly preceeded. */
	private static double getValue(String symbolstring, String searchstring, String startTag) {
	    double val = 0.0;	//return value
	    
	    //try to open the NASDAQ URL for the stock query
	    try {
		//open the URL and it's input stream
		URL u = new URL(URLSTRING + symbolstring);
		BufferedReader theHTML = new BufferedReader(new InputStreamReader(u.openStream()));
		
		//init the search vars
		//boolean searchbool = true;
		// Was the searchstring found?  If yes, next line has Value in it.
		boolean flag = false;
		
		//keep searching every line of the HTML for the searchstring
		String thisLine;
		// Goto line 130, that's where the fun starts (Note, this may not work for all stocks, but it should.)
		// Skip 6950 characters
		//theHTML.skip(6950); // Speed up the data processing.
		while(((thisLine = theHTML.readLine()) != null)) {
		    //System.out.print("\n" + thisLine);

				//for every line of HTML, start checking for the searchstring from the beginning
		    if (!flag) {
			for (int i=0; i < thisLine.length() - searchstring.length(); i++) {			    
			    //if we have found it, set the flag to get the value from the next line and break from the loop.
			    if (thisLine.regionMatches(true, i, searchstring, 0, searchstring.length())) {
				flag = true;
				break;
			    }  
			}
		    }

		    else {
			// Grab the value:
			int start = thisLine.indexOf(startTag) + startTag.length();      // Start of the value string
			int end = thisLine.indexOf("<", start);     // end of the value string
			    
			String sval = thisLine.substring(start, end);
			
			try {
			    val = (Double.valueOf(trimString(sval, ','))).doubleValue();
			} catch (NumberFormatException e) { System.out.print("Number format error when grabbing data.  Sorry.\n"); }
			
			//Close the HTML input stream
			try{
			    theHTML.close();
			} catch (IOException e) { System.err.println(e); }

			return (val);
		    }
		}
	    }
	    catch (MalformedURLException e) { System.err.println(e); }
	    catch (IOException e) { System.err.println(e); }	    

	    return (-1.0);
	}
    
    
    
    /** Private:  Takes in a stock symbolstring and a searchstring for parsing the HTML.
	Returns the double value the the search string directly preceeded. */
    /*    
    private static double getNetChangeValue(String symbolstring) {
	
	double val = (double) 0;	//return value
	
	//try to open the NASDAQ URL for the stock query
	try {
	    //open the URL and it's input stream
	    URL u = new URL(URLSTRING + symbolstring);
	    BufferedReader theHTML = new BufferedReader(new InputStreamReader(u.openStream()));
	    
	    //init the search vars
	    boolean searchbool = true;
	    
	    //keep searching every line of the HTML for the NETCHANGESTRING
	    String thisLine;
	    while(((thisLine = theHTML.readLine()) != null) && searchbool) {
				//for every line of HTML, start checking for the NETCHANGESTRING from the beginning
		for (int i=0; i < thisLine.length() - NETCHANGESTRING.length(); i++) {
		    
		    //if we have found it, set the return value and break from this loop and the while loop
		    if (thisLine.regionMatches(true, i, NETCHANGESTRING, 0, NETCHANGESTRING.length())) {
			
			//get the offset in the current HTML line for the beginning of the value
			int valuestart = i + NETCHANGESTRING.length();	// the beginning offset for the value string
			int valueend = valuestart;	// the end offset for the value string
			
			//must do extra processing for the netchange, since it can be both + and -
			while (thisLine.charAt(valueend) != '\"')
			    valueend++;
			
			//grab the valuestring
			String valuestring = thisLine.substring(valuestart, valueend);
			
			//now we must do the extra processing to determine the net change
			String chngstring = "";
			//check whether the change is + or minus and store it
			if (valuestring.compareTo("FF0000") == 0)  //else "009900" for +
			    chngstring = "-";
			
			//now we need to grab the actual value
			valueend += 2;		//gets rid of the "> at the end of the color value
			valuestart = valueend;	
			while (thisLine.charAt(valueend) != '<')
			    valueend++;
			
			//grab the value of the change and append the "+" or "-"
			valuestring = thisLine.substring(valuestart, valueend);
			valuestring = chngstring + valuestring;
			
			//trim unnecessary chars from the string
			valuestring.trim();
			valuestring = trimString(valuestring, ',');
			
			//Convert the valuestring to a double for the return value
			try {
			    val =  (Double.valueOf(valuestring)).doubleValue();
			} catch (NumberFormatException e) { System.err.println(e); }
			
			//break from the for and while loops
			searchbool = false;
			break;
		    }
		}
	    }
	}
	catch (MalformedURLException e) { System.err.println(e); }
	catch (IOException e) { System.err.println(e); }
	
	return val;
    }
    
    */    
    
    /** Private:  Takes in a String and a character, c.
	Returns the given string with all instances of c removed.
	NOTE: Doesn't modify string passed in */
    private static String trimString(String s, char c) {
	
	String tmpstring = new String(s);		//copy the value of s, so we don't modify it
	String trimmedstring = "";				//holds the value of the trimmed string
	
	int index = tmpstring.indexOf(c);
	while (index != -1) {
	    //add the part of the string before the c to the trimmedstring
	    trimmedstring = trimmedstring + tmpstring.substring(0, index);
	    //update the tmpstring
	    tmpstring = tmpstring.substring(index+1);
	    //update the new index
	    index = tmpstring.indexOf(c);
	}
	//add the part of the tmpstring after the final c to the trimmedstring (can be "")
	trimmedstring = trimmedstring + tmpstring;
	
	
	return trimmedstring;
    }
}

