// $Id: Hive.java,v 1.1 2000/05/18 22:00:47 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.


package net.hivecell.hive;

import java.rmi.AlreadyBoundException;
import java.rmi.RemoteException;
import java.util.Properties;

import net.hivecell.hive.Global;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.MalformedHiveURLException;
import net.hivecell.hive.support.async.AsyncCall;
import net.hivecell.hive.support.async.AsyncCaller;

import gnu.getopt.*;


public class Hive {

    ///////////////////////////////////////////////////////////////////////////
    // Hive startup code. Not for agents                                     //
    ///////////////////////////////////////////////////////////////////////////
    private static void usage() {
	System.err.println( "Hive version: " + Global.version + " Java " + System.getProperty( "java.version" ) );
	System.err.println( "Copyright (c) 1998, 1999, 2000 by the Massachusetts Institute of Technology." );
	System.err.println( "All rights reserved. http://hive.media.mit.edu/" );
	System.err.println();
        System.err.println( "Usage: Hive [port] [options]");
        System.err.println( "  [port]                   port server runs on. Default is " + Global.defaultPort );

        System.err.println( "  --configdir            directory to load config files\n");
        System.err.println( "                         default is " + Global.getConfigDir() );
        System.err.println( "  --celldesc             cell description, which contains startup options." );
        System.err.println( "  --debuglevel           only show messages of this level or higher.");
        System.err.println( "                         values: notice, warning, error, critical, failure");
        System.err.println( "  <DebugLevel>           Name of this tag in " + Global.CellDescription);
        System.err.println( "  --nojoin               do not join any cell list.");
        System.err.println( "  --join                 join this specific hive:// URL as cell list.");
        System.err.println( "                         if self, this cell will act as master.");
        System.err.println( "  <JoinCellList>         Tag name in " + Global.CellDescription + ",");
	System.err.println( "                         value true|false|address of master cell");
        System.err.println( "  --fullasync            async messages are never queued.");
        System.err.println( "  --noasync              turns off async notifies (Hive may not work)" );
        System.err.println( "  --nogui                do not allow any graphics");
        System.err.println( "  <UI>                   tag name, value true or false");
        System.err.println( "  --noguiagent           do not create a GUI agent for the Hive cell.");
	System.err.println( "  <GUIAgent>             tag name, true or false");
        System.err.println( "  --help,-h              displays this help message." );

    }


    public static void main( String[] args ) {
	Debug.setDefaultLevel( Debug.NOTICE ); // default level.
	Debug.setFallthrough( true );
        
	AsyncCall.setAsyncCallerClass( Global.asyncCallerClass );

	String[] newargs = null;
	CellAddress ca = null;
	RemoteCell rs = null;
	try {
	    try {
		int port = Integer.parseInt( args[0] );
		ca = CellAddress.createLocalCellAddress( port );

		newargs = new String[args.length - 1];
		System.arraycopy( args, 1, newargs, 0, args.length - 1 );
	    } catch ( NumberFormatException error ) {
                ca = CellAddress.createLocalCellAddress();
                
                newargs = new String[args.length];
                System.arraycopy( args, 0, newargs, 0, args.length );
	    }
	} catch( ArrayIndexOutOfBoundsException error ) {
	    ca = CellAddress.createLocalCellAddress();
	    
	    newargs = new String[args.length];
	    System.arraycopy( args, 0, newargs, 0, args.length );
	}
	try {
	    rs = ca.getRemoteCell();
	} catch( CellConnectException error ) {
	    ;; // Leave rs as null.
	}

	int c;
	String arg;

	LongOpt[] longopts = new LongOpt[9];
	longopts[0] = new LongOpt( "help", LongOpt.NO_ARGUMENT, null, 'h' );
	longopts[1] = new LongOpt( "configdir", LongOpt.REQUIRED_ARGUMENT, null, 'c' );
	longopts[2] = new LongOpt( "celldesc", LongOpt.REQUIRED_ARGUMENT, null, 'd' );
	longopts[3] = new LongOpt( "fullasync", LongOpt.NO_ARGUMENT, null, 'f' );
	longopts[4] = new LongOpt( "nogui", LongOpt.NO_ARGUMENT, null, 'u' );
	longopts[5] = new LongOpt( "noguiagent", LongOpt.NO_ARGUMENT, null, 'a' );
	longopts[6] = new LongOpt( "nojoin", LongOpt.NO_ARGUMENT, null, 'j' );
	longopts[7] = new LongOpt( "join", LongOpt.REQUIRED_ARGUMENT, null, 's' );
	longopts[8] = new LongOpt( "debuglevel", LongOpt.REQUIRED_ARGUMENT, null, 'l' );


	Getopt g = new Getopt( "Hive", newargs, "-h", longopts );
	g.setOpterr( false );

	// From here, we're probably going to boot a cell, so start collecting options..
	Properties props = System.getProperties();
	while( ( c = g.getopt() ) != -1 ) {
	    switch( c ) {
	    case 'h':
		usage();
		System.exit( 0 );
		break;

	    case 'u':
		props.put( Global.UI, "false" );
		props.put( Global.GUIAgent, "false" );
		Global.argList.addElement("UI");  // Let the cell know to ignore these opts. in its desc
		Global.argList.addElement("GUIAgent");
		break;

	    case 'a':
		props.put( Global.GUIAgent, "false" );
		Global.argList.addElement("GUIAgent");
		break;

	    case 'j':
		props.put( Global.JoinCellList, "false" );
		Global.argList.addElement("JoinCellList");
		break;
	    
	    case 'c':
		props.put( Global.ConfigDir, g.getOptarg() );
		break;

	    case 'd':
		props.put( Global.CellDescription, g.getOptarg() );
		break;
		/*
	    case 'w':
		props.put( Global.ShadowConfig, g.getOptarg() );
		break;

	    case 'g':
		props.put( Global.AgentConfig, g.getOptarg() );
		break;
		*/
	    case 'f':
	        Debug.notice( "using full asynchronous messaging without async calling queues" );
		AsyncCall.setAsyncCallerClass( new AsyncCaller() );
		break;		
	    case 's':
		props.put( Global.JoinCellList, "true" );
		props.put( Global.MasterCellListCell, g.getOptarg() );
		Global.argList.addElement("JoinCellList");
		Global.argList.addElement("MasterCellListCell");
		break;
	    case 'l':
		Global.argList.addElement("DebugLevel");
		String debug = g.getOptarg();
		if ( debug.equalsIgnoreCase("notice") ) {
		    Debug.setDefaultLevel( Debug.NOTICE );
		}
		else if ( debug.equalsIgnoreCase("warning") ) {
		    Debug.setDefaultLevel( Debug.WARNING );
		}
		else if ( debug.equalsIgnoreCase("error") ) {
		    Debug.setDefaultLevel( Debug.ERROR );
		}
		else if ( debug.equalsIgnoreCase("critical") ) {
		    Debug.setDefaultLevel( Debug.CRITICAL );
		}
		else if ( debug.equalsIgnoreCase("failure") ) {
		    Debug.setDefaultLevel( Debug.FAILURE );
		}
		else {
		    Debug.warning("Invalid argument for --debuglevel: " + debug + ".  Only recognizes notice, warning, error, critical and failure.");
		}
		break;

		
	    default:
		Debug.error( "unrecognized command line flag: " + g.getOptopt() );
		break;
	    }
	}

	// test to make sure we have not asked for there to be no
	// agent component manager shadow and also visual shadow
	// configuration -- if they were both specified this will
	// break
	if( System.getProperty( Global.VisualShadowConfig, "false" ).equals( "true" ) &&
	    System.getProperty( Global.UI, "true" ).equals( "false" ) ) {
	    Debug.error( "cannot both have visual shadow configuration and not allow a ui" );
	    System.exit( 1 );
	}

	if( rs == null ) {
	    if( ca.isLocal() ) {
		Cell server = createCell( ca.getPort() );
	    } else {
		Debug.error( "cannot create cell remotely: " + ca.toString() );
	    }
	} else {
	    Debug.error( "a server already exists on port " + ca.getPort() );
	}
    }

    private static Cell createCell( int port ) {
	// Booting new local cell..
	Cell server = null;
	try {
	    server = new Cell( port );
	} catch( AlreadyBoundException ex ) {
	    Debug.critical( "port " + port + " already in use." );
	    System.exit( 1 );

	} catch( CellConnectException ex ) {
	    Debug.critical( "cannot get the RMI reference to ourselves", ex );
	    System.exit( 1 );

	} catch( RemoteException ex ) {
	    Debug.critical( "remote exception creating cell", ex );
	    System.exit( 1 );

	}
	return server;
    }
}
