package net.hivecell.hive.agent;

/**
 * this interface is used so the event receiving agent can list the
 * objects that it can respond to best.  event receiving agents are
 * supposed to be designed so they can repsond to any generic event,
 * and then take special notice of certain events (ie -- string
 * display agents will display whatever event they will get mouse
 * events, etc, etc -- but it will display a string if it gets a
 * string.  so the String object is a specific event that the
 * StringDisplayAgentImpl can handle.
 *
 * so this interface allows agent authors to list the objects that it
 * can specially handle.  this information is not stuck in the
 * Description of the agent because this is a very symantic thing (as
 * opposed to a semantic item), and therefore it needs to be type
 * checked.
 *
 * if an event receiving agent does not implement this interface, it
 * does not mean that the agent does not take special care to specific
 * objects -- it only means that the agent's author has not provided
 * the courtesy of listing these objects 
 */
public interface EventReceivingDescription {

    /**
     * this method is used to enumerate the classes that this agent
     * would like in its event objects.  an agent can return null from
     * this method if it would like to say that it is a completely
     * generic event object.  or it can list all the classes it would
     * like.
     *
     * @return null if it is a completely generic event receiver, or the classes that it would like to receive.
     */
    public Class[] getAgentSpecificEventClasses();

}
