// $Id: EventSendingAgent.java,v 1.1 2000/05/18 22:00:49 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent;
import net.hivecell.hive.event.*;

import java.rmi.RemoteException;
import java.util.Vector;

/** Interface for agents that send events to other agents.
 ** @see RemoteEvent
 ** @see EventReceivingAgent
 **
 ** @version $Revision: 1.1 $
 ** @author Nelson Minar <nelson@media.mit.edu>
 **/
public interface EventSendingAgent extends Agent {
  /** Adds the specified agent as a listener.
    * @param subscriber The agent to subscribe **/
  public void addListener(EventReceivingAgent subscriber) throws RemoteException;

  /** Adds the specified agent as a listener with a handback object.
   * @param subscriber The agent to subscribe
   * @param handback An object to be returned in every event **/
  public void addListener(EventReceivingAgent sender, MarshalledObject handback) throws RemoteException;

  /** Adds the specified agent as a listener.
    * @param subscriber The agent to subscribe **/
  public void addListener(EventReceivingAgent subscriber, long eventMask) throws RemoteException;

  /** Adds the specified agent as a listener with a handback object.
   * @param subscriber The agent to subscribe
   * @param handback An object to be returned in every event **/
  public void addListener(EventReceivingAgent sender, long eventMask, MarshalledObject handback) throws RemoteException;

  /** Removes the specified agent from listening. **/
  public void removeListener(EventReceivingAgent sender) throws RemoteException;

  /** Returns a count of the number of events. **/
  public int getEventCount(Agent sender) throws RemoteException;

}
