// $Id: EventTransceivingAgentImpl.java,v 1.2 2000/05/19 19:55:53 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent;

import net.hivecell.hive.*;
import net.hivecell.hive.event.*;
import net.hivecell.hive.cell.*;
import net.hivecell.hive.support.Debug;

import java.rmi.RemoteException;
import java.util.*;

/** Support for agents that both send and receive agents.
 ** @version $Revision: 1.2 $
 ** @author Nelson Minar <nelson@media.mit.edu>
 **/

// IMPORTANT (nelson) - if you modify this class, see if the modifications
// are appropriate to EventSendingAgentImpl as well. It has code
// cut-and-pasted from here to do multiple inheritance.

public abstract class EventTransceivingAgentImpl 
extends EventReceivingAgentImpl
implements EventTransceivingAgent, EventSendingErrorListener {

    protected transient EventSendingHelper eventSendingHelper;

    public EventTransceivingAgentImpl() 
	throws RemoteException { 
	super(); 
    }

    /** When we arrive, create an event sending helper. **/
    public void doLocalSetup() throws AgentInitializationException {
	super.doLocalSetup();
        if( eventSendingHelper == null ) eventSendingHelper = new EventSendingHelper( this, myCell );
	synchronized( this ) {
	    notify();
	}
    }
  

    public void addListener( EventReceivingAgent subscriber ) {
	while( eventSendingHelper == null )
	    try {
		synchronized( this ) {
		    wait();
		}
	    } catch( InterruptedException error ) { }
        eventSendingHelper.addListener( subscriber );
    }

    public void addListener( EventReceivingAgent subscriber, MarshalledObject handback ) {
        Debug.warning( "Warning, handbacks are not currently supported." );
        this.addListener( subscriber );
    }

    public void addListener( EventReceivingAgent subscriber, long eventMask ) {
	while( eventSendingHelper == null )
	    try {
		synchronized( this ) {
		    wait();
		}
	    } catch( InterruptedException error ) { }
        eventSendingHelper.addListener( subscriber, eventMask );
    }

    public void addListener( EventReceivingAgent subscriber, long eventMask, MarshalledObject handback ) {
        Debug.warning( "Warning, handbacks are not currently supported." );
        this.addListener( subscriber, eventMask );
    }

    public void removeListener( EventReceivingAgent sender ) {
        eventSendingHelper.removeListener( sender );
    }

    /** Return a count of events fired. **/
    public int getEventCount( Agent sender ) {
        return eventSendingHelper.getEventCount( sender );
    }
    
    /** Called asynchronously if an attempt to deliver an event failed.
     * Typically, the asynchronous transport itself calls this method.
     * This method is best-effort only, there is no guarantee that a failure
     * will result in this being called.
     */
    public void eventSendingError( Object o, Exception error ) {
	Debug.notice( "Removing " + Global.shortString( o ) + " because of the messaging failure: " + error );
	removeListener( (EventReceivingAgent)o );
    }

    /**
     * Unfortunately in event transceiving agents it will not tell you
     * which way the connections are -- it will just tell you that you
     * have a connection 
     */
    public Vector listAllIncomingConnections() {
	return agentSubscriptions;

    }

    public Vector listAllOutgoingConnections() {
	if( eventSendingHelper == null )
	    return null;
	return eventSendingHelper.getListeners();

    }

}

// IMPORTANT (nelson) - if you modify this class, see if the modifications
// are appropriate to EventSendingAgentImpl as well. It has code
// cut-and-pasted from here to do multiple inheritance.
