// $Id: GenericSignalPassthroughAgentImpl.java,v 1.2 2000/05/19 19:55:53 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent;

import java.rmi.*;

import net.hivecell.hive.*;
import net.hivecell.hive.agent.SignalPassthrough;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.*;
import net.hivecell.hive.support.Debug;

/** An agent that passes through signals from a shadow as Hive events.
 ** The shadow to use is set by the variable shadowType, which can be
 ** set by an RDF file, via bean methods, or programmatically.
 **/
public class GenericSignalPassthroughAgentImpl  
extends SignalPassthrough {

    protected String shadowType = null;
    protected transient EventPublisherShadow myShadow;

    public GenericSignalPassthroughAgentImpl() throws RemoteException {
        super(); 
    }
    
    /**
     * Get the value of shadowType.
     * @return Value of shadowType.
     */
    public String getShadowType() {
	return shadowType;
    }
    
    /**
     * Set the value of shadowType.
     * @param v  Value to assign to shadowType.
     */
    protected void setShadowType( String v ) {
	this.shadowType = v;
    }

    /** Do the local setup.
     ** Grab the requested shadow as an event publisher and subscribe to it.
     **/
    public void doLocalSetup() throws AgentInitializationException  {
        super.doLocalSetup();

        // First check that we have a shadow type set
	while( getShadowType() == null ){
	    Debug.notice("Waiting for my shadow..." );
	    try { 
		Thread.sleep(1000); 
	    } catch( Exception e ) { }
	}

        // Now grab the shadow from the database
        Shadow s = null;
	try {
	    s = myCell.getShadow( getShadowType() );
	    if( ( s == null ) || !( s instanceof EventPublisherShadow ) ) {
		Debug.error( getShadowType() + " was not an event publisher!" );
		throw new AgentInitializationException( getShadowType() + " was not an event publisher!" );

	    }
	} catch( ShadowNotFoundException error ) {
	    Debug.error( getShadowType() + " cannot be found" );
	    throw new AgentInitializationException( getShadowType() + " cannot be found!" );

	}
        myShadow = (EventPublisherShadow)s;
        myShadow.addListener( this );
	
	if (myShadow.getDescription() != null) {
	    //setDescription(myShadow.getDescription().child( this, "TagReaderAgent" ) ); NOTE: This makes a copy of the description
	    // NOTE: with different references.  This should do the same:
	    setDescription( new Description(this, myShadow.getDescription().getNode()) );

	}

    }

    public void doLocalCleanup() {
        if (myShadow != null)
            myShadow.removeListener( this );
    }

    public void doBehavior() {
	waitUntilDeath();
    }
}
