// $Id: TwoWayConnectionAgentImpl.java,v 1.1 2000/05/18 22:00:49 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/** 
 ** $Id: TwoWayConnectionAgentImpl.java,v 1.1 2000/05/18 22:00:49 tucker Exp $
 ** Agent that creates a two way connection for events between itself and 
 ** its subscriber. 
 **/

package net.hivecell.hive.agent;

import java.util.*;
import java.rmi.RemoteException;

import net.hivecell.hive.*;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.agent.EventTransceivingAgentImpl;
import net.hivecell.hive.agent.TwoWayConnectionAgent;
import net.hivecell.hive.event.HiveEvent;


public abstract class TwoWayConnectionAgentImpl extends EventTransceivingAgentImpl implements TwoWayConnectionAgent {
    
    protected static boolean connecting = false;     // Is the TWCAI in the process of connecting with another (to prevent inf recurse)
    protected static boolean disconnecting = false;  // Is the TWCAI in the process of disconnecting from another (to prevent inf recurse)

    public TwoWayConnectionAgentImpl() throws RemoteException { 
	super();
	connecting = false;
	disconnecting = false;
    }
    
    
    public void setConnecting() { connecting = true; }
    public void setDisconnecting() { disconnecting = true; }
    
    /** Forms a two-way connection between this agent and the one it's subscribing to.
     ** Cannot form a connection with itself.
     **/
    public boolean connectTo(Agent otherAgent)
    {
	// TwoWayAgents cannot form connections with themselves.
	if (otherAgent.equals(this))
	    return false;


	boolean connected = super.connectTo(otherAgent);
	if (connected && !connecting)
	    {
		try {
		    if (otherAgent instanceof TwoWayConnectionAgent) 
			{
			    ((TwoWayConnectionAgent)otherAgent).setConnecting();
			    connected = otherAgent.connectTo(this);
			}
		    
		} catch (RemoteException e) { 
		    Debug.error("Remote Exception thrown while attempting to connect: ", e);
		    connected = false; 
		} catch (Exception x) { 
		    Debug.failure("Non-RemoteException caught while trying to connect. (damn)", x); 
		    connected = false;
		}
	    }	
	connecting = false;
	return (connected);
    }
    
    
    
    /** Disconnects from all of the agents it has subscribed to, 
     ** and attempts to make them unsubscribe from it.
     **/
    public void disconnectFrom(Agent otherAgent)
    {
	super.disconnectFrom(otherAgent);
	if (!disconnecting)
	    {
		try {
		    if (otherAgent instanceof TwoWayConnectionAgent)
			((TwoWayConnectionAgent)otherAgent).setDisconnecting();
		    otherAgent.disconnectFrom(this);
		    
		} catch (RemoteException e) { Debug.error("Remote exception thrown while disconnecting: ", e); }
	    }
	disconnecting = false;
    }

    /** Inherited from eventSendingAgent: modify to call disconnectFrom() */
    
    public void eventSendingError( Object o, Exception error ) {
	super.eventSendingError(o, error);

	// Don't bother trying to get 'o' to disconnect from you.  It's already dead.
	super.disconnectFrom((EventReceivingAgent)o);
    }
};
