// $Id: CellControlAgentImpl.java,v 1.2 2000/05/19 19:55:55 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.cell.AgentDeliveryException;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.CellControlShadow;
import net.hivecell.hive.shadow.cell.CellControlShadowImpl;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.description.Description;

import java.rmi.RemoteException;


/**
 * The agent which will provide the remote access to the
 * CellControlShadow.  If any security is to be done to "protect"
 * the server, it needs to be done here 
 */
public class CellControlAgentImpl
extends AgentImpl 
implements CellControlAgent {

    /**
     * Our hook to the server's CellControlShadow
     */
    private CellControlShadow cellControl;
    
    
    /**
     * Initialize the cellcontrolagentimpl
     */
    public CellControlAgentImpl() 
    throws RemoteException {
	super();
    }

    public void doLocalSetup() 
    throws AgentInitializationException {
	try {
	    cellControl = (CellControlShadow)myCell.getShadow( (CellControlShadowImpl.class).getName() );
	    
	} catch( ShadowNotFoundException error ) {
	    // if this happens -- the system is dieing
	    Debug.error( "cell control shadow not found!" );
	    throw new AgentInitializationException();

	}

    }
    
    /**
     * Get our hook to the cellcontrolshadow
     */
    public void doBehavior() { 
	waitUntilDeath();

    }

    /** This method should be called when other agents 'connect' to this one.
     *  This agent does not explicitly store agents that are connected to it, but
     *  We still want to note the agent connection in the UI.
     *  We don't do any type checking, because any type of agent can call the methods
     *  on this one. (But only certain ones will want to note a connection.)
     *
     * @param otherAgent reference to the agent to connect to
     * @return whether the connection was successful.
     */
    public boolean connectTo( Agent otherAgent ) {
	myCell.noteAgentConnect( this, this, otherAgent );
	return true;
    }

    /** 
     * This method should be called when other agents 'disconnect' to
     * this one.  This agent does not explicitly store agents that are
     * connected to it, but We still want to note the agent
     * disconnection in the UI.
     *
     * @param otherAgent reference to the agent to connect to
     * @return whether the connection was successful.  
     */
    public void disconnectFrom( Agent otherAgent ) {
	myCell.noteAgentDisconnect( this, this, otherAgent );
    }


    /**
     * Ask the server control shadow to create this agent
     *
     * @param agentName the name of the agent to create
     * @return the remote reference to the agent we have created
     * @exception ClassNotFoundException when the server doesn't have this
     * @exception InstantiationException if something goes wrong with creating the agent
     * @exception RemoteException if something goes wrong with the remote call
     */
    public Agent createAgent( String agentName )
    throws ClassNotFoundException, InstantiationException, RemoteException {
	return cellControl.createAgent( agentName );

    }

    /**
     * Ask the server control shadow to create this agent
     *
     * @param agentName the name of the agent to create
     * @param description the description we want to give to this agent
     * @return the remote reference to the agent we have created
     * @exception ClassNotFoundException when the server doesn't have this
     * @exception InstantiationException if something goes wrong with creating the agent
     * @exception RemoteException if something goes wrong wtih the remote call
     */
    public Agent createAgent( String agentName, Description description )
    throws ClassNotFoundException, InstantiationException, RemoteException {
	return cellControl.createAgent( agentName, description );

    }

    /**
     * Ask the server control shadow to kill this agent
     *
     * @param agent the agent which you wish to kill
     * @exception RemoteException if something goes wrong wtih the remote call
     */
    public void killAgent( Agent agent ) 
    throws RemoteException {
	cellControl.killAgent( agent );

    }

    /**
     * Ask the server control shadow to kill this agent
     *
     * @param agent the agent which you wish to kill
     * @return the remote reference to the moved agent
     * @exception RemoteException if something goes wrong wtih the remote call
     */
    public Agent moveAgent( Agent agent, CellAddress address ) 
    throws RemoteException, AgentDeliveryException {
	return cellControl.moveAgent( agent, address );

    }

    /**
     * Ask the server control shadow to connect these two agents
     *
     * @param from the agent to make the connection from
     * @param to the agent to connect to
     * @return true if the agents were connected
     * @exception RemoteException if something goes wrong wtih the remote call
     */
    public boolean connectAgents( Agent from, Agent to ) 
    throws RemoteException {
	return cellControl.connectAgents( from, to );

    }

    /**
     * Ask the server control shadow to disconnect everything from
     * this server
     *
     * @param agent the agent to disconnect from everything 
     * @exception RemoteException if something goes wrong wtih the remote call
     */
    public void disconnectAgentAll( Agent agent ) 
    throws RemoteException {
	cellControl.disconnectAgentAll( agent );

    }

    /**
     * Ask the server control shadow to disconnect these two agents
     *
     * @param from the agent to disconnect from
     * @param to the agent to disconnect to
     * @exception RemoteException if something goes wrong wtih the remote call
     */
    public void disconnectAgents( Agent from, Agent to )
    throws RemoteException {
	cellControl.disconnectAgents( from, to );

    }

    /**
     * Ask the server control shadow to shutdown the server
     * @exception RemoteException if something goes wrong wtih the remote call
     */
    public void shutDownCell()
    throws RemoteException {
	cellControl.shutDownCell();

    }

}






