// $Id: CellListAgent.java,v 1.1 2000/05/18 22:00:51 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.cell;

import java.rmi.RemoteException;
import java.util.Vector;

import net.hivecell.hive.agent.EventTransceivingAgent;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.support.CellAddress;

public interface CellListAgent 
extends EventTransceivingAgent, CellAgent {
    
    /**
     * return a vector of cell list info objects which has
     * information on whether cells are up or down
     *
     * @param sender the agent requesting the information
     * @return a vector of server list info objects 
     */
    public Vector listCells( Agent sender ) 
    throws RemoteException;

    /**
     * query all the cells which are up and that this agent knows
     * about for this specific semantic description
     *
     * @param requester the agent requesting the query
     * @param semantic an array of semantic description strings
     * @return a desc set mapping remote servers to their descritptions 
     */
    public AgentSet queryCells( Agent requester, String[] semantic )
    throws RemoteException;

    /**
     * announce a server to the hive federation 
     *
     * @param sender the agent asking for the annoucement
     * @param ca the cell address of the server that is being announced
     * @param rs the remote server object of the server that is being announced
     * @return true if the announcement went through
     */
    public boolean announceCell( Agent sender, CellAddress ca, RemoteCell rs ) 
    throws RemoteException;

    /**
     * withdraw a server from the hive federation 
     *
     * @param sender the agent asking for the annoucement
     * @param ca the cell address of the server that is being announced
     * @param rs the remote server object of the server that is being announced
     * @return true if the withdrawl went through
     */
    public boolean withdrawCell( Agent sender, CellAddress ca, RemoteCell rs )
    throws RemoteException;

}
