// $Id: CellListInfo.java,v 1.1 2000/05/18 22:00:51 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.cell;

import java.io.Serializable;
import java.rmi.RemoteException;
import net.hivecell.hive.cell.*;
import net.hivecell.hive.support.*;

/** Small utility class to track information about a server. Used
 ** by CellListAgents. **/

// TODO - should implement equals, hashcode here?
public class CellListInfo implements Serializable {
    /** the URL for the server **/
    protected final CellAddress address;
    /** the server stub, if server is up **/
    protected RemoteCell server;
    /** how stale the information about this server is. Time in MS **/
    protected long lastInfo;
    /** Is the server up? **/
    protected boolean up = false;

    public CellListInfo(CellAddress addr, RemoteCell rs, boolean status) {
        this.address = addr;
        this.server = rs;
        this.up = status;
        this.lastInfo = System.currentTimeMillis();
    }

    public CellAddress getAddress() { return address; }
    public RemoteCell getCell() { return server; }
    public long getLastInfo() { return lastInfo; }
    public boolean isUp() { return up; }

    public void updateLastInfo() {
        setLastInfo(System.currentTimeMillis());
    }
    public void setLastInfo(long timeMS) { lastInfo = timeMS; }

    /** Set the status of a server. Note, in the case where the server
     ** has just come up, this may do a network lookup to find the
     ** remote server from the cell address.
     **/
    public void setUp(boolean newStatus) {
        // Four cases:
        //   if was down and still down, just refresh info
        //   if was up and still up, just refresh info
        //   if was up and now down, clear out the server variable
        //   if was down and now up, look up the server
        if (up == false && newStatus == false)
            updateLastInfo();
        else if (up == true && newStatus == true)
            updateLastInfo();
        else if (up == true && newStatus == false) {
            this.up = false;
            this.server = null;
            updateLastInfo();
        } else if (up == false && newStatus == true) {
            this.up = true;
            try {
                this.server = address.getRemoteCell();
            } catch (CellConnectException cce) {
                Debug.error("Was told cell " + address + " was up, but got an error when trying to connect to it.");
            }
            updateLastInfo();
        } else
            Debug.assert(false);
    }
    
    public String toString() {
        return ((up ? "up" : "down") +
                " " + (System.currentTimeMillis() - this.lastInfo)/1000 +
                " " + this.address.toString());
    }
}
