// $Id: BaseGUIAgentImpl.java,v 1.2 2000/05/19 19:55:56 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.hiveui;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.cell.UnableToMonitorCellException;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.MalformedHiveURLException;
import net.hivecell.hive.support.menu.MenuElement;
import net.hivecell.hive.support.menu.MenuFolder;
import net.hivecell.hive.support.menu.MenuItem;
import net.hivecell.hive.support.menu.MenuSeparator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Canvas;
import java.awt.Choice;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.InputStream;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Vector;


/**
 * this class is the next level up on top of the base ui agent -- the
 * purpose of this is so we have a standard way to get the popup boxes
 * that we like 
 *
 * @author Raffi Krikorian <raffi@mit.edu>
 */
public abstract class BaseGUIAgentImpl
extends BaseUIAgentImpl {


    /** 
     * a handle to the ACMS so we don't need to look it up every time.
     */
    protected ComponentManagerShadow acms;

    
    public BaseGUIAgentImpl() throws RemoteException { super(); }
	

    /**
     * do our local setup which just involves letting the BaseUIAgent
     * do its stuff, and then we can lookup our agent component
     * manager shadow
     */  
    public void doLocalSetup() throws AgentInitializationException {

	super.doLocalSetup();

	// get our handle to our agent component manager shadow
	try {
	    acms = getComponentManagerShadow();
	} catch( ShadowNotFoundException error ) {
	    Debug.warning( "this agent cannot start on a cell that does not have graphical capabilities.", error );
	    myCell.killAgent( (AgentImpl)this );
	}

    }

    public void doBehavior() { super.doBehavior(); }

    /**
     * a helper function that reads the RDF files and stuff and
     * creates the default menus that we are going to use 
     */
    protected void constructMenus() {
	super.constructMenus();

	// we just need to now implement some of the things that we
	// did not implement in the BaseUIAgent's implementation
	MenuFolder serverMenuFolder = (MenuFolder)menu.getMenuElement( "Cell" );
	serverMenuFolder.replaceMenuElement( "Monitor remote cell", new MenuItem() {
	    public String getName() { return "Monitor remote cell"; }
	    public boolean invoke() {
		showStartMonitorCellDialogBox();
		return true;
	    }
	});
	
	serverMenuFolder.replaceMenuElement( "Stop monitoring remote cell", new MenuItem() {
	    public String getName() { return "Stop monitoring remote cell"; }
	    public boolean invoke() {
		showStopMonitorCellDialogBox();
		return true;
	    }
	});
	
	MenuFolder agentMenuFolder = (MenuFolder)menu.getMenuElementAt( menu.getMenuElementSize() - 1 );
	agentMenuFolder.replaceMenuElement( "Create agent by classname...", new MenuItem() {
	    public String getName() { return "Create agent by classname..."; }
	    public boolean invoke() {
		showCreateAgentByNameDialogBox();
		return true;
	    }
	});

	// go through all the create agent menus and add the dialog
	// box if there are more than one server we are monitoring
	replaceInvokeCommands( agentMenuFolder );

    }
    
    /**
     * helper function to go through menu folders and replace all the
     * menu items with new menu items with different invoke statements
     *
     * @param folder the folder to go thorugh 
     */
    private void replaceInvokeCommands( MenuFolder folder ) {

	Enumeration e = folder.getMenuItems();
	while( e.hasMoreElements() ) {
	    final MenuElement me = (MenuElement)e.nextElement();
	    if( me instanceof CreateAgentMenuItem ) {
		folder.replaceMenuElement( ((CreateAgentMenuItem)me).getName(), new CreateAgentMenuItem() {
		    public String getName() { return ((CreateAgentMenuItem)me).getName(); }
		    public String getClassName() { return ((CreateAgentMenuItem)me).getClassName(); }
		    public boolean invoke() {
			Vector monitored = getMonitoredCells();
			if( monitored.size() == 0 ) return false;
			else if( monitored.size() == 1 ) return ((CreateAgentMenuItem)me).invoke();
			else {
			    showCreateAgentDialogBox( this.getClassName() );
			    return true;
			}
		    }
		});
	    } else if( me instanceof MenuFolder )
		replaceInvokeCommands( (MenuFolder)me );
	}

    }


    /**
     * method to bring up a dialog box containing which servers we
     * want to shut down 
     */
    public abstract void showShutdownCellDialogBox();

    /**
     * method to bring up a dialog box to start monitoring another cell
     */
    public abstract void showStartMonitorCellDialogBox();

    /**
     * method to bring up a dialog box containing which server we want
     * to stop monitoring 
     */
    public abstract void showStopMonitorCellDialogBox();
 
    /**
     * method to bring up a dialog box to create an agent by its java
     * class name 
     */
    public abstract void showCreateAgentByNameDialogBox();

    /**
     * method to bring up the dialog box with the agent icon and stuff like that
     *
     * @param agent the name of the agent to create -- this has to be correct, am making assumptions at this level
     */
    public abstract void showCreateAgentDialogBox( String agent );

    /**
     * method to bring up the dialog box with the agent icon for movement
     *
     * @param agen the agent to move
     */
    public abstract void showMoveAgentDialogBox( Agent agent );

    /**
     * method to bring up a variable sheet editor
     *
     * @param agent the agent to bring the editor up for
     */
    public abstract void showConfigureAgentDialogBox( Agent agent );

    /**
     * deal with action commands (probably from the agent icon's popup
     * menu) -- this method overrides the one i the BaseUIAgent so we
     * can popup dialog boxes
     *
     * @param a the agent we wish to do the command on
     * @param command a string which is the command 
     */
    public boolean invokeMenuCommand( Agent a, String command ) {

	if( command.equals( moveCmd ) ) {
	    showMoveAgentDialogBox( a );
	    return true;

	} else if( command.equals( propertiesCmd ) ) {
	    showConfigureAgentDialogBox( a );	    
	    return true;
	    
	}

	return super.invokeMenuCommand( a, command );

    }
    
}

