// $Id: BaseSwingGUIAgentImpl.java,v 1.2 2000/05/19 19:55:56 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.hiveui;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.cell.UnableToMonitorCellException;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.IconLoader;
import net.hivecell.hive.support.MalformedHiveURLException;
import net.hivecell.hive.support.UnknownVariablePropertyName;
import net.hivecell.hive.support.VariablePropertyManipulationException;
import net.hivecell.hive.support.VariablePropertyManipulatorGUI;
import net.hivecell.hive.support.menu.TopMenu;
import net.hivecell.hive.support.menu.MenuCheckBoxItem;
import net.hivecell.hive.support.menu.MenuElement;
import net.hivecell.hive.support.menu.MenuFolder;
import net.hivecell.hive.support.menu.MenuItem;
import net.hivecell.hive.support.menu.MenuObject;
import net.hivecell.hive.support.menu.MenuSeparator;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
import java.io.InputStream;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * this class is the next level up on top of the base ui agent -- the
 * purpose of this is so we have a standard way to get the popup boxes
 * that we like 
 *
 * @author Raffi Krikorian <raffi@mit.edu>
 */
public abstract class BaseSwingGUIAgentImpl
extends BaseGUIAgentImpl {

    /** 
     * a handle to the CMS so we don't need to look it up every time.
     */
    protected ComponentManagerShadow cms;

    
    public BaseSwingGUIAgentImpl() throws RemoteException { super(); }
	

    /**
     * do our local setup which just involves letting the BaseUIAgent
     * do its stuff, and then we can lookup our agent component
     * manager shadow
     */  
    public void doLocalSetup() throws AgentInitializationException {

	super.doLocalSetup();

	// get our handle to our agent component manager shadow
	try {
	    cms = getComponentManagerShadow();
	} catch( ShadowNotFoundException error ) {
	    Debug.warning( "this agent cannot start on a cell that does not have a UI.", error );
	    myCell.killAgent( (AgentImpl)this );
	}

    }

    public void doBehavior() { super.doBehavior(); }

    /**
     * a helper function that reads the RDF files and stuff and
     * creates the default menus that we are going to use 
     */
    protected void constructMenus() {
	super.constructMenus();

	// we just need to now implement some of the things that we
	// did not implement in the BaseUIAgent's implementation
	MenuFolder serverMenuFolder = (MenuFolder)menu.getMenuElement( "Cell" );
	serverMenuFolder.replaceMenuElement( "Monitor remote cell", new MenuItem() {
	    public String getName() { return "Monitor remote cell"; }
	    public boolean invoke() {
		showStartMonitorCellDialogBox();
		return true;
	    }
	});
	
	serverMenuFolder.replaceMenuElement( "Stop monitoring remote cell", new MenuItem() {
	    public String getName() { return "Stop monitoring remote cell"; }
	    public boolean invoke() {
		showStopMonitorCellDialogBox();
		return true;
	    }
	});
	
	MenuFolder agentMenuFolder = (MenuFolder)menu.getMenuElementAt( menu.getMenuElementSize() - 1 );
	agentMenuFolder.replaceMenuElement( "Create agent by classname...", new MenuItem() {
	    public String getName() { return "Create agent by classname..."; }
	    public boolean invoke() {
		showCreateAgentByNameDialogBox();
		return true;
	    }
	});

    }

    /**
     * a function that will return an AWT menu that is constructed off
     * the information that is in the menu object
     *
     * @return the awt menu 
     */
    public JMenuBar getMenuBar() {

	JMenuBar mb = new JMenuBar();
	Enumeration e = menu.getMenuItems();
	while( e.hasMoreElements() ) {
	    MenuElement me = (MenuElement)e.nextElement();
	    mb.add( (JMenu)createMenuItem( me ) );
	}
	return mb;

    }

    /**
     * recursive function meant for creating the menu items
     *
     * @param me the menu element that we want to deal wiht
     * @return a menu item to be added
     */
    private JMenuItem createMenuItem( final MenuElement me ) {  
	
	if( me instanceof MenuFolder ) {
	    JMenu m = new JMenu( ((MenuFolder)me).getName() );
	    Enumeration e = ((MenuFolder)me).getMenuItems();
	    while( e.hasMoreElements() ) {
		MenuElement scanElement = (MenuElement)e.nextElement();
		if( scanElement instanceof MenuSeparator) 
		    m.addSeparator();
		else
		    m.add( createMenuItem( scanElement ) );
	    }
	    return m;
	    
	} else if( me instanceof MenuCheckBoxItem ) {
	    JCheckBoxMenuItem cbmi = new JCheckBoxMenuItem( ((MenuCheckBoxItem)me).getName(), ((MenuCheckBoxItem)me).getState() );
	    cbmi.addChangeListener( new ChangeListener() {
		public void stateChanged( ChangeEvent ce ) {
		    ((MenuCheckBoxItem)me).setState( ((JCheckBoxMenuItem)ce.getSource()).isSelected() );
		    ((MenuCheckBoxItem)me).invoke();
		}
	    });
	    return cbmi;

	} else if( me instanceof MenuItem ) {
	    JMenuItem mi = new JMenuItem( ((MenuItem)me).getName() );
	    mi.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent ae ) {
		    ((MenuItem)me).invoke();
		}
	    });
	    return mi;

	}

	return null;

    }

    /**
     * method to bring up a dialog box containing which servers we
     * want to shut down 
     */
    public void showShutdownCellDialogBox() { 

	// if we have more than one server to choose from, then bring
	// up that dialog box -- otherwise just shut down the only
	// server we are monitoring
	if( cellControlAgents.size() > 1 ) {
	    new BaseSwingGUIShutDownCellUI( this, cms );

	} else if( cellControlAgents.size() == 1 ) {
	    Enumeration e = cellControlAgents.keys();
	    shutdownCellCommand( (CellAddress)e.nextElement() );

	}

    }

    /**
     * method to bring up a dialog box to start monitoring another cell
     */
    public void showStartMonitorCellDialogBox() { new BaseSwingGUIMonitorCellUI( this, cms ); }

    /**
     * method to bring up a dialog box containing which server we want
     * to stop monitoring 
     */
    public void showStopMonitorCellDialogBox() { new BaseSwingGUIStopMonitorCellUI( this, cms); }
 
    /**
     * method to bring up a dialog box to create an agent by its java
     * class name 
     */
    public void showCreateAgentByNameDialogBox() { new BaseSwingGUICreateAgentByNameUI( this, cms); }

    /**
     * method to bring up the dialog box with the agent icon and stuff like that
     *
     * @param agent the name of the agent to create -- this has to be correct, am making assumptions at this level
     */
    public void showCreateAgentDialogBox( String agent ) { new BaseSwingGUICreateAgentUI(  this, cms, agent ); }

    /**
     * method to bring up the dialog box with the agent icon for movement
     *
     * @param agen the agent to move
     */
    public void showMoveAgentDialogBox( Agent agent ) { new BaseSwingGUIMoveAgentUI(  this, cms, agent ); }

    /**
     * method to bring up a variable sheet editor
     *
     * @param agent the agent to bring the editor up for
     */
    public void showConfigureAgentDialogBox( Agent agent ) { 
	try {
	    new VariablePropertyManipulatorGUI( cms, agent ); 
	} catch( VariablePropertyManipulationException error ) {
	    Debug.error( "error setting up a property manipulator for agent", error );
	}
    }

}

/**
 * an abstract class so we can display a text area, get the text
 * area and then do something with that string 
 */
abstract class BaseSwingGUITextManipulationUI {

    protected BaseSwingGUIAgentImpl guiAgent;
    protected ComponentManagerShadow cms;

    private Frame myFrame;
    private JPanel myPanel;

    private JPanel panel;
    private JTextField textField;

    public BaseSwingGUITextManipulationUI( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms ) {

	this.guiAgent = guiAgent;
	this.cms = cms;
	    
	myPanel = new JPanel( new BorderLayout() );

	panel = new JPanel();
	panel.add( new JLabel( getTextLabel() ) );
	panel.add( textField = new JTextField( getInitialLabel(), 35 ) );
	textField.addActionListener( new MonitorCell() );
	    
	myPanel.add( panel, "Center" );
	    
	JPanel buttonPanel = new JPanel( new BorderLayout() );
	JButton button = new JButton( getButtonLabel() );
	button.addActionListener( new MonitorCell() );
	buttonPanel.add( button, "Center" );
	myPanel.add( buttonPanel, "South" );

	this.myFrame = cms.acceptComponentFrom( guiAgent, this.myPanel, getTitle() );

    }

    public abstract String getInitialLabel();
    public abstract String getTitle();
    public abstract String getTextLabel();
    public abstract String getButtonLabel();
    public abstract void doCommand( String string );

    class MonitorCell 
	implements ActionListener {
	public void actionPerformed( ActionEvent ae ) { 
	    doCommand( textField.getText().trim() ); 
	    cms.removeComponentFrom( guiAgent, BaseSwingGUITextManipulationUI.this.myPanel );
	}
    }

}
    
/**
 * craete a dialog box that will cause the agent to start
 * monitoring another cell 
 */
class BaseSwingGUIMonitorCellUI 
extends BaseSwingGUITextManipulationUI {

    public BaseSwingGUIMonitorCellUI( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms ) { super( guiAgent, cms ); }

    public String getInitialLabel() { return ""; }
    public String getTitle() { return "Monitor server"; }
    public String getTextLabel() { return "Cell to monitor:"; }
    public String getButtonLabel() { return "Monitor"; }
    public void doCommand( String string ) {
	String cellAddress = string.trim();
	if( !cellAddress.startsWith( "hive://" ) )
	    cellAddress = "hive://" + cellAddress;
	try {
	    guiAgent.monitorCell( new CellAddress( cellAddress ) );
	} catch( MalformedHiveURLException error ) {
	    Debug.notice( "unable to parse celladdress." );
	} catch( UnableToMonitorCellException error ) {
	    Debug.notice( "unable to monitor cell at " + cellAddress, error );
	}
    }

}
    
/**
 * create a dialog box that will cause an agent to be created on
 * the local cell by its class name 
 */
class BaseSwingGUICreateAgentByNameUI
extends BaseSwingGUITextManipulationUI {

    public BaseSwingGUICreateAgentByNameUI( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms ) { super( guiAgent, cms ); }

    public String getInitialLabel() { return ""; }
    public String getTitle() { return "Create agent"; }
    public String getTextLabel() { return "Agent to create: "; }
    public String getButtonLabel() { return "Create"; }
    public void doCommand( String string ) {
	try {
	    guiAgent.createNewAgentCommand( string, guiAgent.getCellAddress() );

	} catch( ClassNotFoundException error ) {
	    Debug.error( "class \"" + string + "\" not found" );

	} catch( InstantiationException error ) {
	    Debug.error( "class \"" + string + "\" could not be created" );

	}

    }

}

/**
 * this class pops up a frame that has the servers listed in a choice
 * field and then a button -- whatever extends this class can set what
 * the title of the frame is, what the button label says, and the
 * action performed when the button is pushed.  right now this class
 * is used for the shut down server and the stop monitoring server
 * boxes 
 */
abstract class BaseSwingGUIDisplayCellListButtonFrame
implements ActionListener {

    protected BaseSwingGUIAgentImpl guiAgent;
    protected ComponentManagerShadow cms;

    private Frame myFrame; // the frame we are going to use
    private JPanel myPanel; // the panel we are sitting in

    /**
     * the actual menu that will be filled with the choices
     */
    protected JComboBox serverListChoice;
    
    public BaseSwingGUIDisplayCellListButtonFrame( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms ) {

	this.guiAgent = guiAgent;
	this.cms = cms;

	this.myPanel = new JPanel( new BorderLayout() );

	JPanel serverListPanel = new JPanel();
	serverListPanel.add( new JLabel( "Cell Address: " ) );
	
	// get a list of the servers we are monitoring
	this.serverListChoice = new JComboBox();
	this.serverListChoice.setEditable( false );
	Enumeration e = guiAgent.getMonitoredCells().elements();
	while( e.hasMoreElements() )
	    this.serverListChoice.addItem( ((CellAddress)e.nextElement()).toString() );
	this.serverListChoice.setSelectedItem( guiAgent.getCellAddress().toString() );
        
	serverListPanel.add( this.serverListChoice );
	this.myPanel.add( "Center", serverListPanel );
	
	JPanel serverListButtonPanel = new JPanel( new BorderLayout() );
	JButton serverListButton = new JButton( this.getButtonLabel() );
	serverListButton.addActionListener( this );
	serverListButtonPanel.add( serverListButton, "Center" );
	this.myPanel.add( "South", serverListButtonPanel );

	this.myFrame = cms.acceptComponentFrom( guiAgent, this.myPanel, this.getTitle() );
    }

    protected abstract String getButtonLabel();
    protected abstract String getTitle();
    protected abstract void buttonPressed();

    public void actionPerformed( ActionEvent e ) {
	this.buttonPressed();
	cms.removeComponentFrom( guiAgent, this.myPanel );
    }

}

/**
 * our class to bring up a dialog box to ask the ui to kill a server 
 */
class BaseSwingGUIShutDownCellUI
extends BaseSwingGUIDisplayCellListButtonFrame {

    public BaseSwingGUIShutDownCellUI( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms ) { super( guiAgent, cms); }

    protected String getButtonLabel() { return "Shut Down"; }
    protected String getTitle() { return "Shut down server..."; }

    protected void buttonPressed() {
	String argument = (String)this.serverListChoice.getSelectedItem();
	try {
	    guiAgent.shutdownCellCommand( new CellAddress( argument ) );
	} catch( MalformedHiveURLException e ) {
	    Debug.error( "Unparseable CellAddress: " + argument );
	}
    }

}
    
/**
 * our class to bring up a dialog box to ask the server to stop
 * monitoring another server 
 */
class BaseSwingGUIStopMonitorCellUI
extends BaseSwingGUIDisplayCellListButtonFrame {

    public BaseSwingGUIStopMonitorCellUI( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms ) { super( guiAgent, cms); }

    protected String getButtonLabel() { return "Stop Monitoring Cell"; }
    protected String getTitle() { return "Stop Monitoring server..."; }

    protected void buttonPressed() {
	String argument = (String)this.serverListChoice.getSelectedItem();
	try {
	    guiAgent.stopMonitoringCell( new CellAddress( argument ) );
	} catch( MalformedHiveURLException e ) {
	    Debug.error( "Unparsable CellAddress: " + argument );
	}
    }

}

/**
 * our class to bring up a dialog box showing the agent icon and
 * stuff like that -- this is used for creating new agents or
 * moving agents 
 */
abstract class BaseSwingGUIAgentIconDisplayPanel
implements ActionListener {

    protected BaseSwingGUIAgentImpl guiAgent;
    protected ComponentManagerShadow cms;

    protected String myAgentName;
    protected Image myAgentImage = null;

    protected JPanel myPanel;
    protected JComboBox serverListChoice;

    public BaseSwingGUIAgentIconDisplayPanel( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms, Image agentImage ) { this( guiAgent, cms, null, agentImage ); }

    public BaseSwingGUIAgentIconDisplayPanel( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms, String agentName ) { this( guiAgent, cms, agentName, null ); }

    public BaseSwingGUIAgentIconDisplayPanel( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms, String agentName, Image agentImage ) {

	this.guiAgent = guiAgent;
	this.cms = cms;

	this.myAgentName = agentName;
	this.myAgentImage = agentImage;

	createDisplayPanel();

    }

    protected void createDisplayPanel() {

	this.myPanel = new JPanel( new BorderLayout() );

	Canvas c = new AgentIconCanvas();
	c.setSize( this.myPanel.getSize().width, 50 );
	this.myPanel.add( "North", c );

	JPanel serverListButtonPanel = new JPanel( new BorderLayout() );
	// get a list of the servers we are monitoring
	this.serverListChoice = new JComboBox();
	this.serverListChoice.setEditable( false );
	
	Enumeration e = guiAgent.monitoredCells.elements();
	while( e.hasMoreElements() ) {
	    String host = ((CellAddress)e.nextElement()).toString();
	    this.serverListChoice.addItem( host );
	}

	JPanel serverListPanel = new JPanel();
	serverListPanel.add( new JLabel( "Cell address: " ) );
	serverListPanel.add( this.serverListChoice );
	serverListButtonPanel.add( "Center", serverListPanel );

	// create the done button
	JButton finishedButton = new JButton( getButtonLabel() );
	finishedButton.addActionListener( this );
	serverListButtonPanel.add( "South", finishedButton );
	this.myPanel.add( "Center", serverListButtonPanel );

	Frame f = cms.acceptComponentFrom( guiAgent, this.myPanel, getFrameTitle() );

    }

    protected abstract String getButtonLabel();
    protected abstract String getFrameTitle();
    protected abstract void buttonPressed();

    public void actionPerformed( ActionEvent event ) {
	cms.removeComponentFrom( guiAgent, this.myPanel );
	buttonPressed();
    }

    class AgentIconCanvas
    extends Canvas
    implements ImageObserver {

	private Image myImage = myAgentImage;

	public void paint( Graphics g ) {
		
	    if( myImage == null ) {
		try {
		    myImage = IconLoader.loadIcon( Class.forName( myAgentName ) ).getImage();
		} catch( ClassNotFoundException error ) {
		    Debug.critical( "huh?  now this should never happen!", error );
		}
	    }

	    int x = ( this.getSize().width - this.myImage.getWidth( this ) ) / 2;
	    int y = ( this.getSize().height - this.myImage.getHeight( this ) ) / 2;
	    g.drawImage( myImage, x, y, this );
	}
	public boolean imageUpdate( Image img, int infoflags, int x, int y, int width, int height ) {
	    if( ( infoflags & ImageObserver.ALLBITS ) != 0 ) {
		repaint();
		this.doLayout();
		return false;
	    }
	    return true;
	}
    }

}

/**
 * use this class to create a new agent on a specific cell
 */
class BaseSwingGUICreateAgentUI
extends BaseSwingGUIAgentIconDisplayPanel {

    private String className;

    public BaseSwingGUICreateAgentUI( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms, String className ) {

	super( guiAgent, cms, className );
	this.className = className;

    }
 
    protected String getButtonLabel() { return "Create Agent"; }
    protected String getFrameTitle() { return "Create Agent..."; }
    protected void buttonPressed() {
	String argument = (String)serverListChoice.getSelectedItem();
	try {
	    guiAgent.createNewAgentCommand( this.className, new CellAddress( argument ) );
	} catch( ClassNotFoundException error ) {
	    Debug.critical( "huh?  this should never happen!", error );

	} catch( InstantiationException error ) {
	    Debug.critical( "huh?  this should never happen!", error );

	} catch( MalformedHiveURLException error ) {
	    Debug.error( "Unparseable CellAddress: " + argument );

	}
    }
}

/**
 * use this class to move an agent to another cell
 */
class BaseSwingGUIMoveAgentUI 
extends BaseSwingGUIAgentIconDisplayPanel {

    private Agent a;

    public BaseSwingGUIMoveAgentUI( BaseSwingGUIAgentImpl guiAgent, ComponentManagerShadow cms, Agent a ) {

	super( guiAgent, cms, guiAgent.getAgentIcon( a ).getImage() );
	this.a = a;

    }
 
    protected String getButtonLabel() { return "Move Agent"; }
    protected String getFrameTitle() { return "Move Agent..."; }
    protected void buttonPressed() {
	String argument = (String)serverListChoice.getSelectedItem();
	try {
	    guiAgent.moveAgentCommand( this.a, new CellAddress( argument ) );
	} catch( MalformedHiveURLException error ) {
	    Debug.error( "unparseable celladdress: " + argument );

	}
    }
}
