// $Id: BaseUIAgentImpl.java,v 1.2 2000/05/19 19:55:56 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.hiveui;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent;
import net.hivecell.hive.agent.cell.CellControlAgent;
import net.hivecell.hive.agent.cell.UnknownAgentException;
import net.hivecell.hive.agent.cell.UnableToMonitorCellException;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.event.cell.*;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.description.HiveXML;  // NOTE: NEED TO REDO THE MENU STUFF LATER!
import net.hivecell.hive.cell.AgentDeliveryException;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.support.AgentGraph;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.PPM;
import net.hivecell.hive.support.SerializableImage;
import net.hivecell.hive.support.VariablePropertyManipulationException;
import net.hivecell.hive.support.VariablePropertyManipulator;
import net.hivecell.hive.support.UnknownVariablePropertyName;
import net.hivecell.hive.support.graph.Edge;
import net.hivecell.hive.support.graph.NotInGraphException;
import net.hivecell.hive.support.menu.TopMenu;
import net.hivecell.hive.support.menu.MenuElement;
import net.hivecell.hive.support.menu.MenuFolder;
import net.hivecell.hive.support.menu.MenuItem;
import net.hivecell.hive.support.menu.MenuObject;
import net.hivecell.hive.support.menu.MenuSeparator;

import java.awt.Dimension;
import java.lang.reflect.InvocationTargetException;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.rmi.RemoteException;

import org.w3c.dom.*;
import com.docuverse.dom.*;


/**
 * the standard base class for all ui agents -- this class will handle
 * the minimum amount of state maintainance, and it will also handles
 * a lot of the communications to the agent monitoring agents 
 *
 * todo list -- the ability to monitor less cells than the agent
 * monitoring agent is monitoring -- this would require that we filter
 * the information the agent monitoring agent is sending and we need
 * to keep track of exactly who we are monitoring
 *
 * @author Raffi Krikorian <raffi@mit.edu> 
 */
public abstract class BaseUIAgentImpl
extends EventReceivingAgentImpl {

    protected AgentMonitoringAgent ama; // the handle to our agent monitoring agent
    protected Hashtable cellControlAgents; // the cell control agents we need access to

    /**
     * our copy of the agent graph -- this graph is constructed from
     * the differential information that we get from the agent
     * monitoring agent -- do NOT modify this graph -- please only read
     * it 
     */
    protected AgentGraph ag;

    protected Vector monitoredCells;

    // a record mapping agents to their agent addresses -- this is
    // useful especially when we stop monitoring a cell, we no longer
    // have a way of determining the address of these agents that were
    // removed -- so we use this
    protected Hashtable agentAddresses; 

    /**
     * a mapping between agents and their BeanManipulators -- this way
     * we do not need to go through reflection all those times 
     */
    protected Hashtable variablePropertyManipulators;

    // this is where the representation of the menu is going to be
    // held
    protected TopMenu menu; 
    

    public BaseUIAgentImpl() 
    throws RemoteException {

	super();
	cellControlAgents = new Hashtable();
	monitoredCells = new Vector();
	agentAddresses = new Hashtable();
	variablePropertyManipulators = new Hashtable();

    }

    /**
     * Returns the cell the uiAgent is running on.
     */
    public CellAddress getCellAddress() { return myCell.getAddress(); }

    /**
     * start the monitoring of our agent monitoring agent -- this will
     * get our handle to it, and it will then start up an event stream
     * to it.  right now it displays all the information the agnet
     * monitoring agent has -- so if somebody else asks the agent
     * monitoring agent to monitor another cell, all that information
     * will be pushed through to the display 
     */
    public synchronized void doLocalSetup() throws AgentInitializationException {

	super.doLocalSetup();

	
	// connect to the local agent monitoring agent
	ama = null;
	try {
	    String[] amaName = { (net.hivecell.hive.agent.cell.AgentMonitoringAgent.class).getName() };
	    AgentSet ds = myCell.queryAgents( this, amaName, null );
	    ama = (AgentMonitoringAgent)ds.pickOne();
	    if( ama == null ) {
		Debug.error( "no agent monitoring agent on local cell -- this agent requires one to run" );
		myCell.killAgent( (AgentImpl)this );
		return;
	    }
	} catch( ClassNotFoundException error ) {
	    Debug.critical( "huh?  this should never happen", error );
	}

	// get our cell's cell control agent
	try {
	    String[] ccaName = { (net.hivecell.hive.agent.cell.CellControlAgent.class).getName() };
	    AgentSet ds = myCell.queryAgents( this, ccaName, null );
	    if( ds.size() == 0 ) {
		Debug.error( "no cell control agent on local cell -- this agent requires one to run" );
		myCell.killAgent( (AgentImpl)this );
		return;
	    } else 
		cellControlAgents.put( myCell.getAddress(), ds.pickOne() ); 
	} catch( ClassNotFoundException error ) {
	    Debug.error( "huh?  this should never happen", error );
	}
	
	// set the monitored cells to be our local cell and ask for
	// the pertinant agent graph
	try {
	    CellAddress[] monitoredCellsArray = { myCell.getAddress() };
	    myCell.noteAgentMessage( this, "addListener", this, ama );
	    ama.addListener( this, monitoredCellsArray );
	    agentSubscriptions.addElement( ama );
	    myCell.noteAgentConnect( this, this, ama );

	    monitoredCells.addElement( myCell.getAddress() );

	    AgentSet allAgents = ama.queryAgents( this, monitoredCellsArray, null, null );

	    // get all the agents into the graph
	    Enumeration e = allAgents.elements();
	    ag = new AgentGraph();
	    while( e.hasMoreElements() ) {
		Agent a = (Agent)e.nextElement();
		ag.addAgent( a );
		agentAddresses.put( a, getAgentAddress( a ) );

	    }

	    // now make all the appropriate connections
	    e = ag.listAllAgents();
	    while( e.hasMoreElements() ) {
		Agent a = (Agent)e.nextElement();
		Vector incoming = listIncomingConnections( a );
		if( incoming != null ) {
		    Enumeration incomings = incoming.elements();
		    while( incomings.hasMoreElements() ) {
			Agent incomingAgent = (Agent)incomings.nextElement();
			if( ag.containsAgent( incomingAgent ) )
			    ag.connectAgents( incomingAgent, a );
			
		    }

		}

		Vector outgoing = listOutgoingConnections( a );
		if( outgoing != null ) {
		    Enumeration outgoings = outgoing.elements();
		    while( outgoings.hasMoreElements() ) {
			Agent outgoingAgent = (Agent)outgoings.nextElement();
			if( ag.containsAgent( outgoingAgent ) )
			    ag.connectAgents( a, outgoingAgent );
			
		    }

		}		    

	    }

	    // at this point the initial state of teh graph has not
	    // been called -- make sure to create it as only
	    // incremental updates will occur between now and then

	} catch( RemoteException error ) {
	    Debug.warning( "remote exception while trying to get a list of monitored cells and the initial graph of agents from the agent monitoirng agent -- shutting down", error );
	    myCell.killAgent( (AgentImpl)this );

	} catch( ClassNotFoundException error ) {

	}

	constructMenus();

    }
 
    /**
     * a helper function that reads the RDF files and stuff and
     * creates the default menus that we are going to use 
     */
    protected void constructMenus() {
	
	menu = new TopMenu();
	
	// make the UI menu first
	MenuFolder uiMenuFolder = new MenuFolder( "UI" );
	uiMenuFolder.addMenuElement( new MenuItem() {
	    public String getName() { return "Show all debug messages"; }
	    public boolean invoke() { 
		Debug.setDefaultLevel( Debug.NOTICE );
		Debug.notice( "now showing all debugging messages." );
		return true;
	    }
	});
	
	uiMenuFolder.addMenuElement( new MenuItem() {
	    public String getName() { return "Show only error debug messages"; }
	    public boolean invoke() {
		Debug.notice( "now showing only error debugging messages." );
		Debug.setDefaultLevel( Debug.WARNING );
		return true;
	    }
	});

	uiMenuFolder.addMenuElement( new MenuSeparator() );

	uiMenuFolder.addMenuElement( new MenuItem() {
	    public String getName() { return "Show Hive credits"; }
	    public boolean invoke() {
		try {
		    createNewAgentCommand( "net.hivecell.hive.agent.desktop.CreditsAgentImpl", myCell.getAddress() ); 
		    return true;

		} catch( ClassNotFoundException error ) {
		    Debug.warning( "unable to find the class for the credits agent", error );
		    return false;

		} catch( InstantiationException error ) {
		    Debug.warning( "unable to create the class for the credits agent", error );
		    return false;

		}

	    }

	});
	
	uiMenuFolder.addMenuElement( new MenuSeparator() );
	
	uiMenuFolder.addMenuElement( new MenuItem() {
	    public String getName() { return "Quit interface"; }
	    public boolean invoke() { 
		myCell.killAgent( (AgentImpl)BaseUIAgentImpl.this );
		return true;
	    }
	});
	    
	menu.addMenuElement( uiMenuFolder );
 
	
	// now make the cell menu
	MenuFolder serverMenuFolder = new MenuFolder( "Cell" );
	serverMenuFolder.addMenuElement( new MenuItem() {
	    public String getName() { return "Monitor remote cell"; }
	    public boolean invoke() {
		Debug.notice( "monitor remote cell is not implemented" );
		return true; 
	    }
	});

	serverMenuFolder.addMenuElement( new MenuItem() {
	    public String getName() { return "Stop monitoring remote cell"; }
	    public boolean invoke() {
		Debug.notice( "stop monitoring remote cell is not implemented" );
		return true;
	    }
	});

	serverMenuFolder.addMenuElement( new MenuSeparator() );

	serverMenuFolder.addMenuElement( new MenuItem() {
	    public String getName() { return "Kill cell"; }
	    public boolean invoke() {
		if( !shutdownCellCommand( myCell.getAddress() ) )
		    return false;
		return true;
	    }
	});
	
	menu.addMenuElement( serverMenuFolder );


	// now create the create agent menu
	//String fs = System.getProperty( "file.separator" );
	//String personalRdfConfigFile = System.getProperty( "user.home" ) + fs + ".hive" + fs + "menu" + fs + "uimenu.rdf";
	InputStream baseIS = null;   // Base hive menus
	InputStream appsIS = null;   // Application menus
	InputStream userIS = null;   // User's menus
	
	MenuFolder baseMenu = null;
	MenuFolder appsMenu = null;
	MenuFolder userMenu = null;


	// Open apps menus, and create them
	appsIS = (BaseUIAgentImpl.class).getResourceAsStream("xml/AppsAgentMenu.xml");
	if (appsIS == null) {
	    Debug.warning( "Application agent menus file '<apps>/agent/hiveui/xml/AppsAgentMenu.xml' could not be opened." );
	} else appsMenu = makeMenu( appsIS, "<apps>/agent/hiveui/xml/AppsAgentMenu.xml" );

	// Open base menus, and create them
	baseIS = (BaseUIAgentImpl.class).getResourceAsStream( "xml/BaseAgentMenu.xml" );
	if (baseIS == null) {
	    Debug.warning( "Base agent menus file './xml/BaseAgentMenu.xml' could not be opened." );
	} else baseMenu = makeMenu( baseIS, "./xml/BaseAgentMenu.xml" );


	try {   // Open user's personal menus, and create them
	    userIS = new FileInputStream( Global.getMenuDir() + "UserAgentMenu.xml" );
	} catch( IOException e ) {
	    Debug.notice( "Personalized agent menus file '" + Global.getMenuDir() + "UserAgentMenu.xml' could not be opened." );
	}
	if (userIS != null) userMenu = makeMenu( userIS, Global.getMenuDir() + "UserAgentMenu.xml" );


	// Merge the menus together
	MenuFolder standardMenu = MenuFolder.mergeMenus(appsMenu, baseMenu);
	MenuFolder completeAgentMenu = MenuFolder.mergeMenus(standardMenu, userMenu);


	
	completeAgentMenu.insertMenuElementAt( new MenuItem() {
	    public String getName() { return "Create agent by classname..."; }
	    public boolean invoke() {
		Debug.notice( "create agent by classname is not implemented" );
		return true;
	    }
	}, 0 );
	completeAgentMenu.insertMenuElementAt( new MenuSeparator(), 1 );

	menu.addMenuElement( completeAgentMenu );
	
    }

    /**
     * helper function to make a menu from a filename
     *
     * @param filename the name of the file
     * @return MenuElement needed 
     */
    private MenuFolder makeMenu( String filename ) {
	InputStream is = (BaseUIAgentImpl.class).getResourceAsStream( filename );  // First look relative to the hiveui dir.
	if( is == null ) {
	    is = (BaseUIAgentImpl.class).getResourceAsStream( "xml/" + filename ); // If that fails, try tacking 'xml' onto the beginning

	    if (is == null) {
		try {
		    is = new FileInputStream( filename );                // If that also fails, treat the filename as absolute.
		} catch( IOException e ) {
		    Debug.error( "file \"" + filename  + "\" or \"xml/" + filename + "\" not found.  menu not created.", e );
		}
	    }
	}
	return( makeMenu( is, filename ) );
    }

    /**
     * helper function to create a menu from the input stream
     *
     * @param is the input stream to create the stuff from
     * @return the MenuElement that makes the create agent menu
     */
    private MenuFolder makeMenu( InputStream is, String fname ) {
	if( is != null ) {
	    HiveXML xml = new HiveXML(is);
	    Node root = xml.getRoot();

	    return ( makeMenu( HiveXML.getFirstNonWhitespaceChild(root) ) );  // Get the first child

	} else return null;
    }


    /**
     * another helper function to create an agent menu from a Node
     *
     * @param node the DOM node to make the menu from
     * @return the menu element that we can build the menus from, or a menuSeparator if the XML was invalid.
     */
    
    private MenuFolder makeMenu( Node node ) { 	
	// First, get make sure the top level node is a menu node.

	MenuFolder toReturn = null;
	String mName = node.getNodeName();

	if (mName.equals("menu")) {
	    NamedNodeMap attrib = node.getAttributes();
	    NodeList kids = node.getChildNodes();

	    String mString = attrib.item(0).getNodeValue();
	    toReturn = new MenuFolder( mString );

	    for (int i = 0; i < kids.getLength(); i++) {
		Node kid = kids.item(i);
		String kname = kid.getNodeName();
		
		// If the node is named "agent"
		if (kname.equals("agent")) {
		    NamedNodeMap aattrib = kid.getAttributes();
		    final String menuString = stripWhiteSpace( ( HiveXML.getFirstNonWhitespaceChild(kid) ).getNodeValue() );
		    NodeList aanl = HiveXML.select(aattrib, "class");
		    Node cnameNode = null;
		    if (aanl != null)
			cnameNode = aanl.item(0);  // Can only be one tag named class in the attributes.

		    if (cnameNode == null) {
			Debug.warning("Malformed HiveXML: couldn't find class name node for '" + menuString + "' in '" + kname + "'");
			toReturn.addMenuElement( new MenuSeparator() );
		    }
		    final String className = cnameNode.getNodeValue();


		    // Make an agent selection with name value of 1st child node and class the value of the property named "class"
		    toReturn.addMenuElement( new CreateAgentMenuItem() {
			public String getName() { return menuString; }
			public String getClassName() { return className; }
			public boolean invoke() {
			    try {
				createNewAgentCommand( className, myCell.getAddress() );
				return true;

			    } catch( ClassNotFoundException error ) {
				Debug.error( "\"" + className + "\" not found.  Cannot be created.", error );
				return false;

			    } catch( InstantiationException error ) {
				Debug.error( "\"" + className + "\" found but cannot be created.", error );
				return false;

			    }

			}
		    });
		}
		
		// If the node is named "hr", insert a separator
		if (kname.equals("hr"))
		    toReturn.addMenuElement( new MenuSeparator() );
		

		// If the node is named "menu"
		if (kname.equals("menu")) {
		    
		    NamedNodeMap mattrib = kid.getAttributes();
		    
		    Node nameNode = null, typeNode = null;
		    NodeList manl = HiveXML.select(mattrib, "name");
		    if (manl != null)
			nameNode = manl.item(0);  // Can only be one tag named "name" in the attributes.
		    manl = HiveXML.select(mattrib, "type");
		    if (manl != null)
			typeNode = manl.item(0);  // Can only be one tag named "type" in the attributes.

		    if (nameNode == null && typeNode == null) {
			Debug.warning("Malformed HiveXML: couldn't find a type or name property for menu '" + kid.getNodeName() + "'");
			toReturn.addMenuElement( new MenuSeparator() );
		    }

		    // if it has property with name "type"
		    if (typeNode != null) {
			// if property named "type" has value "file"
			if (typeNode.getNodeValue().equals("file")) {
			    Node fnameNode = HiveXML.getFirstNonWhitespaceChild(kid);
			    if (fnameNode == null) {
				Debug.warning("Malformed HiveXML: File not specified for menu node '" + kid.getNodeName() + "'");
				toReturn.addMenuElement( new MenuSeparator() );
			    }

			    // recurse with value of 1st child node (which is the filename)
			    String fname = stripWhiteSpace(fnameNode.getNodeValue());
			    toReturn.addMenuElement( makeMenu( fname ) );
			}
		    }

		    // if it has property with name "name" (meaning there will be a submenu specified directly, not in a separate file)
		    if (nameNode != null) {
			// make a new MenuFolder with name the value of property "name."
			toReturn.addMenuElement( makeMenu(kid) );
		    }
		}
	    }
	}

	else {
	    Debug.warning("First Node in list needs to be a menu node (<menu>).  Name was " + mName);
	    toReturn.addMenuElement( new MenuSeparator() );					
	}

	return toReturn;
    }

    private String stripWhiteSpace( String raw ) {
	int pos = 0;
	while( pos < raw.length() && 
	       ( raw.charAt( pos ) == ' ' || 
		 raw.charAt( pos ) == '\n' || 
		 raw.charAt( pos ) == '\t') ) 
	    pos++;
	return( raw.substring( pos ).trim() );

    }

    public void doBehavior() { waitUntilDeath(); }

    /**
     * handle the event stream as it comes in
     */
    public synchronized void notify( HiveEvent event ) {

	// next off -- make sure this is a server event
	if( event instanceof CellEvent ) {

	    // make sure this event has to do with one of the servers that
	    // we care about
	    CellEvent se = (CellEvent)event;
	    
	    boolean properEvent = false;
	    Enumeration e = monitoredCells.elements();
	    while( e.hasMoreElements() )
		if( se.matchesCellAddress( (CellAddress)e.nextElement() ) ) {
		    properEvent = true;
		    break;
		}
	    if( !properEvent ) {
		Debug.warning( "got an event concerning a server we are not monitoring" );
		return;
	    }

	    // if the server event is something we have to deal with --
	    // then do it
	    if( ( se.eventCode() & ( CellEvent.AGENTBORN | 
				     CellEvent.AGENTDYING |
				     CellEvent.AGENTMOVINGTO |
				     CellEvent.AGENTMOVINGFROM |
				     CellEvent.AGENTSCONNECT |
				     CellEvent.AGENTSDISCONNECT |
				     CellEvent.AGENTDISCONNECTALL |
				     CellEvent.AGENTMESSAGE |
				     CellEvent.SERVERGOINGDOWN |
				     CellEvent.SERVERREPORT ) ) != 0 ) {

		if( se instanceof AgentBornCellEvent )
		    addAgent( ((AgentBornCellEvent)se).getAgent() );

		else if( se instanceof AgentDeathCellEvent )
		    removeAgent( ((AgentDeathCellEvent)se).getAgent() );

		else if( se instanceof AgentMovingFromCellEvent )
		    addAgent( ((AgentMovingFromCellEvent)se).getAgent() );

		else if( se instanceof AgentMovingToCellEvent )
		    removeAgent( ((AgentMovingToCellEvent)se).getAgent() );
	    
		else if( se instanceof AgentConnectCellEvent )
		    connectAgents( ((AgentConnectCellEvent)se).getSourceAgent(), ((AgentConnectCellEvent)se).getDestinationAgent() );

		else if( se instanceof AgentDisconnectCellEvent )
		    disconnectAgents( ((AgentDisconnectCellEvent)se).getSourceAgent(), ((AgentDisconnectCellEvent)se).getDestinationAgent() );

		else if( se instanceof AgentDisconnectFromAllCellEvent )
		    disconnectAgentFromAll( ((AgentDisconnectFromAllCellEvent)se).getAgent() );

		else if( se instanceof AgentMessageCellEvent ) 
		    renderAgentMessage( ((AgentMessageCellEvent)se).getSourceAgent(), ((AgentMessageCellEvent)se).getDestinationAgent() );

		else if( se instanceof CellGoingDownCellEvent )
		    stopMonitoringCell( ((CellGoingDownCellEvent)se).getAddress() );

		else if( se instanceof CellReportCellEvent  )
		    renderCellReport( (CellReportCellEvent)se );

	    }
	}
    }
    
    /**
     * get rid of all our active references
     */
    public void doLocalCleanup() {
 
	super.doLocalCleanup();

	disconnectFrom( ama );

    }


    /**
     * START HANDLE RENDERING COMMANDS
     */

    /**
     * this method is used internally to add another agent to the
     * graph and then to cause the UI to update itself because this
     * new agent has come onto the scene
     *
     * @param a the agent that we are adding to the graph 
     */
    private synchronized void addAgent( Agent a ) {
	
	if( ag.containsAgent( a ) ) return;
	ag.addAgent( a );
	agentAddresses.put( a, getAgentAddress( a ) );

	renderNewAgent( a );

    }

    /**
     * a method which needs to be implemented -- this means that
     * another agent has arrived on the scene and it needs to be drawn
     * to the UI
     *  
     * @param a the new agent to be rendered 
     */
    public abstract void renderNewAgent( Agent a );

    /**
     * this method is used internally to remove an agent from the
     * agent graph and then to cause the UI to update itself because
     * this new agent has come onto the scene 
     * 
     * @param a the agent that we are removing from the graph
     */
    private synchronized void removeAgent( Agent a ) {
	
	Agent[] incomingAgents = null;
	Agent[] outgoingAgents = null;
	try {
	    incomingAgents = ag.listIncomingConnections( a );
	    outgoingAgents = ag.listOutgoingConnections( a );
	} catch( NotInGraphException error ) {
	    Debug.error( "asked to remove an agent that is not in our graph" );
	    return;
	}
	ag.removeAgent( a );

	CellAddress ca = (CellAddress)agentAddresses.remove( a );
	renderRemoveAgent( a, ca, incomingAgents, outgoingAgents );

    }

    /**
     * a method which needs to be implemented -- this means that an
     * agent has been removed from the scene and this change needs to
     * be rendered.  this method also includes a list of agents that
     * this agent was connected to so we can remove those connections
     * also
     *
     * @param a the agent that we are removing from the scene
     * @param ca the cell address of the agent we are removing
     * @param incoming the agents that are pointing to this agent
     * @param outgoing the agents that this agent is pointing to 
     */
    public abstract void renderRemoveAgent( Agent a, CellAddress ca, Agent[] incoming, Agent[] outgoing );

    /**
     * this method is used internally so we can create a connection
     * between two agents
     *
     * @param s the source of the connection
     * @param d the destination of the connection 
     */
    private synchronized void connectAgents( Agent s, Agent d ) {

	if( !ag.containsAgent( s ) ) {
	    addAgent( s );
	    agentAddresses.put( s, getAgentAddress( s ) );
	}
	if( !ag.containsAgent( d ) ) {
	    addAgent( d );
	    agentAddresses.put( d, getAgentAddress( d ) );
	}
	if( ag.isConnected( s, d ) )
	    return;

	ag.connectAgents( s, d );
	renderConnectAgents( s, d );

    }
    
    /**
     * a method which needs to be implemented -- when two agents
     * connect, this method will be called.  the two agents should
     * already be on the graph -- two agents should not need to be
     * connected if they are not on the graph yet
     *
     * @param s the source agent
     * @param d the destination agent 
     */
    public abstract void renderConnectAgents( Agent s, Agent d );

    /**
     * this is an internal method to handle the disconnection of two agents
     *
     * @param s the source agent in the connection
     * @param d the destiantion agent in the connection
     */
    private synchronized void disconnectAgents( Agent s, Agent d ) {
	if( ag.disconnectAgents( s, d ) ) {
	
	    renderDisconnectAgents( s, d );  

	    // if this connection was made to another cell that is not
	    // part of the cells we are monitoring, then we should
	    // just remove it
	    if( !monitoredCells.contains( ((CellAddress)agentAddresses.get( s )) ) )
		removeAgent( s );
	    if( !monitoredCells.contains( ((CellAddress)agentAddresses.get( d )) ) )
		removeAgent( d );

	}

    }

    /**
     * a method which needs to be implemented -- when two agents
     * disconnect, this method will be called to display the change to
     * the screen
     *
     * @param s the source of the disconnection
     * @param d the destination of the disconnection 
     */
    public abstract void renderDisconnectAgents( Agent s, Agent d );

    /**
     * this is another internal method to handle when an agent
     * disconnects from all other agents -- this is not implemented as
     * just a series of calls to disconnectAgents as an optimization
     *
     * @param a the agent that is disconnecting from everything 
     */
    private synchronized void disconnectAgentFromAll( Agent a ) {

	if( !ag.containsAgent( a ) )
	    return;

	Agent[] incoming = null;
	Agent[] outgoing = null;
	try {
	    incoming = ag.listIncomingConnections( a );
	    outgoing = ag.listOutgoingConnections( a );
	} catch( NotInGraphException error ) {
	    Debug.error( "huh?  this shouldn't happen -- there must be corruption in our data structures" );
	}
	if( ag.disconnectFromAll( a ) )
	    renderDisconnectFromAll( a, incoming, outgoing );

	for( int count=0;count < incoming.length;count++ )
	    if( !monitoredCells.contains( ((CellAddress)agentAddresses.get( incoming[count] )) ) )
		removeAgent( incoming[count] );
	for( int count=0;count < outgoing.length;count++ )
	    if( !monitoredCells.contains( ((CellAddress)agentAddresses.get( outgoing[count] )) ) )
		removeAgent( outgoing[count] );

    }

    /**
     * this method needs to be implemented for when an agent disconnects from all
     *
     * @param a the agent that is disconnecting from all
     * @param incoming all the agents that are connected to this agnet
     * @param outgoing all the agnets that are connected to by this agent
     */
    public abstract void renderDisconnectFromAll( Agent a, Agent[] incoming, Agent[] outgoing );

    /**
     * this method needs to be implemented so agent messaging can be drawn
     *
     * @param s the source of the message
     * @param d the destination of the message
     * @param message the message
     */
    public abstract void renderAgentMessage( Agent s, Agent d );

    /**
     * this method needs to be implemented so the server reports go somwhere
     *
     * @param message the server message
     */
    public abstract void renderCellReport( CellReportCellEvent report );

    /**
     * STOP HANDLE RENDERING COMMANDS
     */


    /**
     * START HANDLING MONITORING AND STOP MONITORING CELL
     */

    /**
     * get a vector of all the cells that we are monitoring.  this
     * simply returns a vector we keep around saying which cells we
     * are monitoring
     *
     * @return an approximate list of all the cells we are monitoring 
     */
    public Vector getMonitoredCells() { return monitoredCells; }

    /**
     * start monitoring another cell.  the way this works is that when
     * you call this method we will ask the agent monitoring agent to
     * start monitoring anther cell.  we will also request the server
     * control agent from that cell and squirrel it away so we can use
     * it
     *
     * @param ca the cell address that we wish to monitor 
     * @exception throws UnableToMonitorCellException when it can't monitor the other cell
     */
    public synchronized void monitorCell( CellAddress ca ) 
    throws UnableToMonitorCellException {

	// first ask the agent monitoring agent to monitor the cell we wish to monitor
	try {
	    ama.monitorCell( this, ca );
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to ask the ama to monitor " + ca, error );
	    throw new UnableToMonitorCellException( "remote exception while trying to ask the ama to monitor " + ca, error );
	}

	// get a hold of the server control agent for that cell
	try {
	    CellAddress[] addresses = { ca };
	    String[] classNames = { (net.hivecell.hive.agent.cell.CellControlAgent.class).getName() };
	    AgentSet ds = ama.queryAgents( this, addresses, classNames, null );
	    CellControlAgent sca = (CellControlAgent)ds.pickOne();
	    if( sca == null )
		Debug.error( "there is no server control agent on " + ca );
	    else
		cellControlAgents.put( ca, sca );
	} catch( ClassNotFoundException error ) {
	    Debug.error( "huh?  this should never be thrown!", error );
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to ask the ama for the server control agent on " + ca, error );
	    try {
		Debug.notice( "trying to tell the ama to stop monitoring " + ca );
		ama.stopMonitoringCell( this, ca );
	    } catch( RemoteException wrappedError ) {
		Debug.notice( "remote exception while trying to get the ama to stop monitoring " + ca );
	    }
	}

	// reconnect to the ama asking to monitor stuff plus this new address
	monitoredCells.addElement( ca );
	try {
	    CellAddress[] monitoredCellsArray = new CellAddress[monitoredCells.size()];
	    monitoredCells.copyInto( monitoredCellsArray );
	    ama.addListener( this, monitoredCellsArray );

	    AgentSet allAgents = ama.queryAgents( this, monitoredCellsArray, null, null );

	    // get all the agents into the graph
	    Enumeration e = allAgents.elements();
	    AgentGraph newAg = new AgentGraph();
	    while( e.hasMoreElements() )
		newAg.addAgent( (Agent)e.nextElement() );

	    // now make all the appropriate connections
	    e = newAg.listAllAgents();
	    while( e.hasMoreElements() ) {
		Agent a = (Agent)e.nextElement();
		try {
		    AgentSet incoming = ama.listIncomingAgentConnections( this, a );
		    if( incoming != null ) {
			Enumeration incomings = incoming.elements();
			while( incomings.hasMoreElements() ) {
			    Agent incomingAgent = (Agent)incomings.nextElement();
			    if( newAg.containsAgent( incomingAgent ) ) {
				connectAgents(incomingAgent, a);
			    }

			}

		    }

		    AgentSet outgoing = ama.listOutgoingAgentConnections( this, a );
		    if( outgoing != null ) {
			Enumeration outgoings = outgoing.elements();
			while( outgoings.hasMoreElements() ) {
			    Agent outgoingAgent = (Agent)outgoings.nextElement();
			    if( newAg.containsAgent( outgoingAgent ) ) {
				connectAgents( a, outgoingAgent );
			    }
			    
			}

		    }

		} catch( UnknownAgentException error ) {
		    Debug.error("Caught an Unknown Agent Exception: " + a.getName() + ", " + a.getAddress(), error);
		}

	    }

	    Agent[] addedAgents = this.ag.getNewAgents( newAg );
	    for( int count=0;count < addedAgents.length;count++ )
		addAgent( addedAgents[count] );
	    Edge[] addedEdges = this.ag.getNewEdges( newAg );
	    for( int count=0;count < addedEdges.length;count++ )
		connectAgents( (Agent)addedEdges[count].getSource(), (Agent)addedEdges[count].getDestination() );
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to get the agent graph from the ama", error );

	    // a better failure case needs to be taken care of here --
	    // perhaps we can put this information in a queue or
	    // something so later we can come back and attempt to get
	    // the full refresh?
	    
	    monitoredCells.removeElement( ca );

	} catch( ClassNotFoundException error ) {
	    Debug.error("Caught a class not found exception: ", error);
	    
	}

    }
    
    /** 
     * stop monitoring another cell.  this just goes through and asks
     * the ama to stop monitoring a certain cell, and it gets rid of
     * the references to the server control agents
     * 
     * @param ca the cell address to stop monitoring 
     */
    public synchronized void stopMonitoringCell( CellAddress ca ) {

	// first ask the ama to stop monitoring the address
	try {
	    ama.stopMonitoringCell( this, ca );
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to ask the ama to stop monitoring " + ca );
	}

	// remove the server control agent reference 
	cellControlAgents.remove( ca );

	// get another copy of the agent graph so we can render it
	monitoredCells.removeElement( ca );
	try {
	    CellAddress[] monitoredCellsArray = new CellAddress[monitoredCells.size()];
	    monitoredCells.copyInto( monitoredCellsArray );
	    ama.addListener( this, monitoredCellsArray );

	    AgentSet allAgents = ama.queryAgents( this, monitoredCellsArray, null, null );

	    // get all the agents into the graph
	    Enumeration e = allAgents.elements();
	    AgentGraph newAg = new AgentGraph();
	    while( e.hasMoreElements() )
		newAg.addAgent( (Agent)e.nextElement() );

	    // now make all the appropriate connections
	    e = newAg.listAllAgents();
	    while( e.hasMoreElements() ) {
		Agent a = (Agent)e.nextElement();
		try {
		    AgentSet incoming = ama.listIncomingAgentConnections( this, a );
		    if( incoming != null ) {
			Enumeration incomings = incoming.elements();
			while( incomings.hasMoreElements() ) {
			    Agent incomingAgent = (Agent)incomings.nextElement();
			    if( ag.containsAgent( incomingAgent ) )
				ag.connectAgents( incomingAgent, a );
			    
			}

		    }
			
		    AgentSet outgoing = ama.listOutgoingAgentConnections( this, a );
		    if( outgoing != null ) {
			Enumeration outgoings = outgoing.elements();
			while( outgoings.hasMoreElements() ) {
			    Agent outgoingAgent = (Agent)outgoings.nextElement();
			    if( ag.containsAgent( outgoingAgent ) )
				ag.connectAgents( a, outgoingAgent );
			    
			}

		    }

		} catch( UnknownAgentException error ) {
		    // this own't happen

		}

	    }

	    // figure out which edges and which agents we have to
	    // remove from our drawing
	    Edge[] removedEdges = this.ag.getRemovedEdges( newAg );
	    for( int count=0;count < removedEdges.length;count++ ) 
		disconnectAgents( (Agent)removedEdges[count].getSource(), (Agent)removedEdges[count].getDestination() );
	    Agent[] removedAgents = this.ag.getRemovedAgents( newAg );
	    for( int count=0;count < removedAgents.length;count++ )
		removeAgent( removedAgents[count] );

	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to get the agent graph from the ama", error );

	    // a better failure case needs to be taken care of here --
	    // perhaps we can put this information in a queue or
	    // something so later we can come back and attempt to get
	    // the full refresh?

	} catch( ClassNotFoundException error ) {
	    // not going to happen

	}

    }

    /**
     * STOP HANDLING MONITORING AND STOP MONITORING CELL
     */


    /**
     * START HANDLING AGENT ACTION COMMANDS
     */

    /**
     * a helper function which will determine the names of the menu
     * items that should go on the popup menu for each agent
     *
     * @param a the agent that we want popup menu for
     * @return an array of strings for the menu 
     */
    public static final String disconnectAllCmd = "Disconnect from all agents";
    public static final String propertiesCmd = "Properties...";
    public static final String connectCmd = "Connect this agent";
    public static final String disconnectCmd = "Disconnect agents";
    public static final String moveCmd = "Move this agent";
    public static final String killCmd = "Kill this agent";
    public static final String[] defaultCmds = { disconnectAllCmd, propertiesCmd, connectCmd, disconnectCmd, moveCmd, killCmd }; 
    public String[] getMenuNames( Agent a ) {

	// perhaps this should do some caching???

	Vector customAgentNames = null;
	try {
	    customAgentNames = a.getActionCommands();
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to get the custom menu names for " + a, error );
	    Debug.error( "remote exception while trying to get the custom menu names for " + a);
	    customAgentNames = new Vector();

	    // soemthing has gone wrong talking to this agent -- is
	    // this agent alive?  has the cell gone down?
	    CellAddress ca = null;
	    if( ( ( ca = (CellAddress)agentAddresses.get( a ) ) != null ) && !pingAddress( ca ) )
		stopMonitoringCell( ca );
	}

	String[] menuNames = new String[ defaultCmds.length + customAgentNames.size() ];
	System.arraycopy( defaultCmds, 0, menuNames, 0, defaultCmds.length );
	String[] customNames = new String[ customAgentNames.size() ];
	customAgentNames.copyInto( customNames );
	System.arraycopy( customNames, 0, menuNames, defaultCmds.length, customNames.length);
	return menuNames;

    }

    /**
     * a helper function to ask the agent to deal with a command
     * (these are the commands that were returned out of the popup
     * menu)
     *
     * @param a the agent that needs to deal
     * @param command the string to deal with 
     * @return true if the command was parsed and dealt with, false otherwise
     */
    public boolean invokeMenuCommand( Agent a, String command ) {

	if( command.equals( disconnectAllCmd ) ) 
	    return disconnectAgentAllCommand( a );

	else if( command.equals( propertiesCmd ) ) {
	    // this is not yet implemented -- this is where the remote
	    // property sheets would go
	    Debug.notice( "the properties command is not implemented" );
	    return false;

	} else if( command.equals( connectCmd ) ) {
	    // this is also not yet implemented
	    Debug.notice( "the connect command is not implemented" );
	    return false;

	} else if( command.equals( disconnectCmd ) ) {
	    Debug.notice( "the disconnect command us not implemented" );
	    return false;

	} else if( command.equals( moveCmd ) ) {
	    Debug.notice( "the move command is not implemented" );
	    return false;

	} else if( command.equals( killCmd ) )
	    return killAgentCommand( a );

	try {
	    if( !a.invokeActionCommand( command ) ) {
		Debug.warning( "nothing in the command chain can deal with \"" + command + "\"" );
		return false;
	    }
	    return true;
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to ask the agent to perform \"" + command + "\"" );
	    return false;

	}

    }

    /**
     * STOP HANDLING AGENT ACTION COMMANDS
     */


    
    /**
     * START HANDLING CELL ACTION COMMANDS
     */

    /** The menu strings for the popup CellList menu */
    public static final String changeMonitoringCellCmd = "Start/Stop monitoring this cell";
    public static final String changeShowCellAgentsCmd = "Start/Stop displaying agents on this cell";
    public static final String killCellCmd = "Kill this cell";
    public static final String[] defaultCellCmds = { changeMonitoringCellCmd, changeShowCellAgentsCmd, killCellCmd };
    
    /**
     * a helper function to ask the UI to deal with a command for a cell
     * (these are the commands that were returned out of the cell list popup
     * menu)
     *
     * @param cell the cell that is being affected
     * @param command the string to deal with 
     * @return true if the command was parsed and dealt with, false otherwise
     */
    public boolean invokeCellMenuCommand( CellAddress cell, String command ) {
	if (command.equals( changeMonitoringCellCmd ) ) {
	    if (monitoredCells.contains(cell))
		stopMonitoringCell(cell);
	    else {
		try {
		    monitorCell( cell );
		} catch( UnableToMonitorCellException error ) {
		    Debug.notice( "unable to monitor cell at " + cell.toString(), error );
		}
	    }
	    return (true);
	} else if (command.equals( changeShowCellAgentsCmd ) ) {
	    Debug.notice( "The show cell agents command is not yet implemented" );
	    return (false);
	} else if (command.equals( killCellCmd ) ) {
	    shutdownCellCommand(cell);
	    return (true);
	} else {
	    Debug.warning( "This command, '" + command + "' not recognized." );
	    return (false);
	}
    }

    /**
     * STOP HANDLING CELL ACTION COMMANDS
     */



    /**
     * START HANDLING CACHED INFORMATION ABOUT AGENTS
     */

    /** 
     * a helper function that will get the icon for a certain agent --
     * this will go pretty hardcore to get an image.  it will first
     * query the agent monitoring agent for an image, if that fails it
     * will ask the agent itself, and if that fails it will go and get
     * a default icon to use
     *
     * @param a the agent that we want the icon for
     * @return the serializable image for the agent 
     */
    private SerializableImage unknownAgentIcon = null;
    public SerializableImage getAgentIcon( Agent a ) {
	
	// if that failed, ask the agent itself directly for its icon
	try {
	    return a.getIcon();
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to ask the agent (" + a + ") itself for its icon", error );
	}

	// we should of gotten some sort of information at this point
	// -- ping the cell to make sure that the cell the agent is on
	// is up and running -- if it is not, then stop monitoring it
	CellAddress ca = null;
	if( ( ( ca = (CellAddress)agentAddresses.get( a ) ) != null ) && !pingAddress( ca ) )
	    stopMonitoringCell( ca );

	// if that fails -- go to the failsafe icon (UnknownAgent.ppm)
	if( unknownAgentIcon != null )
	    return unknownAgentIcon;
	InputStream is = (AgentImpl.class).getResourceAsStream( "icons/UnknownAgent.ppm" );
	try {
	    unknownAgentIcon = (SerializableImage)(new PPM( is ));
	    return unknownAgentIcon;
	} catch( Exception error ) { 
	    Debug.error( "problem deserializing the icon file (UnknownAgent.ppm)" );
	}

	// ok -- if we got this far, then we are really screwed, just return a black square
	return new SerializableImage( new int[ 32 * 32 ], new Dimension( 32, 32 ) );

    }

    /**
     * another helper function -- this one gets the name of the agent
     * so it can display it
     * 
     * @param a the agent to get the name of 
     * @return the name to display 
     */
    public String getAgentName( Agent a ) {

	// query the agent directly for its name
	try {
	    return a.getName();
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to query the agent (" + a + ") for its name", error );
	}

	// if we are resorting to this, then we need to make sure the
	// cell is still up and running
	CellAddress ca = null;
	if( ( ( ca = (CellAddress)agentAddresses.get( a ) ) != null ) && !pingAddress( ca ) )
	    stopMonitoringCell( ca );

	// just return the short string for this agent
	return Global.shortString( a );

    }

    /**
     * another helper to find out what cell this agent is sitting on
     * -- check if this returns null!  the worst case scenario will
     * have this return null as the cell address.  this will only
     * happen in the case that the ama does not know this agent's cell
     * address, and we can't talk to the agent
     *
     * @param a the agent to get the address of
     * @return the cell address it is sitting on 
     */
    public CellAddress getAgentAddress( Agent a ) {
	
	// query the agent monitoring agent for the address of this agnet
	try {
	    return ama.getCellAddress( this, a );
	} catch( UnknownAgentException error ) {
	    // Debug.error( "the agent monitoring agent does not have this agent (" + a + ") in its cache", error );
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to ask the agent monitoring agent for the address of " + a, error );
	}

	// query the agent directly for its cell address
	try {
	    return a.getCell().getAddress();
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to ask the agent for its cell address", error );
	}

	// worst case scenario
	Debug.error( "cannot get the cell address for " + a + "!" );

	// something has really gone wrong at this point -- look into
	// our hashtable to see if we have the address of this agent,
	// if we do, ping its cell and if that returns false then we
	// need to stop monitoring that cell
	CellAddress ca = null;
	if( ( ( ca = (CellAddress)agentAddresses.get( a ) ) != null ) && !pingAddress( ca ) )
	    stopMonitoringCell( ca );

	return null;

    }

    public Vector listIncomingConnections( Agent a ) {
	try {
	    AgentSet incoming = ama.listIncomingAgentConnections( this, a );
	    Enumeration e = incoming.elements();
	    Vector incomingEdges = new Vector();
	    while( e.hasMoreElements() )
		incomingEdges.addElement( e.nextElement() );
	    return incomingEdges;

	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to determining incoming connections to " + a, error );
	    return new Vector();

	} catch( UnknownAgentException error ) {
	    return null;
	    
	}

    }

    public Vector listOutgoingConnections( Agent a ) {
	try {
	    AgentSet outgoing = ama.listOutgoingAgentConnections( this, a );
	    Enumeration e = outgoing.elements();
	    Vector outgoingEdges = new Vector();
	    while( e.hasMoreElements() )
		outgoingEdges.addElement( e.nextElement() );
	    return outgoingEdges;

	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to determining outgoing connections to " + a, error );
	    return new Vector();

	} catch( UnknownAgentException error ) {
	    return null;
	    
	}

    }
    
    /**
     * STOP HANDLING CACHED INFORMATION ABOUT AGENTS
     */


    /**
     * START HANDLING UI TYPE COMMANDS
     */

    /**
     * Call this to shutdown a server.  
     *
     * @param ca the cell address of a server to shutdown
     * @return true if the command was sent through, false otherwise
     */
    public synchronized boolean shutdownCellCommand( CellAddress ca ) {

	// if we don't have a handle to the server control agent on
	// the other server, then just give up
	if( !cellControlAgents.containsKey( ca ) )
	    return false;
	
	CellControlAgent sca = (CellControlAgent)cellControlAgents.get( ca );
	try {
	    sca.shutDownCell();
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to shut down the cell at " + ca );
	    return false;
	}

	if( !ca.equals( myCell.getAddress() ) ) {
	    // we need to see a new graph of the agents at this point
	    try {
		AgentSet allAgents = ama.queryAgents( this, null, null, null );

		// get all the agents into the graph
		Enumeration e = allAgents.elements();
		AgentGraph newAg = new AgentGraph();
		while( e.hasMoreElements() )
		    newAg.addAgent( (Agent)e.nextElement() );

		// now make all the appropriate connections
		e = newAg.listAllAgents();
		while( e.hasMoreElements() ) {
		    Agent a = (Agent)e.nextElement();
		    try {
			AgentSet incoming = ama.listIncomingAgentConnections( this, a );
			if( incoming != null ) {
			    Enumeration incomings = incoming.elements();
			    while( incomings.hasMoreElements() ) {
				Agent incomingAgent = (Agent)incomings.nextElement();
				if( ag.containsAgent( incomingAgent ) )
				    ag.connectAgents( incomingAgent, a );
			    
			    }

			}
			
			AgentSet outgoing = ama.listOutgoingAgentConnections( this, a );
			if( outgoing != null ) {
			    Enumeration outgoings = outgoing.elements();
			    while( outgoings.hasMoreElements() ) {
				Agent outgoingAgent = (Agent)outgoings.nextElement();
				if( ag.containsAgent( outgoingAgent ) )
				    ag.connectAgents( a, outgoingAgent );
			    
			    }

			}

		    } catch( UnknownAgentException error ) {
			// this own't happen

		    }

		}

		// figure out which edges and which agents we have to
		// remove from our drawing
		Edge[] removedEdges = this.ag.getRemovedEdges( newAg );
		for( int count=0;count < removedEdges.length;count++ ) 
		    disconnectAgents( (Agent)removedEdges[count].getSource(), (Agent)removedEdges[count].getDestination() );
		Agent[] removedAgents = this.ag.getRemovedAgents( newAg );
		for( int count=0;count < removedAgents.length;count++ )
		    removeAgent( removedAgents[count] );

		return true;
	    } catch( RemoteException error ) {
		Debug.error( "remote exception while trying to get the agent graph from the ama", error );

	    // a better failure case needs to be taken care of here --
	    // perhaps we can put this information in a queue or
	    // something so later we can come back and attempt to get
	    // the full refresh?
		return false;
	    } catch( ClassNotFoundException error ) {
		return false;

	    }
	} else {

	    diePlease();
	    myCell.killAgent( (AgentImpl)this );
	    return true;

	}

    }

    /**
     * Call this to create an agent on a particular server
     *
     * @param name the name of the class we wish to create
     * @param ca the address of the cell that we want to create it on
     * @return true ifthe command went through, false otherwise
     * @exception class not found exception is thrown if the class does not exist
     * @exception instantiation exception if cannot create the agent
     */
    public boolean createNewAgentCommand( String name, CellAddress ca ) 
    throws ClassNotFoundException, InstantiationException {

	// if we don't have a handle to the server control agent on
	// the other server, then just give up
	if( !cellControlAgents.containsKey( ca ) )
	    return false;

	CellControlAgent sca = (CellControlAgent)cellControlAgents.get( ca );
	try {
	    sca.createAgent( name );
	    return true;

	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to create " + name + " on " + ca, error );
	    
	    // something went wrong while we are trying to create an
	    // agent on the other remote cell -- ping that cell, and
	    // if it is not responding, then stop monitoring it
	    if( !pingAddress( ca ) )
		stopMonitoringCell( ca );

	    return false;

	}

    }

    /**
     * Call this to kill an agent 
     *
     * @param a the agent that we want to kill
     * @return false if the command does not go through, true otherwise
     */
    public boolean killAgentCommand( Agent a ) {
	
	// first find out what cell this agent is on
	CellAddress ca = getAgentAddress( a );
	if( !cellControlAgents.containsKey( ca ) )
	    return false;

	CellControlAgent sca = (CellControlAgent)cellControlAgents.get( ca );
	try {
	    sca.killAgent( a );
	    return true;
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to kill " + a, error );

	    // something went wrong while trying to kill this agent --
	    // ping the cell that the cell control agent is on, and if
	    // that cell is not responding, then stop monitoring it
	    if( !pingAddress( ca ) )
		stopMonitoringCell( ca );

	    return false;
	}

    }

    /**
     * Call this to move an agent from one server to another one
     *
     * @param a the agent that we wish to move
     * @param ca the destination cell address
     * @return true if the command goes through
     */
    public boolean moveAgentCommand( Agent a, CellAddress ca ) {

	// first find out what cell this agent is on
	CellAddress sourceAddress = getAgentAddress( a );
	if( !cellControlAgents.containsKey( sourceAddress ) )
	    return false;

	CellControlAgent sca = (CellControlAgent)cellControlAgents.get( sourceAddress );
	try {
	    sca.moveAgent( a, ca );
	    return true;
	} catch( AgentDeliveryException error ) {
	    Debug.error( "agent delivery exception trying to move " + a + " to " + ca, error );

	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to move " + a + " to " + ca, error );

	} finally {

	    // we shouldn't be here unless an exception was thrown -
	    // if an exception was thrown, then the agent was not
	    // moved, if the agent was not moved, we need to ping the
	    // cell that the cell control agent is on, if the ping
	    // fails then we need to stop monitoring that cell

	    if( !pingAddress( ca ) ) 
		stopMonitoringCell( ca );

	    return false;

	}
	
    }

    /**
     * Call this to connect two agents together
     *
     * @param s the agent that we want to start the connection on
     * @param d the agent that we want to stop the connection at
     * @return true if the command goes through, false otherwise
     */
    public boolean connectAgentsCommand( Agent s, Agent d ) {
	
	// find out what cell the first agent is on
	CellAddress ca = getAgentAddress( s );
	if( !cellControlAgents.containsKey( ca ) )
	    return false;

	CellControlAgent sca = (CellControlAgent)cellControlAgents.get( ca );
	try {
	    sca.connectAgents( s, d );
	    return true;
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to connect " + s + " and " + d );

	    // something went wrong while trying to connect these two
	    // agents -- ping the cell the cell control agent is on,
	    // and if the ping doesn't work, then stop monitoring that
	    // cell
	    if( !pingAddress( ca ) ) 
		stopMonitoringCell( ca );

	    return false;
	}
	

    }
    
    /**
     * Call this to disconnect two agents
     *
     * @param s the agent that we want to start the disconnection
     * @param d the agent that we want to stop
     * @return true if the command goes through, false otherwise
     */
    public boolean disconnectAgentsCommand( Agent s, Agent d ) {
	
	// find out what cell the first agent is on
	CellAddress ca = getAgentAddress( s );
	if( !cellControlAgents.containsKey( ca ) )
	    return false;

	CellControlAgent sca = (CellControlAgent)cellControlAgents.get( ca );
	try {
	    sca.disconnectAgents( s, d );
	    return true;
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to disconnect " + s + " and " + d );

	    // something went wrong while trying to disconnect these
	    // agents -- stop monitoring the cell that the control
	    // agent is on
	    if( !pingAddress( ca ) )
		stopMonitoringCell( ca );

	    return false;
	}
	

    }
    
    /**
     * Call this to disconnect all the agents
     *
     * @param a the agent that we want to start the disconnection
     * @return true if the command goes through, false otherwise
     */
    public boolean disconnectAgentAllCommand( Agent a ) {
	
	// find out what cell the first agent is on
	CellAddress ca = getAgentAddress( a );
	if( !cellControlAgents.containsKey( ca ) )
	    return false;

	CellControlAgent sca = (CellControlAgent)cellControlAgents.get( ca );
	try {
	    sca.disconnectAgentAll( a );
	    return true;
	} catch( RemoteException error ) {
	    // Debug.error( "remote exception while trying to disconnect " + a + " from all", error );
	    
	    // something went wrong when trying to disconnect this
	    // agent from all -- lets ping its cell, and if we return
	    // false, then we should stop monitoring this cell
	    if( !pingAddress( ca ) )
		stopMonitoringCell( ca );

	    return false;
	}

    }

    /**
     * STOP HANDLING UI TYPE COMMANDS
     */

    
    /**
     * START BEAN CONFIGURATION OF AGENTS
     */
    
    /**
     * helper function to get or create a variable property manipulator object
     *
     * @param a the agent that we want the variable property manipulator for
     * @return the bean manipulator object, or null if something went wrong.  if null, then exit whatever function calling from fast
     */
    private VariablePropertyManipulator getVariablePropertyManipulator( Agent a ) {

	VariablePropertyManipulator bm = (VariablePropertyManipulator)variablePropertyManipulators.get( a );
	if( bm == null ) {
	    try {
		bm = new VariablePropertyManipulator( a );
		variablePropertyManipulators.put( a, bm );
	    } catch( VariablePropertyManipulationException error ) {

		// make sure that this whole cell just did not go offline
		CellAddress ca = null;
		if( ( ( ca = (CellAddress)agentAddresses.get( a ) ) != null ) && !pingAddress( ca ) )
		    stopMonitoringCell( ca );

		return null;

	    }
	}
	return bm;

    }

    /**
     * get the variable names for the agent configuration stuff
     *
     * @param a the agent that we want this bean information from
     * @return an array of strings which is the variable names
     */
    public String[] getAccessibleVariableNames( Agent a ) {

	VariablePropertyManipulator bm = getVariablePropertyManipulator( a );
	if( bm == null ) {
	    String[] emptyString = { };
	    return emptyString;
	}

	return bm.getAccessibleVariableNames();

    }

    /**
     * get the variable type for a certain varible in an agent
     *
     * @param a the agent that we want the information from
     * @param variableName the name of the variable that we want the information from
     * @return a class object which is the type of this variable, null if something went wrong
     * @exception UnknownVariablePropertyName if the variable name is not known of
     */
    public Class getAccessibleVariableType( Agent a, String variableName )
    throws UnknownVariablePropertyName {
	
	VariablePropertyManipulator bm = getVariablePropertyManipulator( a );
	if( bm == null ) return null;

	return bm.getAccessibleVariableType( variableName );

    }

    /**
     * get the variable inside a certain agent
     *
     * @param a the agent that we want to get the variable for
     * @param s the variable that we want to get
     * @return the value of the variable or null if something went wrong
     * @exception UnknownVariablePropertyName if the variable that we weant to set is not part of the variables we can set
     */
    public Object getAccessibleVariable( Agent a, String variableName ) 
    throws UnknownVariablePropertyName {

	VariablePropertyManipulator bm = getVariablePropertyManipulator( a );
	if( bm == null ) return null;

	try {
	    return bm.getAccessibleVariable( variableName );
	} catch( VariablePropertyManipulationException error ) {

	    Debug.error( error );

	    // we want to figure out if it was some sort of remote
	    // exception, if it was then we need to ping the host to
	    // make sure that it is still up
	    Exception innerError = error.getException();
	    if( innerError instanceof InvocationTargetException ) {
		InvocationTargetException ite = (InvocationTargetException)innerError;
		Throwable t = ite.getTargetException();

		if( t instanceof RemoteException ) {
		    // make sure that this whole cell just did not go offline
		    CellAddress ca = null;
		    if( ( ( ca = (CellAddress)agentAddresses.get( a ) ) != null ) && !pingAddress( ca ) )
			stopMonitoringCell( ca );

		}

	    }
	    return null;
	    
	}

    }

    /**
     * set a variable for a certain agent
     *
     * @param a the agent that we want to set the variable for
     * @param s the variable that we want to set
     * @param o the object that we are going to set the variable to
     * @return the previous value of the variable or null if something went wrong
     * @exception UnknownVariablePropertyName if the variable that we want to set is not part of the variables we can set 
     */
    public Object setAccessibleVariable( Agent a, String variableName, Object o ) 
    throws UnknownVariablePropertyName {

	VariablePropertyManipulator bm = getVariablePropertyManipulator( a );
	if( bm == null ) return null;

	try {
	    return bm.setAccessibleVariable( variableName, o );
	} catch( VariablePropertyManipulationException error ) {

	    // we want to figure out if it was some sort of remote
	    // exception, if it was then we need to ping the host to
	    // make sure that it is still up
	    Exception innerError = error.getException();
	    if( innerError instanceof InvocationTargetException ) {
		InvocationTargetException ite = (InvocationTargetException)innerError;
		Throwable t = ite.getTargetException();

		if( t instanceof RemoteException ) {
		    // make sure that this whole cell just did not go offline
		    CellAddress ca = null;
		    if( ( ( ca = (CellAddress)agentAddresses.get( a ) ) != null ) && !pingAddress( ca ) )
			stopMonitoringCell( ca );

		}

	    }
	    return null;
	    
	}

    }
    
    /**
     * STOP BEAN CONFIGURATION OF AGENTS
     */
    
  
    /**
     * a helper function for ourselves to ping a cell.  this is useful
     * to see if a cell unexpectidly went down -- and if it did, then
     * we need to stop monitoring it
     *
     * @param ca the cell address that we should test
     * @return true if the cell is up, false if anything went wrong
     * with the ping 
     */
    private boolean pingAddress( CellAddress ca ) {
	// first get the remote server handle
	RemoteCell rs = null;
	try {
	    rs = ca.getRemoteCell();
	} catch( CellConnectException error ) {
	    // we could not convert the cell address to the remote
	    // server!  something went wrong.
	    return false;
	}

	// now try pinging the remote server
	try {
	    return rs.ping();
	} catch( RemoteException error ) {
	    // for some reason could not execute the ping on the
	    // remote server
	    return false;
	}
    }

}
