// $Id: CellListCanvas.java,v 1.1 2000/05/18 22:00:53 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/**
 ** Panel showing the cells being monitored and this information about them:
 ** - uptime
 ** - colored border legend
 ** - name
 ** - displaying icons or not (can be set)
 **
 **
 ** NOTE: This currently does not display the uptimes of remote cells.  This is due
 ** to the fact that the CellListAgentImpl relies on a centralized "master" cell to
 ** give it updates every so often (this is currently pinot-noir, which is out of date.
 **
 ** This also desperately needs to be made a little more efficient.
 ** right now it gets a vector of celllistinfo objects every time some
 ** happens -- it needs to be able to deal with more incremental
 ** information to deal with the bandwidth better
 **/
package net.hivecell.hive.agent.hiveui.iconicui;

import net.hivecell.hive.*;
import net.hivecell.hive.agent.hiveui.BaseGUIAgentImpl;
import net.hivecell.hive.agent.hiveui.BaseUIAgentImpl;
import net.hivecell.hive.cell.RemoteCell;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.agent.cell.CellListAgentImpl;
import net.hivecell.hive.agent.cell.CellListInfo;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.MalformedHiveURLException;

import net.hivecell.hive.support.BaseCanvas;

import java.awt.*;
import java.awt.event.*;
import java.util.*;


public class CellListCanvas
extends BaseCanvas 
implements MouseListener, MouseMotionListener {

    protected BaseGUIAgentImpl uiAgent;
    protected Vector monitoredCells;  // The names of all the cells we're monitoring in the order we began to monitor them.
    protected Vector orderedCells;    // All of the cells we know about, with the monitoredCells first, then the unmonitored ones.
    protected Hashtable cellColors;   // Cell name -> border color
    protected Hashtable cellInfo;     // Cell name -> CellListInfo
    protected Hashtable cellRects;    // Cell name -> rectangle in which their data is displayed.
    protected Hashtable shortNames;   // Cell name string -> short name

    protected Dimension rectSize = new Dimension(100, 45);  // Size of the bounding rectangle for the cell info
    protected CellAddress selectedCell;                     // The cell that the mouse clicked on
    protected Color unmonitoredCellColor;                   // The color of a cell that isn't being monitored, but that we know about.
    

    public CellListCanvas( BaseGUIAgentImpl uiAgent ) {
	super();
	setSize(100, 300);

	monitoredCells = new Vector();
	orderedCells = new Vector();
	cellColors = new Hashtable();
	cellInfo = new Hashtable();
	cellRects = new Hashtable();
	shortNames = new Hashtable();

	unmonitoredCellColor = Color.gray;

	this.uiAgent = uiAgent;

	(new Thread( new Runnable() {
	    public void run() {
		while( true ) {
		    try {
			Thread.sleep( 1000 );
		    } catch( InterruptedException error ) { }
		    render();
		}
	    }
	})).start();

	addMouseListener(this);

	setVisible(true);
    }


    /**
     * Create a color that the UI will use to distinguish this
     * cell from the others.
     *
     * @param remoteCellAddress the cell we need the color for
     * @return the color for this cell 
     */
    public Color addMonitoredCell( CellAddress cell ) {
	Color col = null;

	// if a new cell, create a new random color, pastelize it and use it
	col = (Color)cellColors.get(cell);
	if (col == null) {
	    col = createNewColor();
	    col = makePastel( col );
	    cellColors.put( cell, col );
	}

	if (!monitoredCells.contains(cell))
	    monitoredCells.addElement(cell);
	if (cellRects.get(cell) == null)
	    cellRects.put( cell, new Rectangle(rectSize) );

	rearrangeOrdered();
	render(super.getGraphics());

	return (col);
    }


    public void addUnmonitoredCell( CellAddress cell ) {
	cellRects.put( cell, new Rectangle(rectSize) );	
    }

    /**
     * Remove a cell and its appropriate color from the database.
     *
     * @param cell the cell to remove for our database
     */
    public void stopMonitoringCell( CellAddress cell ) { 
	cellColors.remove( cell ); 
	monitoredCells.removeElement( cell );
	cellRects.remove( cell );
	rearrangeOrdered();
    }


    /** Get the color of cell's border. */
    public Color getColor( CellAddress cell ) { return ((Color)cellColors.get(cell)); }
	
    public BaseGUIAgentImpl getUI() { return (uiAgent); }


    /**
     * A "magic" function -- i really don't know how to make colors
     * pastel, but nelson said this might do it :)
     *
     * @param c the color we wish to "pastelize"
     * @return the pastel version of the color we set it 
     */
    private Color makePastel( Color c ) {
	
	// uhm - maybe this makes the colors pastel?  nelson said it
	// would :)
	float[] hsb = Color.RGBtoHSB( c.getRed(), c.getGreen(), c.getBlue(), null );
	hsb[1] = (float)0.4;
	hsb[2] = (float)0.95;
	return Color.getHSBColor( hsb[0], hsb[1], hsb[2] );
	
    }

    /**
     * Do nelson's idea of only having like seven colors, and then we
     * cycle through it -- hopefully we are not looking at more than 7
     * cells 
     */
    private Color createNewColor() {

	Color colors[] = { Color.red, Color.blue, Color.green, Color.white, Color.magenta, Color.black };
	
	int nextColor = ( cellColors.size() % colors.length ) + 1;
	return colors[ nextColor ];

    }


    private void rearrangeOrdered() {

	orderedCells = new Vector();   // Start from scratch each time

	// Go through the cells and arrange them in order: 
	// First are the ones we are monitoring, in the order we began monitoring them
	Enumeration e = monitoredCells.elements();
	while (e.hasMoreElements()) { orderedCells.addElement(e.nextElement()); }
	
	// Next are the ones we know about, but aren't monitoring.
	e = cellInfo.keys();
	while (e.hasMoreElements()) { 
	    CellAddress k = (CellAddress)e.nextElement();
	    if (!monitoredCells.contains(k))
		orderedCells.addElement(k);
	}
	
    }

    /** Called by the CellListAgentImpl in its update loop,  */
    public void newCellInfo(Vector v) {    // V is a vector of CellListInfo types.

	Enumeration e = v.elements();
	while (e.hasMoreElements()) {
	    CellListInfo cli = (CellListInfo)e.nextElement();
	    if ( !cellInfo.containsKey(cli.getAddress()) ) {
		cellInfo.put(cli.getAddress(), cli);
		addUnmonitoredCell(cli.getAddress());
		rearrangeOrdered();
	    }
	}	
	render( getGraphics() );
    }



    // this method is a big hack to get the spacing right.  Needs to be cleaned up later.
    public void render(Graphics g) {
	//	printCOLS();
	if (g == null) return;
	Point pos = new Point(5, 15);

	Color oldCol = g.getColor();
	Color bg = getBackground();
	g.setColor(bg);
	g.fillRect(0, 0, this.getSize().width, this.getSize().height);


	// Now, go through and render them all.
	Enumeration e = orderedCells.elements();
	while (e.hasMoreElements()) {
 	    CellAddress cell = (CellAddress)e.nextElement();

	    ((Rectangle)cellRects.get(cell)).setLocation(pos.x - 5, pos.y - 15);

	    
	    g.setColor( pickTextCol(bg) );

	    // Draw the name of the cell first
	    String name = stripName(cell.toString());
	    g.drawString(name, pos.x, pos.y);

	    pos.y += 5;      // jump down 5
	    // If the agent is being monitored, draw a colored, raised box.
	    if (monitoredCells.contains(cell)) {
		g.setColor((Color)cellColors.get(cell));
		g.fill3DRect(pos.x, pos.y, 15, 15, true);
	    }
	    else {
		g.setColor(unmonitoredCellColor);
		g.fill3DRect(pos.x, pos.y, 15, 15, false);
	    }
	    //  If not, draw a gray sunken one.

	    pos.x += 25;     // Move 10 beyond edge of the rect
	    pos.y += 12;     // Move down 12 pixels from the top edge of the rectangle
	    g.setColor( pickTextCol(bg) );
	    if (cellInfo.containsKey(cell)) {
		CellListInfo tcli = (CellListInfo)cellInfo.get(cell);
		String outString = (tcli.isUp() ? "up" : "down");
		g.drawString( outString, pos.x, pos.y );
	    }
	    
	    pos.y += 30;
	    pos.x = 5;
	}

	g.setColor(oldCol);
    }

    
    /**
     ** This selects a text color that contrasts with bg, so that
     ** text can be read on any color background.
     **
     ** @param bg The color to contrast against.
     **/
    private Color pickTextCol(Color bg) {
	return ( new Color(255 - bg.getRed(), 255 - bg.getGreen(), 255 - bg.getBlue()) );
    }

    
    private String stripName(String raw) {

	if (shortNames.containsKey(raw))
	    return ((String)shortNames.get(raw));
	
	String ret;
	
	// Strip of the "hive://"
	ret = raw.substring(raw.lastIndexOf("/") + 1);

	// Strip off the port number for now.
	int pStart = ret.lastIndexOf(":");
	String port = "";
	if (pStart != -1) {
	    port = ret.substring( pStart );  // Capture the ':'
	    ret = ret.substring( 0, pStart );
	}


	// Get the domain from ret and this domain
	String domain = ret.substring(ret.indexOf(".") + 1);
	String myDomain = uiAgent.getCellAddress().toString();
	// Strip the "hive://"
	myDomain = myDomain.substring(myDomain.lastIndexOf("/") + 1);
	// Strip the port
	int endp = myDomain.lastIndexOf(":");
	if (endp != -1)
	    myDomain = myDomain.substring(myDomain.indexOf(".") + 1, endp);
	else myDomain = myDomain.substring(myDomain.indexOf(".") + 1);


	// if they are in the same domain, don't show it.
	if (myDomain.equals(domain)) {
	    int firstDot = ret.indexOf(".");  // is this a FQDN?
	    if (firstDot > 0)	              // yes, strip out domainname
		ret = ret.substring( 0, ret.indexOf(".") ) + port;
	    else		              // no, just use hostname
		ret = ret + port;
	} else 
	    ret += port;

	shortNames.put(raw, ret);
	return (ret);
    }


    /** Mouse handling code */
    
    // Mouse pressed - start a drag here. (This is where we should put in
    // the click-an-agent-to-edit stuff)
    public void mousePressed(MouseEvent event) {
	
	// Figure out which mouse button it was from the meta/alt flags.
	int mouseButton = -1;
	selectedCell = null;

	if (event.isAltDown()) {
	    mouseButton = 1;
	} else if (event.isMetaDown()) {
	    mouseButton = 2;
	} else {
	    mouseButton = 0;
	}


	// Figure out which icon was pressed.
	Enumeration e = orderedCells.elements();
	while (e.hasMoreElements()) {
	    CellAddress tCell = (CellAddress)e.nextElement();
	    if ( ((Rectangle)cellRects.get(tCell)).contains(event.getX(), event.getY()) ) {
		selectedCell = tCell;
		break;
	    }
	}

	if ( selectedCell == null ) {  	    // Whoops, mouse missed.
	    Debug.notice("There's nothing at (" + event.getX() + ", " + event.getY() + ")");
	    return;

	} else if ( selectedCell != null ) {

	    if (mouseButton == 0) {
	    } else if (mouseButton == 2) {           // Handle Popup menu stuff.

		String cellName = selectedCell.toString();
		PopupMenu pm = new PopupMenu( cellName );
		
		if( Global.windowsOS == true ) {     // hack stupid Windows JDK bug in popups
		    pm.add(cellName);
		    pm.add("-");
		}
		
		
		for (int i = 0; i < uiAgent.defaultCellCmds.length; i++)
		    pm.add(uiAgent.defaultCellCmds[i]);
		
		pm.addActionListener( new ActionListener() { 
		    public void actionPerformed(ActionEvent ae) {
			String command = ae.getActionCommand();/*

			    if (monitoredCells.contains(selectedCell)) {
				addUnmonitoredCell(selectedCell);    // Not being monitored, but still not gone.
				rearrangeOrdered();
			    }
			    //else addMonitoredCell(selectedCell);
			    //rearrangeOrdered();

			} else if (command.equals(BaseUIAgentImpl.changeShowCellAgentsCmd)) {
			    
			} else if (command.equals(BaseUIAgentImpl.killCellCmd)) {
			    
			}
							       */
			uiAgent.invokeCellMenuCommand(selectedCell, command);
			if (command.equals(BaseUIAgentImpl.changeMonitoringCellCmd))
			    rearrangeOrdered();
						    
		    }
		});
		
		add(pm);
		pm.show(this, event.getX(), event.getY());


	    } else if (mouseButton == 1) {                // middle button
	    } else {
		Debug.error("Unknown mouse button.  What the hell is going on?");     // shouldn't reach here.
	    }
	}

    }


    /** We don't care about these events */
    public void mouseReleased(MouseEvent event) {}
    public void mouseDragged(MouseEvent event) {}
    public void mouseClicked(MouseEvent event) {}
    public void mouseEntered(MouseEvent event) {}
    public void mouseExited(MouseEvent event) {}
    public void mouseMoved(MouseEvent event) {}

}
