// $Id: IconicUIAgentImpl.java,v 1.2 2000/05/19 19:55:56 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/**
 ** UIAgent implementation of the basic 
 ** boxes and lines UI for Hive.
 **/

package net.hivecell.hive.agent.hiveui.iconicui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.rmi.RemoteException;
import java.util.*;

//import net.hivecell.hive.rdf.*;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.hiveui.BaseUIAgentImpl;
import net.hivecell.hive.agent.hiveui.BaseAWTGUIAgentImpl;
import net.hivecell.hive.agent.cell.CellControlAgent;
import net.hivecell.hive.agent.cell.CellAgent;
import net.hivecell.hive.agent.cell.CellListAgent;
import net.hivecell.hive.agent.cell.UnableToMonitorCellException;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.event.cell.*;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.support.*;
import net.hivecell.hive.support.graph.*;
import net.hivecell.hive.support.menu.MenuCheckBoxItem;
import net.hivecell.hive.support.menu.MenuFolder;
import net.hivecell.hive.support.menu.MenuSeparator;

/**
  * The UIAgent for a boxes and lines style GUI.
  *
  * BUGS:   - Test this (from uiCanvas): Let the Canvas know it shouldn't start any new renders, as its agent is being killed.
  *                                    This might only be necessary because of the IcUIAg icon hack, test later.
  *         - A monitors B, A goes down, A comes back up, A tries to monitor B again, B throws rmi errors (problem with rmi, 
  *                                    but might be possible to do a workaround). (not me, not raffi, sun.)
  *         - A monitors B, A kills self, A throws null. 
  *                 OR EVEN A monitors B, A stops monitoring B, A kills self, A throws null. (not me)
  *         - IcUIAg stops monitoring its own cell, then AWTUI kills it, throws null (might not want to worry about this one...)
  *
  *         - Resizing window causes problems...
  *
  *         - When a new cell joins the master, the master doesn't tell older cells subscribed to it about the new cell that joined.
  *
  *         - When a RemoteException occurs, should let CellListAgentImpl know that that cell is down.
  * 
  **/


// Graphical connection arrow in IconicUILayoutManager (ln 134)

// Here:
//      protected IconicUICanvas uiCanvas;
//      protected IconicUILayoutManager layout;



public class IconicUIAgentImpl 
extends BaseAWTGUIAgentImpl 
implements ActionListener {

    protected Frame frame;
    protected Panel uiPanel;
    protected GridBagConstraints gbcs;
    
    protected TextArea messageWindow;

    /** This is needed because of an error when messages are appended 
	to the message window before it has been popped up.  This catches
	all messages that are sent when the message window isn't visible
	so that they can be printed when it is. **/
    protected String messageBuffer = "";

    
    //    protected BaseUICanvas uiCanvas;
    //protected BaseLayoutManager layout;
    protected IconicUICanvas uiCanvas;
    protected IconicUILayoutManager layout;
    protected CellListCanvas clc;

    protected CellListAgent cla;

    public static Color bgColor;
    

    /**
     * this is the flag that will tell this UI whether to render
     * server agents or not 
     */
    private boolean renderCellAgents = false;


    public IconicUIAgentImpl() throws RemoteException { 
	super(); 
	
	/*
	  *  Modified IconicUIAgentImpl to set it's icon in the constructor.  The
	  *  normal way of the icon getting loaded by the AgentImpl causes a
	  *  flicker on startup, and doing it this way prevents that, while
	  *  allowing us to have an icon to display on remote cells.
	  * 
	  * 
	  *  That's why this mess is here.
	   */
 	InputStream is = null;
	is = getClass().getResourceAsStream( "icons/RemoteIconicUIAgentImpl.ppm" ); 

	if (is == null)
	    Debug.warning("Error trying to load default icon for Remote UI.");
	else {
	    try {
		icon = new PPM(is);
	    } catch( Exception ex ) { 
		Debug.warning( "Error Creating UI agent's icon ", ex ); 
	    }
	}
    }


    /**
     * setup this agent for activity -- mostly this just sets up the
     * visible part of the UI and displays it and then it renders all
     * the agents onto the screen 
     */
    public void doLocalSetup() throws AgentInitializationException {

	super.doLocalSetup();

	bgColor = Color.white;
	Description desc = this.getDescription();
	if (desc != null) {
	    Description conf = desc.configContext();

	    if (conf != null) {
		if (conf.selectFirst("backgroundColor")) {
		    conf = conf.getMatch();
		    
		    int r = 255, g = 255, b = 255;
		    if (conf.selectFirst("red"))
			r = Integer.parseInt( conf.getMatch().getText() );	    
		    if (conf.selectFirst("green"))
			g = Integer.parseInt( conf.getMatch().getText() );
		    if (conf.selectFirst("blue"))
			b = Integer.parseInt( conf.getMatch().getText() );
		    
		    bgColor = new Color(r, g, b);
		}
	    }
	}
	
	/**
	 * setup all the visible ui stuff
	 */

	gbcs = new GridBagConstraints();
	gbcs.insets = new Insets(1, 1, 1, 1);  // 1 pixel border surrounds each component
	gbcs.fill = GridBagConstraints.BOTH;   // should grow in both directions	
	uiPanel = new Panel( new GridBagLayout() );

	
	// create the messaging area
	messageWindow = new TextArea( "", 8, 40, TextArea.SCROLLBARS_VERTICAL_ONLY );	
	messageWindow.setEditable( false );
	messageWindow.setBackground(bgColor);

	gbcs.gridx = 0; gbcs.gridy = 2;              // Text panel is on bottom of frame
	gbcs.gridwidth = 4; gbcs.gridheight = 1;
	gbcs.weightx = 1.0; gbcs.weighty = 1.0;      // and should expand horizontally to fill free space
	uiPanel.add( messageWindow, gbcs );
	

	// Create the canvas for the cell list
	clc = new CellListCanvas(this);

	gbcs.gridx = 0; gbcs.gridy = 0;          // Cell list is on left
	gbcs.gridwidth = 1; gbcs.gridheight = 2;
	gbcs.weightx = 0; gbcs.weighty = 1.0;    // and should expand horizontally and vertically to fill free space.
	uiPanel.add( clc, gbcs );
	clc.setBackground(bgColor);
	clc.addMonitoredCell(getCellAddress());

	
	// Create the main panel for the agent icons
	layout = new IconicUILayoutManager();
	uiCanvas = new IconicUICanvas( (BaseUIAgentImpl)this, (BaseLayoutManager)layout );
	
	gbcs.gridx = 1; gbcs.gridy = 0;          // Icon pane is on right
	gbcs.gridwidth = 3; gbcs.gridheight = 2;
	gbcs.weightx = 1; gbcs.weighty = 1.0;    // and should expand horizontally and vertically to fill free space.
	uiPanel.add( uiCanvas, gbcs );

		
	// put the frame up and put the menu bar there
	frame = acms.acceptComponentFrom( this, uiPanel, getCellAddress().toString() );
	frame.setMenuBar( getMenuBar() );

	// Set the background color for everything.
	setBackgroundColor( bgColor );
	

	/**
	 * finish setting up all the visible ui stuff
	 */

	// get a connection to the local cell list agent
	try {
	    String[] claName = { (net.hivecell.hive.agent.cell.CellListAgent.class).getName() };
	    AgentSet ds = myCell.queryAgents( this, claName, null );
	    cla = (CellListAgent)ds.pickOne();
	    if( !connectTo( cla ) )
		Debug.notice( "cannot get a handle to the cell list agent -- not going to be getting the updates" );

	    clc.newCellInfo( cla.listCells( this ) );

	} catch( ClassNotFoundException error ) {

	} catch( RemoteException error ) {
	    Debug.notice( "cannot get a handle to the cell list agent -- not going to be getting the updates" );
	}

	// Now render the agent graph (minus the server agents)
	Enumeration e = ag.listAllAgents();
  	while( e.hasMoreElements() ) {
	    Agent a = (Agent)e.nextElement();
	    renderNewAgent( a );
	}

	// And the connections
	e = ag.listAllAgents();
	while( e.hasMoreElements() ) {

	    Agent a = (Agent)e.nextElement();

	    try {
		Agent[] inc = ag.listIncomingConnections( a );
		Agent[] out = ag.listOutgoingConnections( a );
		
		for( int i=0;i < inc.length;i++ ) 
		    renderConnectAgents( inc[i], a );
		
		for( int i=0;i < out.length;i++ )
		    renderConnectAgents( a, out[i] );
		
	    } catch( NotInGraphException y ) {
		Debug.warning("Hmmm... Agent which is being read from graph is not in graph.  If you're reading this, you're in the TZ.");
	    }
	}

	setIsReady( true );

    }

    /**
     * catch the notify messages -- this way we can listen to stuff
     * from our cell list agent and then we can pass on anything else
     * to the underlying code 
     */
    public void notify( HiveEvent he ) {

	blockUntilReady();
	
	if( ( cla != null ) && he.getSource().equals( cla ) && ( he instanceof CellEvent ) ) {
	    CellEvent ce = (CellEvent)he;
	    if( ( ce instanceof CellBornCellEvent ) || ( ce instanceof CellGoingDownCellEvent ) ) {
		try {
		    clc.newCellInfo( cla.listCells( this ) );
		} catch( RemoteException error ) { 
		}
	    }

	}
	    
	super.notify( he );

    }

    /**
     * add the "display server agents" and the "animate agent icons"
     * menu items to the UI menu 
     */
    protected void constructMenus() {
	super.constructMenus();

	// put the menu items to dislpay server agents and to animate
	// agent icons here
	MenuFolder uiMenuFolder = (MenuFolder)menu.getMenuElement( "UI" );

	class displayCellAgentsMenuCheckBoxItem
	extends MenuCheckBoxItem {
	    public displayCellAgentsMenuCheckBoxItem() { super( "Display server agents", false ); }
	    public boolean invoke() {
		renderCellAgents = getState();
		if( renderCellAgents ) {
		    // ok -- we need to walk the agent graph for all
		    // the agents that are there and then display
		    // those agents and those connections which
		    // pertain to CellAgents
		    Enumeration e = ag.listAllAgents();
		    while( e.hasMoreElements() ) {
			Agent a = (Agent)e.nextElement();
			if( a instanceof CellAgent )
			    renderNewAgent( a );
		    }

		    e = ag.listAllConnections();
		    while( e.hasMoreElements() ) {
			AgentGraphEdge age = (AgentGraphEdge)e.nextElement();
			if( ( age.getSourceAgent() instanceof CellAgent ) ||
			    ( age.getDestinationAgent() instanceof CellAgent ) )
			    renderConnectAgents( age.getSourceAgent(), age.getDestinationAgent() );
		    }

		} else {
		    // go through the graph and remove all the server
		    // agents
		    Enumeration e = ag.listAllAgents();
		    while( e.hasMoreElements() ) {
			Agent a = (Agent)e.nextElement();
			if( a instanceof CellAgent ) {
			    CellAddress ca = getAgentAddress( a );
			    
			    Agent[] incoming = null;
			    Agent[] outgoing = null;
			    try {
				incoming = ag.listIncomingConnections( a );
				outgoing = ag.listOutgoingConnections( a );
			    } catch( NotInGraphException error ) {
				// this can't happen
			    }
			    
			    renderRemoveAgent( a, ca, incoming, outgoing );
			}
		    }

		}
		return true;
	    }
	}
	uiMenuFolder.insertMenuElementAt( new displayCellAgentsMenuCheckBoxItem(), 0 );

	class animateIconsMenuCheckBoxItem
	extends MenuCheckBoxItem {
	    public animateIconsMenuCheckBoxItem() { super( "Animate agent icons", true ); }
	    public boolean invoke() {
		if( getState() == true ) 
		    Debug.notice( "turning on agent icon animations" );
		else
		    Debug.notice( "turning off agent icon animations" );
		uiCanvas.setAnimateIcons( getState() );
		return true;
	    }
	}
	uiMenuFolder.insertMenuElementAt( new animateIconsMenuCheckBoxItem(), 1 );

	uiMenuFolder.insertMenuElementAt( new MenuSeparator(), 2 );

    }

    /** 
     * do cleanup because we are dying -- stop our rendering and stop
     * our animation loops 
     */
    public void onDying() {

	uiCanvas.stopRenders();
	uiCanvas.stopAnim();

    }


    /**
     * ALL THE RENDERING METHODS (START)
     */

    /**
     * an agent has arrived on the scene and it needs to be drawn
     * to the UI
     *  
     * @param a the new agent to be rendered 
     */
    public void renderNewAgent( Agent a ) {

	// the icon has been added, but if we are not displaying
	// server agents, then we should not move onto rendering
	if( !renderCellAgents && ( a instanceof CellAgent ) ) return;

	// create the base agent icon for the agent
	Image iconImage = getAgentIcon( a ).getImage();
	if( layout == null )
	    Debug.warning( "SSSSSSSSSSSSSSSSSSSSSSSHHHHHHHHHHHHHHHHHIIIIIIIIIIIIIIIIIIIIITTTTTTTTTTTTT" );
	BaseAgentIcon ai = layout.arrangeAddAgent( a, iconImage );

	// find out what address the agent is from and then give the
	// icon the proper color border -- if we don't have an
	// address, then give the icon a black border
	CellAddress add = getAgentAddress( a );
	if( add != null ) {
	    Color c = clc.getColor( add );
	    if( c == null ) {
		Debug.notice( "got an agent from a cell that we are not monitoring.  need to add its color to the legend: " );
		c = clc.addMonitoredCell( add );
	    }
	    ai.setBorderColor( c );

	} else {
	    Debug.warning( "couldn't tell what server the agent is from, using a black border color" );
	    ai.setBorderColor( Color.black );

	}

	uiCanvas.renderAgentIcon( ai );
	// This allows the animated UIIcon to register new Icons immediately.
	uiCanvas.renderUIIcon();

    }
    
    /**
     * a method which needs to be implemented -- this means that an
     * agent has been removed from the scene and this change needs to
     * be rendered.  this method also includes a list of agents that
     * this agent was connected to so we can remove those connections
     * also
     *
     * @param a the agent that we are removing from the scene
     * @param ca the cell address of the agent we are removing
     * @param incoming the agents that are pointing to this agent
     * @param outgoing the agents that this agent is pointing to 
     */
    public void renderRemoveAgent( Agent a, CellAddress ca, Agent[] incoming, Agent[] outgoing ) {

	renderDisconnectFromAll( a, incoming, outgoing );
	layout.arrangeRemoveAgent( a );
	uiCanvas.render( uiCanvas.getGraphics() );

    }
    
    /**
     * a method which needs to be implemented -- when two agents
     * connect, this method will be called.  the two agents should
     * already be on the graph -- two agents should not need to be
     * connected if they are not on the graph yet
     *
     * @param s the source agent
     * @param d the destination agent 
     */
    public void renderConnectAgents( Agent s, Agent d ) {

	// if we are not connecting server agents, then this should
	// not go through for the actual connection
	if( !renderCellAgents && 
	    ( ( s instanceof CellAgent ) || ( d instanceof CellAgent ) ) )
	    return;

	GraphicalConnectionArrow ai = (GraphicalConnectionArrow)layout.arrangeConnectAgents( s, d );
	uiCanvas.renderArrow( ai );
	uiCanvas.renderUIIcon();
    }

    /**
     * a method which needs to be implemented -- when two agents
     * disconnect, this method will be called to display the change to
     * the screen
     *
     * @param s the source of the disconnection
     * @param d the destination of the disconnection
     */
    public void renderDisconnectAgents( Agent s, Agent d ) {

	layout.arrangeDisconnectAgents( s, d );
	uiCanvas.render( uiCanvas.getGraphics() );
	uiCanvas.renderUIIcon();
    }

    /**
     * this method needs to be implemented for when an agent disconnects from all
     *
     * @param a the agent that is disconnecting from all
     * @param incoming all the agents that are connected to this agnet
     * @param outgoing all the agnets that are connected to by this agent
     */
    public void renderDisconnectFromAll( Agent a, Agent[] incoming, Agent[] outgoing ) {

	for( int i=0; i < outgoing.length; i++ )
	    renderDisconnectAgents( a, outgoing[i] );

	for( int i=0; i < incoming.length; i++ )
	    renderDisconnectAgents( incoming[i], a );
    }

    /**
     * this method needs to be implemented so agent messaging can be drawn
     *
     * @param s the source of the message
     * @param d the destination of the message
     * @param message the message
     */
    public void renderAgentMessage( Agent s, Agent d ) { 

	if( !renderCellAgents && 
	    ( ( s instanceof CellAgent ) || ( d instanceof CellAgent ) ) )
	    return;
	
	layout.arrangeAgentMessage( s, d ); 
    
    }

    /**
     * this method needs to be implemented so the server reports go somwhere
     *
     * @param message the server message
     */
    public void renderCellReport( CellReportCellEvent report ) {
	if (messageWindow == null) return;
	if (messageWindow.isShowing() == false) {
	    String txt = report.getMessage();
	    String serverName = report.getAddress().toString();

	    if (serverName.equals(getCellAddress().toString()))
		serverName = new String("");
	    else serverName = "(" + serverName + "): ";

	    messageBuffer += serverName + txt + "\n";
	    return;
	}
	/**
	 * The above test needs to be explained: when a text area has text inserted into it
	 * while it is still hidden (in this case, hasn't been popped up yet), from then on,
	 * inserting text into it will NOT cause it to scroll to the bottom, which is what we
	 * want it to do.  
	 * This is detailed at http://developer.java.sun.com/developer/bugParade/bugs/4053740.html
	 *
	 * Other workarounds which worked, but will hopefully not have to be implemented are: 
	 *
	 * If the isShowing method fails on some platforms, another workaround is below, 
	 * although it involves a deprecated function.
	 *	if (messageWindow.getPeer() == null) return;
	 *
	 * finally, messageWindow.setCaretPosition(999999999);
	 * after the "append" will also work, as long as the peer has been created.
	 * (check for an exception to be thrown)
	 **/

	String txt = report.getMessage();
	String serverName = report.getAddress().toString();
	if (serverName.equals(getCellAddress().toString()))
	    serverName = new String("");
	else serverName = "(" + serverName + "): ";
	
	messageWindow.append( messageBuffer + serverName + txt + "\n" );
	messageBuffer = "";
    }

    /**
     * ALL THE RENDERING METHODS (END)
     */



    /**
     ** Override monitorCell and stopMonitorCell to provide
     ** additional functionality.
     **/
    public synchronized void monitorCell( CellAddress ca ) 
    throws UnableToMonitorCellException {
	super.monitorCell(ca);
	clc.addMonitoredCell(ca);
    }


    public synchronized void stopMonitoringCell( CellAddress ca ) {
	super.stopMonitoringCell(ca);
	clc.stopMonitoringCell(ca);
	clc.addUnmonitoredCell(ca);
    }



    
    /** 
     * Needs to listen to the popup menu 
     *
     * @param e the action event that is coming from the popup menus
     */
    public void actionPerformed( ActionEvent e ) { invokeMenuCommand( uiCanvas.getSelectedIcon().getAgent(), e.getActionCommand() ); }
    public boolean invokeMenuCommand( Agent a, String command ) {

	if( command.equals( connectCmd ) ) {
	    uiCanvas.setConnectMode();
	    return true;

	} else if( command.equals( disconnectCmd ) ) {
	    uiCanvas.setDisconnectMode();
	    return true;

	} else if( command.equals( killCmd ) )
	    return killAgentCommand( a );

	return super.invokeMenuCommand( a, command );

    }


    protected void loadIcon() {
	
	// this should prevent the icon from being loaded 
	icon = new SerializableImage( new int[]{ 1 }, new java.awt.Dimension( 1, 1 ) );

    }


    /** Get the background color */
    public Color getBackgroundColor() { return (bgColor); }

    /** Set the background color of the ui. */
    public void setBackgroundColor(Color nc) {
	bgColor = nc;
	messageWindow.setBackground(bgColor);
	uiCanvas.setBackground(bgColor);
	clc.setBackground(bgColor);
    }

}
