// $Id: CellAddressTreeFilterer.java,v 1.1 2000/05/18 22:00:57 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.hiveui.treeui;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.hiveui.BaseGUIAgentImpl;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent;
import net.hivecell.hive.agent.cell.UnknownAgentException;
import net.hivecell.hive.support.AgentGraph;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;

import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;


/**
 * this class takes the agent monitoring agent and queries it for all
 * the agnets it knows about and collapses the information into a tree
 * that is filtered on cell addresses 
 */
public class CellAddressTreeFilterer
extends GraphTreeFilterer {

    /**
     * use this as a mapping between DefaultMutableTreeNodes and the
     * agents that they represent 
     */
    private Hashtable nodeAgents = new Hashtable();
    
    /**
     * and use this as the reverse
     */
    private Hashtable agentNodes = new Hashtable();
    
    /**
     * use this to keep track of all the agents on a particular cell.
     * so this maps a cell address to a vector of agents 
     */
    private Hashtable agentAddresses = new Hashtable();
    
    /**
     * use this to keep track of a mapping between cell addresses and
     * the dmtn for the tree 
     */
    private Hashtable addressDMTN = new Hashtable();

    /**
     * the top of the tree
     */
    private DefaultMutableTreeNode top;


    public CellAddressTreeFilterer( BaseGUIAgentImpl bga ) { super( bga ); }

    
    public Agent getAgentNode( DefaultMutableTreeNode dmtn ) { return (Agent)nodeAgents.get( dmtn ); }
    
    public JTree renderAgentGraph( AgentGraph ag ) { 

	// fill our hashtable of agent addresses so we have a mapping
	// of cell addresses to agents
	Enumeration e = ag.listAllAgents();
	while( e.hasMoreElements() ) {
	    Agent a = (Agent)e.nextElement();
	    CellAddress ca = bga.getAgentAddress( a );
	    Vector toAddOn = (Vector)agentAddresses.get( ca );
	    if( toAddOn == null ) {
		toAddOn = new Vector();
		agentAddresses.put( ca, toAddOn );
	    }
	    toAddOn.addElement( a );
	}

	// now render our tree
	top = new DefaultMutableTreeNode( "Cell Addresses" );
	e = agentAddresses.keys();
	while( e.hasMoreElements() ) {
	    CellAddress ca = (CellAddress)e.nextElement();
	    DefaultMutableTreeNode cellAddressDMTN = new DefaultMutableTreeNode( ca.toString() );
	    Enumeration agents = ((Vector)agentAddresses.get( ca )).elements();
	    while( agents.hasMoreElements() ) {
		Agent a = (Agent)agents.nextElement();
		DefaultMutableTreeNode agentDMTN = new DefaultMutableTreeNode( bga.getAgentName( a ) );
		nodeAgents.put( agentDMTN, a );
		agentNodes.put( a, agentDMTN );
		cellAddressDMTN.add( agentDMTN );
		
	    }
	    addressDMTN.put( ca, cellAddressDMTN );
	    top.add( cellAddressDMTN );
	}

	jtree = new JTree( top );
	addListeners( jtree );
	expandTree( jtree );

	return jtree;
	
    }

    public JTree renderNewAgent( Agent a ) { 

	// first figure out what cell this agent is on
	CellAddress ca = bga.getAgentAddress( a );
	if( !agentAddresses.containsKey( a ) )
	    agentAddresses.put( ca, new Vector() );
	((Vector)agentAddresses.get( ca )).addElement( a );

	DefaultMutableTreeNode cellAddressDMTN = (DefaultMutableTreeNode)addressDMTN.get( ca );
	if( cellAddressDMTN == null ) {
	    cellAddressDMTN = new DefaultMutableTreeNode( ca );
	    addressDMTN.put( ca, cellAddressDMTN );
	    top.add( cellAddressDMTN );
	}

	String agentName = bga.getAgentName( a );
	DefaultMutableTreeNode agentDMTN = new DefaultMutableTreeNode( agentName );

	nodeAgents.put( agentDMTN, a );
	agentNodes.put( a, agentDMTN );
	cellAddressDMTN.add( agentDMTN );

	jtree = new JTree( top );
	addListeners( jtree );
	expandTree( jtree );

	return jtree;

    }

    public JTree renderRemoveAgent( Agent a, CellAddress ca, Agent[] incoming, Agent[] outgoing ) { 

	// first figure out what cell this agent is on
	if( agentAddresses.containsKey( ca ) ) {
	    ((Vector)agentAddresses.get( ca )).removeElement( a );
	    if( ((Vector)agentAddresses.get( ca )).isEmpty() )
		agentAddresses.remove( ca );
	}

	DefaultMutableTreeNode cellAddressDMTN = (DefaultMutableTreeNode)addressDMTN.get( ca );
	DefaultMutableTreeNode agentDMTN = (DefaultMutableTreeNode)agentNodes.remove( a );
	nodeAgents.remove( agentDMTN );

	cellAddressDMTN.remove( agentDMTN );
	if( cellAddressDMTN.getChildCount() == 0 ) {
	    addressDMTN.remove( ca );
	    top.remove( cellAddressDMTN );
	}

	jtree = new JTree( top );
	addListeners( jtree );
	expandTree( jtree );

	return jtree;

    }

}





