// $Id: TreeUIAgentImpl.java,v 1.2 2000/05/19 19:55:57 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.agent.hiveui.treeui; 

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.EventReceivingAgentImpl;
import net.hivecell.hive.agent.hiveui.BaseUIAgentImpl;
import net.hivecell.hive.agent.hiveui.BaseGUIAgentImpl;
import net.hivecell.hive.agent.hiveui.BaseSwingGUIAgentImpl;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent; 
import net.hivecell.hive.agent.cell.CellControlAgent;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.event.cell.CellEvent;
import net.hivecell.hive.event.cell.CellReportCellEvent;
//import net.hivecell.hive.rdf.DescSet;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.AgentGraph;
import net.hivecell.hive.support.Debug;

import java.awt.BorderLayout;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;


/**
 * a UI that displays the information from the agent
 * monitoring agent in a tree like format.  this class allows
 * different classes to be used for filtering and displaying the
 * information
 */
public class TreeUIAgentImpl 
extends BaseSwingGUIAgentImpl {

    /**
     * store our server control agents to the different servers so we
     * can command the agents on the servers 
     */
    protected Hashtable cellControlAgents = new Hashtable();
    
    /**
     * the class that we are using to filter our information into the
     * jtree 
     */
    protected GraphTreeFilterer gtf = new CellAddressTreeFilterer( this );


    protected JFrame frame; // the frame that we are putting everything into
    protected JPanel panel; // the master level panel
    protected JScrollPane scrollPane; // what we are going to use for our tree


    public TreeUIAgentImpl() throws RemoteException { super(); }

    public void doLocalSetup() throws AgentInitializationException {

	super.doLocalSetup();

	// set things up for drawing
	panel = new JPanel( new BorderLayout() );
	frame = (JFrame)acms.acceptComponentFrom( this, panel, "TreeUIAgent" );
	frame.setJMenuBar( getMenuBar() );
	frame.setSize( 350, 300 );
	frame.setVisible( true );

	try {
	    CellAddress[] address = { getCellAddress() };
	    String[] className = { (net.hivecell.hive.agent.cell.CellControlAgent.class).getName() };
	    AgentSet ds = ama.queryAgents( this, address, className, null );
	    if( ds.size() == 0) {
		Debug.error( "no server control agent on " + getCellAddress() + ".  this agent requires one to work" );
		myCell.killAgent( (AgentImpl)this );
	    } else
		cellControlAgents.put( getCellAddress(), (CellControlAgent)ds.pickOne() );
	} catch( ClassNotFoundException error ) {
	    Debug.critical( "huh?  this should -never- happen!", error );
	    
	} catch( RemoteException error ) {
	    Debug.error( "remote exception while trying to ask the agent monitoring agent for the server control agent -- this agent requires the server control agent to work" );
	    myCell.killAgent( (AgentImpl)this );
	}

	// tell everything else that we are ready to party
	readyFlag = true;
	synchronized( this ) {
	    notifyAll();
	}

	// Render the Agent Graph canvas:
	renderTree( gtf.renderAgentGraph( ag ) );

    }

    public void doBehavior() { super.doBehavior(); }

    public void doLocalCleanup() { 

	super.doLocalCleanup();
	frame.dispose();

    }

    public String[] getMenuNames( Agent a ) {

	String[] customAgentNames = super.getMenuNames( a );

	// get all the default menu names minus the connect and disconnect commands
	String[] menuNames = new String[ customAgentNames.length - 2 ];
	int position = 0;
	for( int count=0;count < customAgentNames.length;count++ )
	    if( !( customAgentNames[count].equals( connectCmd ) || customAgentNames[count].equals( disconnectCmd ) ) )
		menuNames[position++] = customAgentNames[count];

	return menuNames;

    }

    public void renderNewAgent( Agent a ) { pauseForReadyFlag(); renderTree( gtf.renderNewAgent( a ) ); }
    public void renderRemoveAgent( Agent a, CellAddress ca, Agent[] incoming, Agent[] outgoing ) { pauseForReadyFlag(); renderTree( gtf.renderRemoveAgent( a, ca, incoming, outgoing ) ); }
    public void renderConnectAgents( Agent s, Agent d ) { pauseForReadyFlag(); renderTree( gtf.renderConnectAgents( s, d ) ); }
    public void renderDisconnectAgents( Agent s, Agent d ) { pauseForReadyFlag(); renderTree( gtf.renderDisconnectAgents( s, d ) ); }
    public void renderDisconnectFromAll( Agent a, Agent[] incoming, Agent[] outgoing ) { pauseForReadyFlag(); renderTree( gtf.renderDisconnectFromAll( a, incoming, outgoing ) ); }
    public void renderAgentMessage( Agent s, Agent d ) { pauseForReadyFlag(); renderTree( gtf.renderAgentMessage( s, d ) ); }
    public void renderCellReport( CellReportCellEvent report ) { 
	pauseForReadyFlag(); renderTree( gtf.renderCellReport( report.getMessage() ) ); 
    }
    
    private void pauseForReadyFlag() {
	while( !readyFlag ) 
	    try { 
		synchronized( this ) {
		    wait();
		}
	    } catch( InterruptedException error ) { }
    }
    
    protected synchronized void renderTree( JTree tree ) {
	
	if( ( panel == null ) || ( frame == null ) )
	    return;

	if( scrollPane != null ) panel.remove( scrollPane );
	scrollPane = new JScrollPane( tree );
	panel.add( scrollPane, "Center" );
	
	frame.show();

    }

}



