// $Id: Cell.java,v 1.3 2000/05/19 23:19:34 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.cell;

import java.io.InputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.lang.reflect.Constructor;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.rmi.ServerRuntimeException;
import java.rmi.AlreadyBoundException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.AgentNotFoundException;
import net.hivecell.hive.agent.AgentUnableToBirthException;
import net.hivecell.hive.agent.AgentInitializationException;
import net.hivecell.hive.agent.cell.AgentMessagingShadowAgentImpl;
import net.hivecell.hive.agent.cell.AgentMonitoringAgent;
import net.hivecell.hive.agent.cell.AgentMonitoringAgentImpl;
import net.hivecell.hive.agent.cell.CellAgent;
import net.hivecell.hive.agent.cell.CellControlAgentImpl;
import net.hivecell.hive.agent.cell.CellListAgentImpl;
import net.hivecell.hive.agent.hiveui.iconicui.IconicUIAgentImpl;
import net.hivecell.hive.description.AgentSet;
import net.hivecell.hive.description.Description;
import net.hivecell.hive.shadow.Shadow;
import net.hivecell.hive.shadow.ShadowImpl;
import net.hivecell.hive.shadow.ShadowInitializationException;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.ShadowStub;
import net.hivecell.hive.shadow.cell.AgentMessagingShadow;
import net.hivecell.hive.shadow.cell.AgentMessagingShadowImpl;
import net.hivecell.hive.shadow.cell.CellControlShadow;
import net.hivecell.hive.shadow.cell.CellControlShadowImpl;
import net.hivecell.hive.shadow.cell.CellEventMonitor;
import net.hivecell.hive.shadow.cell.CellEventMonitorImpl;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.shadow.cell.ComponentManagerShadowImpl;
import net.hivecell.hive.shadow.cell.HTTPServerShadow;
import net.hivecell.hive.shadow.cell.HTTPServerShadowImpl;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.CellConnectException;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.support.VariablePropertyManipulator;

import com.docuverse.dom.DOM;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;


// NOTE: cleanup item: kill distinction between configFile and semanticFile in createAgent, etc.

/** 
 * A server implementation for Hive.
 *
 * @see RemoteCell
 * @version $Revision: 1.3 $
 */
public class Cell 
extends UnicastRemoteObject 
implements RemoteCell {
    /** 
     * The security manager used by Hive (will be changed in JDK 1.2
     * -- if we ever decide to go that way) 
     */
    private SecurityManager securityManager;

    /** 
     * The gatekeeper does the security checking for all the agents
     * trying to enter the cell, and provides the tokens to all agents
     * trying to go to a remote cell.  
     */
    private AgentGatekeeper agentGatekeeper;

    /** 
     * The DOM object used by the cell to parse the agent's semantic
     * descriptions.  
     */
    private DOM dom;

    /** 
     * Pointer to cell's XML description.
     */
    protected Description description = new Description( this, "Hive Cell" );
  

    private static Hashtable localToRemoteObjects = new Hashtable();
    private static Hashtable remoteToLocalObjects = new Hashtable();

    /**
     * use this to keep track of the agents on the cell
     */
    private AgentTable agentTable = new AgentTable();


    /*******************
     * default shadows *
     *******************/

    /**
     * use this to keep track of the shadows on the cell
     */
    private ShadowTable shadowTable = new ShadowTable();

    /** 
     * Handles the components provided by the agents 
     */
    private ComponentManagerShadow componentManagerShadow;

    /** 
     * Handles the remote control of the cell 
     */
    private CellControlShadow cellControlShadow;

    /** 
     * Handles the event sending/receiving for this cell 
     */
    private AgentMessagingShadow agentMessagingShadow;

    /** 
     * The server used to serve class bytes for RMI (will be changed in 1.2) 
     */
    private HTTPServerShadow httpCellShadow;

    /** 
     * A shadow that publishes server events. 
     */
    private CellEventMonitor cellEventMonitor;



    /// Miscellany
    private static Registry registry;
    private CellAddress myAddress;
    private RemoteCell myRemoteCell;

    /** 
     * A flag that is set when the server is about to go down.
     * Setting it will do nothing, but it is useful to check it to
     * see if the server is in shutdown procedures
     **/
    private boolean shuttingDown = false;


    /** 
     * A list of those agents which are in the middle of shutdown procedures
     */
    private Hashtable shuttingDownAgents;

    /**
     * A list of those agents which are in the middle of moving
     */
    private Vector movingAgents;
    
    ///////////////////////////////////////////////////////////////////////////
    // Remotely callable methods, the services the server provides to the net//
    ///////////////////////////////////////////////////////////////////////////

    /** Get the semantic description of this cell.  
     *  This method only returns the semantic description, not the config one.
     */
    public Description getDescription() {

	// Don't let anyone see the entire description, only the semantic part.
	Description clean = new Description(this, "HiveCell");

	clean.addTag("semantic", null, null, null);
	clean = clean.semanticContext();
	
	try {
	    Node cur = clean.getNode();

	    if (cur != null) {
		Document doc = cur.getOwnerDocument();
		Element a = doc.createElement("semantic");
		Node n = description.semanticContext().getNode();  // Grab the semantic
		
		cur.appendChild(a);
		a.appendChild(n);
	    }
  	} catch (Exception f) {
  	    Debug.error("An error occured while trying to add nodes.", f);
  	}

	clean = clean.noContext();
	return (clean);
    }

    /** Set the semantic description of this cell.
     */
    public void setDescription( Description desc ) { description = desc; }


    /**
     * Get the host:port address of the server.
     *
     * @param the host:port address 
     */
    public CellAddress getAddress() { return myAddress; }


    /**
     * can we say hack?  get the exported version of the agent.  i'm
     * not sure if this is the right version to do it -- i'm keeping a
     * hashtable around holding all the agents that we are going to
     * export -- this way we can ask for the remote version of
     * something and this will hold it.
     *
     * this will need to be fixed, big time!  right now it is caching
     * everything.  that is soooooooooooooooooo wrong.  only cache
     * agents or clean up our cache or something otherwise we will be
     * -eating- memeory.
     *
     * @param o the object that we are looking to get the remote version of
     * @return the remote stub of the object we are looking at -- if
     *         you give us something that we can't get the remote object of,
     *         then it just returns the same object 
     */
    public Object getRemoteObject( Object o ) {
	// lookup the object in the hashtable.  if we can't find it,
	// then export the object.  exporting the object (because we
	// overrode it), as the sideeffect of putting it in the
	// hashtable that we can later use
	Object remoteObject = (Agent)localToRemoteObjects.get( o );
	if( remoteObject == null )
	    try {
		remoteObject = myRemoteCell.returnRemoteObject( o );
		localToRemoteObjects.put( o, remoteObject );

	    } catch( Exception error ) {
		// this should really return an exception -- let's
		// just make sure this works, then we need to convert
		// everything to throw the exception
		Debug.critical( error );

	    }
	return remoteObject;

    }

    public Object returnRemoteObject( Object o ) {
	return o;

    }


    /**
     * Just do the ping on the cell -- it will just return true in
     * all cases.  useful for network testing.  
     */
    public boolean ping() { return true; }



    /**
     * Query this Cell for the agents on it.
     *
     * @param sender the object doing the query
     * @param syntatic the java type of the agent we are searching for
     * @param semantic the semantic information of the agent we are searching for
     */
    public AgentSet queryAgents( Object sender, String[] syntactic, String[] semantic )
    throws ClassNotFoundException {
	report( this, "query agents from " + Global.shortString( sender ) );

	// this -needs- to not send an enumeration of the
	// implementations to the QueryAgentsHelper, it needs to send
	// an enumeration of the stubs instead.  unfortunatly there is
	// no way to convert the implementations to stubs in java 1.1
	// -> we need to wait until java 2 so we can do this
	return QueryAgentsHelper.queryAgents( agentTable.getImplementations(), syntactic, semantic );

    }

    /**
     * Accept agent from a remote host.  This call is made with the
     * serialized copy of the agent to be passed in through an array
     * of bytes
     *
     * @param agentBytes a serialized copy of the agent to be passed
     * @param sourceAddress the address the agent came from
     * @param token A token authorizing this agent to get into this cell.
     * @return the remote reference to the agent if it successfully moved, false otherwise
     * @exception DenyMobileAgentException if we are rejecting this agent from coming in
     * @exception RemoteException if something went wrong with accepting this gent
     */
    public Agent acceptAgent( byte[] agentBytes, CellAddress sourceAddress, Object token )
    throws DenyMobileAgentException, RemoteException {
	if( !agentGatekeeper.getAcceptAgent() || !agentGatekeeper.verifyToken( token ) ) {
	    Debug.notice( "asked to accept agent, but am rejecting it" );
	    throw new DenyMobileAgentException( "rejecting mobile agent due to security check" );

	}

	try {
	    Debug.notice( "cell is supposed to accept agent " + agentBytes.length + " bytes." );
	    ByteArrayInputStream bais = new ByteArrayInputStream( agentBytes );
	    MobileAgentInputStream mais = new MobileAgentInputStream( bais );

	    Debug.notice( "reconsituting agent from the byte stream" );
	    AgentImpl newAgent = (AgentImpl)mais.readObject();

	    Debug.notice( "starting the incoming agent" );
	    handleMovedAgent( newAgent, sourceAddress );

	    Debug.notice( "returning the reference back to the sender" );
	    return newAgent;

	} catch( Exception ex ) {
	    Debug.error( "couldn't reconstitute agent, not accepting it.", ex );
	    throw new DenyMobileAgentException( "couldn't reconsititute agent", ex );

	}

    }
    
    private void handleMovedAgent( final AgentImpl newAgent, final CellAddress sourceAddress ) {
	// Note the existence of the agent
	report( this, "New agent " + Global.shortString( newAgent ) );

	if( !agentTable.containsImplementation( newAgent ) ) {
	    agentTable.runAgentThread( newAgent );
	    noteAgentMovedFrom( this, newAgent, sourceAddress );

	} else
	    Debug.error( "agent being moved here, but it is already in our data structure as being local" );

    }

    /////////////////////////////////////////////////////////////////////////
    // Services that the cell provides to resident agents                  //
    /////////////////////////////////////////////////////////////////////////
    
    /**
     * ask the cell to create a new shadow -- if it works, it will
     * return the stub of the shadow that we created.  if it doesn't
     * work, it will probably throw an exception explaining itself.
     *
     * as a note, it might be possible to (in the future) allow the
     * starting and stopping of shadow's at any time.  we would need
     * to have the proxy for the cell first so nobody could just call
     * this method out of the blue
     *
     * @param shadow the name of the shadow that we want to create
     * @param propertiesFile the name of the file that we are going to load the properties of the shadow from
     * @return the shadow's stub 
     * @exception ClassNotFoundException if the shadow name doesn't map to any class
     * @exception ShadowInitializationException if the shadow doesn't initalize properly, or if we are not allowed to create any more shadows 
     */
    public Shadow createShadow( String shadow, String propertiesFile ) 
    throws ShadowInitializationException, ClassNotFoundException {
	return shadowTable.createShadow( shadow, propertiesFile );

    }
    
    /**
     * get a specific shadow
     *
     * @param string the name of the shadow we want to get.  this is the name of the Shadow or the ShadowImpl
     * @return the stub of the shadow we want
     * @exception ShadowNotFoundException if we don't have this shadow in our data structures at all
     */
    public Shadow getShadow( String shadow )
    throws ShadowNotFoundException {
	return shadowTable.getShadow( shadow );

    }

    /**
     * get a handle to the RMI registry that hive uses so you can
     * stick your own stuff in there if you need it
     *
     * @return a handle to our HiveRMIRegistry */
    public Registry getRMIRegistry() {
	return registry;

    }
    
    /**
     * Create an agent of a given class name.
     *
     * @param className the name of the agent to create
     * @param descFile the description file for this agent
     * @return the newly created agent
     * @exception ClassNotFoundException if the class cannot be found in the CLASSPATH
     * @exception InstantiationException if something goes wrong with the creation of the agent
     */
    public AgentImpl createAgent( String className, String descFile )
    throws ClassNotFoundException, InstantiationException {
	Class toCreate = null;
	try {
	    toCreate = Class.forName( className );

	} catch( ClassNotFoundException error ) {
	    Debug.error( "Asked to create class \"" + className + "\" which does not exist!" );
	    throw error;
	}
	return createAgent( toCreate, descFile );

    }

    /**
     * Create an agent of a given class name.
     *
     * @param className the name of the agent to create
     * @return the newly created agent
     * @exception ClassNotFoundException if the class cannot be found in the CLASSPATH
     * @exception InstantiationException if something goes wrong with the creation of the agent
     */
    public AgentImpl createAgent( String className )
    throws ClassNotFoundException, InstantiationException {
	return createAgent( className, null );

    }

    /** 
     * Create an agent of the given class. Instantiate it, load it into
     * our data structures, and set it running.
     *
     * @param agentClass the Class to implement (needs to be AgentImpl)
     * @return the newly created agent
     * @exception InstantiationException if something goes wrong with the creation of the agent
     */
    public AgentImpl createAgent( Class agentClass ) 
    throws InstantiationException {
	return createAgent( agentClass, null );

    }

    /** 
     * Create an agent of the given class. Instantiate it, load it into
     * our data structures, and set it running.
     *
     * @param agentClass the Class to implement (needs to be AgentImpl)
     * @param descFile the description file for this agent
     * @return the newly created agent
     * @exception InstantiationException if something goes wrong with the creation of the agent
     */
    public AgentImpl createAgent( Class agentClass, String descFile ) 
    throws InstantiationException {
	try {
	    AgentImpl newAgent = (AgentImpl)agentClass.newInstance();

	    // Load the agents description and set it
	    if( descFile != null ) {
		InputStream is = Global.getConfigAsInputStream( descFile );
	    
		if( is != null ) {
		    Document doc = dom.openDocument(is);
		    Description desc = new Description( newAgent, doc );
		
		    if( desc != null ) {
			newAgent.setDescription( desc );
			newAgent.configure( this, desc );

		    } else 
			Debug.warning( "Error: couldn't create Description from file \"" + descFile + "\".  Not setting description." );

		} else
		    Debug.notice( "Agent's Description file \"" + descFile + "\" not found." );

	    }

	    if( newAgent.getDescription() == null )  // Make sure the agent has a non-null description.  This should not ever be possible tho.
		newAgent.setDescription( new Description( newAgent, Global.shortClassName( newAgent ) ) );

	    return newAgent;

	} catch( Exception ex ) {
	    Debug.critical( "Error creating " + agentClass + ". ", ex );
	    throw new InstantiationException();

	}

    }

    /** 
     * Move the agent to the specified destination.
     *
     * @param agent The agent to move
     * @param address The host:port address
     * @return the remote reference to the moved agent
     */
    public Agent moveAgent( final AgentImpl agent, CellAddress address ) 
    throws AgentDeliveryException {
	// add this agent to a list of agents which are in the process of moving
	if( agentTable.containsImplementation( agent ) )
	    movingAgents.addElement( agent );

	try {
	    RemoteCell remoteCell = address.getRemoteCell();
	    agent.onMoving(); // give the agent warning that it is about to move

	    ByteArrayOutputStream bos = new ByteArrayOutputStream();
	    MobileAgentOutputStream maos;
	    maos = new MobileAgentOutputStream( bos, this );
	    maos.writeObject( agent );
	    maos.flush();

	    Debug.notice( "serialized " + Global.shortString( agent ) + " " + bos.size() + " bytes. Sending to " + address );

	    // If this cell was started merely so that we can launch an agent,
	    // then createHiveObjects will not have been called, and the
	    // agentGatekeeper will not be instantiated. There is a potential
	    // problem though that if the security-tokens file is in some non
	    // standard place, this cell might not find it, and the
	    // agent move might fail.
	    if( agentGatekeeper == null )
		agentGatekeeper = new AgentGatekeeper();
	    Object token = agentGatekeeper.getToken( address );
	    Agent transferred = remoteCell.acceptAgent( bos.toByteArray(), getAddress(), token );

	    // notify the community that this agent is moving
	    if( agentTable.containsImplementation( agent ) ) {
		noteAgentMovedTo( this, agent, address );
		try {
		    agentTable.removeAgent( agent, AgentImpl.AGENTMOVED );

		} catch( AgentNotFoundException error ) {

		}

	    }

	    movingAgents.removeElement( agent );
	    return transferred;

	} catch( CellConnectException error ) {
	    throw new AgentDeliveryException( "cannot connect to cell at " + address );

	} catch( DenyMobileAgentException error ) {
	    Debug.warning( "attempt to move agent " + Global.shortString( agent ) + " to " + address + " failed." );
	    throw new AgentDeliveryException( address + ".acceptAgent() failed" );
	    
	} catch( ServerRuntimeException error ) {
	    if( error.detail instanceof NullPointerException ) {
		Debug.notice( "security is not set up properly on the cell you are attempting to move the agent to" );
		throw new AgentDeliveryException( "security not set up properly on remote cell" );
	    } else
		throw new AgentDeliveryException(error.getMessage());

	} catch( IOException error ) {
	    throw new AgentDeliveryException( error.getMessage() );

	}

    }

    /** 
     * Is the agent local to this cell?
     * @param a the Agent interface we wish to test for locality
     * @return true if local, false otherwise
     */
    public boolean isAgentLocal( Agent a ) {
        return ( getLocalReference( a ) != null );

    }

    /**
     * Given an agent stub, return the local AgentImpl for it if it
     * is available, or else null.
     *
     * @param agent the agent we want to get the local reference to
     * @return the local reference, or null
     */ 
    public AgentImpl getLocalReference( Agent a ) {
        if( a instanceof AgentImpl ) {
            return (AgentImpl)a;

        } else
            // The thing we have is only a stub, we need to look up the
            // real object from localAgentRefs.
	    try {
		return agentTable.getImplementation( a );

	    } catch( AgentNotFoundException error ) {
		if( a.toString().indexOf( "(remote)" ) < 0 )
                    Debug.warning( "Cell found a pointer to an Agent that was local, but not in the local list: " + a );
                return null;
		
	    }

    }


    /////////////////////////////////////////////////////////////////////////
    // Methods that the cell uses to track what the agents are up to.      //
    /////////////////////////////////////////////////////////////////////////
  
    /** 
     * Note when an agent sends a message to another agent. Agents
     * should call this on all messages so the cell can keep
     * track. Ideally, the cell should be able to extract this from
     * the RMI base.  Messages that have a return should also include
     * a notification of both "message" and "message repl"
     *
     * @param methodName the method that was called. (Add " repl" for a reply.)
     * @param sender the object asking for the broadcast of this notiifcation
     * @param from The agent making the call.
     * @param to The agent receiving the call.  */
    public void noteAgentMessage( Object sender, String s, Agent from, Agent to ) {
      	if( cellEventMonitor != null )
      	    cellEventMonitor.sendAgentMessage( sender, s, from, to );

    }

    /** 
     * Note when one agent connects to another. Agents should call this
     * so the cell can keep track.
     *
     * @param sender the object asking to broadcast this notification
     * @param from The agent initiating the connection
     * @param to The agent receiving the connection
     */
    public void noteAgentConnect( Object sender, Agent from, Agent to ) {
  	if( cellEventMonitor != null )
	    cellEventMonitor.sendAgentsConnecting( sender, from, to );
      	
    }

    /** 
     * Note when an agent disconnects from another. Agents should call
     * this so the cell can keep track.
     *
     * @param sender the object asking to broadcast this notification
     * @param from The agent breaking the connection
     * @param to The agent that was receiving the connection
     */
    public void noteAgentDisconnect( Object sender, Agent from, Agent to ) {
    	if( cellEventMonitor != null )
	    cellEventMonitor.sendAgentsDisconnecting( sender, from, to );

    }

    /**
     * note when an agent disconnects from all -- agents should call
     * this so the cell can keep track
     *
     * @param sender the object sending this information
     * @param agent the agent that is disconnecting 
     */
    public void noteAgentDisconnectFromAll( Object sender, Agent from ) {
	if( cellEventMonitor != null ) 
	    cellEventMonitor.sendAgentDisconnectingAll( sender, from );
	
    }

    /** 
     * Note when an agent has appeared somewhere on the scene. Cell
     * uses this to keep track of things and echo it to the UI.
     * 
     * @param sender the object asking to broadcast this notification
     * @param agent The agent who is around 
     */
    public void noteAgentBorn( Object sender, Agent agent ) {
	if( cellEventMonitor != null )
	    cellEventMonitor.sendAgentBorn( sender, agent );

    }

    /** 
     * Note when an agent has disappeared somewhere. Cell uses this to
     * keep track of things and echo it to the UI.
     * 
     * @param sender the object asking to broadcast this notification
     * @param agent The agent who is around 
     */
    public void noteAgentDisappear( Object sender, Agent agent ) {
	if( cellEventMonitor != null )
	    cellEventMonitor.sendAgentDying( sender, agent);

    }

    /** 
     * Note when an agent has moved somewhere. Cell uses this to
     * keep track of things and echo it to the UI.
     * 
     * @param sender the object asking to broadcast this notification
     * @param agent The agent who is around 
     * @param destinationAddress the address the agent is moving to
     */
    public void noteAgentMovedTo( Object sender, Agent agent, CellAddress destinationAddress ) {
	if( cellEventMonitor != null )
	    cellEventMonitor.sendAgentMovingTo( sender, agent, destinationAddress);

    }

    /** 
     * Note when an agent has moved in. Cell uses this to
     * keep track of things and echo it to the UI.
     * 
     * @param sender the object asking to broadcast this notification
     * @param agent The agent who is around 
     * @param sourceAddress the address the agent has moved in from
     */
    public void noteAgentMovedFrom( Object sender, Agent agent, CellAddress sourceAddress ) {
	if( cellEventMonitor != null )
	    cellEventMonitor.sendAgentMovingFrom( sender, agent, sourceAddress);

    }

    /** 
     * Report a message to the user in a nice way.
     * Typically, agents shouldn't need to call this.
     *
     * @param sender the object asking to broadcast this notification
     * @param msg the message to display via the UI
     */
    public void report( Object sender, String msg ) {
	if( cellEventMonitor != null )
	    cellEventMonitor.sendCellReport( sender, msg );

    }

    ///////////////////////////////////////////////////////////////////////////
    // Methods to keep track of the agent population. Not for agents.        //
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Do everything necessary to create and launch an agent of this type
     *
     * @param agentClassName the class name of the agent we want to create
     * @param semanticFile the description for the agent we want to create
     * @param configFile the configuration file for the agent we want to create
     * @exception ClassNotFoundException thrown when the agentClassName doesn't map to anything
     * @exception InstantiationException thrown if we have problems creating the agent
     */
    public AgentImpl createNewAgent( String agentClassName, String descFile ) 
    throws ClassNotFoundException, InstantiationException {
	AgentImpl ai = createAgent( agentClassName, descFile );

	try {
	    return birthNewAgent( ai );
	} catch (AgentUnableToBirthException e) {
	    throw new InstantiationException("Agent " + Global.shortString(ai) + " was not created successfully.");
	}
    }

    /**
     * Do everything necessary to create and launch an agent of this type
     *
     * @param agentClassName the class name of the agent we want to create
     * @exception ClassNotFoundException thrown when the agentClassName doesn't map to anything
     * @exception InstantiationException thrown if we have problems creating the agent
     */
    public AgentImpl createNewAgent( String agentClassName ) 
    throws ClassNotFoundException, InstantiationException {
	AgentImpl ai = createAgent( agentClassName );

	try {
	    return birthNewAgent( ai );
	} catch (AgentUnableToBirthException e) {
	    throw new InstantiationException("Agent " + Global.shortString(ai) + " was not created successfully.");
	}
    }

    /**
     * Do everything necessary to create and launch an agent of this type
     *
     * @param agentClassName the class name of the agent we want to create
     * @param description the description of the agent that we want to birth
     * @exception ClassNotFoundException thrown when the agentClassName doesn't map to anything
     * @exception InstantiationException thrown if we have problems creating the agent
     */
    public AgentImpl createNewAgent( String agentClassName, Description description ) 
    throws ClassNotFoundException, InstantiationException {
	AgentImpl ai = createAgent( agentClassName );
	ai.setDescription( description );

	try {
	    return birthNewAgent( ai );
	} catch (AgentUnableToBirthException e) {
	    throw new InstantiationException("Agent " + Global.shortString(ai) + " was not created successfully.");
	}
    }

    /**
     * Do everything necessary to create and launch an agent of this type
     *
     * @param agentClass the class object of the agent we want to create
     * @param semanticFile the description for the agent we want to create
     * @exception InstantiationException thrown if we have problems creating the agent
     */
    public AgentImpl createNewAgent( Class agentClass, String descFile ) 
    throws InstantiationException {
	AgentImpl ai = createAgent( agentClass, descFile );

	try {
	    return birthNewAgent( ai );
	} catch (AgentUnableToBirthException e) {
	    throw new InstantiationException("Agent " + Global.shortString(ai) + " was not created successfully.");
	}
    }

    /**
     * Do everything necessary to create and launch an agent of this type
     *
     * @param agentClass the class object of the agent we want to create
     * @exception InstantiationException thrown if we have problems creating the agent
     */
    public AgentImpl createNewAgent( Class agentName ) 
    throws InstantiationException {
	AgentImpl ai = createAgent( agentName );

	try {
	    return birthNewAgent( ai );
	} catch (AgentUnableToBirthException e) {
	    throw new InstantiationException("Agent " + Global.shortString(ai) + " was not created successfully.");
	}
    }

    /** 
     * Handle a new agent - account for it, then fork a thread for it. 
     * 
     * @param agent the agent to have this cell handle  
     * @return the argument, just to facilitate assignment without intermediate storage.
     */
    public synchronized AgentImpl birthNewAgent( final AgentImpl agent ) 
    throws AgentUnableToBirthException {
	report( this, "new agent " + Global.shortString(agent) );
	
	if( !agentTable.containsImplementation( agent ) ) {
	    agentTable.runAgentThread( agent );

	    (new Thread( new Runnable() {
		public void run() {
		    agent.blockUntilReady();
		    	    
		    try {
			agentTable.getRemoteReference( agent );
			noteAgentBorn( this, agent );
			
		    } catch( AgentNotFoundException error ) {
			Debug.error( Global.shortString( agent ) + " must not have started up appropriately" );

		    }

		}
	    })).start();

	} else
	    Debug.error( "agent being born, but it is already in our data structures for being local?" );

	return agent;

    }

    /** 
     * handle killing the agent -- this will pass it down to the agent
     * table which will do the actual killing
     *
     * @param agent the agent to kill 
     */
    public boolean killAgent( final AgentImpl agent ) {
	try {
	    // first ask the agent table to remove the agent (this will
	    // involve killing the agent)
	    boolean returnValue = agentTable.removeAgent( agent, AgentImpl.AGENTKILLED );

	    // tell everybody that this agent has disappeared from the system
	    noteAgentDisappear( this, agent );

	    return returnValue;

	} catch( AgentNotFoundException error ) {
	    Debug.error( "asked to kill " + Global.shortString( agent ) + " which is not in the tables" );
	    return false;

	}

    }

    /////////////////////////////////////////////////////////////////////////
    // Methods that the cell uses internally. Not for agents.              //
    /////////////////////////////////////////////////////////////////////////
  
    /** 
     * Calculate the port of the RMI registry, given the base port number. 
     * 
     * @param basePort the base port
     * @return the port of the RMI registry
     */
    private static int calcRegistryPort( int basePort ) { return basePort; }

    /** 
     * Get the port that the registry for this cell is running on. 
     *
     * @return the port that the registry is running on
     */
    private int getRegistryPort() { return calcRegistryPort( myAddress.getPort() ); }
    /** 
     * Get the port that the code cell for this cell is running on. 
     *
     * @return the port that the code cell is on
     */
    private int getCodeCellPort() { return httpCellShadow.getPort(); }

    /** 
     * Constructs an RMI Registry on the specified port if possible.
     * If the port is taken, throws an AlreadyBoundException.
     * If the argument is a 0, keeps trying ports until an open one
     * is found, or gives up after 10 tries and throws an
     * AlreadyBoundException.
     * Then binds this to "cell";
     * Returns the port that the registry was
     * eventually bound on.
     *
     * this is a horrible piece of code.  this should be a lot easier,
     * maybe put it into the HiveRegistry to let it to try to do its
     * best 
     */
    protected int bindRegistry( int port ) 
    throws AlreadyBoundException {

	/// RMI internal stuff
	/** The RMI registry used to bootstrap finding the cell. **/
	registry = null;
	if( port < 0 )
	    throw new AlreadyBoundException(); // There should not be ports < 0;

	else if( port > 0 ) {
	    // Specified port...
	    try {
		registry = new HiveRMIRegistry( port );
		Debug.notice( "Creating a RMI registry on port " + ((HiveRMIRegistry)registry).getPort() );
	    } catch( RemoteException e ) {
		// There is already something using this port, so we cannot have it.
		throw new AlreadyBoundException( "Unable to attach to port " + port );
	    }

	} else {
	    // port = 0, Find an open port...
	    boolean success = false;

	    for(int i = 0; ( i < 10 ) && ( success == false ); i++ ) {
		port = (int)( Math.random() * 30000 ) + 2000;
		try {
		    registry = new HiveRMIRegistry( port );
		    success = true;
		}
		catch( RemoteException e ) { }
	    }

	    if( success == false )
		throw new AlreadyBoundException();

	}

	try {
	    registry.bind( "cell", this );
	} catch (RemoteException e) {
	    throw new AlreadyBoundException();
	}
	return port;
    }
    
    /** 
     * construct a cell implementation with the specified base port.
     * also sets up the security manager, a class file cell, an rmi
     * registry, and registers the cell at the registry as "cell".
     *
     * @param port the port to start on
     * @exception remoteexception, ioexception, alreadyboundexception
     * @see createhiveobjects()
     */
    public Cell( int port ) 
    throws RemoteException, CellConnectException, AlreadyBoundException {
	
	super();

	// initialize our home directory for ~/.hive and create the cell's dom object
 	Global.setConfigDir( System.getProperty( Global.ConfigDir, Global.getConfigDir() ) );
	dom = new DOM();


	// Try and grab an XML file to set the cell's description
	InputStream is = Global.getConfigAsInputStream( Global.CellDescription );
	if( is != null ) {

	    Document doc = dom.openDocument(is);
	    Description cellDesc = new Description(this, doc);
	    cellDesc.addSemantic("upSince", Long.toString(System.currentTimeMillis()), null, null);
	    setDescription(cellDesc);
	}
	else {
	    Debug.notice( "Cell Description file not found: '" + Global.CellDescription + "'. No description set for this cell." );
        }

	// See which global options are specified in the cell's description.
	configureStartupCellOptions();


	// create the security manager for the cell
	createSecurityManager();

	// Create an RMI Registry on the specified port if available.
	port = bindRegistry( port );

	// Find and set our address.
	myAddress = CellAddress.createLocalCellAddress( port );
	myRemoteCell = myAddress.getRemoteCell();

	// Figure out what OS we're running on.
	String osType = System.getProperty( "os.name" );
	if( osType.startsWith( "Windows" ) ) {
	    Debug.notice( "OS type \"" + osType + "\" looks like Windows." );
	    Global.windowsOS = true;

	} else {
	    Debug.notice( "OS type \"" + osType + "\" doesn't look like Windows." );
	    Global.windowsOS = false;
	}

	// Create a thread group for all the happy agents

	// set up the lists which will keep track of the agent states
	shuttingDownAgents = new Hashtable();
	movingAgents = new Vector();

	boolean noUI = !Boolean.getBoolean( Global.UI );
	if( noUI )
	    Debug.notice( "Running without the agent component manager shadow." );

	// Create the Hive Objects.
	createHiveObjects( noUI );

	// figure out what shadows need to be instantiated
	configureStartupShadows();
	shadowTable.lockShadows();

	// Create the Hive Agents.

        // Should we create a GUI agent?
        boolean makeUIAgent;
        boolean userWantsGUIAgent = Boolean.valueOf(System.getProperty(Global.GUIAgent, "true")).booleanValue();
        if (noUI || !userWantsGUIAgent)
            makeUIAgent = false;
        else
            makeUIAgent = true;
        createHiveAgents(!makeUIAgent);
	
	// figure out which agents need to be instantiated
	configureStartupAgents();

	report( this, "" );
	report( this, "Copyright (c) 1998-1999 by the Massachusetts Institute of Technology. All rights reserved." );
	report( this, "Hive has no warranty, not even the implied warranty of merchantability or fitness for a particular purpose.");
	report( this, "Hive is free software. For details, see the GNU General Public License, COPYING.GPL");
	report( this, "For more information on Hive, see http://hive.media.mit.edu/");
	report( this, "Hive version: " + Global.version + " Cell $Revision: 1.3 $" +" Java " +System.getProperty( "java.version" ));
	report( this, "Running on " + getAddress() );
	report( this, "" );
    }


    protected void configureStartupCellOptions() { // Put the ability to change port in description too.
	Properties props = System.getProperties();
	Description root = description.configContext();
	if (root == null) return;

	// Go through the config section and set any variables that are specified, but haven't been set on the commandline
	if ( root.selectFirst("DebugLevel") && !Global.argList.contains("DebugLevel") ) {
	    String debug = root.getMatch().getText();
	    if ( debug.equalsIgnoreCase("notice") ) {
		Debug.setDefaultLevel( Debug.NOTICE );
	    }
	    else if ( debug.equalsIgnoreCase("warning") ) {
		Debug.setDefaultLevel( Debug.WARNING );
	    }
	    else if ( debug.equalsIgnoreCase("error") ) {
		Debug.setDefaultLevel( Debug.ERROR );
	    }
	    else if ( debug.equalsIgnoreCase("critical") ) {
		Debug.setDefaultLevel( Debug.CRITICAL );
	    }
	    else if ( debug.equalsIgnoreCase("failure") ) {
		Debug.setDefaultLevel( Debug.FAILURE );
	    }

	}

	if ( root.selectFirst("GUIAgent") && !Global.argList.contains("GUIAgent") ) {
	    String ui = root.getMatch().getText();
	    if ( ui == null || ui.equalsIgnoreCase("true") ) {
		props.put( Global.GUIAgent, "true" );
	    }
	    else {
		props.put( Global.GUIAgent, "false" );
	    }
	}

	if ( root.selectFirst("UI") && !Global.argList.contains("UI") ) {
	    String ui = root.getMatch().getText();
	    if ( ui == null || ui.equalsIgnoreCase("true") ) {
		props.put( Global.UI, "true" );
	    }
	    else {
		props.put( Global.UI, "false" );
		props.put( Global.GUIAgent, "false" );
	    }
	}

	if ( root.selectFirst("JoinCellList") && !Global.argList.contains("JoinCellList") ) {
	    String join = root.getMatch().getText();
	    if ( join == null || !join.equalsIgnoreCase("false") ) {
		props.put( Global.JoinCellList, "true" );
		if (join != null && !join.equalsIgnoreCase("true") ) { // Not null, not true, not false: must be a cell address
		    props.put( Global.MasterCellListCell, join );
		}

	    } else {
		props.put( Global.JoinCellList, "false" );
	    }

	}

    }



    protected void configureStartupShadows() {

	Description root = description.configContext();
	if (root == null) return;


	// get the shadows
	int num = root.select("shadow");
	String[] className = new String[num];
	String[] descName = new String[num];

	Description shadow;
	for (int i = 0; i < num; i++) {
	    shadow = root.getMatch(i);
	    if (!shadow.selectFirst("class")) {
		className[i] = null;
		descName[i] = null;
		continue;
	    }

	    className[i] = shadow.getMatch().getText();
	    if (!shadow.selectFirst("desc")) {
		descName[i] = null;
		continue;
	    }

	    descName[i] = shadow.getMatch().getText();
	}


	for( int i = 0; i < className.length; i++ ) {

	    try {
		Debug.notice("Going to create shadow " + className[i] + ".");
		if( createShadow( className[i], descName[i] ) == null )
		    Debug.error( className[i] + " does not seem to be a valid shadow to create" );
		else {  // Add the shadow to the semantic description of the cell
		    description.addSemantic("shadow", className[i], null, null);
		}
	    } catch( ClassNotFoundException error ) {
		Debug.error( className[i] + " does not seem to be a valid shadow to create" );
	    } catch( ShadowInitializationException error ) {
		Debug.error( className[i] + " did not initialize properly" );
	    }
	}
    }


    protected void configureStartupAgents() {
	
	Description root = description.configContext();
	if (root == null) return;

	// get the agents
	int num = root.select("agent");

	Hashtable namedAgentHash = new Hashtable();
	Vector connectToList = new Vector();
	String[] className = new String[num];
	Description[] agent = new Description[num];
	String[] descName = new String[num];

	for (int i = 0; i < num; i++) {
	    agent[i] = root.getMatch(i);
	    if (!agent[i].selectFirst("class")) {
		className[i] = null;
		descName[i] = null;
		Debug.error("Malformed Agent Tag in " + Global.CellDescription + ".  Agent requires <class> tag.");
		continue;
	    }
	    className[i] = agent[i].getMatch().getText();


	    if (!agent[i].selectFirst("desc")) {
		descName[i] = null;
		continue;
	    }
	    descName[i] = agent[i].getMatch().getText();
	}


	// Class which holds an agent and the name of another agent it wants to connect to. The next loop makes the connections, if it can
	class ConnectingAgents {
	    AgentImpl from;
	    String to;
	    ConnectingAgents(AgentImpl ag, String name) {
		from = ag;
		to = name;
	    }

	    public AgentImpl getFromAgent() { return (from); }
	    public String getToName() { return (to); }
	}


	// Now create the agents, and store the information needed to configure them.
	for( int i = 0; i < num; i++ ) {
	    if (className[i] == null) continue;

	    Debug.notice( "creating agent " + className[i] );
	    AgentImpl ai = null;
	    try {
		// AgentImpl ai = createNewAgent( Class.forName(className[i]), descName[i] , null );
		// NOTE: Treat the config file and the semantic file the same.  They're not separate anymore.
		if( agent[i].selectFirst( "inlineDescription" ) ) {  // Give the agent the inline description
		    Description iDesc = agent[i].getMatch();
		    if( iDesc.selectFirst( "description" ) )
			iDesc = iDesc.getMatch();
		    ai = createNewAgent( className[i], new Description( ai, iDesc.getNode().cloneNode( true ) ) );

		} else
		    ai = createNewAgent( Class.forName( className[i] ), descName[i] );

		ai.blockUntilReady();  // NOTE: Raffi, this stops up the whole cell now.  Do you know what happened to it?

	    } catch( ClassNotFoundException error ) {
		Debug.error( "\"" + className[i] + "\" as specified in the agent config file is not found" );
		continue;

	    } catch( InstantiationException error ) {
		Debug.error( "\"" + className[i] + "\" as specified cannot be instantiated" );
		continue;

	    } 
	    
	    if (ai == null) {
		Debug.error( "\"" + className[i] + "\" as specified in the agent config file could not be created!" );
		continue;
	    }


	    // Now, after it has been successfully created, store the agent's name if it has one, may be used later when connecting them.
	    String aName = agent[i].getAttribute("name");
	    
	    if (aName != null) {
		if (namedAgentHash.containsKey(aName))
		    Debug.error("In " + Global.CellDescription + ":  More than one agent given the name '" + aName +"'.  Names must be unique.  Skipping.");
		else
		    namedAgentHash.put(aName, ai);
	    }


	    // See if the agent should connect to anything, and store it if it does
	    int numConnects = agent[i].select("connectTo");
	    for (int j = 0; j < numConnects; j++) {

		Description conn = agent[i].getMatch(j);
		String cName = conn.getText();

		connectToList.addElement(new ConnectingAgents(ai, cName));
	    }
	}


	// Go through the list of which agents want to connect to other agents, and connect them.
	for (int i = 0; i < connectToList.size(); i++) {
	    
	    AgentImpl ag = ((ConnectingAgents)connectToList.elementAt(i)).getFromAgent();
	    String cName = ((ConnectingAgents)connectToList.elementAt(i)).getToName();

	    if (namedAgentHash.containsKey(cName))
		ag.connectTo((AgentImpl)namedAgentHash.get(cName));
	    else
		Debug.warning("In " + Global.CellDescription + ":  Agent told to connect to '" + cName +"', which was not specified.  Not connecting.");
	}
    }




    /**
     * set up the security manager for this cell
     */
    protected void createSecurityManager() {
        // Override Hive security manager..

	// this should be commented out, right?
	/*
	DescSet cellDesc = new DescSet();
	if( description != null )
	    cellDesc.addDesc( description );
	*/
       
	// Set up a security manager (Change in JDK 1.2)
	Class securityManagerClass = (net.hivecell.hive.cell.HiveSecurityManager.class);


	//DescSet securityManagerDescSet = cellDesc.select( Description.HIVE_CELL + "securityManager" );
	Description securityManagerDesc = null;
	if ( description != null && description.selectFirst("securityManager") ) 
	    securityManagerDesc = description.getMatch();


	if( securityManagerDesc != null) { //.size() != 0 ) {
	    //DescSet securityManagerClassNameDescSet = securityManagerDescSet.select( Description.HIVE_CELL + "className" );
	    Description securityManagerClassNameDesc = null;
	    if (securityManagerDesc.selectFirst("className")) {
		securityManagerClassNameDesc = securityManagerDesc.getMatch();
	    } else {
		Debug.warning( "security manager class name not specified -- using default hive security manager" );
	    }


	    if (securityManagerClassNameDesc != null) {
		String securityManagerClassName;
		if( !( securityManagerClassName = securityManagerClassNameDesc.getText().trim()).equals( "" ) ) {
		    try {
			Class securityManagerProposedClass = Class.forName( securityManagerClassName );
			if( !(SecurityManager.class).isAssignableFrom( securityManagerProposedClass ) )
			    Debug.warning( "\"" + securityManagerClassName + "\" is not a valid security manager -- using default hive security manager" );

			else {
			    Debug.notice( "setting security manager type to \"" + securityManagerClassName + "\"" );
			    securityManagerClass = securityManagerProposedClass;
			}

		    } catch( ClassNotFoundException error ) {
			Debug.error( "\"" + securityManagerClassName + "\" is not a valid security manager -- using default hive security manager" );
		    }
		}
	    }
	} else 
	    Debug.notice( "setting up the standard HiveSecurityManager" );



	// if we have a custom description -- and it contains more
	// than just the className tag, then let's pass it into
	// the security manager -- otherwise just construct one
	// without custom constructor
	try {
	    if( description != null && !securityManagerClass.equals(net.hivecell.hive.cell.HiveSecurityManager.class) ) {
		Constructor[] constructors = securityManagerClass.getConstructors();

		for(int count = 0; count < constructors.length; count++) {
		    Class[] constructorClasses = constructors[count].getParameterTypes();
		    if( constructorClasses.length == 1 && 
			constructorClasses[count].equals( (net.hivecell.hive.description.Description.class) ) ) {

			Object[] constructorObjects = { securityManagerDesc };
			securityManager = (SecurityManager)constructors[count].newInstance( constructorObjects );
		    }
		}
	    } else 
		securityManager = (SecurityManager)securityManagerClass.newInstance();

	} catch( Exception error ) {
	    // if we get either an instantiation exception, or an
	    // illegal access exception -- then just puke and setup
	    // the standard HiveSecurityManager
	    Debug.error( "could not create a \"" + securityManagerClass.getName() + "\" defaulting to the HiveSecurityManager", error );
	    securityManager = new HiveSecurityManager();
	}
	System.setSecurityManager( securityManager );

    }

    /** 
     * add Hive-specific objects to the Cell. 
     *
     * @param noUI whether we are booting this cell with or without a GUI
     */
    protected void createHiveObjects( boolean noUI ) {

	// this has to be cleaned up big time.  it is a mess, we can't tell where the errors are ocming from
	try {
	    httpCellShadow = (HTTPServerShadow)createShadow( (HTTPServerShadowImpl.class).getName(), null );
	    Debug.notice( "codebase = " + httpCellShadow.getCodebase() );

	    cellEventMonitor = (CellEventMonitor)createShadow( (CellEventMonitorImpl.class).getName(), null );

	    // setup the default font and also set up the agent component
	    // manager shadow, but only if we are running with a GUI
	    Global.defaultFont = new java.awt.Font( "Helvetica", java.awt.Font.PLAIN, 10 );
	    if( !noUI )
		componentManagerShadow = (ComponentManagerShadow)createShadow( (ComponentManagerShadowImpl.class).getName(), null );

	    // set up the cell control shadow to allow for commands to be
	    // sent into the cell
	    cellControlShadow = (CellControlShadow)createShadow( (CellControlShadowImpl.class).getName(), null );
	    
	    // create the agent messaging shadow
	    agentMessagingShadow = (AgentMessagingShadow)createShadow( (AgentMessagingShadowImpl.class).getName(), null );

	} catch( Exception error ) {
	    Debug.error( "errors setting up the internal shadows for the cell", error );
	    System.exit( 0 );

	}

	// start the agent gatekeeper -- the security manager for
	// accepting incoming agents
	agentGatekeeper = new AgentGatekeeper();
	agentGatekeeper.setAcceptAgent( true );

    }

    /** 
     * create agents for Hive startup.  after each agent is
     * individually created, the hive cell waits until they have fully
     * started up. this prevents intra-cell deadlocks
     *
     * @param noGUIAgent should the gui agent be started up
     */
    protected void createHiveAgents( boolean noGUIAgent ) {
	try {
	    // start the agent monitoring agent
	    createNewAgent( AgentMonitoringAgentImpl.class ).blockUntilReady();
	
	    // start the agent messaging agent
	    createNewAgent( AgentMessagingShadowAgentImpl.class ).blockUntilReady();

	    // start the cell control agent
	    createNewAgent( CellControlAgentImpl.class ).blockUntilReady();

	    // start the cell list agent
	    createNewAgent( CellListAgentImpl.class ).blockUntilReady();

	    if( !noGUIAgent ) {
		IconicUIAgentImpl auai = (IconicUIAgentImpl)createAgent( net.hivecell.hive.agent.hiveui.iconicui.IconicUIAgentImpl.class, null );

		birthNewAgent( auai );
		auai.blockUntilReady();
	    }

	} catch( InstantiationException error ) {
	    // this should never happen -- but if it does, oh shit
	    Debug.critical( error );
	} catch (AgentUnableToBirthException e) {
	    Debug.critical("Unable to create agent which is necessary for hive.  Dying now.", e);
	}
	
    }



    
    /** 
     * Tell the cell it needs to shut down. This code: <ul>
     * <li>stops serialization so it does not record that agents are dying
     * <li>kills all agents
     * <li>massive cleanup
     * <li>exits
     * </ul>
     * There really really should be some sort of check for permission.
     */
    public void quit() {
	Debug.notice( "Cell is quitting!" );
	shuttingDown = true;

	// launch off a reaper to wait for 30 seconds.  some of the
	// agent killers might need 15 seconds and this should provide
	// enough time for all of them to shut down
	class shutdownRunnable
	implements Runnable {
	    int toWait;
	    
	    public shutdownRunnable( int time ) { this.toWait = time; }

	    public void run() {
		Debug.notice( "Killing cell in " + toWait + " seconds to make sure all agents are dead" );
		
		try {
		    Thread.sleep( toWait * 1000 );
		} catch( InterruptedException error ) {
		    return;
		}

		System.exit( 0 );
		Debug.assert( false, "We called System.exit(0) and yet we're still running!" );
	    }
	}
	Thread shutdownThread = new Thread( new shutdownRunnable( 30 ) );
	shutdownThread.start();

	Debug.notice( "Turning off agent accept services" );
	agentGatekeeper.setAcceptAgent( false );

	if( cellEventMonitor != null ) {
	    Debug.notice( "Sending a shutdown through the cell event monitor." );
	    cellEventMonitor.sendCellGoingDown( this );
	    // Debug.notice( "Removing our event connections" );
	    // AgentMonitoringAgent.disconnectFromAll();
	}
	
	Debug.notice( "Killing off indiginous agents" );
	// Kill off the non-cell agents and set them up for garbage collection
	AgentImpl[] toKill = agentTable.getNonCellAgentArray();
	for( int count=0;count<toKill.length;count++ )
	    killAgent( toKill[count] );

	// now kill off the rest of the agents
	toKill = agentTable.getImplementationArray();
	for( int count=0;count<toKill.length;count++ )
	    killAgent( toKill[count] );

	// sit and wait until all our asynchronous messages have gone through just to make 
	// sure all the notificiation about killing off agents have completed
	Debug.notice( "Waiting until all asynchronous messages have been sent" );
	Global.asyncCallerClass.blockUntilDone();

    }

    ///////////////////////////////////////////////////////////////////////////
    // Debugging support and miscellany                                      //
    ///////////////////////////////////////////////////////////////////////////

    public static void main( String args[] ) {
        System.err.println( "to start hive, run the class net.hivecell.hive.Hive" );
        System.exit( 1 );

    }


    /**
     * this is just simpler than doing the synchronization and stuff
     * inside the Cell.java 
     */
    class ShadowTable {
	/**
	 * the flag stating whether we have locked the database
	 */
	private boolean locked = false;

	/**
	 * the mapping from shadows to their stubs
	 */
	private Hashtable shadowToStub = new Hashtable();

	/**
	 * the mapping from stubs to their shadows
	 */
	private Hashtable stubToShadow = new Hashtable();

	/**
	 * the mapping from the name to the shadow
	 */
	private Hashtable nameToShadow = new Hashtable();

    
	/**
	 * lock the shadow table down so we won't create any more shadows
	 */
	public synchronized void lockShadows() {
	    locked = true;

	}

	/**
	 * get whether we have locked the shadow database
	 *
	 * @return true if the table is locked
	 */
	public synchronized boolean isLocked() {
	    return locked;

	}

	/**
	 * create a shadow and add it to the table
	 *
	 * @param shadowName the shadow that we want to create
	 * @param paramFile the parameter file for the shadow
	 * @return the stub to the shadow
	 * @exception ClassNotFoundException if the shadow name passed is not a valid shadow
	 * @exception ShadowInitializationException if we can't initailize the shadow
	 */
	public synchronized Shadow createShadow( String shadowName, String propertiesFile ) 
	throws ShadowInitializationException, ClassNotFoundException {
	    // make sure we are allowing new shadows to be created
	    if( shadowTable.isLocked() )
		throw new ShadowInitializationException( "cannot create any more shadows" );
	
	    // load the class, and then make sure the class is a shadow
	    // before goign any further
	    Class shadowClass = Class.forName( shadowName );
	    if( !( ShadowImpl.class ).isAssignableFrom( shadowClass ) )
		throw new ShadowInitializationException( shadowName + " is not a shadow implementation" );
	    
	    // try creating the shadow
	    try {
		Debug.notice( "instantiating " + shadowName );
		ShadowImpl s = (ShadowImpl)shadowClass.newInstance();
		s.setCell( Cell.this );
		
		// configure the shadow using the properties file
		Debug.notice( "loading " + propertiesFile + " as the properties file for " + shadowName );
		s.loadDescription( propertiesFile );
		VariablePropertyManipulator.configure( s );
		
		Debug.notice( "initializing " + shadowName );
		s.init();
		
		// record this shadow
		shadowToStub.put( s, s.getStub() );
		stubToShadow.put( s.getStub(), s );
		nameToShadow.put( s.getClass().getName(), s );
		return (Shadow)s.getStub();
	
	    } catch( Exception error ) {
		throw new ShadowInitializationException( error );
		
	    }

	}

	/**
	 * get a shadow from the database
	 *
	 * @param shadow the name of the shadow we want to pull out
	 * @return the shadow
	 * @exception ShadowNotFoundException if we can't find it
	 */
	public synchronized Shadow getShadow( String shadow )
	    throws ShadowNotFoundException {
	    try {
		Class searching = Class.forName( shadow );
		if( !( ShadowImpl.class ).isAssignableFrom( searching ) ) {
		    if( !( Shadow.class ).isAssignableFrom( searching ) )
			throw new ShadowNotFoundException( shadow + " being requested is not a shadow" );
		
		    // ok - some smart alec passed us the Shadow interface
		    // that he wanted, not the implementation.  argh.  ok.
		    // let's just go through all our stubs, and if we find
		    // anything which implements what they want -- then
		    // let's just return that.  dumb ass.
		    synchronized( shadowToStub ) {
			Enumeration e = stubToShadow.keys();
			while( e.hasMoreElements() ) {
			    ShadowStub shadowStub = (ShadowStub)e.nextElement();
			    if( searching.isAssignableFrom( shadowStub.getClass() ) )
				return (Shadow)shadowStub;

			}
			throw new ShadowNotFoundException( "no shadows implement " + shadow );

		    }

		} else
		    try {
			// woo-hoo!  we can just do a bunch of look ups in the
			// hashtables and return him the shadow he wants
			return (Shadow)shadowToStub.get( ((ShadowImpl)nameToShadow.get( shadow )) );
		    
		    } catch( NullPointerException error ) {
			throw new ShadowNotFoundException();

		    }

	    } catch( ClassNotFoundException error ) {
		throw new ShadowNotFoundException( "do not know anything about " + shadow );

	    }

	}

	/**
	 * get an enumeration of all the shadows in the ssytem
	 *
	 * @return the keys in the shadow to stub mapping
	 */
	public Enumeration getShadows() {
	    return shadowToStub.keys();

	}
 
    }

    /**
     * this is a table holding all the information about the agents on the
     * system -- this maintains the mapping between the real agent, and
     * its stub.  this only keeps track of the agents local to this
     * system.  it is not the job of this mapping (or the cell in fact) to
     * keep track of agents which are not on this cell.
     *
     * this also maintains all the thread information about the agents
     */
    class AgentTable {
	/**
	 * the mapping from the implementation to the stub
	 */
	private Hashtable implementationToStub = new Hashtable();
    
	/**
	 * the mapping from teh stub to the implementation
	 */
	private Hashtable stubToImplementation = new Hashtable();

	/**
	 * the mapping from the agent implementation to the thread
	 * which is running the agent 
	 */
	private Hashtable implementationToThread = new Hashtable();

	/**
	 * the mapping from the agent implementation to the agent's
	 * thread group 
	 */
	private Hashtable implementationToThreadGroup = new Hashtable();

	/**
	 * the thread group for all the agents
	 */
	private ThreadGroup agentThreadGroup;
	
	/**
	 * the number of threads that we have created so far for agents
	 */
	private int numAgentThreads = 0;

	
	/**
	 * construct a new agent table.  this basically just
	 * initalizes the agent thread group 
	 */
	public AgentTable() {
	    agentThreadGroup = new AgentThreadGroup( "Agents" ); 
	    agentThreadGroup.setDaemon( false );
	    
	}

	/**
	 * start the agent off and running
	 *
	 * @param implementation the agent that we want to create this thread group for
	 * @return the agent thread group
	 */
	public synchronized void runAgentThread( final AgentImpl implementation ) {
	    final Agent remoteAgent = (Agent)getRemoteObject( implementation );

	    final ThreadGroup threadGroup = new ThreadGroup( agentThreadGroup, "A-" + numAgentThreads++ + " (" + Global.shortClassName( implementation ) + ")" );


	    class ThreadContainer extends Thread {

		ThreadContainer(String name) {
		    super(name);
		}


		public void run() {
		    try {
			implementation.doLocalSetup();

			// put everything into the tables
			synchronized( Cell.AgentTable.this ) {
			    implementationToStub.put( implementation, remoteAgent );
			    stubToImplementation.put( remoteAgent, implementation );
			    implementationToThread.put( implementation, this );
			    implementationToThreadGroup.put( implementation, threadGroup );

			}

			implementation.setIsReady( true );

			// start the agent running
			implementation.doBehavior();

			// ok, the agent has finished its behavior, did it really
			// mean to stop?
			if( !implementation.isTimeToStop() )
			    Debug.warning( "warning - doBehavior exited prematurely for " + Global.shortString( implementation ) );

		    } catch( AgentInitializationException e ) {
			Debug.error( "error doing local setup for " + Global.shortString( implementation ) + ", " + e );
			implementation.setIsReady(true);  // NOTE: Big hack here.  Set isReady so blockUntilReady goes thru, then 
			// in birthNewAgent do a remote Lookup of the agent, and it will fail if 
			// the agent died in its doLocalSetup.
		    }

		    // remove this agent from our data structures
		    synchronized( Cell.AgentTable.this ) {
			implementationToStub.remove( implementation );
			stubToImplementation.remove( remoteAgent );
			implementationToThread.remove( implementation );
			implementationToThreadGroup.remove( implementation );
			    
		    }
			
		}
	    }

	    
	    Thread agentThread = new ThreadContainer("main thread");

	    // set the cell of this agent to this and set the thread group to this
	    implementation.setCell( Cell.this );	
	    implementation.setThreadGroup( threadGroup );

	    agentThread.start();

	}

	/**
	 * remove an implementation from the system
	 *
	 * @param implementation the implementation that we want to remove from the system
	 * @param code the code used to kill the agent (agent moving or agent killed)
	 * @return true if we removed it, false otherwise
	 * @exception AgentNotFoundException if we don't have the agent in the data structure
	 */
	public synchronized boolean removeAgent( final AgentImpl implementation, int code ) 
	throws AgentNotFoundException {
	    // first make sure that we have the agent implementation
	    // in the data structures
	    if( !implementationToStub.containsKey( implementation ) )
		throw new AgentNotFoundException();

	    // get al lthe thread information from the agent
	    final Thread agentThread = (Thread)implementationToThread.get( implementation );
	    final ThreadGroup agentThreadGroup = (ThreadGroup)implementationToThreadGroup.get( implementation );

	    Debug.notice( "killing " + Global.shortString( implementation ) );

	    // this si the thread which will "nail" the agent if it
	    // does not die politely
	    class killAgentDash9Runnable
	    implements Runnable {
		// this will be set to true if we don't need this anymore
		boolean notNeeded = false;

		// this will set the flag to true
		public void setNotNeeded() {
		    notNeeded = true;

		}

		// wait 15 seconds (or until we are interrupted), then
		// find out if we are needed anymore -- if we are,
		// kill the agent harshly
		public void run() {
		    try {
			Thread.sleep( 15000 );
			
		    } catch( InterruptedException error ) {
			
		    }
		    if( !( notNeeded || ( agentThreadGroup.activeCount() == 0 ) ) )
			// this will kill the agent hard
			agentThreadGroup.stop();
		    

		    // remove the components from this agent
		    if( componentManagerShadow != null ) {
			Debug.notice( "removing components from " + Global.shortString( implementation ) );
			componentManagerShadow.removeComponentsFrom( implementation );
			
		    }
		    
		    // ask the agent to free all its local resources
		    implementation.doLocalCleanup();

		}

	    }
	    final killAgentDash9Runnable killAgentDash9Object = new killAgentDash9Runnable();
	    final Thread killAgentDash9Thread = new Thread( killAgentDash9Object, Global.shortString( implementation ) + " kill agent watcher" );

	    // this thread attempts to have the agent die cleanly.  we
	    // have to first notify the agent that we are about to
	    // kill it, then we set it as not ready, then we tell it
	    // is time to stop (and we pass it the code), we interrupt
	    // the agent's thread in case it is asleep and it is not
	    // checking the variables.  then we have to wait to see if
	    // the agent thread stops running.  if it does, we make
	    // sure we remove all the components, we call do local
	    // cleanup 
	    final Thread killAgentThread = new Thread( new Runnable() {
		    public void run() {
			// notify the agent that we are going to kill it.
			implementation.onDying();
	
			// this is the "funky" part -- check here to see if
			// this agent has started moving somewhere --
			// eventually we need to make this more elaborate and
			// see if the move actually succeedced if the agent
			// has started moving.  if the move did not succeed,
			// then this agent needs to be killed
			if( movingAgents.contains( implementation ) ) {
			    killAgentDash9Object.setNotNeeded();
			    killAgentDash9Thread.interrupt();

			    Debug.notice( Global.shortString( implementation ) + " has chosen to move, rather than being killed" );
			    return;

			}

			// tell the agent to not report that it is
			// ready anymore and ask the agent to get out
			// of its main loop and interrupt the agent's
			// thread in case it is in a wait or a sleep
			// so it can check out the new value of its
			// flags and start the shutdown
			implementation.setIsReady( false );
			implementation.setTimeToStop( AgentImpl.AGENTKILLED );
			agentThread.interrupt();

			// wait for the agent's thread to die
			try {
			    agentThread.join();
			    killAgentDash9Object.setNotNeeded();
			    killAgentDash9Thread.interrupt();
			    Debug.notice( Global.shortString( implementation ) + " has died politely" );

			} catch( InterruptedException error ) {

			}

		    }

		}, Global.shortString( implementation ) + " agent killer" );

	    // first remove the agent from all the tables so that he
	    // can't be picked up by anybody else while we are killing
	    // it
	    Agent agent = (Agent)implementationToStub.remove( implementation );
	    stubToImplementation.remove( agent );
	    implementationToThread.remove( implementation );
	    implementationToThreadGroup.remove( implementation );

	    // now start the threads for killing the agent
	    killAgentThread.start();
	    killAgentDash9Thread.start();

	    return true;

	}

	/**
	 * get the remote reference for the agent.  we won't have the
	 * implementation of an agent unless we have this agent, so
	 * then we just look up for the remote reference.  if they
	 * pass us a remote reference to begin with, then we simply
	 * return it.
	 *
	 * @param agent the agent that we want the remote reference for
	 * @return the remote reference to the agent as it is in the hashtable
	 * @exception AgentNotFoundException if the actual imlementation cannot be found in the lookup table

	 */
	public synchronized Agent getRemoteReference( AgentImpl agent ) 
	throws AgentNotFoundException {
	    if( !implementationToStub.contains( agent ) )
		throw new AgentNotFoundException();
	    return (Agent)implementationToStub.get( agent );

	}

	/**
	 * get the real reference to this remote reference.  
	 *
	 * @param agent the remote reference to the agent that we want to get
	 * @return the real reference to the agent
	 * @exception AgentNotFoundException if the aremote reference is not in our tables
	 */
	public synchronized AgentImpl getImplementation( Agent agent ) 
	throws AgentNotFoundException {
	    Agent toLookup = agent;
	    if( toLookup instanceof AgentImpl )
		toLookup = getRemoteReference( (AgentImpl)toLookup );
	    if( !stubToImplementation.contains( toLookup ) )
		throw new AgentNotFoundException();
	    return (AgentImpl)stubToImplementation.get( toLookup );

	}

	/**
	 * test to see if we have a certain implementation in the system
	 *
	 * @param agent the agentimpl that we want to test out
	 */
	public synchronized boolean containsImplementation( Agent agent ) {
	    return implementationToStub.containsKey( agent );

	}
	
	/**
	 * test to see if we have a certain remote reference in the system
	 *
	 * @param agent the remote reference we want to test out
	 */
	public synchronized boolean containsRemoteReference( Agent agent ) {
	    Agent toLookup = agent;
	    if( toLookup instanceof AgentImpl )
		try {
		    toLookup = getRemoteReference( (AgentImpl)toLookup );

		} catch( AgentNotFoundException error ) {
		    return false;

		}
	    return stubToImplementation.containsKey( toLookup );

	}

	/**
	 * get the thread of an agent
	 *
	 * @param implementation the agent we want the thread of
	 * @return the thread of the agent (or null, if we don't yet have the thread information)
	 * @exception AgentNotFoundException if we don't know anything about this implementation
	 */
	public synchronized Thread getAgentThread( AgentImpl implementation ) 
	throws AgentNotFoundException {
	    if( !implementationToStub.contains( implementation ) )
		throw new AgentNotFoundException();
	    return (Thread)implementationToThread.get( implementation );

	}

	/**
	 * get the thread group of an agent
	 *
	 * @param implementation the agent we want the thread group of
	 * @return the thread group of the agent (or null, if we don't yet have the thread information)
	 * @exception AgentNotFoundException if we don't know anything about this implementation
	 */
	public synchronized ThreadGroup getAgentThreadGroup( AgentImpl implementation ) 
	throws AgentNotFoundException {
	    if( !implementationToStub.contains( implementation ) )
		throw new AgentNotFoundException();
	    return (ThreadGroup)implementationToThreadGroup.get( implementation );

	}

	/**
	 * get a listing of all the implemenations that we have in the system
	 *
	 * @return an enumeration of all the agentimpl
	 */
	public synchronized Enumeration getImplementations() {
	    return implementationToStub.keys();

	}

	/**
	 * get an array of all the implementations in the system
	 *
	 * @return an array of the AgentImpl
	 */
	public synchronized AgentImpl[] getImplementationArray() {
	    AgentImpl[] agents = new AgentImpl[implementationToStub.size()];
	    Enumeration e = getImplementations();
	    int count = 0;
	    while( e.hasMoreElements() )
		agents[count++] = (AgentImpl)e.nextElement();
	    return agents;

	}

	/**
	 * get an array of all the non cell agents
	 *
	 * @return an array of AgentImpls which are not CellAgents
	 */
	public synchronized AgentImpl[] getNonCellAgentArray() {
	    AgentImpl[] agents = getImplementationArray();
	    Vector nonCellAgents = new Vector();
	    for( int count=0;count<agents.length;count++ )
		if( !( agents[count] instanceof CellAgent ) )
		    nonCellAgents.addElement( agents[count] );
	    agents = new AgentImpl[nonCellAgents.size()];
	    nonCellAgents.copyInto( agents );
	    return agents;

	}

	/**
	 * get a listing of all the remote references to the agents
	 *
	 * @return an enumeration of all the agents
	 */
	public synchronized Enumeration getRemoteReferences() {
	    return stubToImplementation.keys();

	}
	
    }

    /**
     * this is the object which will deny or accept different agents on
     * the basis of a flag which tells us whether we can accept agents, or
     * whether or not the agent has the proper tokens.  this is a truly
     * crappy way to do it -- we need some form of signing or something
     * like that 
     */
    class AgentGatekeeper {
	/**
	 * this is the flag which will tell us whether we can accept
	 * agents or not 
	 */
	private boolean acceptAgents = false;


	/**
	 * the string which is setting for the default accept
	 */
	private static final String DEFAULTACCEPT = "default-accept";

	/**
	 * the string which is the setting for the default provide
	 */
	private static final String DEFAULTPROVIDE = "default-provide";
    

	/**
	 * the file name of the security tokens
	 */
	private static final String FILENAME = "security-tokens";

	/**
	 * the header for the hive security tokens
	 */
	private static final String HEADER = "Hive V2.0 Security Tokens";


	/**
	 * the properties which holds our tokens
	 */
	private Properties tokens;

	/** 
	 * Initialize the gatekeeper, loading data from the standard
	 * location.
	 */
	public AgentGatekeeper() {
	    tokens = new Properties();
	    InputStream is = Global.getConfigAsInputStream( FILENAME );
	    if( is != null )
		try {
		    tokens.load( is );

		} catch( IOException e ) {
		    Debug.warning( "Unable to load config file from stream." );

		}

	}

	/**
	 * set the flag on whether we are accepting agents or not
	 *
	 * @param flag true if we want to accept, false otherwise
	 */
	public void setAcceptAgent( boolean flag ) {
	    acceptAgents = flag;

	}

	/**
	 * tell us whether we are accepting agents or not
	 *
	 * @return true if we are accepting agents
	 */
	public boolean getAcceptAgent() {
	    return acceptAgents;

	}

	/** 
	 * Returns true if the supplied token authorizes access to this cell. 
	 *
	 * @param token the token that we want to see if it can authorize
	 * @return true if we are authorized, false otherwise
	 */
	public boolean verifyToken( Object token ) {
	    if( containsToken( tokens.get( Global.mungeAddress( myAddress.toString() ) ), token ) )
		return true;

	    Object defaultAccept = tokens.get( DEFAULTACCEPT );
	    if( token.equals( defaultAccept ) )
		return true;
	    return false;

	}

	/** 
	 * Returns the object needed to authorize access to a remote
	 * cell.  If there are multiple present in the security
	 * properties file, then the first one is returned. This is so
	 * that a securityfile can be used unmodified between multiple
	 * cell. (Although this doesn't provide much security)
	 *
	 * @param address the address that we want to get the token for 
	 * @return the token that we need to move to that address */
	public Object getToken( CellAddress address ) {
	    String token = (String)tokens.get( Global.mungeAddress( address.toString() ) );
	    if( token != null ) {
		int index = token.indexOf( "," );
		if( index == -1 )
		    return token;

		else
		    return token.substring( 0, index );

	    } else
		return tokens.get( DEFAULTPROVIDE );

	}

	/** 
	 * Return true iff token is in list. (A comma delimited list) 
	 *
	 * @param list the list that we want to see if the token is in
	 * @param token the token that we want to test
	 * @return true if the list contains the token
	 */
	private boolean containsToken( Object list, Object token ) {
	    if( list == null )
		return false;

	    StringTokenizer st = new StringTokenizer( (String)list, "," );	
	    while( st.hasMoreTokens() )
		if( token.equals( st.nextToken() ) )
		    return true;
	    return false;

	}

	/** 
	 * For debugging purposes. 
	 *
	 * @return the gatekeeper with all the tokens
	 */
	public String toString() {
	    return "AgentGatekeeper: " + tokens.toString();

	}

	//      /** This static main method creates a dummy tokens file that can later
	//       *  Be modified by hand. It is just here for convenience. */
	//      public static void main(String[] args) {
	//  	Properties props = new Properties();
	//  	if (args.length != 2) {
	//  	    System.out.println("AgentGatekeeper DEFAULTACCEPT DEFAULTPROVIDE");
	//  	    System.exit(0);
	//  	}
	//  	props.put(DEFAULTACCEPT, args[0]);
	//  	props.put(DEFAULTPROVIDE, args[1]);
	//  	props.put("host-port", "You must use hyphens (-) instead of colons (:) for addresses to avoid confusing the properties parser");
	//  	try {
	//  	    FileOutputStream fos = new FileOutputStream(FILENAME, false);
	//  	    props.save(fos, HEADER);
	//  	    fos.flush();
	//  	    fos.close();
	//  	}
	//  	catch (IOException e) {
	//  	    Debug.warning("Unable to create dummy tokens file");
	//  	}
	//  	Debug.notice("Creating dummy tokens file as " + FILENAME);
	//  	Debug.notice("This file needs to go into your config directory");
	//      }

    }


}



