// $Id: HiveSecurityManager.java,v 1.1 2000/05/18 22:01:04 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.Shadow;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;
import net.hivecell.hive.support.Debug;

import java.io.FileDescriptor;
import java.net.InetAddress;
import java.util.Vector;


/** 
 * this security manager is the default manager that hive will run
 * with.  the more advanced version, the MIT version, is meant for
 * higher security and will attempt to setup a virtual firewall
 *
 * this will be changed big time come jdk 1.2 
 */
public class HiveSecurityManager 
extends SecurityManager {

    /**
     * this method tests to see if a piece of remote code is allowing
     * for an incoming connection, if it is then we need to throw our
     * security exception
     *
     * @param host the host that the connection is comming from
     * @param port the port the connection is coming in 
     */
    public void checkAccept( String host, int port ) { 

	String error = "remote code is attempting to accept a connection from " + host + " on port " + port;
	testRemoteExecuting( error );

    }

    /**
     * we need to test here to make sure if remote code is tampering
     * with thread groups.  remote code can only mess with its own
     * thread group, otherwise we are going to go nuts
     *
     * @param g the thread group that is being messed with 
     */
    public void checkAccess( ThreadGroup g ) { 

    }

    /**
     * this should follow the same convention as the checkAccess for
     * thread groups
     *
     * @param t the thread
     */
    public void checkAccess( Thread t ) { 

    }

    /**
     * we are letting anything get through to the awt event queue
     */
    public void checkAwtEventQueueAccess() { 

    }
    
    /**
     * we are probably going to have to let this one go since the
     * mobile code will need to be in contact with its implementation.
     * if we go bezerk, we might be able to do some test to make sure
     * the mobile code is only talking to its implementation and not
     * something else
     *
     * @param host the host that the code is connecting to
     * @param port the port that it is connecting to 
     */
    public void checkConnect( String host, int port ) { 

    }

    /**
     * same deal as the other checkConnect method
     */
    public void checkConnect( String host, int port, Object context ) { 

    }
    
    /**
     * nothing is allowed to delete files!  throw a warning here immediately
     *
     * @param file the file that something is attempting to delete
     */
    public void checkDelete( String file ) {
        Class[] c = getClassContext();
        // (nelson) javacomm 2.0 does a bogus checkDelete() as a security
        // check. Argh! We will let this one class "delete files".
        if( javax.comm.CommPortIdentifier.class.equals( c[1] ) && primordialClass( c[1] ) )
            return;
        else
            throw new SecurityException( "something is attempting to delete \"" + file + "\"" );
    }

    /**
     * ok.  no remote code should ever be calling anything that is
     * causing something to be execced -- unless there is a shadow in
     * the stack that is causing this execution.  shadows are trusted
     * code.
     *
     * @param cmd the command that somebody is trying to execute 
     */
    public void checkExec( String cmd ) { 

	// just go right up the stack.. if we bump into a shadow, then
	// we are good and we should exit.  if we bump into remote
	// code, throw the exception
	String error = "a piece of remote code is attempting to exec \"" + cmd + "\"";
	if( !testShadowExecuting( error ) )
	    // hmm - we got all the way up the stack -- something is
	    // wrong.  ok...  just throw an exception.  if this gets
	    // thrown like this then we'll have to inspect what is
	    // happening
	    throw new SecurityException( "got all the way up the stack and there is no shadow execcing \"" + cmd + "\"" );

    }
    
    /** 
     * as of right now, nothing should linking in code.  but we'll *
     * follow the same policy as we did when checking for stuff that
     * execced code
     *
     * @param lib the name of the library 
     */
    public void checkLink( String lib ) { 
	
	if( !testJavaOrShadowExecuting( "a piece of remote code is attempting to link in \"" + lib + "\"" ) )
	    throw new SecurityException( "got all the way up the stack and there is no shadow linking \"" + lib + "\"" );

    }

    /**
     * as of right now, we are just not letting any mobile code to
     * receive incoming connections or even to listen for them...  not
     * sure if this is right, but i'll know if it breaks
     *
     * @param port the port they are trying to listen in on 
     */
    public void checkListen( int port ) { 
	testRemoteExecuting( "remote code is attempting to listen for incoming connections on " + port );
    }

    /**
     * for now we will just let this one go through -- not sure what
     * we will implement to make it more difficult later
     *
     * @param testingClass the class they are trying to access
     * @param which no clue what this is here for 
     */
    public void checkMemberAccess( Class testingClass, int which ) { 

    }

    /**
     * nobody should be doing multicast currently -- let's just turn
     * it off, i don't want to think of security breaches coming
     * through multicast
     *
     * @param maddr the address for multicast 
     */
    public void checkMulticast( InetAddress maddr ) { 
	throw new SecurityException( "somebody is attempting to send something through multicast" );
    }

    /**
     * again, nothing should be sending anything over multicast
     *
     * @param maddr the multicast address
     * @param ttl the value of multicast in use
     */
    public void checkMulticast( InetAddress maddr, byte ttl ) { 
	throw new SecurityException( "somebody is attempting to send something through multicast" );
    }

    /**
     * see if the current thread is allowed to access this package.
     * for now, i'm not doing anything as i am not sure what i can do.
     * i could perhaps not allow remote code access to certain
     * packages, but i'm just not sure which ones
     *
     * @param pkg the package that is being accessed 
     */
    public void checkPackageAccess( String pkg ) { 

    }
    
    /**
     * again, i'm not sure what to do here.  what does it mean that "a
     * class is not allowed to define another class in this package"?
     * nelson wants net.hivecell.hive.* to be special, so we'll see,
     * maybe if the class trying to define something in
     * net.hivecell.hive is attempting to create something in
     * net.hivecell.hive then we flag it? For now, let all through.
     *
     * @param pkg the package that we are looking at 
     */
    public void checkPackageDefinition( String pkg ) {
        // checks removed, see version 1.10 for more.
    }
    
    /**
     * nothing should be printing anything in hive.  but just to leave
     * the possibiltiy for a printing shadow, i will test for a shadow
     * in the stack before remote code 
     */
    public void checkPrintJobAccess() { 
    
	if( !testRemoteExecuting( "remote code is attempting to send a print job" ) )
	    // huh -- we scanned to the top of the stack, and didn't find
	    // anything that fit the criteria, just throw an exception
	    // then
	    throw new SecurityException( "something is attempting to send a print job" );

    }
    
    /**
     * remote code should never be attempting to access the properties
     */
    public void checkPropertiesAccess() { 
	
	testJavaOrShadowExecuting( "remote code is attempting to access the properties" );

	// usually we would just throw an excption at this point, but
	// we won't here because perhaps something as Cell.java is
	// accessing the properties
	
    }
    
    /**
     * same policy as the other property access method
     *
     * @param key the key that something is trying to access
     */
    public void checkPropertyAccess( String key ) { 
    		
	testJavaOrShadowExecuting( "remote code is attempting to access the property \"" + key + "\"" );

	// usually we would just throw an excption at this point, but
	// we won't here because perhaps something as Cell.java is
	// accessing the properties

    }
    
    /** 
     * remote code should never (on its own) be accessinf a file
     *
     * @param fd the file descriptor that is being accessed
     */
    public void checkRead( FileDescriptor fd ) { 

	testRemoteExecuting( "remote code is attempting to access a file" );

	// usually we would just throw an excption at this point, but
	// we won't here because perhaps something as Cell.java is
	// accessing a file

    }
    
    /**
     * remote code should never on its own be accessing a file
     *
     * @param file the name of the file that is being attempted to read
     */
    public void checkRead( String file ) { 

	testRemoteExecuting( "remote code is attempting to access a \"" + file + "\"" );

	// usually we would just throw an excption at this point, but
	// we won't here because perhaps something as Cell.java is
	// accessing a file

    }
    
    /**
     * check read on a file -- again, a piece of remote code, on its
     * own, should never be accessing files
     *
     * @param file the file that is being accessed
     * @param context a system dependant context
     */
    public void checkRead( String file, Object context ) { 
    
	testRemoteExecuting( "remote code is attempting to access a \"" + file + "\"" );

	// usually we would just throw an excption at this point, but
	// we won't here because perhaps something as Cell.java is
	// accessing a file

    }
    
    /**
     * not sure what to do on this one -- this is supposed to be
     * testing access to the security API.  i'll just leave it blank
     * for now and somebody can go back and take a look at it
     * 
     * @param action the action that is being taken 
     */
    public void checkSecurityAccess( String action ) { 

    }

    /**
     * nobody should be setting their own factories -- eventually Hive
     * will need to, but for right now this is defintaely a security
     * risk 
     */
    public void checkSetFactory() {
      	throw new SecurityException( "something is attempting to set its own socket factory" );
    }

    /**
     * nobody is allowed access to the system clipboard.  eventually
     * we would like access to it, but we will need to think about it
     * for now.  right now i don't want a malcious agent transmitting
     * the values in my clipboard off my system 
     */
    public void checkSystemClipboardAccess() { 
	throw new SecurityException( "something is attempting to access the system clipboard" );
    }

    /** 
     * some code checking will need to be done on this method.  right
     * now this piece of code will not throw any exceptions about code
     * being allowed to display a top level window, but for now i will
     * just flag those as untrusted.  but remote code should -not- be
     * putting up windows on its own
     *
     * @param window the window that is being displayed 
     */
    public boolean checkTopLevelWindow( Object window ) { 

  	Class c[] = getClassContext();
  	for( int count=0;count < c.length;count++ ) {
  	    if( (ComponentManagerShadow.class).isAssignableFrom( c[count] ) && primordialClass( c[count] ) )
  		return true;
  	    else if( !primordialClass( c[count] ) )
  		throw new SecurityException( "remote code is attempting to display a window" );
  	}

  	return false;

    }

    /**
     * not sure if this is right, but for now nothing should be allowed to write any files
     *
     * @param file the name of the file that people are trying to write
     */
    public void checkWrite( String file ) { 
	testRemoteExecuting( "something is attempting to write to \"" + file + "\"" );
    }
    
    /**
     * same policy as the other check write method
     */
    public void checkWrite( FileDescriptor fd ) { 
	testRemoteExecuting( "something is attempting to write to a file" );
    }

    /**
     * remote code -cannot- cause a shutdown.  in fact, nothing except
     * the cell should be able to call exit.  and even then, the only
     * people allowed to tell the cell to shutdown is the cell itself
     * and the cell control shadow.  except there is no real way to
     * test this, so the only thing i can test is that the cell itself
     * is final class calling exit
     *
     * @param code some code 
     */
    public void checkExit( int code ) {
	
	// this is pretty pathetic -- this means that any piece of code can call quit on the Cell

	Class c[] = getClassContext();
	for( int count=1;count < c.length;count++ ) {
	    if( c[count].getName().startsWith( "java." ) ) {

	    } else if( c[count].getName().startsWith( "net.hivecell.hive.cell.Cell" ) )
		return;

	    else
		throw new SecurityException( "something besides the cell tried to be the final say on the shutdown" );
		
	}
	    
    }

    /** 
     * Just want to note when this happens 
     **/
    public void checkCreateClassLoader() {
	Debug.notice( "class loader created!" );
    }

    /**
     * a simple test to see if this class was loaded in the same class
     * loader as the security manager's (which means that this class
     * was loaded by the primordeal class loader
     *
     * @param toTest the class that we are testing
     * @return true if this class was loaded by the primoderal class loader, false otherwise 
     */
    protected boolean primordialClass( Class toTest ) {

	if( ( toTest.getClassLoader() == null ) ||
	    toTest.getClassLoader().equals( this.getClass().getClassLoader() ) )
	    // then this came from the primordial loader
	    return true;
	return false;

    }

    /**
     * just test to see if there is a piece of remote code in the stack
     */
    protected boolean testRemoteExecuting( String error ) 
    throws SecurityException {

	Class[] c = getClassContext();
	for( int count=0;count < c.length;count++ )
	    if( !primordialClass( c[0] ) )
		throw new SecurityException( error );
	return false;

    }

    /**
     * test to see if a shadow is executing code, and is not soley been executed by remote code
     */
    protected boolean testShadowExecuting( String error ) 
    throws SecurityException {

	Class[] c = getClassContext();
	for( int count=0;count < c.length;count++ ) {
	    if( (Shadow.class).isAssignableFrom( c[count] ) && primordialClass( c[count] ) )
		return true;
	    else if( !primordialClass( c[count] ) )
		throw new SecurityException( error );
	}

	return false;

    }

    /**
     * test to see if a shadow is executing code, but also make an
     * exception if code has been done stuff through the java package
     */
    protected boolean testJavaOrShadowExecuting( String error ) 
    throws SecurityException {

	Class[] c = getClassContext();
	for( int count=0;count < c.length;count++ ) {
	    if( c[count].getName().startsWith( "java." ) ||
		c[count].getName().startsWith( "sun." ) ) 
		return true;
	    else if( (Shadow.class).isAssignableFrom( c[count] ) && primordialClass( c[count] ) )
		return true;
	    else if( !primordialClass( c[count] ) ) {
		throw new SecurityException( error );
	    }
	}

	return false;

    }

    

}








