// $Id: MobileAgentInputStream.java,v 1.1 2000/05/18 22:01:04 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.cell;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.io.IOException;
import java.io.StreamCorruptedException;
import java.rmi.server.RMIClassLoader;
import java.net.URL;
import java.net.MalformedURLException;
import net.hivecell.hive.support.Debug;

/** A class that handles codebase annotation to do mobile agents.
 ** This class is currently buggy - managing ClassLoader namespace issues
 ** is proving to be difficult. Also, this class needs to deal with
 ** buffering the class bytes somehow, in case the agent wants to hop again.
 ** We need to implement our own class loader to manage things.
 **
 ** @version $Revision: 1.1 $
 ** @author Nelson Minar <nelson@media.mit.edu>
 **/
public class MobileAgentInputStream extends ObjectInputStream {
  public MobileAgentInputStream(InputStream is) throws IOException, StreamCorruptedException {
    super(is);
  }

  /** Resolve a class based on the annotation. Currently a bit buggy, see
   ** notes for the class.
   **/
  protected Class resolveClass(ObjectStreamClass v) throws IOException, ClassNotFoundException {
    String className = v.getName();
    
    try {
      Class c = super.resolveClass(v);
      // Debug.notice("Found class " + className + " locally, not loading.");
      return c;
    } catch (ClassNotFoundException cnfe) {
      // ignore the exception, let's keep resolving.
    }
    
    String codebase = readUTF();
    URL codebaseURL = null;
    try {
      codebaseURL = new URL(codebase);
    } catch (MalformedURLException mue) {
      Debug.error("While resolving class " + className + " got a bad URL " +
                  codebase);
      return null;
    }

    Debug.notice("Resolving class " + className + " from " + codebaseURL.toString());
    return RMIClassLoader.loadClass(codebaseURL, className);
  }
}
