// $Id: MobileAgentOutputStream.java,v 1.1 2000/05/18 22:01:04 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.cell;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.HTTPServerShadow;
import net.hivecell.hive.shadow.cell.HTTPServerShadowImpl;
import net.hivecell.hive.support.Debug;

/** 
 * A class that handles codebase annotation to do mobile agents.
 *
 * @version $Revision: 1.1 $
 *
 */
public class MobileAgentOutputStream 
extends ObjectOutputStream {

    /**
     * the code base we are annotating this with -- we get this
     * informaiton from the HTTPServerShadow 
     */
    protected String codebase;

    /**
     * Construct an object stream suitable for mobile agents. Handles
     * annotating the standard ObjectOutputStream with codebase information.
     *
     * @param os the output stream that we are working with
     * @param cell the cell we are working with
     * @exception IOException if something goes wrong with the creation
     */
    public MobileAgentOutputStream( OutputStream os, Cell cell )
    throws IOException {
	super( os );
	try {
	    codebase = ((HTTPServerShadow)cell.getShadow( (HTTPServerShadowImpl.class).getName() )).getCodebase();

	} catch( ShadowNotFoundException error ) {
	    // this is a bad thing if it happens -- this should be
	    // started with the standard hive stuff
	    throw new IOException();

	}

    }
    
    /** 
     * Produce an annotation that allows a reader to find this class.
     * The annotation is just the codebase.
     *
     * @param cl the class that we want to annotate 
     * @exception IOException if something goes wrong with the annotation
     */
    protected void annotateClass( Class cl ) 
    throws IOException {
	writeUTF( codebase );    

    }

}
