// $Id: HiveXML.java,v 1.1 2000/05/18 22:01:06 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/**
 ** This class handles browsing an XML file like a DescSet browsing an RDF file.
 **/

package net.hivecell.hive.description;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;
import java.io.*;

import org.xml.sax.HandlerBase;
import org.xml.sax.AttributeList;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

import org.w3c.dom.*;

import com.docuverse.dom.util.SAXReader;
import com.docuverse.dom.util.*;
import com.docuverse.dom.*;

import net.hivecell.hive.support.Debug;


/** An XML Config file for hive 
 ** This is the XML equivalent of DescSet.
 **/

public class HiveXML implements Serializable {
    

    public static final String parserClass = "com.microstar.xml.SAXDriver";

    protected DOM dom;        /** The DOM object.  Use this to change the default Reader, Writer, Factory and Document. */
    protected Document root;  /** This XML document/root Node */
    public static DOMUtil dutil = new DOMUtil();


    /** Create an empty Config file for writing a dynamically created xml file. */
    /** NOTE: HAVE THE CONSTRUCTOR CATCH THE ERRORS AND SET root TO NULL! **/
    public HiveXML() {
	// instantiate a DOM implementation
        // default DOMReader is com.docuverse.dom.util.SAXReader
	dom = new DOM();

        // SAXReader needs a SAX implementation driver so configure it
        // to use Microstar AElfred XML parser
	dom.setProperty("sax.driver", parserClass);
	
        // Create a new BasicDocument (a docuverse type)
	root = new BasicDocument();
    } 

    public HiveXML(Reader xmlDoc) {
	
        // instantiate a DOM implementation
        // default DOMReader is com.docuverse.dom.util.SAXReader
	dom = new DOM();

        // SAXReader needs a SAX implementation driver so configure it
        // to use Microstar AElfred XML parser
	dom.setProperty("sax.driver", parserClass);
	
        // read in document from xmlDoc.
        // Input parameter can be anything the current DOMReader
        // can handle.  SAXReader accepts URL, InputStream, Reader,
        // or InputSource.
	root = dom.openDocument(xmlDoc);
    }

    public HiveXML(InputStream xmlDoc) {
	this(new InputStreamReader(xmlDoc));
    }


    /**
     ** Write this DOM Object to the specified output.
     **
     ** @param node The node to write out
     ** @param output The output to be written to
     **/
    public void write(Object output) { 
	DOMWriter writer = dom.getWriter();
	writer.write(dom, root, output);
    }
    public void writeNode(Node node, Object output) {
	SAXWriter writer = new SAXWriter();
	dom.setWriter(writer);
	dom.writeNode(node, output);
    }


    
    /** 
     ** Return the Nodes named 'name' from the children of current 
     ** 
     ** @param current the node whose children are being searched
     ** @param name the name of the tag to search for
     **/
    public static NodeList select(Node current, String name) {
	
	NodeList nl = current.getChildNodes();
	NodeListImpl currentList = new NodeListImpl();
	int len = nl.getLength();

	if (len != 0) {
	    for (int i = 0; i < len; i++) {
		Node c = nl.item(i);
		if (c.getNodeName().equals(name)) {
		    currentList.appendNode(c);
		}
	    }
	}
	

	if (currentList.getLength() == 0)   // Cause a null ptr exception.  Easier to track than an empty NodeList.
	    return (null);
	return (currentList);
    }


    /**
     ** Return the Nodes named 'name' from a node list
     **
     ** @param list the NodeList whose elements are being searched
     ** @param name the name of the tag to search for
     **/
    public static NodeList select(NodeList list, String name) {
	NodeListImpl currentList = new NodeListImpl();

	for (int i = 0; i < list.getLength(); i++)
	    if (list.item(i).getNodeName().equals(name))
		currentList.appendNode(list.item(i));


	if (currentList.getLength() == 0)   // Cause a null ptr exception.  Easier to track than an empty NodeList.
	    return (null);
	return (currentList);
    }


    /**
     ** Return the Nodes named 'name' from a NamedNodeMap
     **
     ** @param map the NamedNodeMap whose elements are being searched
     ** @param name the name of the tag to search for
     **/
    public static NodeList select(NamedNodeMap map, String name) {
	NodeListImpl currentList = new NodeListImpl();

	for (int i = 0; i < map.getLength(); i++)
	    if (map.item(i).getNodeName().equals(name))
		currentList.appendNode(map.item(i));


	if (currentList.getLength() == 0)   // Cause a null ptr exception.  Easier to track than an empty NodeList.
	    return (null);
	return (currentList);
    }


    /**
     ** Return the Node which is named name and has value val
     **
     ** @param name the name of the tag to search for
     ** @param val the value that the Node of name must have
     **/
    public static NodeList select(Node current, String name, String val) { 
	
	NodeList nl = current.getChildNodes();
	NodeListImpl currentList = new NodeListImpl();
	int len = nl.getLength();
	
	if (len != 0) {
	    for (int i = 0; i < len; i++) {

		Node c = nl.item(i);
		if (c.getNodeName().equals(val)) {
		    
		    Node sub = c.getFirstChild();
		    if (sub != null) {
			if ( sub.getNodeType() == Node.TEXT_NODE && sub.getNodeValue().equals(val) )
			    currentList.appendNode(c);
			
		    }
		}
		
	    }
	}
	
	
	if (currentList.getLength() == 0) 
	    return (null);
	else return (currentList);
    }


    /**
     ** Return the Nodes named 'name' from a node list
     **
     ** @param list the NodeList whose elements are being searched
     ** @param name the name of the tag to search for
     ** @param val the value that the tag with name "name" must have.
     **/
    public static NodeList select(NodeList list, String name, String val) {
	NodeListImpl currentList = new NodeListImpl();

	for (int i = 0; i < list.getLength(); i++) {
	    Node c = list.item(i);

	    if (c.getNodeName().equals(name)) {

		Node sub = c.getFirstChild();
		if (sub != null) {
		    if ( sub.getNodeType() == Node.TEXT_NODE && sub.getNodeValue().equals(val) )
			currentList.appendNode(c);
		}

	    }
	}
	

	if (currentList.getLength() == 0) 
	    return (null);
	else return (currentList);
    }


    /**
     ** Return the Nodes named 'name' from a NamedNodeMap
     **
     ** @param map the NamedNodeMap whose elements are being searched
     ** @param name the name of the tag to search for
     ** @param val the value that the tag with name "name" must have.
     **/
    public static NodeList select(NamedNodeMap map, String name, String val) {
	NodeListImpl currentList = new NodeListImpl();

	for (int i = 0; i < map.getLength(); i++) {
	    Node c = map.item(i);

	    if (c.getNodeName().equals(name)) {

		Node sub = c.getFirstChild();
		if (sub != null) {
		    if ( sub.getNodeType() == Node.TEXT_NODE && sub.getNodeValue().equals(val) )
			currentList.appendNode(c);
		}

	    }
	}
	

	if (currentList.getLength() == 0) 
	    return (null);
	else return (currentList);
    }


    /** 
     ** Here are convenience select functions for when 
     ** you only need the first node that matches.  Otherwise
     ** they are exactly the same as the other select fucntions.
     **/
    public static Node selectFirst(Object node, String name) {	
	NodeList l = null;

	if (node instanceof Node)
	    l = select((Node)node, name);
	else if (node instanceof NodeList)
	    l = select((NodeList)node, name);
	else if (node instanceof NamedNodeMap)
	    l = select((NamedNodeMap)node, name);
	else {
	    Debug.error("Trying to select on a non-Node, NodeList or NamedNodeMap obejct.");
	    return (null);
	}
	
	if (l == null || l.getLength() == 0)
	    return null;
	else return (l.item(0));
    }
    public static Node selectFirst(Object node, String name, String val) {
	NodeList l = null;

	if (node instanceof Node)
	    l = select((Node)node, name, val);
	else if (node instanceof NodeList)
	    l = select((NodeList)node, name, val);
	else if (node instanceof NamedNodeMap)
	    l = select((NamedNodeMap)node, name, val);
	else {
	    Debug.error("Trying to select on a non-Node, NodeList or NamedNodeMap obejct.");
	    return (null);
	}
	
	if (l == null || l.getLength() == 0)
	    return null;
	else return (l.item(0));
    }

    /** A convenience method to read config values out of Hive config
     ** XML files. Assumes the XML document is structured like this:
     ** <config><keyword>value</keyword><keyword2>value2</keyword></config>
     **
     ** @param doc The pointer to the root of the XML document. This
     **    node type should be the XML tag <config>.
     ** @param keyword The keyword whose value you are looking up.
     ** @returns The value of the keyword, or null if not found.
     **/
    public static String getConfigValue(Document doc, String keyword) {
        // Grab the "config" node
        Node config = HiveXML.selectFirst(doc, "config");
        if (config == null)
            return null;

        // Search for the keyword inside
        Node valueNode = HiveXML.selectFirst(config, keyword);
        if (valueNode == null)
            return null;

        String value = HiveXML.getFirstNonWhitespaceChild(valueNode).getNodeValue();
        // Special hack - if the user writes "null", turn into null pointer.
        if (value.equals("null"))
            return null;
        else
            return value;
    }

    public static Node getFirstNonWhitespaceChild(Node parent) {
	NodeList kids = parent.getChildNodes();
	WhitespaceFilter whitespace = new WhitespaceFilter();	

	for (int i = 0; i < kids.getLength(); i++)
	    if (!dutil.isWhitespace( kids.item(i) ))
		return (kids.item(i));

	return (null);
    }


    public Document getRoot() {
	return (root);
    }

    public DOM getDOM() {
	return (dom);
    }

    
    /**
     ** Prints out node's properties, node's attributes, and does the 
     ** same for all of node's children.
     **
     ** @param node The node to print out.
     ** @param depth How many levels deep we are (to know how many tabs to use when indenting)
     **/
    public void printTree() { printTree(root, 0, true); }
    public void printTree(boolean showWS) { printTree(root, 0, showWS); }
    public static void printTree(Node node) { printTree(node, 0, true); }
    public static void printTree(Node node, boolean showWS) { printTree(node, 0, showWS); }
    public static void printTree(Node node, int depth, boolean showWS) {

	if (!showWS && dutil.isWhitespace(node))
	    return;
	
	/* USING A FILTER:
	WhitespaceFilter whitespace = new WhitespaceFilter();
	if (whitespace.filterNode(node) == NodeFilter.ACCEPT)
	    return;
	*/
	
	for (int j = 0; j < depth; j++)
	    System.out.print("    ");
	System.out.println();


	if (depth != 0)
	    System.out.print(depth + "---");
	for (int i = 1; i < depth; i++)
	    System.out.print("----");

	// Print out the values for this node
	System.out.println("Name: " + node.getNodeName() + 
			 ", Type: " + getNameforType(node.getNodeType()) +
			   ", Value: " + node.getNodeValue());



	// Print out the properties (extra tags) for this node
	NamedNodeMap nnm = node.getAttributes();

	if (nnm != null && nnm.getLength() > 0) {
	    for (int i = 0; i < depth; i++)
		System.out.print("    ");
	    
	    System.out.println("        Properties (" + nnm.getLength() + "): ");

	    
	    for (int i = 0; i < nnm.getLength(); i++) {
		for (int j = 0; j < depth; j++)
		    System.out.print("    ");
		System.out.print("            ");
		    
		Node p = nnm.item(i);
		System.out.println(i + ") Name: " + p.getNodeName() +
				   ", Type: " + getNameforType(p.getNodeType()) +
				   ", Value: " + p.getNodeValue());
	    }
	}


	NodeList kids = node.getChildNodes();
	for (int i = 0; i < kids.getLength(); i++)
	    printTree(kids.item(i), depth + 1, showWS);
    }


    public static String getNameforType(short type) {
	if (type == Node.TEXT_NODE)
	    return ("TEXT_NODE");
	else if (type == Node.DOCUMENT_NODE)
	    return ("DOCUMENT_NODE");
	else if (type == Node.ATTRIBUTE_NODE)
	    return ("ATTRIBUTE_NODE");
	else if (type == Node.ENTITY_NODE)
	    return ("ENTITY_NODE");
	else if (type == Node.ELEMENT_NODE)
	    return ("ELEMENT_NODE");
	else if (type == Node.CDATA_SECTION_NODE)
	    return ("CDATA_SECTION_NODE");

	else return (Integer.toString(type));
    }

    public static void main(String args[]) {

	FileInputStream is = null;
	Vector dynArgs = new Vector();
	String filen = null;
	if (args.length == 0)
	    printUsage();
	else filen = args[0];

	try {
	    is = new FileInputStream(filen);
	} catch (FileNotFoundException e) {
	    Debug.failure("File '" + filen + "'  not found.");
	}

	if (is == null)
	    Debug.failure("Error opening file " + filen);
	
	HiveXML hx = new HiveXML(is);

	boolean showWS = true;
	if (args.length >= 2) {
	    args[1].trim();
	    if (args[1].equals("-W"))
		hx.dutil.removeWhitespaces(hx.getRoot());
	    else if (args[1].equals("-w"))
		showWS = false;

	    // Handle dynamic tag creation.
	    int tpos = -1;
	    if (args[1].equals("-t"))
		tpos = 1;
	    else if (args.length > 2 && args[2].equals("-t"))
		tpos = 2;

	    if (tpos != -1) {
		if (args.length < tpos + 3)
		    printUsage();

		for (int i = tpos + 1; i < args.length; i++)
		    dynArgs.addElement(args[i]);
	    }
	}


	hx.printTree(showWS);
	System.out.println();


	// Add the user-specified nodes onto the tree, and write it out to a file.
	try {
	    if (dynArgs.size() > 0) {
		Enumeration e = dynArgs.elements();
	

		// BEGIN DYNAMIC TREE BUILDING HERE:
		Element a = hx.getRoot().createElement((String)e.nextElement());
		Node n = hx.getRoot().createTextNode((String)e.nextElement());
		
		while (e.hasMoreElements())
		    a.setAttribute((String)e.nextElement(), (String)e.nextElement());
		
		hx.getFirstNonWhitespaceChild(hx.getRoot()).appendChild(a);
		a.appendChild(n);
		// END DYNAMIC NODE BUILDING HERE

		System.out.println(hx.getRoot().toString());
	    } 

	} catch (Exception f) {
	    Debug.error("An error occured while trying to add nodes.  See usage info:", f);
	    printUsage();
	}
    }
    
    private static void printUsage() {
	System.out.println("USAGE: java net.hivecell.hive.HiveXML <xmlFile.xml> [-w|-W]");
	System.out.println("\t-w   Hide whitespace nodes");
	System.out.println("\t-W   Remove whitespace nodes");
	System.out.println("\t-t   <tag name> <text> [<property> <value>]*");
	System.out.println("\tNOTE: -t must be the last tag passed as an argument.");
	System.out.println("\n\tEX:  java net.hivecell.hive.description.HiveXML poem.xml -w -t \n\t\telement \"Hello World\" is id");
	System.exit(1);
    }
}
