// $Id: XMLTest.java,v 1.1 2000/05/18 22:01:06 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

/**
 ** This class handles browsing an XML file like a DescSet browsing an RDF file.
 ** Definintion of an interface to XML that fully replaces RDF and is better designed than HiveXML
 ** 
 ** Thesis:  You have a DOM tree.  This is used to search the tree in different ways.
 ** 	 Once you find the node you are looking for, pull it out with getMatch(int) and query
 ** 	 it with the wrapper functions to the DOM commands.
 **/

package net.hivecell.hive.description;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;
import java.io.*;

import org.xml.sax.HandlerBase;
import org.xml.sax.AttributeList;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

import org.w3c.dom.*;

import com.docuverse.dom.util.SAXReader;
import com.docuverse.dom.util.*;
import com.docuverse.dom.*;

import net.hivecell.hive.support.Debug;
import net.hivecell.hive.Global;



public class XMLTest {
    public XMLTest() {}

    public static void main(String args[]) {
	
	DOM dom = new DOM();
	
        // SAXReader needs a SAX implementation driver so configure it
        // to use Microstar AElfred XML parser
	dom.setProperty("sax.driver", "com.microstar.xml.SAXDriver");
	
        // read in document from xmlDoc.
        // Input parameter can be anything the current DOMReader
        // can handle.  SAXReader accepts URL, InputStream, Reader,
        // or InputSource.
	Vector dynArgs = new Vector();
	String filen = null;
	if (args.length == 0) {
	    Debug.failure("NO! You MUST specify an XML Doc for this test to work!");
	    System.exit(1);
	}
	
	else filen = args[0];

	FileInputStream is = null;
	try {
	    is = new FileInputStream(filen);
	} catch (FileNotFoundException e) {
	    Debug.failure("File '" + filen + "'  not found.");
	}

	if (is == null)
	    Debug.failure("Error opening file " + filen);
	
	Document doc = dom.openDocument(is);
	Description root = new Description(new Object(), doc);

	root.printTree();
	int numMatch = 0;

	System.out.println("\nNOW ADDING TAG:\n");

	root.selectFirst("poem");
	root = root.getMatch();
	String[] attrTag  = {"attr"};
	String[] attrVal = {"1"};
	root = root.semanticContext();
	root.addTag("YOYO", "MAN", attrTag, attrVal);
	root.printTree();

	System.out.println("\nXML LOOKS LIKE:\n" + root.toString());
	

	String attr = null;
	String val = "null";
	System.out.print("\nSelecting attribute '"+attr+"'   ---------->   ");
	int numm = root.findAttribute(attr);
	if (numm != 0) {
	//	if (root.containsAttributeValue("id", val)) {
	    System.out.println("YES:  ");
	    for (int i = 0; i < numm; i++) {
		Description mat = root.getMatch(i);
		System.out.println("\t"+mat.getText());
	    }
	} else {
	    System.out.println("NO");
	}
	
	/*
	System.out.println("\nSearch on an attribute '" + attr + "'");
	int numMatches = root.findAttribute(attr);
	for (int q = 0; q < numMatches; q++) {
	    Description attrSearch = root.getMatch(q);
	    System.out.println("Value of tag containing attribute '" + attr + "' is: " + attrSearch.getText());
	    System.out.println("Value of attribute '" + attr + "' is: " + attrSearch.getAttribute(attr));
	} 
	if (numMatches == 0) {
	    System.out.println("Attribute '" + attr + "' not found.\n");
	}
	*/

	System.exit(1);

	Description desc = root.semanticContext();

	if (desc.selectFirst("line")) {
	    Description line = desc.getMatch();
	    //	    System.out.println(line.getText());
	}

	int num = desc.select("line");
	for (int i = 0; i < num; i++) {
	    Description line = desc.getMatch(i);
	    System.out.println(line.getText());
	}

	if (!root.selectFirst("nestedNote")) {
	    System.out.print("'nestedNote' not a child of root.  Is it in the tree: ");
	    if (root.findFirst("nestedNote")) System.out.println("yes");
	    else System.out.println("no");
	}


	if (root.containsTagValue("line2", null)) { 
	    Debug.critical("THE LINE IS IN THERE!");
	} else {
	    Debug.warning("THE LINE IS *NOT* IN THERE!");
	}

	if (!root.findFirst("line", Description.ANYWHERE)) {
	    Debug.error("Couldn't find line...");
	}
	else {
	    Description m = root.getMatches();
	    //for (int i = 0; i < numMatch; i++) {
		HiveXML.printTree(m.getNode());
		Debug.warning("Value of text is: " + m.getText());
		Debug.warning("Value of unstripped text is: " + m.getUnstrippedText());
		Debug.warning("Value of property 'id' is: " + m.getAttribute("id"));
		m = m.getNext();
		//}
	    /*
	    if (!m.selectFirst("line3")) {
		Debug.error("Couldn't find line3...");
	    }
	    else {
		Description f = m.getMatches();
		HiveXML.printTree(f.getNode());
	    }
	    */
	}


	/*
	boolean showWS = true;
	if (args.length >= 2) {
	    args[1].trim();
	    if (args[1].equals("-W"))
		hx.dutil.removeWhitespaces(hx.getRoot());
	    else if (args[1].equals("-w"))
		showWS = false;

	    // Handle dynamic tag creation.
	    int tpos = -1;
	    if (args[1].equals("-t"))
		tpos = 1;
	    else if (args.length > 2 && args[2].equals("-t"))
		tpos = 2;

	    if (tpos != -1) {
		if (args.length < tpos + 3)
		    printUsage();

		for (int i = tpos + 1; i < args.length; i++)
		    dynArgs.addElement(args[i]);
	    }
	}


	hx.printTree(showWS);
	System.out.println();


	// Add the user-specified nodes onto the tree, and write it out to a file.
	try {
	    if (dynArgs.size() > 0) {
		Enumeration e = dynArgs.elements();
	

		// BEGIN DYNAMIC TREE BUILDING HERE:
		Element a = hx.getRoot().createElement((String)e.nextElement());
		Node n = hx.getRoot().createTextNode((String)e.nextElement());
		
		while (e.hasMoreElements())
		    a.setAttribute((String)e.nextElement(), (String)e.nextElement());
		
		hx.getFirstNonWhitespaceChild(hx.getRoot()).appendChild(a);
		a.appendChild(n);
		// END DYNAMIC NODE BUILDING HERE

		System.out.println(hx.getRoot().toString());
	    } 

	} catch (Exception f) {
	    Debug.error("An error occured while trying to add nodes.  See usage info:", f);
	    printUsage();
	}
	*/
    }
}
