// $Id: AgentBornCellEvent.java,v 1.1 2000/05/18 22:01:08 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.event.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.support.CellAddress;

public class AgentBornCellEvent 
extends CellEvent {

    private Agent bornAgent; // the handle to the agent which was just born
    private CellAddress serverBornOn; // the address of the server this agent was born on
    
    /**
     * Create an agent born server event.
     *
     * @param broadcastingAgent the agent which is going to be sending out this event
     * @param eventCount the number of event which the helper is going to send out
     * @param bornAgent the agent which was just born
     * @param serverBornOn the address of the cell this agent was born on
     */
    public AgentBornCellEvent( Object broadcastingAgent, long eventCount, Agent bornAgent, CellAddress serverBornOn ) {

	super( broadcastingAgent, eventCount, CellEvent.AGENTBORN );
	this.bornAgent = bornAgent;
	this.serverBornOn = serverBornOn;

    }

    /**
     * Get the handle to the agent which has just born
     *
     * @return the handle to the agent
     */
    public Agent getAgent() { return this.bornAgent; }

    /**
     * Get the address to the cell that this agent was born on
     *
     * @return the address of the cell
     */
    public CellAddress getAgentAddress() { return this.serverBornOn; }

    public boolean matchesCellAddress( CellAddress toTest ) {
	return getAgentAddress().equals( toTest );
    }

    public CellAddress[] getCellAddresses() { 
	CellAddress[] toReturn = { serverBornOn, };
	return toReturn;
    }

    public CellEvent createNewEvent( Object newSender, long eventCount ) {
	return new AgentBornCellEvent( newSender, eventCount, getAgent(), getAgentAddress() );
    }


}



