// $Id: AgentConnectCellEvent.java,v 1.1 2000/05/18 22:01:08 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.event.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.support.CellAddress;

/**
 * This encapsulates an agent connect server event.  Usually, the
 * sourceAddress and the destinationAddrses are null as set by the
 * CellEventMonitor.  When the AgentMonitoringAgent rebroadcasts
 * these events out -- it will set these appropriatly from its cache 
 */
public class AgentConnectCellEvent 
extends CellEvent 
implements TwoAgentCellEvent {

    private Agent sourceAgent, destinationAgent; // the two agents which are connection
    private CellAddress sourceAddress, destinationAddress;

    /**
     * Create an agent connect server event.
     *
     * @param broadcastingAgent the agent which is going to be sending out this event
     * @param eventCount the number of event which the helper is going to send out
     * @param sourceAgent the agent which is originating the connection
     * @param destinationAgent the agent which is receiving the connection
     */
    public AgentConnectCellEvent( Object broadcastingAgent, long eventCount, Agent sourceAgent, CellAddress sourceAddress, Agent destinationAgent, CellAddress destinationAddress ) {

	super( broadcastingAgent, eventCount, CellEvent.AGENTSCONNECT );
	this.sourceAgent = sourceAgent;
	this.destinationAgent = destinationAgent;
	this.sourceAddress = sourceAddress;
	this.destinationAddress = destinationAddress;

    }

    public AgentConnectCellEvent( Object broadcastingAgent, long eventCount, Agent sourceAgent, Agent destinationAgent ) {

	super( broadcastingAgent, eventCount, CellEvent.AGENTSCONNECT );
	this.sourceAgent = sourceAgent;
	this.destinationAgent = destinationAgent;

    }

    /**
     * Get the handle to the agent which has started the connection
     *
     * @return the handle to the agent
     */
    public Agent getSourceAgent() { return this.sourceAgent; }

    /**
     * Get the address to the source agent.  There is no guarantee
     * this will not be null -- it depends on whether the appropriate
     * constructor was set
     *
     * @return the cell address of the source agent 
     */
    public CellAddress getSourceAgentAddress() { return this.sourceAddress; }

    /**
     * Get the address to the agent receiving the connection
     *
     * @return the address of the cell
     */
    public Agent getDestinationAgent() { return this.destinationAgent; }

    /**
     * Get the address to the destination agent.  There is no guarantee
     * this will not be null -- it depends on whether the appropriate
     * constructor was set
     *
     * @return the cell address of the destination agent 
     */
    public CellAddress getDestinationAgentAddress() { return this.destinationAddress; }

    public boolean matchesCellAddress( CellAddress toTest ) {
	if( ( this.sourceAddress == null ) || ( this.destinationAddress == null ) ) return false;
	return ( this.sourceAddress.equals( toTest ) || this.destinationAddress.equals( toTest ) );
    }

    public CellAddress[] getCellAddresses() { 
	CellAddress[] toReturn = { sourceAddress, destinationAddress, };
	return toReturn; 
    }

    public CellEvent createNewEvent( Object newSender, long eventCount ) {
	return new AgentConnectCellEvent( newSender, eventCount, getSourceAgent(), getSourceAgentAddress(), getDestinationAgent(), getDestinationAgentAddress() );
    }

    public CellEvent createNewEvent( Object newSender, long eventCount, CellAddress sourceAddress, CellAddress destinationAddress ) {
	return new AgentConnectCellEvent( newSender, eventCount, getSourceAgent(), sourceAddress, getDestinationAgent(), destinationAddress );
    }

}



