// $Id: AgentDeathCellEvent.java,v 1.1 2000/05/18 22:01:08 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.event.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.support.CellAddress;

public class AgentDeathCellEvent
extends CellEvent {

    private Agent diedAgent; // the handle to the agent which was just killed
    private CellAddress serverDiedOn; // the address of the server this agent was killed on

    /**
     * Create an agent death event.
     *
     * @param broadcastingAgent the agent which is going to be sending out this event
     * @param eventCount the number of event which the helper is going to send out
     * @param diedAgent the agent which was just killed
     * @param serverDiedOn the address of the cell this agent was killed on
     */
    public AgentDeathCellEvent( Object broadcastingAgent, long eventCount, Agent diedAgent, CellAddress serverDiedOn ) {

	super( broadcastingAgent, eventCount, CellEvent.AGENTDYING );
	this.diedAgent = diedAgent;
	this.serverDiedOn = serverDiedOn;

    }

    /**
     * Get the handle to the agent which has just killed
     *
     * @return the handle to the agent
     */
    public Agent getAgent() { return this.diedAgent; }

    /**
     * Get the address to the cell that this agent was killed on
     *
     * @return the address of the cell
     */
    public CellAddress getAgentAddress() { return this.serverDiedOn; }

    public boolean matchesCellAddress( CellAddress toTest ) {
	return getAgentAddress().equals( toTest );
    }

    public CellAddress[] getCellAddresses() { 
	CellAddress[] toReturn = { serverDiedOn, };
	return toReturn; 
    }

    public CellEvent createNewEvent( Object newSender, long eventCount ) {
	return new AgentDeathCellEvent( newSender, eventCount, getAgent(), getAgentAddress() );
    }
}



