// $Id: AgentDisconnectFromAllCellEvent.java,v 1.1 2000/05/18 22:01:08 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.event.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.support.CellAddress;

/**
 * This encapsulates an agent connect server event.  Usually, the
 * sourceAddress and the destinationAddrses are null as set by the
 * CellEventMonitor.  When the AgentMonitoringAgent rebroadcasts
 * these events out -- it will set these appropriatly from its cache 
 */
public class AgentDisconnectFromAllCellEvent 
extends CellEvent {

    private Agent agent; // the agent disconnecting from all
    private CellAddress address;

    /**
     * Create an agent from all disconnect server event.
     *
     * @param broadcastingAgent the agent which is going to be sending out this event
     * @param eventCount the number of event which the helper is going to send out
     * @param agent the agent which is originating the disconnection
     */
    public AgentDisconnectFromAllCellEvent( Object broadcastingAgent, long eventCount, Agent agent, CellAddress address ) {

	super( broadcastingAgent, eventCount, CellEvent.AGENTDISCONNECTALL );
	this.agent = agent;
	this.address = address;

    }

    public AgentDisconnectFromAllCellEvent( Object broadcastingAgent, long eventCount, Agent agent ) {

	super( broadcastingAgent, eventCount, CellEvent.AGENTDISCONNECTALL );
	this.agent = agent;

    }

    /**
     * Get the handle to the agent which has started the disconnection
     *
     * @return the handle to the agent
     */
    public Agent getAgent() { return this.agent; }

    /**
     * Get the cell address of the agent.  There is no guarantee that
     * this will not return null -- it depends on whether or not the
     * appropriate constructor was called
     *
     * @return the cell address of the agent 
     */
    public CellAddress getAgentAddress() { return this.address; }
    
    public boolean matchesCellAddress( CellAddress toTest ) {
	if( this.address == null ) return false;
	return this.address.equals( toTest );
    }

    public CellAddress[] getCellAddresses() { 
	CellAddress[] toReturn = { address, };
	return toReturn;
    }

    public CellEvent createNewEvent( Object newSender, long eventCount ) {
	return new AgentDisconnectFromAllCellEvent( newSender, eventCount, getAgent(), getAgentAddress() );
    }

}
