// $Id: AgentMovingFromCellEvent.java,v 1.1 2000/05/18 22:01:08 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.event.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.support.CellAddress;

public class AgentMovingFromCellEvent
extends CellEvent {

    private Agent movingAgent; // the handle to the agent which is moving
    private CellAddress originCell, destinationCell; // the servers this agent is moving from to

    /**
     * Create an agent moving to event.
     *
     * @param broadcastingAgent the agent which is going to be sending out this event
     * @param eventCount the number of event which the helper is going to send out
     * @param movingAgent the agent which is moving
     * @param originCell the server this agent is originating from
     * @param destinationCell the server this agent is going to
     */
    public AgentMovingFromCellEvent( Object broadcastingAgent, long eventCount, Agent movingAgent, CellAddress originCell, CellAddress destinationCell ) {

	super( broadcastingAgent, eventCount, CellEvent.AGENTMOVINGFROM  );
	this.movingAgent = movingAgent;
	this.originCell = originCell;
	this.destinationCell = destinationCell;

    }

    /**
     * Get the handle to the agent which is moving
     *
     * @return the handle to the agent
     */
    public Agent getAgent() { return this.movingAgent; }

    /**
     * Get the address to the cell that this agent is moving from
     *
     * @return the address of the cell
     */
    public CellAddress getOriginAddress() { return this.originCell; }

    /**
     * Get the address to the cell that this agent is moving to
     *
     * @return the address of the cell
     */
    public CellAddress getDestinationAddress() { return this.destinationCell; }

    public boolean matchesCellAddress( CellAddress toTest ) {
	return ( getOriginAddress().equals( toTest ) || getDestinationAddress().equals( toTest ) );
    }

    public CellAddress[] getCellAddresses() { 
	CellAddress[] toReturn = { originCell, destinationCell, };
	return toReturn; 
    }


    public CellEvent createNewEvent( Object newSender, long eventCount ) {
	return new AgentMovingFromCellEvent( newSender, eventCount, getAgent(), getOriginAddress(), getDestinationAddress() );
    }
}
