// $Id: CellEvent.java,v 1.1 2000/05/18 22:01:08 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.event.cell;

import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.support.CellAddress;

import java.util.Vector;

/**
 * This is the event data for the server event -- it is designed to be
 * able to be remitted.  The case in which this becomes important is
 * when then CellEventMonitor first creates this -- it will get a
 * different sender and count code.  These codes will be changed when
 * they hit the agent that emits them, but the superclass will still
 * contain the original code
 *
 * @see net.hivecell.hive.shadow.CellEventMonitor 
 */
public abstract class CellEvent 
extends HiveEvent {

    protected Object source; // these are put here so this object can be mutable
    protected long eventCount;

    private Vector passLog = new Vector();


    /**
     * This needs to be set by anything which subclasses this so we
     * can figure which eventID it is 
     */
    protected long serverEventCode = ALL_EVENTS;

    // event codes
    public static final long AGENTBORN = 1;
    public static final long AGENTDYING = 2;
    public static final long AGENTMOVINGTO = 4;
    public static final long AGENTMOVINGFROM = 8;
    public static final long AGENTSCONNECT = 16;
    public static final long AGENTSDISCONNECT = 32;
    public static final long AGENTDISCONNECTALL = 64;
    public static final long AGENTMESSAGE = 128;
    public static final long SERVERBORN = 256;
    public static final long SERVERGOINGDOWN = 512;
    public static final long SERVERREPORT = 1024;
    public static final long SERVERWITHDRAWN = 2048;
    

    protected CellEvent( Object sourceAgent, long count, long eventID ) {

	super( sourceAgent, count, null, System.currentTimeMillis(), eventID );
	this.serverEventCode = eventID;

    }

    /**
     * This methods -needs- to be overridden by the subclass of this
     * message object to return the event code which pertains to them
     *  
     * @return the class's specific return code 
     */
    public long eventCode() { return serverEventCode; }

    /**
     * This method should return true if the cell address that it
     * comparing with pertains to this server event -- it will return
     * false otherwise
     *
     * @param toTest the address to test on
     * @return true if the toTest pertains to this server event 
     */
    public abstract boolean matchesCellAddress( CellAddress toTest );

    /**
     * This methoud should return all the servers taht this server
     * event pertains to -- "pertains to" is defined by the server
     * event itself
     *
     * @return an array of server addresses this event pertains to 
     */
    public abstract CellAddress[] getCellAddresses();

    /**
     * This method is used so a new CellEvent type can be created
     * with this data, but with a different source and a different
     * count 
     *
     * @param newSender the new sender to create a server event with
     * @param eventCount the new event count
     * @return a new server event with the new senders and the count
     */
    public abstract CellEvent createNewEvent( Object newSender, long eventCount );

    /**
     * add a cell to this passLog to state taht we have gone through this cell
     *
     * @param address teh address of the cell that we want to add to the pass through
     */
    public void addPassThrough( CellAddress address ) {
	passLog.addElement( address );

    }

    /**
     * test to see if this event has passed through a particular cell
     *
     * @param address the address that we want to test against
     * @return true if this event has gone through this address
     */
    public boolean passedThrough( CellAddress address ) {
	return passLog.contains( address );

    }

}



