// $Id: TwoAgentCellEvent.java,v 1.1 2000/05/18 22:01:08 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.event.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.support.CellAddress;

/**
 * This interface is especially useful inside the agent monitoring
 * agent so it can get at these two agents and fill in the missing
 * information when this event is sent through the server event
 * monitor 
 */
public interface TwoAgentCellEvent {

    /**
     * Get the handle to the agent which has started the connection
     *
     * @return the handle to the agent
     */
    public Agent getSourceAgent();

    /**
     * Get the address to the source agent.  There is no guarantee
     * this will not be null -- it depends on whether the appropriate
     * constructor was set
     *
     * @return the cell address of the source agent 
     */
    public CellAddress getSourceAgentAddress();

    /**
     * Get the address to the agent receiving the connection
     *
     * @return the address of the cell
     */
    public Agent getDestinationAgent();

    /**
     * Get the address to the destination agent.  There is no guarantee
     * this will not be null -- it depends on whether the appropriate
     * constructor was set
     *
     * @return the cell address of the destination agent 
     */
    public CellAddress getDestinationAgentAddress();

    /**
     * This method is used so a new CellEvent type can be created
     * with this data, but with a different source and a different
     * count 
     *
     * @param newSender the new sender to create a server event with
     * @param eventCount the new event count
     * @return a new server event with the new senders and the count
     */
    public abstract CellEvent createNewEvent( Object newSender, long eventCount );

    /**
     * This method is used so a new CellEvent type can be created
     * with this data, but with a different source and a different
     * count 
     *
     * @param newSender the new sender to create a server event with
     * @param eventCount the new event count
     * @return a new server event with the new senders and the count
     */
    public abstract CellEvent createNewEvent( Object newSender, long eventCount, CellAddress sourceAddress, CellAddress destinationAddress );

}

