// $Id: EventPublisherHelper.java,v 1.1 2000/05/18 22:01:12 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow;

import net.hivecell.hive.support.Debug;

import java.util.Enumeration;
import java.util.Vector;



/**
 * this is a helper class for any shadows which are interested in
 * sending out events to agents which subscribe to them 
 */
public class EventPublisherHelper {

    /**
     * this is the shadow we are helping
     */
    private EventPublisherShadow shadowHelped;

    /**
     * the list of agents which are listening to us for events
     */
    private Vector listeners = new Vector();

    /**
     * the number of events which have been sent through this helper 
     */
    private int count = 0;

    
    /**
     * create an event publisher helper for this particular shadow
     *
     * @param shadow the shadow that we are helping out
     */
    public EventPublisherHelper( EventPublisherShadow shadow ) {
	this.shadowHelped = shadow;
    }

    /**
     * add an agent to this publisher helpers list
     *
     * @param signal the signalable that we want to start sending events to
     * @return true if the signal was added, false otherwise
     */
    public boolean addListener( Signalable signal ) {
	if( listeners.contains( signal ) )
	    return false;
	listeners.addElement( signal );
	return true;

    }

    /**
     * remove an signal from our list to publish to
     *
     * @param signal the signal that we want to remove from our lists
     * @return true if the signal was removed, false otherwise
     */
    public boolean removeListener( Signalable signal ) {
	return listeners.removeElement( signal );

    }

    /**
     * get a list of the signals which are listening to us
     *
     * @return the vector of signals
     */
    public Vector getListeners() {
	return listeners;

    }
  
    /**
     * Get a count of all the events sent out
     *
     * @return the count
     */
    public int getCount() {
	return count++;
    }

    /**
     * Send a null event out to all subscribers
     */
    public void notifySubscribers() {
	notifySubscribers( null );
    }

    /**
     * send an event out to all our subscribers
     *
     * @param object the event that we want to send out to all the shadows
     */
    public void notifySubscribers( Object object ) {
	Enumeration e = getListeners().elements();
	while( e.hasMoreElements() )
	    notifySubscriber( (Signalable)e.nextElement(), object );
    }

    /**
     * send a signal out to a specific signalable
     *
     * @param signalable the signalable that we want to send information to
     * @param object the object that we are sending out as the event
     */
    public void notifySubscriber( Signalable signal, Object object ) {
	try {
	    signal.signal( shadowHelped, object );
	} catch( Exception error ) {
	    Debug.error( "exception while trying to send a signal to " + signal, error );
	}
    }

}



