// $Id: SerialStreamShadowImpl.java,v 1.1 2000/05/18 22:01:12 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow;

import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.support.Debug;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import javax.comm.*;

public abstract class SerialStreamShadowImpl
extends StreamShadowImpl
implements SerialStreamShadow {

    /**
     * the information needed to configure the serial port with
     */
    protected int baudRate;
    protected int serialPort;
    protected int stopBits = SerialPort.STOPBITS_1;

    /**
     * get the baud rate for this serial port
     *
     * @return the baud rate
     */
    public int getBaudRate() { return baudRate; }

    /**
     * set the baud rate to set the serial port with
     *
     * @param br the baud rate to set it to
     */
    public void setBaudRate( int br ) { baudRate = br; }

    /**
     * get which serial port we are configured at 
     * 
     * @return the serial port number
     */
    public int getSerialPort() { return serialPort; }

    /**
     * set which serial port we are configured at
     *
     * @param s the serial port that we are configured to
     */
    public void setSerialPort( int s ) { serialPort = s; }


    /**
     * do the initialization -- in this case it sets up the serial port
     */
    public void init() 
    throws ShadowInitializationException {
	
	// first make sure that we have the appropriate serial port
	if( serialPort < 1 ) {
	    Debug.error( serialPort + " is an invalid serial port number" );
	    throw new ShadowInitializationException( serialPort + " is an invalid serial port number" );
	}

	// get a list of all the ports to deal with
	Debug.notice( "getting ports" );
	Enumeration ports = CommPortIdentifier.getPortIdentifiers();
	if( ( ports == null ) || !ports.hasMoreElements() ) {
	    Debug.error( "no serial ports" );
	    throw new ShadowInitializationException( "no serial ports" );
	}
	int serialPortNumber = serialPort;
	CommPortIdentifier portId = null;
	while( ports.hasMoreElements() && ( serialPortNumber-- > 0 ) ) {
	    portId = (CommPortIdentifier)ports.nextElement();
	}
	if( portId == null ) {
	    Debug.error( serialPort + " is an invalid serial port number" );
	    throw new ShadowInitializationException( serialPort + " is an invalid serial port number" );
	}
                
	// actually open the serial port
	Debug.notice( "trying to open " + portId.getName() + ", with id " + portId );
	SerialPort sp;
	try {
	    sp = (SerialPort)portId.open( "hive", 1000 );
	} catch( Exception error ) {
	    Debug.error( "couldn't open serial port", error );
	    throw new ShadowInitializationException( "couldn't open serial port", error );
	}
	
	// configure the serial port
	Debug.notice( "configuring " + portId.getName() + " at " + baudRate );
	try {
	    sp.setSerialPortParams( baudRate, SerialPort.DATABITS_8, stopBits, SerialPort.PARITY_NONE );
	    sp.setFlowControlMode( SerialPort.FLOWCONTROL_NONE );
        } catch( Exception e ) {
            Debug.error( "serial port configure failed" );
	    throw new ShadowInitializationException( "serial port configure failed" );
        }

	Debug.notice( "getting some streams for the serial port" );
	// get the input and the output stream
	try {
	    is = sp.getInputStream();
	    os = sp.getOutputStream();
	} catch( Exception e ) {
	    Debug.error( "couldn't talk to the serial port" );
	    throw new ShadowInitializationException( "couldn't talk to the serial port" );
	}

	Debug.notice( "all done setting up serial port" );

    }

}

