package net.hivecell.hive.shadow.cell;

import net.hivecell.hive.agent.EventReceivingAgent;
import net.hivecell.hive.event.HiveEvent;
import net.hivecell.hive.shadow.EventPublisherShadow;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.async.AsyncResult;

import java.util.Vector;


/**
 * this class will allow us to remove the need for each individual
 * agent to do its own event sending and receiving through RMI -- this
 * will allow us to remove the need to spawn off so many asynchronous
 * messaging threads, and will also allow us to cut down on the number
 * of sockets hive needs especuially when there is a significant
 * amount of messaging occuring.
 *
 * by default, the event shadow will open a random port there will be
 * an agent which will sit on the cell to help shadows negotiate what
 * ports they are sending to.  right now (and therefore probably
 * never) the port on which the shadow is operating can ont be changed
 * on the fly.  that just seems like too much of a headache to
 * negotiate that one.  
 */
public interface AgentMessagingShadow
extends EventPublisherShadow {

    /**
     * get the port number we are listening on
     *
     * @return the port number of the eventReceivingSocket
     */
    public int getEventListeningPort();

    /**
     * set the port number of the event receiving port.  right now
     * this does nothing -- it is just here for java beans type stuff
     *
     * @param portNumber the port number we want to set as 
     */
    public void setEventListeningPort( int portNumber );

    /**
     * call this method to send an event.  this method will wrap the
     * HiveEvent in another object so that the AgentMessagingShadow
     * will be able to decode the destination of the event.  this
     * needs to throw exceptions just so in case it doesn't work
     *
     * @param destinationAgents the agents which is meant to receive the HiveEvent
     * @param destinationAddresses the cell addresses of teh agent that we are going to
     * @param he the HiveEvent which is going to be sent to the other cell 
     * @return an async result object which has the result of the event being sent
     */
    public AsyncResult sendEvent( EventReceivingAgent[] destinationAgents, CellAddress[] destinationAddresses, HiveEvent he );

}

