// $Id: CellControlShadow.java,v 1.1 2000/05/18 22:01:15 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow.cell;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.*;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.agent.AgentImpl;
import net.hivecell.hive.agent.cell.CellAgent;
import net.hivecell.hive.cell.AgentDeliveryException;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.*;
import net.hivecell.hive.support.CellAddress;
import net.hivecell.hive.support.Debug;
import net.hivecell.hive.description.Description;

/**
 * This shadow manages the "remote control" of the server.  Killing
 * agents, connecting agents, and disconnecting agents will be echoed
 * through this server via the CellControlAgent.
 */
public interface CellControlShadow
extends Shadow {
    
    /**
     * Ask the server to create an agent
     * 
     * @param agentName classname of the agent to create
     * @return the remote reference to the agent we have created
     * @exception ClassNotFoundException if the class we want to create does not exist
     * @exception InstantiationException something went wrong with creating the class
     */
    public Agent createAgent( String agentName ) 
    throws ClassNotFoundException, InstantiationException;

    /**
     * ask the server to create a specific agent with a particular description
     *
     * @param agentName classname of the agent to create
     * @param description the Description that we want to assign to this agent
     * @return the remote reference to the agent we have created
     * @exception ClassNotFoundException when the calss to create does not exist
     * @exception InstantiationException something went wrong with creating the class
     */
    public Agent createAgent( String agentName, Description description ) 
    throws ClassNotFoundException, InstantiationException;

    /**
     * Ask the server to kill this agent.  Can this properly throws a
     * RemoteException?  Need to think about this.
     *
     * @param agent the agent to be killed 
     */
    public void killAgent( Agent agent ) 
    throws RemoteException;

    /**
     * Ask the server to move this agent.  Can this properly throws a
     * RemoteException?  Need to think about this.
     *
     * @param agent the agent to be killed 
     * @return the new remote reference to the agent
     */
    public Agent moveAgent( Agent agent, CellAddress address )
    throws RemoteException, AgentDeliveryException;

    /**
     * Tell the first agent to connect to the second one.
     *
     * @param from the "source" Agent
     * @param to the "destination" Agent, or the Agent to connect to
     * @return true if the connection has been made, false otherwise
     */
    public boolean connectAgents( Agent from, Agent to );

    /**
     * Tell an agent to disconnect from everybody 
     *
     * @param agent the Agent that should disconnect
     */
    public void disconnectAgentAll(Agent agent);

    /**
     * Ask an agent to disconnect from another
     *
     * @param from to disconnect from
     * @param to to disconnect to
     */
    public void disconnectAgents( Agent from, Agent to );

    /**
     * Ask the server to shutdown
     */
    public void shutDownCell();

}




