// $Id: CellEventMonitor.java,v 1.1 2000/05/18 22:01:15 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.
package net.hivecell.hive.shadow.cell;

import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.shadow.Shadow;
import net.hivecell.hive.shadow.EventPublisherShadow;
import net.hivecell.hive.support.CellAddress;

/** 
 * Monitor class that tells listeners about server events.
 */
public interface CellEventMonitor
extends EventPublisherShadow {

    /** 
     * Tell subscribers that an agent was born. 
     *
     * @param sender the object that wishes to send this notification out
     * @param agent the agent which has just come alive
     */
    public void sendAgentBorn( Object sender, Agent agent );

    /** 
     * Tell subscribers that an agent is dying. 
     * 
     * @param sender the object that wishes to send this notification out
     * @param the agent which has gone down
     */
    public void sendAgentDying( Object sender, Agent agent );

    /** 
     * Tell subscribers that an agent is dying. 
     * 
     * @param sender the object that wishes to send this notification out
     * @param the agent which has gone down
     */
    public void sendAgentMovingTo( Object sender, Agent agent, CellAddress destinationAddress );

    /** 
     * Tell subscribers that an agent is dying. 
     * 
     * @param sender the object that wishes to send this notification out
     * @param the agent which has gone down
     */
    public void sendAgentMovingFrom( Object sender, Agent agent, CellAddress sourceAddress );

    public void sendAgentsConnecting( Object sender, Agent from, Agent to );

    /** 
     * Tell subscribers that two agents are connecting. 
     *
     * @param sender the object that wishes to send this notification out
     * @param from the agent the connection is coming from
     * @param to the agent the connection is going to
     */
    public void sendAgentsConnecting( Object sender, Agent from, CellAddress fromAddress, Agent to, CellAddress toAddress );

    /**
     *  Tell subscribers that two agents are disconnecting. 
     * 
     * @param sender the object that wishes to send this notification out
     * @param from the agent the connection is coming from
     * @param to the agent the connection is going to
     */
    public void sendAgentsDisconnecting( Object sender, Agent from, Agent to );
    
    /** 
     * Tell subscribers that an agent is disconnecting from all. 
     * 
     * @param sender the object that wishes to send this notification out
     * @param from the agent which is disconnecting from all
     */
    public void sendAgentDisconnectingAll( Object sender, Agent from );

    /** 
     * Tell subscribes that an agent is messaging another. 
     *
     * @param sender the object that wishes to send this information out
     * @param from the agent which the message is coming from
     * @param to the agent which the message is going to
     */
    public void sendAgentMessage( Object sender, String message, Agent from, Agent to );

    /** 
     * Tell subscribers that we're about to go down. 
     *
     * @param sender the object that wishes to send this information out
     */
    public void sendCellGoingDown( Object sender );

    /** 
     * Tell subscribers that we're about to go down. 
     *
     * @param sender the object that wishes to send this information out
     */
    public void sendCellWithdrawing( Object sender );

    /** 
     * Tell subscribers of a report. 
     *
     * @param sender the object that wishes to send this information out
     * @param mess the message to display
     */
    public void sendCellReport( Object sender, String message );

}

