// $Id: ComponentManagerShadowImpl.java,v 1.1 2000/05/18 22:01:15 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.shadow.cell;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.JFrame;

import net.hivecell.hive.Global;
import net.hivecell.hive.agent.Agent;
import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.ShadowImpl;
import net.hivecell.hive.shadow.ShadowInitializationException;
import net.hivecell.hive.support.Debug;

public class ComponentManagerShadowImpl
extends ShadowImpl
implements Serializable, ComponentManagerShadow {

    protected Hashtable objectComponents = new Hashtable(); // map of object -> comp
    protected Hashtable frameOwners = new Hashtable(); // map of frame -> object
    protected Hashtable componentFrames = new Hashtable(); // map of comp -> frame

    public ComponentManagerShadowImpl() {
	super();
	
    }
    
    /**
     * just make ourselves ready for action
     */
    public void init() 
    throws ShadowInitializationException {
	setReady( true );
    }
    
    /**
     * helper function to create a frame for a specific agent with a
     * specific componenet
     *
     * @param o the object that the frame is for
     * @param c the component that it will contain
     * @param s the title of the frame
     * @return a frame 
     */
    private Frame constructFrame( final Object o, Component c, String s ) {

	final Frame f = new Frame( s );
	f.setFont( Global.defaultFont );
	f.add( c );
	f.pack();
	f.show();
	f.addWindowListener( new WindowAdapter() {
	    public void windowClosing( WindowEvent event ) {
		Debug.notice( "closing " + o.getClass().getName() + "'s frame, but not destroying the object" );
		f.dispose();

	    }
	});
	return f;

    }

    /**
     * helper function to create a JFrame for a specific agent with a
     * specific JComponent
     *
     * @param o the object the JFrame is for
     * @param c the JCompoent that it will contain
     * @param s the title of the JFrame
     * @return a JFrame 
     */
    private JFrame constructSwingFrame( final Object o, JComponent c, String s ) {

	final JFrame f = new JFrame( s );
	f.setFont( Global.defaultFont );
	f.getContentPane().add( c );
	f.pack();
	f.show();
	f.addWindowListener( new WindowAdapter() {
	    public void windowClosing( WindowEvent event ) {
		Debug.notice( "closing " + o.getClass().getName() + "'s frame, but not destroying the object" );
		f.dispose();
	    }
	});
	return f;

    }

    /**
     * ask the agennt component manager shadow to render a component
     * for a specific object
     *
     * @param o the object
     * @param c the component
     * @param s the title of the frame that it is going to be stuck in 
     */
    public Frame acceptComponentFrom( Object o, Component c, String s ) {

	
	// first create the frame -- determine whether to create a
	// jframe or a regular frame
	Frame f = null;

	if( c instanceof JComponent )
	    f = constructSwingFrame( o, (JComponent)c, s );
	else
	    f = constructFrame( o, c, s );
	
	// record teh component and the ownerx
	Vector components = (Vector)objectComponents.get( o );
	if( components == null ) {
	    components = new Vector();
	    objectComponents.put( o, components );
	}
	components.addElement( c );
	frameOwners.put( f, o );
	componentFrames.put( c, f );

	return f;

    }

    /** 
     * remove a component that an object has given us. only an object
     * can destroy it's own component. in this case, there is only one
     * component per object, but this might change in the future.
     *
     * @param o the object requesting the component be removed
     * @param c the component to remove
     */
    public void removeComponentFrom( Object o, Component c ) {

	// first remove the component
	Vector components = (Vector)objectComponents.get( o );
	if( components == null ) {
	    Debug.notice( o.getClass().getName() + " not found in list of objects which have components" );
	    return;
	}
	components.removeElement( c );

	// remove the frame
	Frame f = (Frame)componentFrames.get( c );
	f.setVisible( false );
	f.dispose();
	componentFrames.remove( c );
	frameOwners.remove( f );

    }

    /**
     * remove all the components for an object
     *
     * @param o the object
     */
    public void removeComponentsFrom( Object o ) {

	Vector components = (Vector)objectComponents.get( o );
	if( components != null ) {
	    
	    while( components.size() != 0 ) {
		Component c = (Component)components.firstElement();

		// remove the component
		components.removeElement( c );
		
		// remove the frame
		Frame f = (Frame)componentFrames.get( c );
		f.setVisible( false );
		f.dispose();
		componentFrames.remove( c );
		frameOwners.remove( f );
	    }

	}
	
	objectComponents.remove( o );

    }
    
}






