// $Id: IconLoader.java,v 1.1 2000/05/18 22:01:17 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support;

import java.io.InputStream;

public class IconLoader {

    /** Default icon for all agents. There should be no need to override this,
     * 
     * @see loadIconPPM
     */
    public static final String defaultIconName = "icons/UnknownAgent.ppm";

    /**  
     * Tries to retrieve the name of the icon If the agent is class
     * <code>net.hivecell.hive.FooAgent</code> then this looks for
     * <code>net.hivecell.hive/icons/FooAgent.ppm</code> in the class
     * path or Jar file.  If that doesn't exist, then it attempts to
     * get the icon for its superclass all the way up to AgentImpl
     * (unknownAgent.ppm)
     *
     * @param obj the class that we want to load -- this assumes that this is an agent class
     * @return an input stream containing the icon
     */
    public static SerializableImage loadIcon( Class obj ) {

	// turn "foo.bar.FooAgent" into "icons/FooAgent.ppm"
	InputStream is = null;
	String className = obj.getName();
	int nameStart = className.lastIndexOf( '.' ) + 1;
	
	// If this is the base Agent class, use the default Icon (UnknownAgent.ppm, usually)
	if( className.substring( nameStart ).equals( "AgentImpl" ) )
	    is = obj.getResourceAsStream( defaultIconName );

	else {
	    String faceName = new String( "icons/" + className.substring( nameStart ) + ".ppm" ); 	
	
	    // Open the icon file as a resource (uses classpath, base dir is class obj's base dir)
	    if( ( is = obj.getResourceAsStream( faceName ) ) == null ) {
		SerializableImage toReturn = null;
		if( ( toReturn = loadIcon( obj.getSuperclass() ) ) != null )
		    return toReturn;
	    }

	}	

	if( is == null ) {
	    // Note: if these warnings are ever thrown, something has
	    // gone very wrong... (UnknownAgent.ppm not in
	    // hive/agent/icons)
	    Debug.warning( "no icon found for " + obj.toString() + ", substituting " + defaultIconName );

	    is = obj.getResourceAsStream( defaultIconName );
	    if( is == null ) {
		Debug.critical( defaultIconName + " doesn't exist. agent icons are broken.  using blank icon." );
		return new SerializableImage( new int[]{ 1 }, new java.awt.Dimension( 1, 1 ) );
	    }

	}

	try {
	    return new PPM( is ); 
	} catch( Exception error ) {
	    Debug.error( "problem creating a ppm from the given input stream", error );
	    return new SerializableImage( new int[]{ 1 }, new java.awt.Dimension( 1, 1 ) );
	}

    }

}
