// $Id: ImageEvent.java,v 1.1 2000/05/18 22:01:17 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support;

/** This class embodies immutable messages that are sent from a an ImageSource to
 * it's listeners. It is a status and framenumber pair.
 * 
 */
public class ImageEvent implements java.io.Serializable {
    /** This status means that the frame is ready to be picked up.
     */
    public static final int READY = 0;

    /** This status means that the camera is currently fetching a frame
     * And will send a ready message shortly.
     */
    public static final int PENDING = 1;

    /** This status means that the camera ignored a request for a new image
     * but returns the index of the most recent image available.
     */
    public static final int IGNORED = 2;

    /** The status of the current message
     */
    protected int status;

    /** The framenumber that the message concerns.
     * 
     */
    protected int frameNum;

    /** This constructor creates a new CameraEvent.
     */
    public ImageEvent(int status, int frameNum) {
	this.status = status;
	this.frameNum = frameNum;
    }

    /** Returns the status of this event.
     */
    public int getStatus() {
	return status;
    }

    /** Returns the frame number of this event.
     */
    public int getFrameNum() {
	return frameNum;
    }
}
