// $Id: Queue.java,v 1.1 2000/05/18 22:01:17 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support;

/** This is a simple Queue (FIFO) data structure. It makes no attempt at type safety.
 * 
 */
public class Queue implements java.io.Serializable {
    
    /** The first element in the queue.
     */
    private Link head = null;
    /** The last element in the queue.
     */
    private Link tail = null;
    /** The number of elements in the queue.
     */
    private int size = 0;
    
    /** Return a simple string representation of the Queue.
     */
    public String toString() {
	return "Queue with " + size + " elements.";
    }
    
    /** 
     * Insert a new value after the tail of the list
     * 
     * @param n - the value to insert
     */
    public synchronized void enQueue(Object n) {
	Link p = new Link(n, null);
	if (head == null) {
	    head = tail = p;
	}
	else {
	    tail.next = p;
	    tail = p;
	}
	size++;
    }

    /** Remove an element from the head of the list.
     * 
     * @return The element that was removed, null if the queue was 
     *     empty.
     */
    public synchronized Object deQueue() {
	if (head == null) {
	    return null;
	}
	Link temp = head;
	if (tail == head) {
	    head = tail = null;
	}
	else {
	    head = head.next;
	}
	size--;
	temp.next = null; // This might seem unneccessary, but if you don't
	// zero out this link you can develop nasty memory leaks.
	return temp.data;
    }

    /** Return the number of elements in the queue.
     */
    public synchronized int getSize() {
	return size;
    }

    /** Returns true if the queue is empty.
     */
    public boolean isEmpty() {
	return (head == null);
    }

}

/** Internal data structure for the queue.
 */
class Link {
  /** The contents of this link in the queue.
   */
  Object data;
  /** The next element in the queue. Null if this is the last element.
   */
  Link next;

  /** Create a new Link
   * 
   * @param n Next Element
   * @param d Data
   */
  Link(Object d, Link n) {
    data = d;
    next = n;
  }

  /** Returns a string representation of the Link
   */
  public String toString() {
    return data.toString();
  }
}


