// $Id: VariablePropertyManipulatorGUI.java,v 1.1 2000/05/18 22:01:17 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support;

import net.hivecell.hive.cell.Cell;
import net.hivecell.hive.shadow.ShadowNotFoundException;
import net.hivecell.hive.shadow.cell.ComponentManagerShadow;

import java.awt.Button;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
import java.util.Hashtable;
import java.util.Enumeration;

public class VariablePropertyManipulatorGUI 
implements ActionListener {

    /**
     * this object is meant to be a "one-time" use.  so this flag
     * signals whether its been used already 
     */
    private boolean done = false;

    /**
     * the component manager so we can throw up a GUI 
     */
    private ComponentManagerShadow cms;

    /**
     * the manipulator we are using so we can play with this object
     */
    protected VariablePropertyManipulator vpm;

    
    /**
     * information on the components and the editors
     */
    protected Hashtable propertyEditors = new Hashtable();
    protected Hashtable components = new Hashtable();

    
    /**
     * the frame which hold our maniapultor
     */
    protected Frame f;
    
    /**
     * the panel on which everything happens
     */
    protected Panel editingPanel;



    /**
     * create a gui for this manipulator
     *
     * @param cell the cell we are operating on
     * @param o the object we wish to manipulate
     * @param stopClass how far down we want to go on the introspection
     */
    public VariablePropertyManipulatorGUI( ComponentManagerShadow cms, Object o, Class stopClass ) 
    throws VariablePropertyManipulationException {

	this.cms = cms;
	if( this.cms == null ) throw new VariablePropertyManipulationException( "there is no component manager shadow" );
	this.vpm = new VariablePropertyManipulator( o, stopClass );

	try {
	    String[] variableNames = this.vpm.getAccessibleVariableNames(); 
	    for( int count=0;count < variableNames.length;count++ ) {
		final PropertyEditor editor = PropertyEditorManager.findEditor( this.vpm.getAccessibleVariableType( variableNames[count] ) );

		if( editor != null ) {
		    Object value = this.vpm.getAccessibleVariable( variableNames[count] );
		    editor.setValue( value );
		    
		    // right now this class only supports a text like
		    // view -- hopefully you will only be trying to
		    // conifgure things which can be viewed in that
		    // mode -- else, write it yourself :)
		    if( editor.getAsText() != null ) {
			class propertyText 
			extends TextField 
			implements KeyListener, FocusListener {
			    public propertyText() {
				super( editor.getAsText(), 30 );
				addKeyListener( this );
				addFocusListener( this );
			    }

			    protected void updateEditor() {
				try {
				    editor.setAsText( getText() );
				} catch( IllegalArgumentException error ) { }
			    }

			    public void focusLost( FocusEvent fe ) {
				updateEditor();
			    }

			    public void keyReleased( KeyEvent ke ) {
				if( ke.getKeyCode() == KeyEvent.VK_ENTER )
				    updateEditor();
			    }

			    public void focusGained( FocusEvent fe ) { }
			    public void keyPressed( KeyEvent ke ) { }
			    public void keyTyped( KeyEvent ke ) { }
			}
			Component c = new propertyText();
			
			propertyEditors.put( variableNames[count], editor );
			components.put( variableNames[count], c );
			
		    }
		}
		
	    }

	    if( components.size() != 0 ) {
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		editingPanel = new Panel( gbl ); 
	    
		Enumeration e = components.keys();
		int yPosition = 0;
		while( e.hasMoreElements() ) {
		    String variableName = (String)e.nextElement();

		    Label variableLabel = new Label( variableName );
		    gbc.gridx = 0;
		    gbc.gridy = yPosition;
		    gbc.gridwidth = 1;
		    gbc.gridheight = 1;
		    gbc.fill = GridBagConstraints.NONE;
		    gbc.anchor = GridBagConstraints.WEST;
		    gbc.insets = new Insets( 2, 7, 2, 7 );
		    gbl.setConstraints( variableLabel, gbc );
		    editingPanel.add( variableLabel );

		    Component c = (Component)components.get( variableName );
		    gbc.gridx = 1;
		    gbc.gridy = yPosition++;
		    gbc.gridwidth = 1;
		    gbc.gridheight = 1;
		    gbc.fill = GridBagConstraints.BOTH;
		    gbc.anchor = GridBagConstraints.CENTER;
		    gbc.insets = new Insets( 2, 7, 2, 7 );
		    gbl.setConstraints( c, gbc );
		    editingPanel.add( c );

		}

		Button doneButton = new Button( "Set Variables" );
		doneButton.addActionListener( this );
		gbc.gridx = 0;
		gbc.gridy = yPosition;
		gbc.gridwidth = 2;
		gbc.insets = new Insets( 1, 2, 1, 2 );
		gbl.setConstraints( doneButton, gbc );
		editingPanel.add( doneButton );

		cms.acceptComponentFrom( this, editingPanel, "Configure " + o );

	    }

	} catch( UnknownVariablePropertyName error ) {
	    // this will never be thrown -- so just ignore it
 
	}

    }

    /**
     * create a gui for this manipulator
     *
     * @param cell the cell we are operating on
     * @param o the object we wish to manipulate
     */
    public VariablePropertyManipulatorGUI( ComponentManagerShadow cms, Object o ) 
    throws VariablePropertyManipulationException {
	this( cms, o, null );
    }

    /**
     * hande the pushing of the configure button
     *
     * @param ae the action event
     */
    public void actionPerformed( ActionEvent ae ) {

	if( done ) {
	    Debug.error( "this manipulator has already been used" );
	    return;
	}
	
	cms.removeComponentFrom( this, editingPanel );
	Enumeration e = propertyEditors.keys();
	try {
	    while( e.hasMoreElements() ) {
		
		String variableName = (String)e.nextElement();
		PropertyEditor pe = (PropertyEditor)propertyEditors.get( variableName );
		Object value = pe.getValue();
		try {
		    this.vpm.setAccessibleVariable( variableName, value );
		} catch( VariablePropertyManipulationException error ) {
		    Debug.error( "error setting " + variableName + " to " + value );
		}
		
	    }
	} catch( UnknownVariablePropertyName error ) {
	    // this should never happen -- so ignore it so we don't generate code

	}

	// set this object as used and release anybody waiting on it
	this.done = true;
	synchronized( this ) {
	    notifyAll();
	}

    }

    /**
     * test to see if this manipulator has already been used
     *
     * @return true if it has configured an object already, or not
     */
    public boolean isUsed() { return done; }

    /**
     * this allows us to lock on the object to wait until it has
     * configured its object 
     */
    public void lockUntilUsed() {

	while( !isUsed() ) {
	    try {
		synchronized( this ) {
		    wait();
		}
	    } catch( InterruptedException error ) {
	    }
	}

    }

    

}

