// $Id: AsyncCallerClass.java,v 1.1 2000/05/18 22:01:20 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support.async;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

import java.rmi.ServerRuntimeException;

public abstract class AsyncCallerClass {

    /**
     * Make the asynchronous call.  This method call should not spawn
     * off a watching thread.
     *
     * @param receiver the object to do the call on
     * @param call the specifics of the call
     * @return the async result of this call 
     */
    public abstract AsyncResult makeCall( Object receiver, AsyncCall call );

    /**
     * Make the asynchronous call with an optional watching thread.
     * If the time given is non-negative, then the watching thread
     * should be spawned off.  If the watching thread comkpletes
     * before the calling thread, then return a
     * AsyncCallNotCompleteException through the AsyncResult
     *
     * @param receiver the object to do the call on
     * @param call the specifics of the call
     * @param millis the number of milliseconds until we should just
     * give up and return a AsyncCallNotCompleteException
     * @return the async result of this call 
     *
     * @see AsyncCallNotCompleteException
     */
    public abstract AsyncResult makeCall( final Object receiver, final AsyncCall call, final long millis );

    /**
     * This method must be overridden by new implementation authors!
     * Since nelson wanted this to be a singleton class, we needed a
     * simple way to get at the instance of the class to use, so this
     * is it.  A static method can't be marked abstract, so just make
     * sure to override this otherwise you will get bizzare null
     * pointer exceptions.  Have this return the singleton -- for
     * examples, see AsyncCaller, or AsyncOneMethodCaller
     *
     * @return the singleton instance to use for the async calling
     *
     * @see AsyncCaller, AsyncOneMethodCaller 
     */
    public static AsyncCallerClass getInstance() { 
	
	return null;

    }

    /**
     * This method should return true if this instance of an async
     * messenger has finished sending all the messages it has been
     * asked to
     *
     * @return true if we have finished sending all the messages we
     * have been asked to 
     */
    public abstract boolean doneWithCalls();

    /**
     * This method should block until all the messages in this caller
     * have been finished
     */
    public abstract void blockUntilDone();

}




