// $Id: AsyncResult.java,v 1.1 2000/05/18 22:01:20 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support.async;

/**
 * The result of the asynchronous call we make
 */
public interface AsyncResult {
    
    /** 
     * Get the object we called on 
     *
     * @return the reference to the object we made the call on
     */
    public Object getReceiver();

    /** 
     * Get the AsyncCall object that we used to call the receiver 
     *
     * @return the AsyncCall object that did the call to get this
     */
    public AsyncCall getCall();
    

    /**
     * Return whether or not we have the result -- this does not block
     *
     * @return true if we have a result, false otherwise
     */
    public boolean isFinished();
    
    /**
     * Return whether or not the result of the call was an exception
     * or not -- this method blocks.  This blocks because there may be
     * a case int which the method call returns void, and we just need
     * to capture any exceptions.  To get around this first call
     * isFinished() and if that returns true, then you can test for an
     * exception
     *
     * @return true if this is an exception */
    public boolean wasException();

    /**
     * The result of the method call.  Could be a Void object, it
     * could be an exception, or anything else this could possibly
     * return.  This method will block, so call isFinished() first.
     *
     * @return the result of running the method 
     */
    public Object getResult();
    

    /**
     * Add a listener which will recieve notices when isFinished()
     * will be returning true
     *
     * @param arl the AsyncResultListener which will recieve the notices 
     */
    public void addListener( AsyncResultListener arl );

    /**
     * Similar to addListener, except these AsyncResultListeners will
     * not get all the events, but they will get the notice if the
     * AsyncResult is an exception
     *
     * @param arl the AsyncResultListener which will recieve only
     * exception notices 
     */
    public void addExceptionOnlyListener( AsyncResultListener arl );

}
