// $Id: GraphDifferenceHelper.java,v 1.1 2000/05/18 22:01:21 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support.graph;

import java.util.Enumeration;
import java.util.Vector;

/**
 * this is a class of static methods which are just useful to help out
 * when we need to know the "difference" between two graphs.  this
 * will help us tell the list of agents which have disappeared, the
 * list of agents that have been added, the list of connections that
 * have disappeared, and the list of connections that have been added 
 */
public class GraphDifferenceHelper {

    /**
     * get a list of all the objects that have been added to the
     * newGraph (ie -- are not in the old graph)
     *
     * @param oldGraph the base graph
     * @param newGraph the graph that we want to see what has been added to
     * @return an array of the objects that have been added 
     */
    public static Object[] getAddedObjects( Graph oldGraph, Graph newGraph ) {

	Vector addedObjects = new Vector();

	Enumeration e = newGraph.nodes();
	while( e.hasMoreElements() ) {
	    Object o = e.nextElement();
	    if( !oldGraph.containsNode( o ) )
		addedObjects.addElement( o );
	}

	Object[] objects = new Object[addedObjects.size()];
	addedObjects.copyInto( objects );
	return objects;

    }

    /**
     * get a list of objects which are in the old graph but not in the
     * new graph.  this is just a shortcut that will make the code
     * look like it makes sense -- in reality you just need to switch
     * the order in which we are testing the graphs in getAddedObjects
     *
     * @param oldGraph the reference graph
     * @param newGraph the graph that we want to see what is missing from
     * @return an array of the objects which have been removed 
     */
    public static Object[] getRemovedObjects( Graph oldGraph, Graph newGraph ) { return getAddedObjects( newGraph, oldGraph ); }

    /**
     * get a list of all the connections which have been made to this graph
     *
     * @param oldGraph the reference graph
     * @param newGraph the graph that we want to see what connections have been added
     * @return an array of the connections that have been added to this graph
     */
    public static Edge[] getAddedConnections( Graph oldGraph, Graph newGraph ) {

	Vector addedEdges = new Vector();

	try {
	    Enumeration e = newGraph.edges();
	    while( e.hasMoreElements() ) {
		Edge edge = (Edge)e.nextElement();
		if( oldGraph.containsNode( edge.getSource() ) && oldGraph.containsNode( edge.getDestination() ) ) {
		    Object[] outgoing = oldGraph.listOutgoingNodes( edge.getSource() );
		    boolean found = false;
		    for( int count=0;count < outgoing.length;count++ ) 
			if( outgoing[count] == edge.getDestination() ) {
			    found = true;
			    break;
			}
		    if( !found )
		    addedEdges.addElement( edge );
		}
		
	    }
	} catch( NotInGraphException error ) {
	    // this should never happen
	}

	Edge[] edges = new Edge[addedEdges.size()];
	addedEdges.copyInto( edges );
	return edges;

    }

    /**
     * get a list of all the edges which have been removed.  thsi is
     * the getAddedConnections method just in reverse
     *
     * @param oldGraph the reference graph for this comparison
     * @param newGraph the graph to make the comparison to 
     */
    public static Edge[] getRemovedConnections( Graph oldGraph, Graph newGraph ) { return getAddedConnections( newGraph, oldGraph ); }

}


