// $Id: Node.java,v 1.1 2000/05/18 22:01:21 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support.graph;

import java.io.Serializable;


public class Node
extends Object
implements Serializable {

    /**
     * The object that is in this node
     */
    private Object nodeObject;

    /**
     * The graph that this node belongs to
     */
    private Graph graph;

    /**
     * The indexing number for the adjacency matrix
     */
    private int indexNumber;


    /**
     * Construct the node object for a specific graph
     *
     * @param o the object that we are putting into the graph
     * @param g the graph this node belongs to
     * @param indexNumber the indexing number for the graph
     */
    public Node( Object o, Graph g, int indexNumber ) {

	this.nodeObject = o;
	this.graph = g;
	this.indexNumber = indexNumber;

    }

    /**
     * Get the object from this node
     *
     * @return the object from this node
     */
    public Object getObject() { return this.nodeObject; }

    /**
     * Get the graph that this Node belongs to
     *
     * @return the graph this node belongs to
     */
    public Graph getGraph() { return this.graph; }

    /**
     * Return the index number that we use for the adjacency matrix
     *
     * @return the index number
     */
    public int getIndexNumber() { return this.indexNumber; }

    public boolean equals( Object o ) {
	
	return ( ( o instanceof Node ) && 
		 ( ((Node)o).getObject().equals( this.getObject() ) ) && 
		 ( ((Node)o).getIndexNumber() == this.getIndexNumber() ) &&
		 ( ((Node)o).getGraph() == this.getGraph() ) );

    }

}


