// $Id: MenuFolder.java,v 1.1 2000/05/18 22:01:22 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support.menu;

import java.util.Vector;
import java.util.Enumeration;

/**
 ** This allows the menu creation of menus which encompass other menu
 ** objects 
 **/
public class MenuFolder  
extends TopMenu
implements MenuObject {

    protected String name;

    public MenuFolder( String name ) { this.name = name; }

    public String getName() { return this.name; }

    public static MenuFolder mergeMenus(MenuFolder base, MenuFolder mergein) {

	if (base == null) return (mergein);
	if (mergein == null) return (base);

	MenuFolder merged = new MenuFolder(base.getName());

	// Step throught the MenuObject base and add each MenuObject to merged
	Enumeration e = base.getMenuItems();
	while (e.hasMoreElements())
	    merged.addMenuElement((MenuElement)e.nextElement());


	// Notice how large it is so we can determine whether or not to insert a separator between the new elements.
	int size = merged.getMenuElementSize();

	// Step through each MenuObject that is a MenuItem in mergein.  If base contains the same 
	// MenuItem on its top (current) level, don't add it to merged. Otherwise, add it.
	e = mergein.getMenuItems();
	while (e.hasMoreElements()) {
	    MenuObject me = (MenuObject)e.nextElement();

	    if (me instanceof MenuItem) {
		if (base.indexOfMenuElement(me.getName()) == -1)  // If the item is not present in base
		    merged.addMenuElement(me);                    // add it
	    }


	    // Step through each MenuObject that is a MenuFolder in mergein.  If base contains the same 
	    // MenuFolder on its top (current) level, then add a separator to merged, and recursively call 
	    // MergeMenus on the two folders.
	    if (me instanceof MenuFolder) {
		
		int index = base.indexOfMenuElement(me.getName());
		if (index == -1)                                               // If the folder isn't in the base
		    merged.addMenuElement(me);                                 // add it
		
		else if (base.getMenuElementAt(index) instanceof MenuFolder) { // Otherwise, merge the two, if they're both folders.
		    //merged.insertMenuElementAt( new MenuSeparator(), index );
		    merged.replaceMenuElementAt( mergeMenus( (MenuFolder)base.getMenuElementAt(index), (MenuFolder)me ),
						 index );
		}
		
		else merged.addMenuElement(me);                                // Or just add it if it isn't a folder.
	    }
	}


	// If any new menu items were added, insert a menu separator.
	if (merged.getMenuElementSize() > size)
	    merged.insertMenuElementAt(new MenuSeparator(), size);

	return (merged);
    }

}

