// $Id: TopMenu.java,v 1.1 2000/05/18 22:01:22 tucker Exp $
// Hive. Copyright (c) 1998-1999, The Massachusetts Institute of Technology.
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package net.hivecell.hive.support.menu;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * this is the top level menu to contain all the other submenu types
 * -- this class provides the abstract representation of the menus 
 */
public class TopMenu {

    /**
     * a list of MenuObjects which represents the things that go in the
     * top menu 
     */
    private Vector menuItems = new Vector();


    public TopMenu() { }
    
    /**
     * add a menu object to the top level menu
     *
     * @param mo the menu object that you want to add
     */
    public void addMenuElement( MenuElement mo ) { menuItems.addElement( mo ); }
    
    /**
     * add a menu object to an arbitrary location
     *
     * @param me the menu element to add
     * @param position the positon in which we wish to add it
     */
    public void insertMenuElementAt( MenuElement me, int position ) { menuItems.insertElementAt( me, position ); }

    /**
     * replace a menu object 
     *
     * @param me the menu element to add
     * @param position the positon in which we wish to add it
     */
    public synchronized void replaceMenuElementAt( MenuElement me, int position ) { 
	menuItems.removeElementAt( position );
	menuItems.insertElementAt( me, position ); 
    }

    /**
     * replace a menu object 
     *
     * @param menuItem the name of the menu item we want to replace
     * @param me the menu element to add
     */
    public synchronized void replaceMenuElement( String menuItem, MenuElement me ) { 
	int i = indexOfMenuElement( menuItem );
	if( i >= 0 )
	    replaceMenuElementAt( me, i );
    }

    /**
     * get the amount of menu items that are on this menu
     *
     * @return the amount of menu objects
     */
    public int getMenuElementSize() { return menuItems.size(); }

    /**
     * return the menu item at a specific position
     *
     * @param position the position of the menu that we want to get the object for
     * @return the menu object that is at that position
     */
    public MenuElement getMenuElementAt( int position ) { return (MenuElement)menuItems.elementAt( position ); }
    
    /**
     * return the first menu element we find with the given name
     *
     * @param name the name that we are searching for
     * @return the menu object that we are looking for
     */
    public MenuElement getMenuElement( String name ) {
	Enumeration e = getMenuItems();
	while( e.hasMoreElements() ) {
	    MenuElement me = (MenuElement)e.nextElement();
	    if( ( me instanceof MenuObject ) && ( ((MenuObject)me).getName().equals( name ) ) )
		return me;
	}
	return null;
    }

    /**
     * return which index menu element has this given name
     *
     * @param name the name that we are searching for
     * @return the index number of the menu object or -1 if the object is not there
     */
    public int indexOfMenuElement( String name ) {
	int count = 0;
	Enumeration e = getMenuItems();
	while( e.hasMoreElements() ) {
	    MenuElement me = (MenuElement)e.nextElement();
	    if( ( me instanceof MenuObject ) && ( ((MenuObject)me).getName().equals( name ) ) )
		return count;
	    count++;
	}
	return -1;
    }
    

    /**
     * return an ordered Enumeration of all the MenuObjects in this menu
     *
     * @return an ordered enumeration of all the MenuObjects
     */
    public Enumeration getMenuItems() {
	return new Enumeration() {
	    int position = 0;

	    public boolean hasMoreElements() {
		if( position < menuItems.size() ) return true;
		return false;
	    }

	    public Object nextElement() {
		return menuItems.elementAt( position++ );
	    }
	};
    }

}

